/*
********************************************************************************
**  SAE J1699-5 Test Source Code
**
**  Copyright (C) 2002 Drew Technologies. http://j1699-3.sourceforge.net/
**
** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  This source code, when compiled and used with an SAE J2534-compatible pass
**  thru device, is intended to run the tests described in the SAE J1699-5
**  document in an automated manner.
**
**  This computer program is based upon SAE Technical Report J1699,
**  which is provided "AS IS"
**
**  See j1699.c for details of how to build and run this test.
**
********************************************************************************
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <windows.h>
#include "j2534.h"
#include "j1699.h"


BOOL  gJ2534_2_Mode = FALSE;


/*
*****************************************************************************
**  ConnectProtocol - Function to connect to a protocol
*****************************************************************************
*/
STATUS ConnectProtocol ( void )
{
	PASSTHRU_MSG  MaskMsg;
	PASSTHRU_MSG  PatternMsg;
	PASSTHRU_MSG  FlowMsg;
	SCONFIG_LIST  ConfigList;
	SCONFIG       ConfigParameter;
	unsigned long RetVal;
	unsigned long EcuIndex;
	unsigned long InitFlags;
	BOOL          ResetEcuIndex;

	unsigned short IgnoreEDUIndex;
	BOOL           IgnoreECUId;

	// Set the request delay, maximum response time and init flags according to the protocol
	InitFlags = 0;

	// 50msec for ISO15765
	gOBDRequestDelay = 50;
	gOBDMaxResponseTimeMsecs = 50;
	gOBDMinResponseTimeMsecs = 0;  // for min time verification
	InitFlags = (gOBDList[gOBDListIndex].InitFlags & CAN_29BIT_ID);
	if ( gJ2534_2_Mode )
	{
		InitFlags |= ISO15765_ADDR_TYPE;
	}

	// Connect to protocol
	RetVal = PassThruConnect ( gulDeviceID,
	                           gOBDList[gOBDListIndex].Protocol,
	                           InitFlags,
	                           gOBDList[gOBDListIndex].BaudRate,
	                           &gOBDList[gOBDListIndex].ChannelID );
	if ( RetVal != STATUS_NOERROR )
	{
		Log ( J2534_FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "%s returned %ld",
		      "PassThruConnect",
		      RetVal );
		return FAIL;
	}

	// Turn on LOOPBACK to see transmitted messages
	ConfigList.NumOfParams = 1;
	ConfigList.ConfigPtr = &ConfigParameter;
	ConfigParameter.Parameter = LOOPBACK;
	ConfigParameter.Value = 1;
	RetVal = PassThruIoctl ( gOBDList[gOBDListIndex].ChannelID,
	                         SET_CONFIG,
	                         &ConfigList,
	                         NULL );
	if ( RetVal != STATUS_NOERROR )
	{
		Log ( J2534_FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "%s returned %ld",
		      "PassThruIoctl(LOOPBACK)",
		      RetVal );
		return FAIL;
	}


	// IF in J2534_2 mode, set CAN_MIXED_FORMAT to CAN_MIXED_FORMAT_ON
	if ( gJ2534_2_Mode )
	{
		ConfigList.NumOfParams = 1;
		ConfigList.ConfigPtr = &ConfigParameter;
		ConfigParameter.Parameter = CAN_MIXED_FORMAT;
		ConfigParameter.Value = CAN_MIXED_FORMAT_ON;
		RetVal = PassThruIoctl ( gOBDList[gOBDListIndex].ChannelID,
		                         SET_CONFIG,
		                         &ConfigList,
		                         NULL );
		if ( RetVal != STATUS_NOERROR )
		{
			Log ( J2534_FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "%s returned %ld",
			      "PassThruIoctl(CAN_MIXED_FORMAT)",
			      RetVal );
			return FAIL;
		}
	}

	// Setup the node IDs, functional message ID and filter(s)
	// Handle both 11-bit and 29-bit ID cases
	if ( gOBDList[gOBDListIndex].InitFlags & CAN_29BIT_ID )
	{
		// Setup ISO15765 flow control filters
		MaskMsg.ProtocolID = gOBDList[gOBDListIndex].Protocol;
		MaskMsg.TxFlags = ISO15765_FRAME_PAD | CAN_29BIT_ID;
		MaskMsg.DataSize = 4;
		MaskMsg.Data[0] = 0xFF;
		MaskMsg.Data[1] = 0xFF;
		MaskMsg.Data[2] = 0xFF;
		MaskMsg.Data[3] = 0xFF;
		PatternMsg.ProtocolID = gOBDList[gOBDListIndex].Protocol;
		PatternMsg.TxFlags = ISO15765_FRAME_PAD | CAN_29BIT_ID;
		PatternMsg.DataSize = 4;
		PatternMsg.Data[0] = 0x18;
		PatternMsg.Data[1] = 0xDA;
		PatternMsg.Data[2] = 0xF1;
		PatternMsg.Data[3] = 0x00;
		FlowMsg.ProtocolID = gOBDList[gOBDListIndex].Protocol;
		FlowMsg.TxFlags = ISO15765_FRAME_PAD | CAN_29BIT_ID;
		FlowMsg.DataSize = 4;
		FlowMsg.Data[0] = 0x18;
		FlowMsg.Data[1] = 0xDA;
		FlowMsg.Data[2] = 0x00;
		FlowMsg.Data[3] = 0xF1;


		ResetEcuIndex = FALSE;
		// Setup a flow control filter for each ECU that responded to SID1 PID0
		for ( EcuIndex = 0; EcuIndex < gOBDNumEcusCan || ResetEcuIndex; EcuIndex++ )
		{
			if ( ResetEcuIndex )
			{
				EcuIndex = 0;
				ResetEcuIndex = FALSE;
			}

			IgnoreECUId = FALSE;
			for ( IgnoreEDUIndex = 0; IgnoreEDUIndex < gECU_Ignore_List_Count; IgnoreEDUIndex++ )
			{
				if ( gOBDResponseTA[EcuIndex] == (gECU_Ignore_List[IgnoreEDUIndex]&0x000000FF) )
				{
					IgnoreECUId = TRUE;
					break;
				}
			}

			// IF the ECU ID is not to be ignored, setup flow control
			if ( IgnoreECUId == FALSE )
			{
				PatternMsg.Data[3] = gOBDResponseTA[EcuIndex];
				FlowMsg.Data[2] = gOBDResponseTA[EcuIndex];
				RetVal = PassThruStartMsgFilter ( gOBDList[gOBDListIndex].ChannelID,
				                                  FLOW_CONTROL_FILTER,
				                                  &MaskMsg,
				                                  &PatternMsg,
				                                  &FlowMsg,
				                                  &gOBDList[gOBDListIndex].FlowFilterID[EcuIndex] );


				// If trying to define too many flow controls...
				if ( RetVal == ERR_EXCEEDED_LIMIT )
				{
					// if currently J2534-1 mode, try J2534-2 mode
					if ( !gJ2534_2_Mode )
					{
						Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
						      "J2534-1 mode will not support required number of ECUs.\n"
						      "Atempting J2534-2 mode.\n" );

						// Disconnect Protocol
						DisconnectProtocol ( );

						// Close J2534 device
						PassThruClose ( gulDeviceID );

						// Try J2534-2 mode
						RetVal = PassThruOpen ( "J2534-2:", &gulDeviceID );
						if ( RetVal != STATUS_NOERROR )
						{
							Log ( J2534_FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
							      "%s returned %ld",
							      "PassThruOpen(J2534-2:)",
							      RetVal );
							return FAIL;
						}


						// restart flow control definitions
						ResetEcuIndex = TRUE;
						// Mode 2
						gJ2534_2_Mode = TRUE;


						// Re-Connect to ISO15765 Protocol
						gOBDRequestDelay = 50;
						gOBDMaxResponseTimeMsecs = 50;
						gOBDMinResponseTimeMsecs = 0;
						InitFlags = (gOBDList[gOBDListIndex].InitFlags & CAN_29BIT_ID);
						InitFlags |= ISO15765_ADDR_TYPE;
						RetVal = PassThruConnect ( gulDeviceID,
						                           gOBDList[gOBDListIndex].Protocol,
						                           InitFlags,
						                           gOBDList[gOBDListIndex].BaudRate,
						                           &gOBDList[gOBDListIndex].ChannelID );
						if ( RetVal != STATUS_NOERROR )
						{
							Log ( J2534_FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
							      "%s returned %ld",
							      "PassThruConnect",
							      RetVal );
							return FAIL;
						}

						// Re-Configure ISO15765 Protocol
						// Turn on LOOPBACK to see transmitted messages
						ConfigList.NumOfParams = 1;
						ConfigList.ConfigPtr = &ConfigParameter;
						ConfigParameter.Parameter = LOOPBACK;
						ConfigParameter.Value = 1;
						RetVal = PassThruIoctl ( gOBDList[gOBDListIndex].ChannelID,
						                         SET_CONFIG,
						                         &ConfigList,
						                         NULL );
						if ( RetVal != STATUS_NOERROR )
						{
							Log ( J2534_FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
							      "%s returned %ld",
							      "PassThruIoctl(LOOPBACK)",
							      RetVal );
							return FAIL;
						}

						// Set CAN_MIXED_FORMAT to CAN_MIXED_FORMAT_ON
						ConfigList.NumOfParams = 1;
						ConfigList.ConfigPtr = &ConfigParameter;
						ConfigParameter.Parameter = CAN_MIXED_FORMAT;
						ConfigParameter.Value = CAN_MIXED_FORMAT_ON;
						RetVal = PassThruIoctl ( gOBDList[gOBDListIndex].ChannelID,
						                         SET_CONFIG,
						                         &ConfigList,
						                         NULL );
						if ( RetVal != STATUS_NOERROR )
						{
							Log ( J2534_FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
							      "%s returned %ld",
							      "PassThruIoctl(CAN_MIXED_FORMAT)",
							      RetVal );
							return FAIL;
						}

					}
					else
					{
						Log ( J2534_FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
						      "J2534-2 mode will not support required number of ECUs.\n"
						      "%s returned %ld",
						      "PassThruStartMsgFilter",
						      RetVal );
						return FAIL;
					}
				}
				else if ( RetVal != STATUS_NOERROR )
				{
					Log ( J2534_FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "%s returned %ld",
					      "PassThruStartMsgFilter",
					      RetVal );
					return FAIL;
				}
			}
		}
	}
	else
	{
		// Setup ISO15765 flow control filters
		MaskMsg.ProtocolID = gOBDList[gOBDListIndex].Protocol;
		MaskMsg.TxFlags = ISO15765_FRAME_PAD;
		MaskMsg.DataSize = 4;
		MaskMsg.Data[0] = 0xFF;
		MaskMsg.Data[1] = 0xFF;
		MaskMsg.Data[2] = 0xFF;
		MaskMsg.Data[3] = 0xFF;
		PatternMsg.ProtocolID = gOBDList[gOBDListIndex].Protocol;
		PatternMsg.TxFlags = ISO15765_FRAME_PAD;
		PatternMsg.DataSize = 4;
		PatternMsg.Data[0] = 0x00;
		PatternMsg.Data[1] = 0x00;
		PatternMsg.Data[2] = 0x00;
		PatternMsg.Data[3] = 0x00;
		FlowMsg.ProtocolID = gOBDList[gOBDListIndex].Protocol;
		FlowMsg.TxFlags = ISO15765_FRAME_PAD;
		FlowMsg.DataSize = 4;
		FlowMsg.Data[0] = 0x00;
		FlowMsg.Data[1] = 0x00;
		FlowMsg.Data[2] = 0x07;
		FlowMsg.Data[3] = 0xDF;

		RetVal = PassThruStartMsgFilter ( gOBDList[gOBDListIndex].ChannelID,
		                                  FLOW_CONTROL_FILTER, &MaskMsg, &PatternMsg, &FlowMsg,
		                                  &gOBDList[gOBDListIndex].FlowFilterID[FlowMsg.Data[3] & 0x07] );
		if ( RetVal != STATUS_NOERROR )
		{
			Log ( J2534_FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "%s returned %ld",
			      "PassThruStartMsgFilter",
			      RetVal );
			return FAIL;
		}


		// Setup ISO15765 flow control filters
		MaskMsg.ProtocolID = gOBDList[gOBDListIndex].Protocol;
		MaskMsg.TxFlags = ISO15765_FRAME_PAD;
		MaskMsg.DataSize = 4;
		MaskMsg.Data[0] = 0xFF;
		MaskMsg.Data[1] = 0xFF;
		MaskMsg.Data[2] = 0xFF;
		MaskMsg.Data[3] = 0xFF;
		PatternMsg.ProtocolID = gOBDList[gOBDListIndex].Protocol;
		PatternMsg.TxFlags = ISO15765_FRAME_PAD;
		PatternMsg.DataSize = 4;
		PatternMsg.Data[0] = 0x00;
		PatternMsg.Data[1] = 0x00;
		PatternMsg.Data[2] = 0x07;
		PatternMsg.Data[3] = 0xE8;
		FlowMsg.ProtocolID = gOBDList[gOBDListIndex].Protocol;
		FlowMsg.TxFlags = ISO15765_FRAME_PAD;
		FlowMsg.DataSize = 4;
		FlowMsg.Data[0] = 0x00;
		FlowMsg.Data[1] = 0x00;
		FlowMsg.Data[2] = 0x07;
		FlowMsg.Data[3] = 0xE0;

		// Setup flow control filters for all allowable 11-bit OBD ID values
		for ( FlowMsg.Data[3] = 0xE0; FlowMsg.Data[3] < 0xE8; FlowMsg.Data[3]++, PatternMsg.Data[3]++ )
		{
			IgnoreECUId = FALSE;
			for ( IgnoreEDUIndex = 0; IgnoreEDUIndex < gECU_Ignore_List_Count; IgnoreEDUIndex++ )
			{
				if ( ((PatternMsg.Data[2]<<8) + PatternMsg.Data[3]) == (unsigned short)(gECU_Ignore_List[IgnoreEDUIndex]) )
				{
					IgnoreECUId = TRUE;
					break;
				}
			}

			// IF the ECU ID is not to be ignored, setup flow control
			if ( IgnoreECUId == FALSE )
			{
				RetVal = PassThruStartMsgFilter ( gOBDList[gOBDListIndex].ChannelID,
				                                  FLOW_CONTROL_FILTER,  &MaskMsg, &PatternMsg, &FlowMsg,
				                                  &gOBDList[gOBDListIndex].FlowFilterID[FlowMsg.Data[3] & 0x07] );
				if ( RetVal != STATUS_NOERROR )
				{
					Log ( J2534_FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "%s returned %ld",
					      "PassThruStartMsgFilter",
					      RetVal );
					return FAIL;
				}
			}
		}
	}

	// Setup tester present keep alive message using PID $F400
	gTesterPresentMsg.ProtocolID = gOBDList[gOBDListIndex].Protocol;
	gTesterPresentMsg.DataSize = 7;
	if ( gOBDList[gOBDListIndex].InitFlags & CAN_29BIT_ID )
	{
		gTesterPresentMsg.TxFlags = ISO15765_FRAME_PAD | CAN_29BIT_ID;
		gTesterPresentMsg.Data[0] = 0x18;
		gTesterPresentMsg.Data[1] = 0xDB;
		gTesterPresentMsg.Data[2] = 0x33;
		gTesterPresentMsg.Data[3] = TESTER_NODE_ADDRESS;
	}
	else
	{
		gTesterPresentMsg.TxFlags = ISO15765_FRAME_PAD;
		gTesterPresentMsg.Data[0] = 0x00;
		gTesterPresentMsg.Data[1] = 0x00;
		gTesterPresentMsg.Data[2] = 0x07;
		gTesterPresentMsg.Data[3] = 0xDF;
	}
	gTesterPresentMsg.Data[4] = 0x22;  // SID
	gTesterPresentMsg.Data[5] = 0xF4;  // DID MSB
	gTesterPresentMsg.Data[6] = 0x00;  // DID LSB

	return PASS;
}


/*
*******************************************************************************
**  StartPeriodicMsg - Function to start tester present message
*******************************************************************************
*/
static BOOL bPeriocicActive = FALSE;

STATUS StartPeriodicMsg ( void )
{
	unsigned long RetVal;

	if ( gPeriodicMsgEnabled == FALSE )
	{
		return PASS;
	}

#ifdef _DEBUG
	Log ( INFORMATION, SCREENOUTPUTOFF, LOGOUTPUTON, NO_PROMPT,
	      "Enter StartPeriodicMsg - STATE %s, ID %x\n",
	      (bPeriocicActive == TRUE) ? "ON" : "OFF",
	      gOBDList[gOBDListIndex].TesterPresentID );

	if ( bPeriocicActive == TRUE )
	{
		Log ( INFORMATION, SCREENOUTPUTOFF, LOGOUTPUTON, NO_PROMPT,
		      "StartPeriodicMsg:: periodic already active - ID: %u\n",
		      gOBDList[gOBDListIndex].TesterPresentID );
	}
#endif



	RetVal = PassThruStartPeriodicMsg ( gOBDList[gOBDListIndex].ChannelID,
	                                    &gTesterPresentMsg,
	                                    &gOBDList[gOBDListIndex].TesterPresentID,
	                                    OBD_TESTER_PRESENT_RATE );

	if ( RetVal != STATUS_NOERROR )
	{
		Log ( J2534_FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "%s returned %ld",
		      "PassThruStartPeriodicMsg",
		      RetVal );
		return FAIL;
	}

	bPeriocicActive = TRUE;



#ifdef _DEBUG
	Log ( INFORMATION, SCREENOUTPUTOFF, LOGOUTPUTON, NO_PROMPT,
	      "Leave StartPeriodicMsg - STATE %s, ID %x\n",
	      (bPeriocicActive == TRUE) ? "ON" : "OFF",
	      gOBDList[gOBDListIndex].TesterPresentID );
#endif



	return PASS;
}


/*
*****************************************************************************
**  StopPeriodicMsg - Function to stop tester present message
*****************************************************************************
*/
STATUS StopPeriodicMsg ( BOOL bLogError )
{
	unsigned long RetVal;

	if ( gPeriodicMsgEnabled == FALSE )
	{
		return PASS;
	}

#ifdef _DEBUG
	// Leave this in for now... for debugging purposes
	Log ( INFORMATION, SCREENOUTPUTOFF, LOGOUTPUTON, NO_PROMPT,
	      "Enter StopPeriodicMsg - STATE %s, ID %x\n",
	      (bPeriocicActive == TRUE) ? "ON" : "OFF",
	      gOBDList[gOBDListIndex].TesterPresentID );

	if ( bPeriocicActive == FALSE )
	{
		Log ( INFORMATION, SCREENOUTPUTOFF, LOGOUTPUTON, NO_PROMPT,
		      "StopPeriodicMsg called, but periodics not active\n" );
	}
#endif


	RetVal = PassThruIoctl ( gOBDList[gOBDListIndex].ChannelID,
	                         CLEAR_PERIODIC_MSGS,
	                         NULL,
	                         NULL );
	if ( RetVal != STATUS_NOERROR )
	{
		if ( bLogError == TRUE )
		{
			Log ( J2534_FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "%s returned %ld",
			      "PassThruIoctl(CLEAR_PERIODIC_MSGS)",
			      RetVal );
			return FAIL;
		}
	}

	RetVal = PassThruIoctl ( gOBDList[gOBDListIndex].ChannelID,
	                         CLEAR_TX_BUFFER,
	                         NULL,
	                         NULL );
	if ( RetVal != STATUS_NOERROR )
	{
		if ( bLogError == TRUE )
		{
			Log ( J2534_FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "%s returned %ld",
			      "PassThruIoctl(CLEAR_TX_BUFFER)",
			      RetVal );
			return FAIL;
		}
	}

	bPeriocicActive = FALSE;



#ifdef _DEBUG
	// Leave this in for now... for debugging purposes
	Log ( INFORMATION, SCREENOUTPUTOFF, LOGOUTPUTON, NO_PROMPT,
	      "Leave StopPeriodicMsg - STATE %s, ID %x\n",
	      (bPeriocicActive == TRUE) ? "ON" : "OFF",
	      gOBDList[gOBDListIndex].TesterPresentID );
#endif



	return PASS;
}
