/*
********************************************************************************
**  SAE J1699-5 Test Source Code
**
**  Copyright (C) 2002 Drew Technologies. http://j1699-3.sourceforge.net/
**
** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  This source code, when compiled and used with an SAE J2534-compatible pass
**  thru device, is intended to run the tests described in the SAE J1699-5
**  document in an automated manner.
**
**  This computer program is based upon SAE Technical Report J1699,
**  which is provided "AS IS"
**
**  See j1699.c for details of how to build and run this test.
**
********************************************************************************
*/
#include <stdio.h>
#include <string.h>
#include <windows.h>
#include "j2534.h"
#include "j1699.h"


/******************************************************************************
**  Protocol Init and Permutation data
******************************************************************************/

typedef struct
{
	unsigned long Protocol;     // generic protocol (for J2534-1)
	unsigned long InitFlags;    // J2534-1 ConnectFlags
	unsigned long HeaderSize;   // size of message header
	unsigned long BaudRate;     // link data rate
	unsigned long ProtocolTag;  // tag that uniquely identifies each protocol
	char *        Name;         // ASCII string for the protocol name
} PROTOCOL_INIT;


// The order of this must match the ProtocolTag definitions in J1699.h
static PROTOCOL_INIT ProtocolInitData[] =
{
	// Protocol  Flags               HeaderSize             Baud    ProtocolTag             Name (<25 char - see PROTOCOL_LIST)
	{ISO15765, 0,                    ISO15765_HEADER_SIZE,  500000, ISO15765_I,             "500K ISO15765 11 Bit"},
	{ISO15765, CAN_29BIT_ID,         ISO15765_HEADER_SIZE,  500000, ISO15765_29_BIT_I,      "500K ISO15765 29 Bit"},
	{ISO15765, 0,                    ISO15765_HEADER_SIZE,  250000, ISO15765_250K_11_BIT_I, "250K ISO15765 11 Bit"},
	{ISO15765, CAN_29BIT_ID,         ISO15765_HEADER_SIZE,  250000, ISO15765_250K_29_BIT_I, "250K ISO15765 29 Bit"}
};


static unsigned int ProtocolPerm[][OBD_MAX_PROTOCOLS] =
{
	{ISO15765_I, ISO15765_29_BIT_I},  // 5.2, 10.1
	{ISO15765_29_BIT_I, ISO15765_I},  // 5.8
	{ISO15765_I, ISO15765_29_BIT_I},  // 6.2
	{ISO15765_29_BIT_I, ISO15765_I},  // 7.2
	{ISO15765_I, ISO15765_29_BIT_I},  // 8.2
	{ISO15765_29_BIT_I, ISO15765_I},  // 9.2
	{ISO15765_I, ISO15765_29_BIT_I},  // 9.9
	{ISO15765_29_BIT_I, ISO15765_I},  // 9.13
	{ISO15765_I, ISO15765_29_BIT_I},  // 9.17
	{ISO15765_29_BIT_I, ISO15765_I}   // 9.21
};
#define NUM_PROTOCOL_PERM   (sizeof ( ProtocolPerm )/sizeof ( ProtocolPerm[0] ))


/*
*****************************************************************************
**  InitProtocolList - Function to initialize the protocol list
*****************************************************************************
*/
void InitProtocolList ( void )
{
	unsigned long i, j;

	gUserInput.MaxProtocols = OBD_MAX_PROTOCOLS;

	// initialize gOBDList
	for ( i = 0; i < gUserInput.MaxProtocols; i++ )
	{
		j = ProtocolPerm[gOBDProtocolOrder][i];

		gOBDList[i].Protocol     = ProtocolInitData[j].Protocol;
		gOBDList[i].InitFlags    = ProtocolInitData[j].InitFlags;
		gOBDList[i].HeaderSize   = ProtocolInitData[j].HeaderSize;
		gOBDList[i].BaudRate     = ProtocolInitData[j].BaudRate;
		gOBDList[i].ProtocolTag  = ProtocolInitData[j].ProtocolTag;
		strcpy_s ( gOBDList[i].Name, MAX_PROTOCOL_NAME, ProtocolInitData[j].Name );

		gOBDList[i].ChannelID  = 0;
	}

	// If at end of order, start over
	if ( ++gOBDProtocolOrder >= NUM_PROTOCOL_PERM )
	{
		gOBDProtocolOrder = 0;
	}
}
