/*
********************************************************************************
**  SAE J1699-5 Test Source Code
**
**  Copyright (C) 2002 Drew Technologies. http://j1699-3.sourceforge.net/
**
** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  This source code, when compiled and used with an SAE J2534-compatible pass
**  thru device, is intended to run the tests described in the SAE J1699-5
**  document in an automated manner.
**
**  This computer program is based upon SAE Technical Report J1699,
**  which is provided "AS IS"
**
**  See j1699.c for details of how to build and run this test.
**
********************************************************************************
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <windows.h>
#include "j2534.h"
#include "j1699.h"


/*
*****************************************************************************
**  IsDTCPending - Function to check if SID $19 Subfunction $42 StatusMask $04 reports a DTC pending
*****************************************************************************
*/
STATUS IsDTCPending ( unsigned long Flags )
{
	SID_REQ        SidReq;
	STATUS         RetCode;
	unsigned long  EcuIndex;

	// Request SID $19 Subfunction $42 StatusMask $04 data
	SidReq.SID    = 0x19;
	SidReq.NumIds = 4;
	SidReq.DID[0] = 0x42;  // Subfunction: reportWWHOBDDTCByMaskRecord
	SidReq.DID[1] = 0x33;  // Functional Group Identifier: Emission
	SidReq.DID[2] = 0x04;  // DTCStatusMask: Pending
	SidReq.DID[3] = 0xFF;  // DTCSeverityMask: all

	RetCode = SidRequest ( &SidReq, Flags );

	if ( RetCode != PASS )
	{
		// Indicate request failed
		return ERRORS;
	}


	RetCode = FAIL;
	// Check if SID $19 Subfunction $42 StatusMask $04 reports DTCs pending
	for ( EcuIndex = 0; EcuIndex < gOBDNumEcus; EcuIndex++ )
	{
		if ( gOBDResponse[EcuIndex].PendDTCHeader.DTCSize == 0 )
		{
			// If no data, ignore
		}
		// Check if there is at least one DTC (chekc High, Middle, and Low bytes)
		else if ( gOBDResponse[EcuIndex].PendDTC[1] != 0 || gOBDResponse[EcuIndex].PendDTC[2] != 0 || gOBDResponse[EcuIndex].PendDTC[3] != 0 )
		{
//			if ( IsDTCIgnored ( EcuIndex, gOBDResponse[EcuIndex].PendDTCSize, &gOBDResponse[EcuIndex].PendDTC[0] ) )
//			{
//				continue;
//			}
			RetCode = PASS;
		}
	}

	return RetCode;
}


/*
*****************************************************************************
**
**  Function:  SaveDTCList
**
**  Purpose:   Save the DTC list from SID $19 Subfunction $42 StatusMask $04 or $08
**
*****************************************************************************
*/
void SaveDTCList ( int nSID )
{
	unsigned long  EcuIndex;
	unsigned long  index;
	unsigned long  DTCIndex;
	unsigned short size;

	if ( nSID != 8 && nSID != 4 )
	{
		return;
	}

	for ( EcuIndex = 0; EcuIndex < gOBDNumEcus; EcuIndex++ )
	{
		size = gDTCList[EcuIndex].Size = (nSID == 8) ? gOBDResponse[EcuIndex].ConfDTCHeader.DTCSize
		                                             : gOBDResponse[EcuIndex].PendDTCHeader.DTCSize;
		for ( index = 0; index < size; index++ )
		{
			gDTCList[EcuIndex].DTC[index] = (nSID == 8) ? gOBDResponse[EcuIndex].ConfDTC[index]
			                                            : gOBDResponse[EcuIndex].PendDTC[index];
		}
	}

	// If this is Test 7.4 and there is a pending DTC, append to DTC list
	if ( nSID == 0x04 &&
	     gTestPhase == eTestConfirmedDTC &&
	     IsDTCPending ( (SID_REQ_NORMAL|SID_REQ_ALLOW_NO_RESPONSE) ) != FAIL )
	{
		// Save DTC list from SID $19 $42 ($04)
		for ( EcuIndex = 0; EcuIndex < gOBDNumEcus; EcuIndex++ )
		{
			size = gOBDResponse[EcuIndex].PendDTCHeader.DTCSize;
			DTCIndex = gDTCList[EcuIndex].Size;
			gDTCList[EcuIndex].Size += gOBDResponse[EcuIndex].PendDTCHeader.DTCSize;
			for ( index = 0; index < size; index++, DTCIndex++ )
			{
				gDTCList[EcuIndex].DTC[DTCIndex] = gOBDResponse[EcuIndex].PendDTC[index];
			}
		}
	}
}
