/*
********************************************************************************
**  SAE J1699-5 Test Source Code
**
**  Copyright (C) 2002 Drew Technologies. http://j1699-3.sourceforge.net/
**
** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  This source code, when compiled and used with an SAE J2534-compatible pass
**  thru device, is intended to run the tests described in the SAE J1699-5
**  document in an automated manner.
**
**  This computer program is based upon SAE Technical Report J1699,
**  which is provided "AS IS"
**
**  See j1699.c for details of how to build and run this test.
**
********************************************************************************
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <windows.h>
#include "j2534.h"
#include "j1699.h"


/*
*******************************************************************************
**  SidResetResponseData - Function to reset SID response data
*******************************************************************************
*/
STATUS SidResetResponseData ( PASSTHRU_MSG *TxMsg )
{
	unsigned long HeaderSize;
	unsigned long EcuIndex;

	// Set the response header size based on the protocol
	HeaderSize = gOBDList[gOBDListIndex].HeaderSize;

	// Reset SID response data for all ECUs
	for ( EcuIndex = 0; EcuIndex < gNumEcusForAlloc; EcuIndex++ )
	{
		gOBDResponse[EcuIndex].bResponseReceived = FALSE;  // reset response received flag

		// Save the data in the appropriate SID/PID/MID/TID/INF
		switch ( TxMsg->Data[HeaderSize] )
		{
			// OBD Services
			// Deprecated
			//SID 1 thru A
			case 0x01:
			case 0x02:
			case 0x03:
			case 0x04:
			case 0x05:
			case 0x06:
			case 0x07:
			case 0x08:
			case 0x09:
			case 0x0A:

			// Reserved
			// SID 0, B thru F
			case 0x00:
			case 0x0B:
			case 0x0C:
			case 0x0D:
			case 0x0E:
			case 0x0F:
			{
				// Invalid services, do not reset anything
			}
			break;


			// UDS Services
			// SID 0x14
			case 0x14:
			{
				// No Data in reply to Clear Codes
				gOBDResponse[EcuIndex].ClrDTCSize = 0;
			}
			break;


			// SID $19
			case 0x19:
			{
				// if this is a DTC with Permanent Status request
				if ( TxMsg->Data[HeaderSize + 1] == 0x55 )
				{
					gOBDResponse[EcuIndex].PermDTCSupported = FALSE;
					gOBDResponse[EcuIndex].PermDTCSize = 0;
					memset ( gOBDResponse[EcuIndex].PermDTC,
					         0,
					         sizeof ( gOBDResponse[EcuIndex].PermDTC ) );
				}
				// if this is DTC by Mask Record request
				else if ( TxMsg->Data[HeaderSize + 1] == 0x42 )
				{
					// if this is a Pending DTC request
					if ( TxMsg->Data[HeaderSize + 3] == 0x04 )
					{
						gOBDResponse[EcuIndex].PendDTCHeader.DTCSupported = FALSE;
						gOBDResponse[EcuIndex].PendDTCHeader.DTCSize = 0;
						memset ( gOBDResponse[EcuIndex].PendDTC,
						         0,
						         sizeof ( gOBDResponse[EcuIndex].PendDTC ) );
					}
					// else this is a Confirmed DTC request
					else
					{
						gOBDResponse[EcuIndex].ConfDTCHeader.DTCSupported = FALSE;
						gOBDResponse[EcuIndex].ConfDTCHeader.DTCSize = 0;
						memset ( gOBDResponse[EcuIndex].ConfDTC,
						         0,
						         sizeof ( gOBDResponse[EcuIndex].ConfDTC ) );
					}
				}
			}  // end case 0x19
			break;

			// SID 0x22
			case 0x22:
			{
				// Process message based on DID number MSB
				switch ( TxMsg->Data[HeaderSize + 1] )
				{
					// PID
					case 0xF4:
					case 0xF5:
					{
						// Process message based on DID number LSB (PID)
						switch ( TxMsg->Data[HeaderSize + 2] )
						{
							case 0x00:
							{
								if ( TxMsg->Data[HeaderSize + 1] == 0xF4 )
								{
									// IF called from VerifyLinkActive or DetermineProtocol,
									if ( gVerifyLink == TRUE ||
									     gDetermineProtocol == TRUE )
									{
										gOBDResponse[EcuIndex].LinkActive = FALSE;
									}
									else if ( gReverseOrderState[EcuIndex] == NOT_REVERSE_ORDER )
									{
										// Clear out the support data when PID 0x00 is requested
										gOBDResponse[EcuIndex].PidSupportSize = 0;

										memset ( &gOBDResponse[EcuIndex].PidSupport[0],
										         0,
										         sizeof ( ID_SUPPORT ) );
									}
								}
							}
							// 0x00 falls through

							case 0x20:
							case 0x40:
							case 0x60:
							case 0x80:
							case 0xA0:
							case 0xC0:
							case 0xE0:
							{
								if ( gReverseOrderState[EcuIndex] == REVERSE_ORDER_REQUESTED )
								{
									// indicate first reverse order response received and SupportSize cleared
									gReverseOrderState[EcuIndex] = REVERSE_ORDER_RESPONSE;

									if ( TxMsg->Data[HeaderSize + 1] == 0xF5 && TxMsg->Data[HeaderSize + 2] == 00 )
									{
										// Clear out the support data when PID 0x00 is requested
										gOBDResponse[EcuIndex].PidSupportSize = 0;
									}

									memset ( &gOBDResponse[EcuIndex].PidSupport[TxMsg->Data[HeaderSize + 2]>>5],
									         0,
									         sizeof ( ID_SUPPORT ) );
								}
							}
							// 0x00-0xE0 falls through

							default:
							{
								// All other requests reset the PID data
								gOBDResponse[EcuIndex].PidSize = 0;

								memset ( gOBDResponse[EcuIndex].Pid,
								         0,
								         sizeof ( gOBDResponse[EcuIndex].Pid ) );
							}
							break;

						}  // end case 0xF4/5 switch ( TxMsg->Data[HeaderSize + 2] )

					}  // end case 0xF4/5
					break;

					// MID
					case 0xF6:
					{
						// Process message based on MID number (DID LSB)
						switch ( TxMsg->Data[HeaderSize + 2] )
						{
							case 0x00:
							{
								// Clear out the support data when MID 0x00 is requested
								gOBDResponse[EcuIndex].MidSupportSize = 0;
								memset ( gOBDResponse[EcuIndex].MidSupport,
								         0,
								         sizeof ( gOBDResponse[EcuIndex].MidSupport ) );
							}
							// 0x00 falls through

							case 0x20:
							case 0x40:
							case 0x60:
							case 0x80:
							case 0xA0:
							case 0xC0:
							case 0xE0:
							{
								// Don't do anything with other MID support requests
							}
							// 0x00-0xE0 falls through

							default:
							{
								// All other requests reset the MID data
								gOBDResponse[EcuIndex].MidSize = 0;
								memset ( gOBDResponse[EcuIndex].Mid,
								         0,
								         sizeof ( gOBDResponse[EcuIndex].Mid ) );
							}
							break;
						}  // end case 0xF6 switch ( TxMsg->Data[HeaderSize + 2] )
					}  // end case 0xF6
					break;

					// INF
					case 0xF8:
					{
						// Process message based on PID number (DID LSB)
						switch ( TxMsg->Data[HeaderSize + 2] )
						{
							case 0x00:
							{
								// Clear out the support data when PID 0x00 is requested
								gOBDResponse[EcuIndex].InfSupportSize = 0;
								memset ( gOBDResponse[EcuIndex].InfSupport,
								         0,
								         sizeof ( gOBDResponse[EcuIndex].InfSupport ) );
							}
							// 0x00 falls through

							case 0x20:
							case 0x40:
							case 0x60:
							case 0x80:
							case 0xA0:
							case 0xC0:
							case 0xE0:
							{
								// Don't do anything with other PID support requests
							}
							// 0x00-0xE0 falls through

							default:
							{
								// All other requests reset the PID data
								gOBDResponse[EcuIndex].InfSize = 0;
								memset ( gOBDResponse[EcuIndex].Inf,
								         0,
								         sizeof ( gOBDResponse[EcuIndex].Inf ) );
							}
							break;
						}  // end case 0xF8 switch ( TxMsg->Data[HeaderSize + 2] )

					}  // end case 0xF8
					break;

				} // end case 0x22 switch ( TxMsg->Data[HeaderSize + 1] )

			}  // end case 0x22
			break;


			default:
			{
				// Unexpected reset of SID data
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  Unexpected reset of data for SID $%02X\n",
				      GetEcuId ( EcuIndex ),
				      TxMsg->Data[HeaderSize] );
				return FAIL;

			}

		}  // end switch ( TxMsg->Data[HeaderSize] )

	}  // end for ( EcuIndex )

	return PASS;
}
