/*
********************************************************************************
**  SAE J1699-5 Test Source Code
**
**  Copyright (C) 2002 Drew Technologies. http://j1699-3.sourceforge.net/
**
** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  This source code, when compiled and used with an SAE J2534-compatible pass
**  thru device, is intended to run the tests described in the SAE J1699-5
**  document in an automated manner.
**
**  This computer program is based upon SAE Technical Report J1699,
**  which is provided "AS IS"
**
**  See j1699.c for details of how to build and run this test.
**
********************************************************************************
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <time.h>
#include <windows.h>
#include "j2534.h"
#include "j1699.h"
#include "ScreenOutput.h"

/* Local Fucntion Prototypes */
STATUS LogIPT ( void );
STATUS VerifyIPTData ( BOOL *pbIptSupported );
STATUS LogInf12 ( IPT pIpt[] );
STATUS VerifyInf12 ( IPT *pIpt );
STATUS StartLogFile ( BOOL *pbTestReEntered );
STATUS RunDynamicTest10 ( unsigned long tEngineStartTimeStamp );
STATUS GetIPTData ( unsigned int EcuIndex, IPT *pIpt );
void   DisplayDynamicTest10Results ( void );

/* External Fucntion Prototypes */
extern STATUS AppendLogFile ( void );
extern STATUS VerifyInf12Data ( unsigned long EcuIndex );
extern STATUS ReadVIN ( void );
extern STATUS ReadCALIDs ( void );
extern STATUS ReadCVNs ( void );
extern STATUS VerifyLogFile ( FILE *hFileHandle );  // upon re-entering Dynamic test, verifies previous data
extern void   LogVersionInformation ( SCREENOUTPUT bDisplay, LOGOUTPUT bLog );

// Initial data for Test 11.x. Declared in TestToVerifyPerformanceCounters.c
extern IPT *Test10_10_IPT;

/*
*******************************************************************************
**  Test 10.x dynamic screen mapping
*******************************************************************************
*/

#define SPACE             7
#define ECU_WIDTH         7
#define NUM_WIDTH         5
#define STRING_WIDTH      7

#define COL1              18

#define COND_IDLE_ROW     7
#define COND_SPEED_ROW    9
#define COND_PHEV_ROW     11

#define CONDITIONS_ROW    13

#define IDLE_TIME_ROW     15
#define SPEED_ROW         15
#define RUN_TIME_ROW      16
#define TOTAL_TIME_ROW    17
#define PHEV_TIME_ROW     18
#define TEST_STATUS_ROW   18
#define ECU_ID_ROW        20
#define INI_OBD_COND_ROW  22
#define CUR_OBD_COND_ROW  23
#define INI_IGN_CNT_ROW   25
#define CUR_IGN_CNT_ROW   26
#define INI_FEO_CNT_ROW   28
#define CUR_FEO_CNT_ROW   29
#define BOTTOM_ROW        31

StaticTextElement  _string_elements_1st_9[] =
{
	{"Drive the vehicle in the following manner, so that the OBD Condition Counter will increment:", 1, 0}
};

StaticTextElement  _string_elements_1st_10[] =
{
	{"Drive the vehicle in the following manner, at an altitude < 8000 ft", 1, 0},
	{"(BARO < 22 in Hg) and ambient temperature > or = 20 deg F,", 1, 1},
	{"so that the OBD Condition Counter will increment:", 1, 2}
};

StaticTextElement  _string_elements10[] =
{
	{"NOTE: Some powertrain control systems have engine controls that can start and", 1, 4},
	{"stop the engine without regard to ignition position.", 1, 5},

	{"- Continuous time > or = 30 seconds with vehicle speed < or = 1 MPH", 1, COND_IDLE_ROW},
	{"  and accelerator pedal released.", 1, 8},
	{"- Cumulative time > or = 300 seconds with vehicle speed > 25MPH/40KPH.", 1, COND_SPEED_ROW},
	{"- Cumulative time since engine start > or = 600 seconds.", 1, 10},

	{"OBD Drive Cycle Status", 1, 13},
	{"BARO:    in Hg      AAT:     C (    F)", 38, CONDITIONS_ROW},

	{" 30 Seconds Idle Timer:", 1, IDLE_TIME_ROW},
	{"300 Seconds at speeds > 25MPH/40KPH Timer:", 1, RUN_TIME_ROW},
	{"600 Seconds Total Drive Timer:", 1, TOTAL_TIME_ROW},

	{"PAGE:    ECU ID:", 1, ECU_ID_ROW},

	{"Initial OBDCOND:", 1, INI_OBD_COND_ROW},
	{"Current OBDCOND:", 1, CUR_OBD_COND_ROW},

	{"Initial IGNCTR:", 1, INI_IGN_CNT_ROW},
	{"Current IGNCTR:", 1, CUR_IGN_CNT_ROW},

	{"Speed:", 60, SPEED_ROW},
	{"       MPH    KPH", 60, SPEED_ROW+1},

	{"Test Status:", 60, TEST_STATUS_ROW},

	{"Press ESC to abort the Drive Cycle and fail the test", 1, BOTTOM_ROW},
	{"", 0, BOTTOM_ROW+1}
};

StaticTextElement  _string_elements10_RPM[] =
{
	{"- Continuous time > or = 30 seconds with engine speed < 1150 RPM.  ", 1, COND_IDLE_ROW},

	{"- Cumulative time > or = 300 seconds with engine speed > or = 1150 RPM.", 1, COND_SPEED_ROW},

	{"300 Seconds at speeds > 1150  RPM  Timer:", 1, RUN_TIME_ROW},

	{"       RPM       ", 60, SPEED_ROW+1}
};

StaticTextElement  _string_elements10_PHEV[] =
{
	{"- Cumulative fueled engine time since engine start > or = 10 seconds for PHEV.", 1, COND_PHEV_ROW},

	{" 10 Seconds Fueled Engine Operation (PHEV only):", 1, PHEV_TIME_ROW},

	{"Initial FEOCNTR:", 1, INI_FEO_CNT_ROW},
	{"Current FEOCNTR:", 1, CUR_FEO_CNT_ROW}
};

const int _num_string_elements_1st_9 = sizeof ( _string_elements_1st_9 )/sizeof ( _string_elements_1st_9[0] );
const int _num_string_elements_1st_10 = sizeof ( _string_elements_1st_10 )/sizeof ( _string_elements_1st_10[0] );
const int _num_string_elements10 = sizeof ( _string_elements10 )/sizeof ( _string_elements10[0] );
const int _num_string_elements10_RPM = sizeof ( _string_elements10_RPM )/sizeof ( _string_elements10_RPM[0] );
const int _num_string_elements10_PHEV = sizeof ( _string_elements10_PHEV )/sizeof ( _string_elements10_PHEV[0] );

DynamicValueElement  _dynamic_elements10[] =
{
	{26, IDLE_TIME_ROW, NUM_WIDTH},                 // (Index 0) 30 seconds idle drive cycle timer
	{45, RUN_TIME_ROW, NUM_WIDTH},                  // (Index 1) 300 seconds at speeds greater then 25 MPH timer
	{33, TOTAL_TIME_ROW, NUM_WIDTH},                // (Index 2) 600 seconds total drive timer
	{51, PHEV_TIME_ROW, NUM_WIDTH},                 // (Index 3) 10 Seconds Fueled Engine Operation (PHEV only)

	{COL1 + 0*SPACE, ECU_ID_ROW, ECU_WIDTH},        // (Index 4) ECU ID 1
	{COL1 + 1*SPACE, ECU_ID_ROW, ECU_WIDTH},        // (Index 5) ECU ID 2
	{COL1 + 2*SPACE, ECU_ID_ROW, ECU_WIDTH},        // (Index 6) ECU ID 3
	{COL1 + 3*SPACE, ECU_ID_ROW, ECU_WIDTH},        // (Index 7) ECU ID 4
	{COL1 + 4*SPACE, ECU_ID_ROW, ECU_WIDTH},        // (Index 8) ECU ID 5
	{COL1 + 5*SPACE, ECU_ID_ROW, ECU_WIDTH},        // (Index 9) ECU ID 6
	{COL1 + 6*SPACE, ECU_ID_ROW, ECU_WIDTH},        // (Index 10) ECU ID 7
	{COL1 + 7*SPACE, ECU_ID_ROW, ECU_WIDTH},        // (Index 11) ECU ID 8

	{COL1 + 0*SPACE, INI_OBD_COND_ROW, NUM_WIDTH},  // (Index 12) initial obdcond, ECU 1
	{COL1 + 1*SPACE, INI_OBD_COND_ROW, NUM_WIDTH},  // (Index 13) initial obdcond, ECU 2
	{COL1 + 2*SPACE, INI_OBD_COND_ROW, NUM_WIDTH},  // (Index 14) initial obdcond, ECU 3
	{COL1 + 3*SPACE, INI_OBD_COND_ROW, NUM_WIDTH},  // (Index 15) initial obdcond, ECU 4
	{COL1 + 4*SPACE, INI_OBD_COND_ROW, NUM_WIDTH},  // (Index 16) initial obdcond, ECU 5
	{COL1 + 5*SPACE, INI_OBD_COND_ROW, NUM_WIDTH},  // (Index 17) initial obdcond, ECU 6
	{COL1 + 6*SPACE, INI_OBD_COND_ROW, NUM_WIDTH},  // (Index 18) initial obdcond, ECU 7
	{COL1 + 7*SPACE, INI_OBD_COND_ROW, NUM_WIDTH},  // (Index 19) initial obdcond, ECU 8

	{COL1 + 0*SPACE, CUR_OBD_COND_ROW, NUM_WIDTH},  // (Index 20) current obdcond, ECU 1
	{COL1 + 1*SPACE, CUR_OBD_COND_ROW, NUM_WIDTH},  // (Index 21) current obdcond, ECU 2
	{COL1 + 2*SPACE, CUR_OBD_COND_ROW, NUM_WIDTH},  // (Index 22) current obdcond, ECU 3
	{COL1 + 3*SPACE, CUR_OBD_COND_ROW, NUM_WIDTH},  // (Index 23) current obdcond, ECU 4
	{COL1 + 4*SPACE, CUR_OBD_COND_ROW, NUM_WIDTH},  // (Index 24) current obdcond, ECU 5
	{COL1 + 5*SPACE, CUR_OBD_COND_ROW, NUM_WIDTH},  // (Index 25) current obdcond, ECU 6
	{COL1 + 6*SPACE, CUR_OBD_COND_ROW, NUM_WIDTH},  // (Index 26) current obdcond, ECU 7
	{COL1 + 7*SPACE, CUR_OBD_COND_ROW, NUM_WIDTH},  // (Index 27) current obdcond, ECU 8

	{COL1 + 0*SPACE, INI_IGN_CNT_ROW, NUM_WIDTH},   // (Index 28) initial ignctr, ECU 1
	{COL1 + 1*SPACE, INI_IGN_CNT_ROW, NUM_WIDTH},   // (Index 29) initial ignctr, ECU 2
	{COL1 + 2*SPACE, INI_IGN_CNT_ROW, NUM_WIDTH},   // (Index 30) initial ignctr, ECU 3
	{COL1 + 3*SPACE, INI_IGN_CNT_ROW, NUM_WIDTH},   // (Index 31) initial ignctr, ECU 4
	{COL1 + 4*SPACE, INI_IGN_CNT_ROW, NUM_WIDTH},   // (Index 32) initial ignctr, ECU 5
	{COL1 + 5*SPACE, INI_IGN_CNT_ROW, NUM_WIDTH},   // (Index 33) initial ignctr, ECU 6
	{COL1 + 6*SPACE, INI_IGN_CNT_ROW, NUM_WIDTH},   // (Index 34) initial ignctr, ECU 7
	{COL1 + 7*SPACE, INI_IGN_CNT_ROW, NUM_WIDTH},   // (Index 35) initial ignctr, ECU 8

	{COL1 + 0*SPACE, CUR_IGN_CNT_ROW, NUM_WIDTH},   // (Index 36) current ignctr, ECU 1
	{COL1 + 1*SPACE, CUR_IGN_CNT_ROW, NUM_WIDTH},   // (Index 37) current ignctr, ECU 2
	{COL1 + 2*SPACE, CUR_IGN_CNT_ROW, NUM_WIDTH},   // (Index 38) current ignctr, ECU 3
	{COL1 + 3*SPACE, CUR_IGN_CNT_ROW, NUM_WIDTH},   // (Index 39) current ignctr, ECU 4
	{COL1 + 4*SPACE, CUR_IGN_CNT_ROW, NUM_WIDTH},   // (Index 40) current ignctr, ECU 5
	{COL1 + 5*SPACE, CUR_IGN_CNT_ROW, NUM_WIDTH},   // (Index 41) current ignctr, ECU 6
	{COL1 + 6*SPACE, CUR_IGN_CNT_ROW, NUM_WIDTH},   // (Index 42) current ignctr, ECU 7
	{COL1 + 7*SPACE, CUR_IGN_CNT_ROW, NUM_WIDTH},   // (Index 43) current ignctr, ECU 8

	{COL1 + 0*SPACE, INI_FEO_CNT_ROW, NUM_WIDTH},   // (Index 44) initial feocntr, ECU 1
	{COL1 + 1*SPACE, INI_FEO_CNT_ROW, NUM_WIDTH},   // (Index 45) initial feocntr, ECU 2
	{COL1 + 2*SPACE, INI_FEO_CNT_ROW, NUM_WIDTH},   // (Index 46) initial feocntr, ECU 3
	{COL1 + 3*SPACE, INI_FEO_CNT_ROW, NUM_WIDTH},   // (Index 47) initial feocntr, ECU 4
	{COL1 + 4*SPACE, INI_FEO_CNT_ROW, NUM_WIDTH},   // (Index 48) initial feocntr, ECU 5
	{COL1 + 5*SPACE, INI_FEO_CNT_ROW, NUM_WIDTH},   // (Index 49) initial feocntr, ECU 6
	{COL1 + 6*SPACE, INI_FEO_CNT_ROW, NUM_WIDTH},   // (Index 50) initial feocntr, ECU 7
	{COL1 + 7*SPACE, INI_FEO_CNT_ROW, NUM_WIDTH},   // (Index 51) initial feocntr, ECU 8

	{COL1 + 0*SPACE, CUR_FEO_CNT_ROW, NUM_WIDTH},   // (Index 52) current feocntr, ECU 1
	{COL1 + 1*SPACE, CUR_FEO_CNT_ROW, NUM_WIDTH},   // (Index 53) current feocntr, ECU 2
	{COL1 + 2*SPACE, CUR_FEO_CNT_ROW, NUM_WIDTH},   // (Index 54) current feocntr, ECU 3
	{COL1 + 3*SPACE, CUR_FEO_CNT_ROW, NUM_WIDTH},   // (Index 55) current feocntr, ECU 4
	{COL1 + 4*SPACE, CUR_FEO_CNT_ROW, NUM_WIDTH},   // (Index 56) current feocntr, ECU 5
	{COL1 + 5*SPACE, CUR_FEO_CNT_ROW, NUM_WIDTH},   // (Index 57) current feocntr, ECU 6
	{COL1 + 6*SPACE, CUR_FEO_CNT_ROW, NUM_WIDTH},   // (Index 58) current feocntr, ECU 7
	{COL1 + 7*SPACE, CUR_FEO_CNT_ROW, NUM_WIDTH},   // (Index 59) current feocntr, ECU 8

	{72, IDLE_TIME_ROW, 3},                         // (Index 60) RPM label
	{67, IDLE_TIME_ROW, NUM_WIDTH},                 // (Index 61) RPM value
	{67, SPEED_ROW, NUM_WIDTH},                     // (Index 62) Speed MPH
	{67 + SPACE, SPEED_ROW, NUM_WIDTH},             // (Index 63) Speed KPH

	{44, CONDITIONS_ROW, 2},                        // (Index 64) Barometric Pressure
	{63, CONDITIONS_ROW, 3},                        // (Index 65) Ambient Air Temperature C
	{70, CONDITIONS_ROW, 3},                        // (Index 66) Ambient Air Temperature F

	{73, TEST_STATUS_ROW, STRING_WIDTH},            // (Index 67) Test Status
	{6,  ECU_ID_ROW, 2}                             // (Index 68) ECU Page number
};

const int _num_dynamic_elements10 = sizeof (_dynamic_elements10)/sizeof (_dynamic_elements10[0]);

#define IDLE_TIMER          0
#define SPEED_25_MPH_TIMER  1
#define TOTAL_DRIVE_TIMER   2
#define PHEV_TIMER          3
#define ECU_ID              4

#define INITIAL_OBDCOND     12
#define CURRENT_OBDCOND     20
#define INITIAL_IGNCTR      28
#define CURRENT_IGNCTR      36
#define INITIAL_FEOCNTR     44
#define CURRENT_FEOCNTR     52

#define RPM_LABEL_INDEX     60
#define RPM_INDEX           61
#define SPEED_INDEX_MPH     62
#define SPEED_INDEX_KPH     63
#define BARO_PRESS_INDEX    64
#define AMB_AIR_TEMP_INDEX  65
#define TESTSTATUS_INDEX    67
#define ECUPAGE_INDEX       68

// used with the on-screen test status
#define SUB_TEST_NORMAL     0x00
#define SUB_TEST_FAIL       0x01
#define SUB_TEST_DTC        0x02
#define SUB_TEST_INITIAL    0x80


#define NORMAL_TEXT         -1
#define HIGHLIGHTED_TEXT    8

#define SetFieldDec(index,val)  (update_screen_dec  ( _dynamic_elements10, _num_dynamic_elements10, index, val ))
#define SetFieldHex(index,val)  (update_screen_hex  ( _dynamic_elements10, _num_dynamic_elements10, index, val ))
#define SetFieldText(index,val) (update_screen_text ( _dynamic_elements10, _num_dynamic_elements10, index, val ))


IPT *pTest10_10_FEOCNTR;
IPT *pLast_Ipt;


/*
*******************************************************************************
**  TestToVerifyInUseCounters - Tests 10.x
**  Function to run test to verify in-use counters with no faults
*******************************************************************************
*/
STATUS TestToVerifyInUseCounters ( BOOL *pbTestReEntered )
{
	STATUS        eResults;
	unsigned long tEngineStartTimeStamp;
	BOOL          bIptSupported = FALSE;

	BOOL          bSubTestFailed = FALSE;


//*******************************************************************************
	// Test 10.1, Prompt user to perform drive cycle to clear I/M readiness bits
	gTestSubsection = 1;
	gOBDgTestSubsectionFailed = FALSE;

	gOBDEngineRunning = FALSE;  // Engine should now be not running
	gOBDProtocolOrder = 0;      // Determine the OBD protocol to use

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Establish Communication, Engine Off)" );

	Log ( PROMPT, SCREENOUTPUTON, LOGOUTPUTON, ENTER_PROMPT,
	      "Turn key on without cranking or starting engine.\n" );

	if ( DetermineProtocol () != PASS )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "Protocol determination unsuccessful.\n" );
		Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
		return FAIL;
	}
	else
	{
		if ( VehicleReport () != PASS )
		{
			bSubTestFailed = TRUE;
		}

		if ( Log ( PROMPT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_PROMPT,
		           "Review the above Vehicle Information.\n\n"
		           "Is the information correct and would you like to continue the test?" ) != 'Y' )
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "Test Ended By User.\n" );
			return EXIT;
		}

		if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS ||
		     gOBDgTestSubsectionFailed == TRUE )
		{
			if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
			{
				return FAIL;
			}
		}
		else
		{
			Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
		}
	}


//*******************************************************************************
	// Test 10.2, Get SID $22 data
	gTestSubsection = 2;
	gOBDgTestSubsectionFailed = FALSE;
	bSubTestFailed = FALSE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Verify Diagnostic Data (Service $22 PIDs), Engine Off)" );

	if ( VerifyDiagnosticSupportAndData ( TRUE ) == FAIL )
	{
		bSubTestFailed = TRUE;
	}

	if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag) != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
	{
		if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
		{
			return FAIL;
		}
	}
	else
	{
		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}


//*******************************************************************************
	// Test 10.3, Get VIN and create (or append) log file
	gTestSubsection = 3;
	gOBDgTestSubsectionFailed = FALSE;
	bSubTestFailed = FALSE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Verify Vehicle Information (Service $22 INFs), Engine Off)" );

	eResults = StartLogFile ( pbTestReEntered );
	if ( eResults == FAIL )
	{
		Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
		return FAIL;
	}
	else if ( eResults == EXIT )
	{
		// Dynamic Tests are all done, just exit
		return EXIT;
	}

	// Add CALIDs to the log file
	if ( ReadCALIDs () != PASS )
	{
		bSubTestFailed = TRUE;
	}

	// Add CVNs to the log file
	if ( ReadCVNs () != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
	{
		if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
		{
			return FAIL;
		}
	}
	else
	{
		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}

	if ( *pbTestReEntered == TRUE )
	{
		return PASS;
	}


//*******************************************************************************
	// Test 10.4, Identify ECUs that support PID $F401
	gTestSubsection = 4;
	gOBDgTestSubsectionFailed = FALSE;
	bSubTestFailed = FALSE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Request current powertrain diagnostic data (Service $22 PIDs), Engine Off)" );

	if ( RequestDIDSupportData ( PIDREQUEST, FALSE ) != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
	{
		if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
		{
			return FAIL;
		}
	}
	else
	{
		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}


	if ( *pbTestReEntered == TRUE )
	{
		return PASS;
	}


//*******************************************************************************
	// Test10.5, Clear DTCs
	gTestSubsection = 5;
	gOBDgTestSubsectionFailed = FALSE;
	bSubTestFailed = FALSE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Clear DTCs (Service $14), Engine Off)" );

	if ( ClearCodes () != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
	{
		if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
		{
			return FAIL;
		}
	}
	else
	{
		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}


//*******************************************************************************
	// Test 10.6, Verify I/M readiness is "not ready"
	gTestSubsection = 6;
	gOBDgTestSubsectionFailed = FALSE;
	bSubTestFailed = FALSE;

	gOBDDTCHistorical = FALSE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Verify IM Readiness (Service $22), Engine Off)" );

	if ( VerifyIM_Ready () != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
	{
		if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
		{
			return FAIL;
		}
	}
	else
	{
		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}


//*******************************************************************************
	// Test 10.7, Verify Monitor Resets
	gTestSubsection = 7;
	gOBDgTestSubsectionFailed = FALSE;
	bSubTestFailed = FALSE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Verify Monitor Test Results (Service $22 MIDs), Engine Off)" );

	if ( VerifyMonitorTestSupportAndResults () != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
	{
		if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
		{
			return FAIL;
		}
	}
	else
	{
		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}


//*******************************************************************************
	// Test 10,8, Verify no pending DTCs (service 7)
	gTestSubsection = 8;
	gOBDgTestSubsectionFailed = FALSE;
	bSubTestFailed = FALSE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Verify Pending DTCs (Service $19 Subfunction $42 StatusMask $04), Engine Off)" );

	if ( VerifyDTCPendingData () != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
	{
		if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
		{
			return FAIL;
		}
	}
	else
	{
		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}


//*******************************************************************************
	// Test10.9, Verify no confirmed DTCs
	gTestSubsection = 9;
	gOBDgTestSubsectionFailed = FALSE;
	bSubTestFailed = FALSE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Verify DTCs (Service $19 Subfunction $42 StatusMask $08), Engine Off)" );

	if ( VerifyDTCStoredData () != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
	{
		if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
		{
			return FAIL;
		}
	}
	else
	{
		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}


//*******************************************************************************
	// Test 10.10, Get in-use performance tracking (service 22 infotype 8 and B)
	gTestSubsection = 10;
	gOBDgTestSubsectionFailed = FALSE;
	bSubTestFailed = FALSE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Verify Vehicle Information (Service $22 INFs), Engine Off)" );

	if ( VerifyIPTData ( &bIptSupported ) != PASS || gOBDgTestSubsectionFailed == TRUE )
	{
		if ( Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT,
		           "Verify IPT Stored Data.\n" ) == 'N' )
		{
			Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
			return FAIL;
		}
		bSubTestFailed = TRUE;
		gOBDgTestSubsectionFailed = FALSE;
	}
	else if ( bIptSupported == FALSE /*&&
	          ( ( (gUserInput.eComplianceType == US_OBDII || gUserInput.eComplianceType == HD_OBD) && gModelYear < 2005 ) ||
	            (gUserInput.eComplianceType == EOBD_NO_IUMPR) ||
	            (gUserInput.eComplianceType == IOBD_NO_IUMPR) ||
	            (gUserInput.eComplianceType == HD_IOBD_NO_IUMPR) ||
	            (gUserInput.eComplianceType == OBDBr_NO_IUMPR) )*/ )
	{
		// allowed to skip the CARB Drive Cycle
		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
		return PASS;
	}

	if ( gOBDPlugInFlag )
	{
		if ( LogInf12 ( pTest10_10_FEOCNTR ) != PASS )
		{
			bSubTestFailed = TRUE;
		}
	}

	Log ( PROMPT, SCREENOUTPUTON, LOGOUTPUTON, ENTER_PROMPT,
	      "Turn ignition off (engine off) for 60 seconds.\n" );

	Log ( PROMPT, SCREENOUTPUTON, LOGOUTPUTON, ENTER_PROMPT,
	      "Turn ignition to crank position and start engine.\n" );

	tEngineStartTimeStamp = GetTickCount ();

	if ( gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
	{
		Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}
	else
	{
		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}


//*******************************************************************************
	// Test 10.11, Engine should now be running
	gTestSubsection = 11;
	gOBDgTestSubsectionFailed = FALSE;

	gOBDEngineRunning = TRUE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Establish Communication, Engine Running)" );

	if ( DetermineProtocol () != PASS )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "Protocol determination unsuccessful.\n" );
		Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
		return FAIL;
	}
	else if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS ||
	          gOBDgTestSubsectionFailed == TRUE )
	{
		if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
		{
			return FAIL;
		}
	}
	else
	{
		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}


//*******************************************************************************
	// Test 10.12, User prompt, run test
	gTestSubsection = 12;
	gOBDgTestSubsectionFailed = FALSE;
	bSubTestFailed = FALSE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Complete CARB drive cycle)" );

	StopPeriodicMsg ( TRUE );
	Sleep ( gOBDRequestDelay );

	gSuspendScreenOutput = TRUE;
	eResults = RunDynamicTest10 ( tEngineStartTimeStamp );
	DisplayDynamicTest10Results ();
	gSuspendScreenOutput = FALSE;

	// re-start tester-present message
	StartPeriodicMsg ();

	if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( (eResults == FAIL) || gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "Errors detected during dynamic test.  View Logfile for more details.\n" );

		if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
		{
			return FAIL;
		}
	}
	else
	{
		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}

	Log ( PROMPT, SCREENOUTPUTON, LOGOUTPUTON, ENTER_PROMPT,
	      "Stop the vehicle in a safe location without turning the ignition off.\n" );


//*******************************************************************************
	// Test 10.13, Verify engine warm data
	gTestSubsection = 13;
	gOBDgTestSubsectionFailed = FALSE;
	bSubTestFailed = FALSE;

	gOBDEngineWarm = TRUE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Verify Diagnostic Data (Service $22 PIDs), Engine Running)" );

	if ( VerifyDiagnosticSupportAndData ( FALSE ) == FAIL )
	{
		bSubTestFailed = TRUE;
	}

	if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
	{
		if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
		{
			return FAIL;
		}
	}
	else
	{
		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}


//*******************************************************************************
	// Test 10.14, Get in-use performance tracking (service 22 infotype 8 or B)
	gTestSubsection = 14;
	gOBDgTestSubsectionFailed = FALSE;
	bSubTestFailed = FALSE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Verify Vehicle Information (Service $22 INFs), Engine Running)" );

	if ( LogIPT () != PASS || gOBDgTestSubsectionFailed == TRUE )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "IPT (INF $F808 or $F80B) data error\n" );
		bSubTestFailed = TRUE;
	}

	if ( gOBDPlugInFlag )
	{
		if ( VerifyInf12 ( &pTest10_10_FEOCNTR[0] ) != PASS )
		{
			bSubTestFailed = TRUE;
		}
	}

	if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
	{
		if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
		{
			return FAIL;
		}
	}
	else
	{
		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}

	return PASS;
}


/*
*******************************************************************************
**  StartLogFile - create new log file or open and append to existing one.
**                 use VIN as log filename.
*******************************************************************************
*/
STATUS StartLogFile ( BOOL *pbTestReEntered )
{
	char   buf[256];
	char   characterset[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890 -_.";
	STATUS eResult = PASS;
	FILE  *hTempFileHandle; // temporary file handle
	int    numchar;


	// Request SID $9 INF support data
	if ( RequestDIDSupportData ( INFREQUEST, TRUE ) != PASS )
	{
		return FAIL;
	}


	// Get VIN
	if ( ReadVIN () == PASS )
	{
		// use VIN as log filename
		strcpy_s ( gLogFileName, MAX_LOGFILENAME, gVIN );
	}
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "Unable to obtain VIN\n" );
		do
		{
			Log ( PROMPT, SCREENOUTPUTON, LOGOUTPUTON, CUSTOM_PROMPT,
			      "Enter a valid filename (<%d characters) for the log file: ",
			      MAX_LOGFILENAME );
			_cgets_s ( buf, sizeof ( buf ), &numchar );
			Log ( BLANK, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "\n" );
		} while ( strlen ( buf ) == 0 ||                           // empty (NULL) string
		          strspn ( buf, characterset ) < strlen ( buf ) || // contains nonalphanumeric characters
		          (strlen ( buf ) + 1) > MAX_LOGFILENAME );        // longer than max filename

		strcpy_s ( gLogFileName, MAX_LOGFILENAME, buf );
	}

	// Check for log file from Tests 5.xx - 9.xx
	strcat_s ( gLogFileName, MAX_LOGFILENAME - strlen ( gLogFileName ), ".log" );
	if ( ghTempLogFile != ghLogFile && ghLogFile != NULL )
	{
		fclose ( ghLogFile );
	}

	// Test if log file already exists
	if ( fopen_s ( &hTempFileHandle, gLogFileName, "r+" ) != 0 )
	{
		// file does not exist - create file and start with test 10
		if ( fopen_s ( &ghLogFile, gLogFileName, "w+" ) != 0 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTOFF, NO_PROMPT,
			      "Cannot open log file %s\n",
			      gLogFileName );
			return FAIL;
		}

		// log file doesn't exist. continue with Test 10.x
		*pbTestReEntered = FALSE;
	}
	else
	{
		// check software version, user input, test already complete, and get results totals in log file
		eResult = VerifyLogFile ( hTempFileHandle );
		if ( eResult == FAIL || eResult == EXIT )
		{
			return eResult;
		}

		ghLogFile = hTempFileHandle;

		// log file already exists, go to Test 11.x
		fputs ( "\n****************************************************\n", ghLogFile );
		*pbTestReEntered = TRUE;
	}

	// Application version, build date, OS, etc
	LogVersionInformation ( SCREENOUTPUTOFF, LOGOUTPUTON );

	// Copy temp log file to actual log file
	if ( AppendLogFile () != PASS )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTOFF, NO_PROMPT,
		      "Error copying temp log file to %s\n",
		      gLogFileName );
		return FAIL;
	}

	// Echo user responses to the log file
	Log ( INFORMATION, SCREENOUTPUTOFF, LOGOUTPUTON, NO_PROMPT,
	      "%s%d\n",
	      g_rgpcDisplayStrings[DSPSTR_PRMPT_MODEL_YEAR], 
	      gModelYear );
	
	Log ( INFORMATION, SCREENOUTPUTOFF, LOGOUTPUTON, NO_PROMPT,
	      "%s%d\n",
	      g_rgpcDisplayStrings[DSPSTR_PRMPT_OBD_ECU],
	      gUserNumEcus );

	Log ( INFORMATION, SCREENOUTPUTOFF, LOGOUTPUTON, NO_PROMPT,
	      "%s%s\n",
	      g_rgpcDisplayStrings[DSPSTR_PRMPT_ENG_TYPE],
	      gEngineTypeStrings[gUserInput.eEngineType] );

	Log ( INFORMATION, SCREENOUTPUTOFF, LOGOUTPUTON, NO_PROMPT,
	      "%s%s\n",
	      g_rgpcDisplayStrings[DSPSTR_PRMPT_PWRTRN_TYPE],
	      gPwrTrnTypeStrings[gUserInput.ePwrTrnType] );

	Log ( INFORMATION, SCREENOUTPUTOFF, LOGOUTPUTON, NO_PROMPT,
	      "%s%s\n",
	      g_rgpcDisplayStrings[DSPSTR_PRMPT_VEH_TYPE],
	      gVehicleTypeStrings[gUserInput.eVehicleType] );

	Log ( INFORMATION, SCREENOUTPUTOFF, LOGOUTPUTON, NO_PROMPT,
	      "%s%s\n",
	      g_rgpcDisplayStrings[DSPSTR_STMT_COMPLIANCE_TYPE],
	      gComplianceTestTypeStrings[gUserInput.eComplianceType] );

	// done
	return PASS;
}


/*
*******************************************************************************
**  VerifyIPTData
*******************************************************************************
*/
STATUS VerifyIPTData ( BOOL *pbIPTSupported )
{
	SID_REQ        SidReq;
	unsigned int   EcuIndex;
	unsigned char  fInf8Supported = FALSE;
	unsigned char  fInfBSupported = FALSE;


	SidReq.SID    = 0x22;
	SidReq.NumIds = 1;

	// INF $F808 request
	if ( IsDIDSupported ( -1, INF_TYPE_IPT_GAS ) == TRUE )
	{
		fInf8Supported = TRUE;
		*pbIPTSupported = TRUE;

		SidReq.DID[0] = INF_TYPE_IPT_GAS;
	}

	// Sid $22 In$F80B request
	if ( IsDIDSupported ( -1, INF_TYPE_IPT_DIESEL ) == TRUE )
	{
		fInfBSupported = TRUE;
		*pbIPTSupported = TRUE;

		SidReq.DID[0] = INF_TYPE_IPT_DIESEL;
	}


	if ( fInf8Supported == FALSE && fInfBSupported == FALSE )
	{
//		if ( ( (gUserInput.eComplianceType == US_OBDII || gUserInput.eComplianceType == HD_OBD) &&
//		       gModelYear >= 2005 ) ||
//		     (gUserInput.eComplianceType == EOBD_WITH_IUMPR ||
//		      gUserInput.eComplianceType == HD_EOBD) )
//		{
//			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//			     "IPT (INF $F808 or INF $F80B) not supported\n" );
//			return FAIL;
//		}
//		else
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "IPT (INF $F808 or INF $F80B) not supported\n" );
			return PASS;
		}
	 }
	 else if ( fInf8Supported == TRUE && fInfBSupported == TRUE )
	{
		for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
		{
			if ( IsDIDSupported ( EcuIndex, INF_TYPE_IPT_GAS ) == TRUE )
			{
				Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  INF $F808 supported\n",
				      GetEcuId ( EcuIndex ) );
			}
			if ( IsDIDSupported ( EcuIndex, INF_TYPE_IPT_DIESEL ) == TRUE )
			{
				Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  INF $F80B supported\n",
				      GetEcuId ( EcuIndex ) );
			}
		}
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "IPT (INF $F808 and INF $F80B) both supported\n" );
		return FAIL;
	}


	if ( SidRequest ( &SidReq, SID_REQ_NORMAL ) == FAIL )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "INF $%04X request\n",
		      SidReq.DID[0] );
		return FAIL;
	}

	for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
	{
		if ( IsDIDSupported ( EcuIndex, INF_TYPE_IPT_GAS ) == TRUE )
		{
			if ( VerifyINF8Data ( EcuIndex ) != FAIL )
			{
				if ( GetIPTData ( EcuIndex, &Test10_10_IPT[EcuIndex] ) != PASS )
				{
					Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "ECU %X  Get IPT (INF $F808) Data\n",
					      GetEcuId ( EcuIndex ) );
					return FAIL;
				}
			}
			else
			{
				return FAIL;
			}
		}
		else if ( IsDIDSupported ( EcuIndex, INF_TYPE_IPT_DIESEL ) == TRUE )
		{
			if ( VerifyINFBData ( EcuIndex ) == PASS )
			{
				if ( GetIPTData ( EcuIndex, &Test10_10_IPT[EcuIndex] ) != PASS )
				{
					Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "ECU %X  Get IPT (INF $F80B) Data\n",
					      GetEcuId ( EcuIndex ) );
					return FAIL;
				}
			}
			else
			{
				return FAIL;
			}
		}
	}

	if ( LogIPT () != PASS )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "Log IPT (INF $F808 or $F80B) Data\n" );
		return FAIL;
	}

	return PASS;
}


/*
*****************************************************************************
**
**  Function:  GetIPTData
**
**  Purpose:   Copy from gOBDResponse[EcuIndex].Inf into common,
**             protocol-independent format.
**
*****************************************************************************
*/
STATUS GetIPTData ( unsigned int EcuIndex, IPT *pIpt )
{
//	unsigned int    SidIndex;
//	unsigned int    IptIndex;
	unsigned short *pData;
	INF            *pINF;
	unsigned int    IPT_MAX;
	STATUS Result = FAIL;


	// clear IPT structure
	memset ( pIpt, 0, sizeof ( IPT ) );

	// point to IPT structure in gOBDResponse
	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// check to ensure that gOBDResponse contains IPT data
	if ( gOBDResponse[EcuIndex].InfSize == 0 ||
	     ( pINF->INFLSB != 0x08 && pINF->INFLSB != 0x0B ) )
	{
		return FAIL;
	}

	memcpy ( pIpt, &(gOBDResponse[EcuIndex].Inf[0]), sizeof ( IPT ) );
	memcpy ( &pLast_Ipt[EcuIndex], &(gOBDResponse[EcuIndex].Inf[0]), sizeof ( IPT ) );

	// If NOT test 9.19, 10.12 or 11.2, log failures
	if ( ( gTestPhase != eTestNoFault3DriveCycle || gTestSubsection != 19 ) &&
	     ( gTestPhase != eTestInUseCounters || gTestSubsection != 12 ) &&
	     ( gTestPhase != eTestPerformanceCounters || gTestSubsection != 2 ) )
	{
		if ( pINF->INFLSB == 0x08 )
		{
			Result = VerifyINF8Data ( EcuIndex );
		}
		else if ( pINF->INFLSB == 0x0B )
		{
			Result = VerifyINFBData ( EcuIndex );
		}

		if ( Result == FAIL )
		{
			return Result;
		}
	} // end if !test 9.19 and !test 10.12 and !test11.2

	// point to start of IPT data in structure
	pData = &(pIpt->Data[0]);

	// calculate number of IPT data bytes
	IPT_MAX = (pIpt->NODI <= INF_TYPE_IPT_NODI) ? (pIpt->NODI)*2 : INF_TYPE_IPT_NODI*2;

	// swap low and high bytes
//	for ( SidIndex = 2, IptIndex = 0; SidIndex <= IPT_MAX; SidIndex+=2 )
//	{
//		*pData++ = gOBDResponse[EcuIndex].Inf[SidIndex] * 256
//		         + gOBDResponse[EcuIndex].Inf[SidIndex+1];
//		pLast_Ipt[EcuIndex].Data[IptIndex++] = gOBDResponse[EcuIndex].Inf[SidIndex] * 256
//		                                       + gOBDResponse[EcuIndex].Inf[SidIndex+1];
//	}


	// data structure is valid
	pIpt->Flags = 1;

	return PASS;
}


/*
******************************************************************************
**  LogIPT
*****************************************************************************
*/
const char szECUID[]     = "ECU ID:";
const char szINF_SIZE[]  = "INF SIZE:";

const char szINF8[][10]  = { "OBDCOND",
                             "IGNCNTR",
                             "CATCOMP1",
                             "CATCOND1",
                             "CATCOMP2",
                             "CATCOND2",
                             "O2SCOMP1",
                             "O2SCOND1",
                             "O2SCOMP2",
                             "O2SCOND2",
                             "EGRCOMP",
                             "EGRCOND",
                             "AIRCOMP",
                             "AIRCOND",
                             "EVAPCOMP",
                             "EVAPCOND",
                             "SO2SCOMP1",
                             "SO2SCOND1",
                             "SO2SCOMP2",
                             "SO2SCOND2",
                             "AFRICOMP1",
                             "AFRICOND1",
                             "AFRICOMP2",
                             "AFRICOND2",
                             "PFCOMP1",
                             "PFCOND1",
                             "PFCOMP2",
                             "PFCOND2" };

const char szINFB[][10]  = { "OBDCOND",
                             "IGNCNTR",
                             "HCCATCOMP",
                             "HCCATCOND",
                             "NCATCOMP",
                             "NCATCOND",
                             "NADSCOMP",
                             "NADSCOND",
                             "PMCOMP",
                             "PMCOND",
                             "EGSCOMP",
                             "EGSCOND",
                             "EGRCOMP",
                             "EGRCOND",
                             "BPCOMP",
                             "BPCOND",
                             "FUELCOMP",
                             "FUELCOND" };

STATUS LogIPT ( void )
{
	SID_REQ       SidReq;
	unsigned int  EcuIndex;
	unsigned int  IptIndex;
	IPT           Ipt;
	unsigned char fInf8Supported = FALSE;
	unsigned char fInfBSupported = FALSE;


	SidReq.SID    = 0x22;
	SidReq.NumIds = 1;

	// Is INF $F808 supported by any ECU?
	if ( IsDIDSupported (-1, INF_TYPE_IPT_GAS ) == TRUE )
	{
		fInf8Supported = TRUE;

		// INF $F808 request
		SidReq.DID[0] = INF_TYPE_IPT_GAS;
	}

	// IS INF $F80B supported by any ECU??
	if ( IsDIDSupported (-1, INF_TYPE_IPT_DIESEL ) == TRUE )
	{
		fInfBSupported = TRUE;

		// INF $F80B request
		SidReq.DID[0] = INF_TYPE_IPT_DIESEL;
	}

	if ( fInf8Supported == FALSE && fInfBSupported == FALSE )
	{
//		if ( ( (gUserInput.eComplianceType == US_OBDII || gUserInput.eComplianceType == HD_OBD) &&
//		       gModelYear >= 2005 ) ||
//		     (gUserInput.eComplianceType == EOBD_WITH_IUMPR || gUserInput.eComplianceType == HD_EOBD) )
//		{
//			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//			     "IPT (INF $F808 or INF $F80B) not supported\n" );
//			return FAIL;
//		}
//		else
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "IPT (INF $F808 or INF $F80B) not supported\n" );
			return PASS;
		}
	}
	else if ( fInf8Supported == TRUE && fInfBSupported == TRUE )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "IPT (INF $F808 and INF $F80B) both supported\n" );
		return FAIL;
	}

	if ( SidRequest ( &SidReq, SID_REQ_NORMAL ) == FAIL )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "INF $%04X request\n",
		      SidReq.DID[0] );
		return FAIL;
	}

	for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
	{
		if ( GetIPTData ( EcuIndex, &Ipt ) == PASS )
		{
			// log OBDCOND, IGNCTR, all OBD monitor and completion counters
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "%-9s 0x%X\n",
			      szECUID,
			      GetEcuId ( EcuIndex ) );
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "%-9s $F8%02X %d\n",
			      szINF_SIZE,
			      Ipt.INFLSB,
			      Ipt.NODI );

			for ( IptIndex = 0; IptIndex < Ipt.NODI && IptIndex < INF_TYPE_IPT_NODI; IptIndex++ )
			{
				Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "%-9s = %u\n",
				      (Ipt.INFLSB == 0x08) ? szINF8[IptIndex] : szINFB[IptIndex],
				      Ipt.Data[IptIndex] );
			}
		}
	}

	return PASS;
}


/*
*****************************************************************************
**  ReadIPT
*****************************************************************************
*/
BOOL ReadIPT ( const char * szTestSectionEnd, IPT Ipt[] )
{
	char  buf[256];
	char *p;
	int   count;
	unsigned int EcuIndex;
	unsigned int EcuId = 0;

	// search for ECU ID
	while ( fgets ( buf, sizeof ( buf ), ghLogFile ) != 0 )
	{
		if ( substring ( buf, szTestSectionEnd ) != 0 )     // end of Test XX section
		{
			return FALSE;
		}

		if ( (p = substring ( buf, szECUID )) != 0 )
		{
			p += sizeof ( szECUID );
			EcuId = strtoul ( p, NULL, 16 );
			break;
		}
	}

	// find EcuIndex
	for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
	{
		if ( EcuId == GetEcuId ( EcuIndex ) )
		{
			break;
		}
	}

	if ( EcuIndex >= gUserNumEcus )
	{
		return FALSE;
	}

	// search for IPT INFOTYPE and SIZE
	while ( fgets ( buf, sizeof ( buf ), ghLogFile ) != 0 )
	{
		if ( substring ( buf, szTestSectionEnd ) != 0 )     // end of Test XX section
		{
			return FALSE;
		}

		if ( (p = substring ( buf, szINF_SIZE )) != 0 )
		{
			p += sizeof ( szINF_SIZE );
			Ipt[EcuIndex].INFLSB = (unsigned char) strtoul ( p, NULL, 16 );
			p += 3;
			Ipt[EcuIndex].NODI = (unsigned char) strtod ( p, NULL );
			break;
		}
	}

	// read counters
	for ( count = 0; count < Ipt[EcuIndex].NODI; count++ )
	{
		fgets ( buf, sizeof ( buf ), ghLogFile );

		if ( ( Ipt[EcuIndex].INFLSB == 0x08 && (p = substring ( buf, szINF8[count] )) != 0 ) ||
		     ( Ipt[EcuIndex].INFLSB == 0x0B && (p = substring ( buf, szINFB[count] )) != 0 ) )
		{
			p = substring ( p, "=" ) + 1;
			Ipt[EcuIndex].Data[count] = (unsigned short)strtoul ( p, NULL, 10 );
			continue;
		}

		return FALSE;
	}

	// Sid 22 Inf $F808 data valid
	Ipt[EcuIndex].Flags |= VALIDDATA;

	return TRUE;
}


/*
******************************************************************************
**  LogInf12
*****************************************************************************
*/
STATUS LogInf12 ( IPT pIpt[] )
{
	unsigned int    EcuIndex;
	SID_REQ         SidReq;
	INF            *pINF;

	STATUS Result = PASS;
	unsigned char   fInf12Supported = FALSE;


	// clear INF structure
	memset ( pIpt, 0, sizeof ( IPT ) * gUserNumEcus + 1 );

	// INF $F812 request
	if ( IsDIDSupported ( -1, INF_TYPE_FEOCNTR ) == FALSE )
	{
//		if ( gUserInput.eComplianceType == US_OBDII &&
//		     gOBDPlugInFlag == TRUE &&
//		     gModelYear >= 2014 )
//		{
//			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//			     "INF $F812 not supported (Required for Plug-In Hybrid vehicles MY 2014 and later).\n" );
//			return FAIL;
//		}
//		else
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "INF $F812 not supported\n" );
			return PASS;
		}
	}

	SidReq.SID    = 0x22;
	SidReq.NumIds = 1;
	SidReq.DID[0] = INF_TYPE_FEOCNTR;

	if ( SidRequest ( &SidReq, SID_REQ_NORMAL ) == FAIL )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "INF $F812 request\n" );
		return FAIL;
	}


	for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
	{
		// point to IPT structure in gOBDResponse
		pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

		// check to ensure that gOBDResponse contains appropriate data
		if ( IsDIDSupported ( EcuIndex, INF_TYPE_FEOCNTR ) == FALSE )
		{
			continue;
		}
		else if ( gOBDResponse[EcuIndex].InfSize == 0 ||
		          pINF->INFLSB != 0x12 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  Invalid response to INF $F812 request\n",
			      GetEcuId ( EcuIndex ) );
			Result = FAIL;
		}
		else
		{
			if ( VerifyInf12Data ( EcuIndex ) == FAIL )
			{
				continue;
			}

			memcpy ( &pIpt[EcuIndex], &(gOBDResponse[EcuIndex].Inf[0]), sizeof ( IPT ) );

			pIpt[EcuIndex].Data[0] = (gOBDResponse[EcuIndex].Inf[2] * 256) + gOBDResponse[EcuIndex].Inf[3];

			// If NOT test 9.19, 10.12 or 11.2, log
			if ( ( gTestPhase != eTestNoFault3DriveCycle  || gTestSubsection != 19 ) &&
			     ( gTestPhase != eTestInUseCounters       || gTestSubsection != 12 ) &&
			     ( gTestPhase != eTestPerformanceCounters || gTestSubsection != 2 ) )
			{
				// log FEOCNTR
				Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  INF $F812  FEOCNTR = %d\n",
				      GetEcuId ( EcuIndex ),
				      pIpt[EcuIndex].Data[0] );
			}
		}
	}

	return Result;
}


/*
******************************************************************************
**  VerifyInf12
*****************************************************************************
*/
STATUS VerifyInf12 ( IPT *pIpt )
{
	SID_REQ         SidReq;
	unsigned int    EcuIndex;
	INF            *pINF;
	unsigned int    FEOCNTR;
	STATUS Result = PASS;
	unsigned char   fInf12Supported = FALSE;


	// INF $F812 request
	if ( IsDIDSupported ( -1, INF_TYPE_FEOCNTR ) == FALSE )
	{
//		if ( gUserInput.eComplianceType == US_OBDII &&
//		     gOBDPlugInFlag == TRUE &&
//		     gModelYear >= 2014 )
//		{
//			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//			     "INF $F812 not supported (Required for Plug-In Hybrid vehicles MY 2014 and later).\n" );
//			return FAIL;
//		}
//		else
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "INF $F812 not supported\n" );
			return PASS;
		}
	}

	SidReq.SID    = 0x22;
	SidReq.NumIds = 1;
	SidReq.DID[0] = INF_TYPE_FEOCNTR;

	if ( SidRequest ( &SidReq, SID_REQ_NORMAL ) == FAIL )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "INF $F812 request\n" );
		return FAIL;
	}


	for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
	{
		// point to IPT structure in gOBDResponse
		pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];


		// check to ensure that gOBDResponse contains appropriate data
		if ( IsDIDSupported ( EcuIndex, INF_TYPE_FEOCNTR ) == FALSE )
		{
			continue;
		}
		else if ( gOBDResponse[EcuIndex].InfSize == 0 ||
		          pINF->INFLSB != 0x12 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  Invalid response to INF $F812 request\n",
			      GetEcuId ( EcuIndex ) );
			Result = FAIL;
		}
		else
		{
			FEOCNTR = (gOBDResponse[EcuIndex].Inf[2] * 256) + gOBDResponse[EcuIndex].Inf[3];

			// log FEOCNTRs
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F812  FEOCNTR = %d\n",
			      GetEcuId ( EcuIndex ),
			      FEOCNTR );

			if ( FEOCNTR <= (unsigned int)(pIpt[EcuIndex].Data[0]) )
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  INF $F812  FEOCNTR has not incremented since Test 10.10.4\n",
				      GetEcuId ( EcuIndex ) );
				Result = FAIL;
			}
		}
	}

	return Result;
}


/*
*******************************************************************************
**  RunDynamicTest10
*******************************************************************************
*/
STATUS RunDynamicTest10 ( unsigned long tEngineStartTimeStamp )
{
	unsigned char  fSubTestStatus = SUB_TEST_NORMAL;
	unsigned char  fLastTestStatus = SUB_TEST_INITIAL; // set to force an update the first time
	unsigned char  PID01C;
	unsigned char  key;
	unsigned int   EcuIndex;
	unsigned int   EcuPageOffset = 0;
	unsigned int   EcuPage = 1;
	BOOL           ECUPageUpdate = FALSE;
	unsigned int   TestState;
	unsigned int   bDone = 0;
	unsigned int   bLoop;
	unsigned int   bFail;
	unsigned int   bRunTimeSupport;
	unsigned int   bVSSSupport;
	unsigned int   bRPMSupport;
	unsigned int   bBAROSupport;
	unsigned int   bAATEMPSupport;
	unsigned int   Baro_InHg;
	unsigned int   AAT_C;
	BOOL           useRPM = FALSE;  // indicate use of RPM for Idle and Drive time calculations
	unsigned int   bIPT = 0;    // support INF for IPT data, default 0 = no support
	unsigned long  t1SecTimer, tDelayTimeStamp;

	BOOL           ErrorPrinted = FALSE;    // indicates whether or not the failure message has already been printed

	BOOL           bPermDTC;

	unsigned short tTestStartTime, tTestCompleteTime;
	unsigned short tTempTime;
	unsigned short tIdleTime;       // time at idle (stops at 30)
	unsigned short tAtSpeedTime;    // time at speeds > 25mph (stops at 300)
	unsigned short RPM;
	unsigned short SpeedMPH;
	unsigned short SpeedKPH;
	unsigned short RunTime;
	unsigned short tFueledRunTime;
	unsigned short tFRTempTime;
	unsigned short tObdCondTimestamp[OBD_MAX_ECUS];

	SID_REQ        SidReq;
	IPT            *pIpt;
	IPT            *pFEOCNTR;
	PID            *pPid;

	STATUS eResult = PASS;


	// determine PID support
	bRunTimeSupport = IsDIDSupported ( -1, 0xF41F );

	bVSSSupport     = IsDIDSupported ( -1, 0xF40D );

	bRPMSupport     = IsDIDSupported ( -1, 0xF40C );

	bBAROSupport    = IsDIDSupported ( -1, 0xF433 );

	bAATEMPSupport  = IsDIDSupported ( -1, 0xF446 );

	if ( bRPMSupport == FALSE && bRunTimeSupport == FALSE )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "PIDs $F40C (RPM) and $F41F (RUNTM) not supported\n" );
		return FAIL;
	}

	//-------------------------------------------------------------------------
	// Request OBD_Type - SID 22 PID $F41C.  Needed to determine wether to use
	// Vehicle Speed (KPH/MPH) or Egine Speed (RPM) to determine idel and drive time.
	//-------------------------------------------------------------------------
	SidReq.SID    = 0x22;
	SidReq.NumIds = 1;
	SidReq.DID[0] = 0xF41C;

	eResult = SidRequest ( &SidReq, SID_REQ_NO_PERIODIC_DISABLE|SID_REQ_IGNORE_NO_RESPONSE );

	if ( eResult == PASS )
	{
		for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
		{
			if ( gOBDResponse[EcuIndex].PidSize > 0 )
			{
				pPid = (PID *)&gOBDResponse[EcuIndex].Pid[0];

				if ( pPid->PIDLSB == 0x1c )
				{
					PID01C = pPid->Data[0];
					break;
				}
			}
		}
	}


	//-------------------------------------------------------------------------
	// Determine whether to use Vehicle Speed (KPH/MPH) or Egine Speed (RPM) for idel and drive time calculations.
	//-------------------------------------------------------------------------
	if ( (gUserInput.eComplianceType == HD_OBD && gOBDDieselFlag == TRUE) ||
	     (gUserInput.eComplianceType == US_OBDII && gOBDDieselFlag == TRUE && PID01C == 0x22) )
	{
		useRPM = TRUE;
	}
	else if ( gUserInput.eComplianceType == HD_EOBD ||
	          (gUserInput.eComplianceType == US_OBDII && gOBDDieselFlag == TRUE && gUserInput.eVehicleType == MD) )
	{
		if ( bVSSSupport )
		{
			useRPM = FALSE;
		}
		else
		{
			useRPM = TRUE;
		}
	}
	else
	{
		useRPM = FALSE;
	}


	if ( IsDIDSupported ( -1, INF_TYPE_IPT_GAS ) == TRUE )
	{
		bIPT = INF_TYPE_IPT_GAS;
	}
	else if ( IsDIDSupported ( -1, INF_TYPE_IPT_DIESEL ) == TRUE )
	{
		bIPT = INF_TYPE_IPT_DIESEL;
	}
//	else if ( ( (gUserInput.eComplianceType == US_OBDII || gUserInput.eComplianceType == HD_OBD) &&
//	            gModelYear >= 2005 ) ||
//	          (gUserInput.eComplianceType == EOBD_WITH_IUMPR || gUserInput.eComplianceType == HD_EOBD) )
//	{
//		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//		     "IPT (INF $F808 or INF $F80B) not supported\n" );
//		return FAIL;
//	}
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "IPT (INF $F808 or INF $F80B) not supported\n" );
	}


	//-------------------------------------------
	/// Initialize Array */
	//-------------------------------------------
	pIpt = (IPT *) malloc ( sizeof ( IPT ) * gNumEcusForAlloc );
	memset ( pIpt, 0x00, sizeof ( IPT ) * gNumEcusForAlloc );

	pFEOCNTR = (IPT *) malloc ( sizeof ( IPT ) * gNumEcusForAlloc );
	memset ( pFEOCNTR, 0x00, sizeof ( IPT ) * gNumEcusForAlloc );


	//-------------------------------------------
	// initialize static text elements
	//-------------------------------------------
	if ( gTestPhase == eTestNoFault3DriveCycle )
	{
		init_screen ( _string_elements_1st_9, _num_string_elements_1st_9 );
	}
	else
	{
		init_screen ( _string_elements_1st_10, _num_string_elements_1st_10 );
	}

	place_screen_text ( _string_elements10, _num_string_elements10 );
	// if using RPM for speed monitoring, update appropriate text
	if ( useRPM == TRUE )
	{
		place_screen_text ( _string_elements10_RPM, _num_string_elements10_RPM );
	}

	// if this is a PHEV, update appropriate text
	if ( gUserInput.ePwrTrnType == PHEV )
	{
		place_screen_text ( _string_elements10_PHEV, _num_string_elements10_PHEV );
	}

	for ( EcuIndex = 0; EcuIndex < gUserNumEcus && EcuIndex < MAX_ECUS_PER_PAGE; EcuIndex++ )
	{
		// initialize ECU IDs
		SetFieldHex ( ECU_ID+EcuIndex, GetEcuId ( EcuIndex ) );

		// initialize initial OBDCONDs
		SetFieldDec ( INITIAL_OBDCOND+EcuIndex, Test10_10_IPT[EcuIndex].Data[0] );

		// initialize initial IGNCTRs
		SetFieldDec ( INITIAL_IGNCTR+EcuIndex, Test10_10_IPT[EcuIndex].Data[1] );

		if ( gOBDPlugInFlag == TRUE )
		{
			// initialize initial FEOCNTRs
			SetFieldDec ( INITIAL_FEOCNTR+EcuIndex, pTest10_10_FEOCNTR[EcuIndex].Data[0] );
		}

		tObdCondTimestamp[EcuIndex] = 0;

		// if INF 8/B not supported, print 0 (INF8/B will never update)
		if ( bIPT == 0 )
		{
			SetFieldDec ( CURRENT_OBDCOND+EcuIndex, pIpt[EcuIndex].Data[0] );
			SetFieldDec ( CURRENT_IGNCTR+EcuIndex,  pIpt[EcuIndex].Data[1] );
		 }
	}
	SetFieldDec ( ECUPAGE_INDEX, EcuPage );

	// flush the STDIN stream of any user input before loop
	clear_keyboard_buffer ();

	RPM = SpeedMPH = SpeedKPH = RunTime = 0;
	tFueledRunTime = 0;
	tFRTempTime = 0;
	tAtSpeedTime = 0;
	tIdleTime = 0;
	tTempTime = 0;
	tTestCompleteTime = 0;
	tTestStartTime = (unsigned short)(tEngineStartTimeStamp / 1000);
	tDelayTimeStamp = t1SecTimer = GetTickCount ();

	TestState = 0xFF;

	//-------------------------------------------
	// loop until test completes
	//-------------------------------------------
	for (;;)
	{
		if ( gTestPhase == eTestNoFault3DriveCycle )
		{
			//-------------------------------------------
			// request Permanent Codes - SID $19 subfunction $55
			//-------------------------------------------
			SidReq.SID    = 0x19;
			SidReq.NumIds = 2;
			SidReq.DID[0] = 0x55;  // reportWWHOBDDTCWithPrmanentStatus
			SidReq.DID[1] = 0x33;  // Functional Group Identifier

			eResult = SidRequest ( &SidReq, SID_REQ_NO_PERIODIC_DISABLE|SID_REQ_IGNORE_NO_RESPONSE );

			if ( eResult == FAIL )
			{
				if ( ErrorPrinted == FALSE )
				{
					Log ( FAILURE, SCREENOUTPUTOFF, LOGOUTPUTON, NO_PROMPT,
					      "SID $19 subfunction $55 request\n" );
					fSubTestStatus |= SUB_TEST_FAIL;
				}
				ErrorPrinted = TRUE;
			}
			else
			{
				if ( eResult != PASS )
				{
					// cover the case where a response was early/late
					fSubTestStatus |= SUB_TEST_FAIL;
				}

				bPermDTC = FALSE;
				for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
				{
					if ( (gOBDResponse[EcuIndex].PermDTC[0] != 0x00) || (gOBDResponse[EcuIndex].PermDTC[1] != 0x00) )
					{
						bPermDTC = TRUE;
					}
				}

				if ( bPermDTC == FALSE )
				{
					Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "SID $19 Subfunction $55 - DTC erased prematurely.\n" );

					free ( pIpt );
					free ( pFEOCNTR );
					gotoxy ( 0, BOTTOM_ROW+2);
					return FAIL;
				}
			}
		}


		//-------------------------------------------
		// request RPM - PID $F40C
		//-------------------------------------------
		if ( bRPMSupport == TRUE )
		{
			SidReq.SID    = 0x22;
			SidReq.NumIds = 1;
			SidReq.DID[0] = 0xF40C;

			eResult = SidRequest ( &SidReq, SID_REQ_NO_PERIODIC_DISABLE|SID_REQ_IGNORE_NO_RESPONSE );

			if ( eResult == FAIL )
			{
				Log ( FAILURE, SCREENOUTPUTOFF, LOGOUTPUTON, NO_PROMPT,
				      "PID $F40C request\n" );
				fSubTestStatus |= SUB_TEST_FAIL;
			}
			else
			{
				if ( eResult != PASS )
				{
					// cover the case where a response was early/late
					fSubTestStatus |= SUB_TEST_FAIL;
				}

				for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
				{
					if ( gOBDResponse[EcuIndex].PidSize > 0 )
					{
						pPid = (PID *)&gOBDResponse[EcuIndex].Pid[0];

						if ( pPid->PIDLSB == 0x0c )
						{
							RPM = pPid->Data[0];
							RPM = RPM << 8 | pPid->Data[1];

							// convert from 1 cnt = 1/4 RPM to 1 cnt = 1 RPM
							RPM >>= 2;
							break;
						}
					}
				}

				if ( EcuIndex >= gUserNumEcus )
				{
					Log ( FAILURE, SCREENOUTPUTOFF, LOGOUTPUTON, NO_PROMPT,
					      "PID $F40C missing response\n" );
					fSubTestStatus |= SUB_TEST_FAIL;
				}
			}
		}

		//-------------------------------------------
		// request Speed - PID $F40D
		//-------------------------------------------
		if ( bVSSSupport == TRUE  )
		{
			SidReq.SID    = 0x22;
			SidReq.NumIds = 1;
			SidReq.DID[0] = 0xF40D;

			eResult = SidRequest ( &SidReq, SID_REQ_NO_PERIODIC_DISABLE|SID_REQ_IGNORE_NO_RESPONSE );

			if ( eResult == FAIL )
			{
				Log ( FAILURE, SCREENOUTPUTOFF, LOGOUTPUTON, NO_PROMPT,
				      "PID $F40D request\n" );
				fSubTestStatus |= SUB_TEST_FAIL;
			}
			else
			{
				if ( eResult != PASS )
				{
					// cover the case where a response was early/late
					fSubTestStatus |= SUB_TEST_FAIL;
				}

				for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
				{
					if ( gOBDResponse[EcuIndex].PidSize > 0 )
					{
						pPid = (PID *)&gOBDResponse[EcuIndex].Pid[0];

						if ( pPid->PIDLSB == 0x0d )
						{
							double dTemp = pPid->Data[0];

							SpeedKPH = pPid->Data[0];

							// convert from km/hr to mile/hr
							SpeedMPH = (unsigned short)( ((dTemp * 6214) / 10000) + 0.5 );
							break;
						}
					}
				}

				if ( EcuIndex >= gUserNumEcus )
				{
					Log ( FAILURE, SCREENOUTPUTOFF, LOGOUTPUTON, NO_PROMPT,
					      "PID $F40D missing response\n" );
					fSubTestStatus |= SUB_TEST_FAIL;
				}
			}
		}

		//-------------------------------------------
		// request engine RunTime - PID $F41F
		//-------------------------------------------
		if ( bRunTimeSupport == TRUE )
		{
			SidReq.SID    = 0x22;
			SidReq.NumIds = 1;
			SidReq.DID[0] = 0xF41F;

			eResult = SidRequest ( &SidReq, SID_REQ_NO_PERIODIC_DISABLE|SID_REQ_IGNORE_NO_RESPONSE );

			if ( eResult == FAIL )
			{
				Log ( FAILURE, SCREENOUTPUTOFF, LOGOUTPUTON, NO_PROMPT,
				      "PID $F41F request\n" );
				fSubTestStatus |= SUB_TEST_FAIL;
			}
			else
			{
				if ( eResult != PASS )
				{
					// cover the case where a response was early/late
					fSubTestStatus |= SUB_TEST_FAIL;
				}

				for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
				{
					if ( gOBDResponse[EcuIndex].PidSize > 0 )
					{
						pPid = (PID *)&gOBDResponse[EcuIndex].Pid[0];

						if ( pPid->PIDLSB == 0x1f )
						{
							RunTime = pPid->Data[0];
							RunTime = RunTime << 8 | pPid->Data[1];    // 1 cnt = 1 sec
							break;
						}
					}
				}

				if ( EcuIndex >= gUserNumEcus )
				{
					Log ( FAILURE, SCREENOUTPUTOFF, LOGOUTPUTON, NO_PROMPT,
					      "PID $F41F missing response\n" );
					fSubTestStatus |= SUB_TEST_FAIL;
				}
			}
		}
		else
		{
			RunTime = (unsigned short)(GetTickCount () / 1000) - tTestStartTime;
		}


		//-------------------------------------------
		// request Barometric Pressure - PID $F433
		//-------------------------------------------
		if ( bBAROSupport == TRUE )
		{
			SidReq.SID    = 0x22;
			SidReq.NumIds = 1;
			SidReq.DID[0] = 0xF433;

			eResult = SidRequest ( &SidReq, SID_REQ_NO_PERIODIC_DISABLE|SID_REQ_IGNORE_NO_RESPONSE );

			if ( eResult != FAIL )
			{
				for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
				{
					if ( gOBDResponse[EcuIndex].PidSize > 0 )
					{
						pPid = (PID *)&gOBDResponse[EcuIndex].Pid[0];

						if ( pPid->PIDLSB == 0x33 )
						{
							Baro_InHg = (unsigned int)((float)(pPid->Data[0]) * 0.295301);  // Convert from kPa to in Hg
						}
					}
				}
			}
		}


		//-------------------------------------------
		// request Ambient Air Temperature - PID $F446
		//-------------------------------------------
		if ( bAATEMPSupport == TRUE )
		{
			SidReq.SID    = 0x22;
			SidReq.NumIds = 1;
			SidReq.DID[0] = 0xF446;

			eResult = SidRequest ( &SidReq, SID_REQ_NO_PERIODIC_DISABLE|SID_REQ_IGNORE_NO_RESPONSE );

			if ( eResult != FAIL )
			{
				for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
				{
					if ( gOBDResponse[EcuIndex].PidSize > 0 )
					{
						pPid = (PID *)&gOBDResponse[EcuIndex].Pid[0];

						if ( pPid->PIDLSB == 0x46 )
						{
							AAT_C = pPid->Data[0] - 40;  // 1 C with -40 offset
						}
					}
				}
			}
		}


		//-------------------------------------------
		// Get SID $22 IPT
		//-------------------------------------------
		if ( bIPT != 0 )
		{
			SidReq.SID    = 0x22;
			SidReq.NumIds = 1;
			SidReq.DID[0] = bIPT;

			eResult = SidRequest ( &SidReq, SID_REQ_NO_PERIODIC_DISABLE|SID_REQ_IGNORE_NO_RESPONSE );

			if ( eResult == FAIL )
			{
				Log ( FAILURE, SCREENOUTPUTOFF, LOGOUTPUTON, NO_PROMPT,
				      "INF $%04X request\n",
				      SidReq.DID[0] );
				fSubTestStatus |= SUB_TEST_FAIL;
			}
			else
			{
				if ( eResult != PASS )
				{
					// cover the case where a response was early/late
					fSubTestStatus |= SUB_TEST_FAIL;
				}

				for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
				{
					if ( GetIPTData ( EcuIndex, &pIpt[EcuIndex] ) == PASS )
					{
						if ( EcuIndex >= EcuPageOffset &&
						     EcuIndex < (EcuPageOffset + MAX_ECUS_PER_PAGE) )
						{
							SetFieldDec ( CURRENT_OBDCOND+(EcuIndex-EcuPageOffset), pIpt[EcuIndex].Data[0] );
							SetFieldDec ( CURRENT_IGNCTR+(EcuIndex-EcuPageOffset),  pIpt[EcuIndex].Data[1] );
						}

						// check when OBDCOND counters increment
						if ( tObdCondTimestamp[EcuIndex] == 0 &&
						     pIpt[EcuIndex].Data[0] > Test10_10_IPT[EcuIndex].Data[0] )
						{
							tObdCondTimestamp[EcuIndex] = RunTime;
							Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
							      "Idle Time = %d;  Speed Time = %d;  Run Time = %d  *OBDCOND DONE*\n",
							      tIdleTime, tAtSpeedTime, RunTime );

							// check for extremely early increments of OBDCOND and FAIL
							// (that is, OBDCOND that increment more than 30 seconds before the test will end)
							if ( tIdleTime == 0 || tAtSpeedTime < 270 || RunTime < 570 )
							{
								Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
								      "OBDCOND incremented much too soon\n" );
								fSubTestStatus |= SUB_TEST_FAIL;
							}
						}
					}
					else
					{
						if ( EcuIndex >= EcuPageOffset &&
						     EcuIndex < (EcuPageOffset + MAX_ECUS_PER_PAGE) )
						{
							SetFieldDec ( CURRENT_OBDCOND+(EcuIndex-EcuPageOffset), 0 );
							SetFieldDec ( CURRENT_IGNCTR+(EcuIndex-EcuPageOffset),  0 );
						}
					}
				}
			}
		}

		//-------------------------------------------
		// Get SID $22 FEOCNTR
		//-------------------------------------------
		if ( gOBDPlugInFlag == TRUE )
		{
			if ( LogInf12 ( pFEOCNTR ) == PASS )
			{
				for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
				{
					if ( EcuIndex >= EcuPageOffset &&
					     EcuIndex < (EcuPageOffset + MAX_ECUS_PER_PAGE) )
					{
						SetFieldDec ( CURRENT_FEOCNTR+(EcuIndex-EcuPageOffset), pFEOCNTR[EcuIndex].Data[0] );
					}
				}
			}
		}


		//-------------------------------------------
		// Update current PIDs, total engine time
		//-------------------------------------------
		if ( bRunTimeSupport == FALSE || useRPM == TRUE )
		{
			SetFieldDec ( RPM_INDEX, RPM );
		}
		else
		{
			SetFieldDec ( SPEED_INDEX_MPH, SpeedMPH );
			SetFieldDec ( SPEED_INDEX_KPH, SpeedKPH );
		}
		SetFieldDec ( TOTAL_DRIVE_TIMER, RunTime );

		if ( bBAROSupport == TRUE )
		{
			SetFieldDec ( BARO_PRESS_INDEX, Baro_InHg );
		}

		if ( bAATEMPSupport == TRUE )
		{
			SetFieldDec ( AMB_AIR_TEMP_INDEX, AAT_C );
			SetFieldDec ( AMB_AIR_TEMP_INDEX+1, ((AAT_C*9)/5)+32 );
		}


		//-------------------------------------------
		// Check for Vehicle Speed > 1 MPH when 0 RPM and 0 Run Time
		//-------------------------------------------
		if ( (bVSSSupport == TRUE && SpeedMPH > 1) &&
		     (bRPMSupport == TRUE && RPM == 0) &&
		     (bRunTimeSupport == TRUE && RunTime == 0) )
		{
			Log ( FAILURE, SCREENOUTPUTOFF, LOGOUTPUTON, NO_PROMPT,
			      "Vehicle Speed > 1 mph with 0 rpm and 0 runtime\n" );
			fSubTestStatus |= SUB_TEST_FAIL;
		}


		//-------------------------------------------
		// Update test status
		//-------------------------------------------
		if ( fSubTestStatus != fLastTestStatus )
		{
			if ( (fSubTestStatus & SUB_TEST_FAIL) == SUB_TEST_FAIL )
			{
				setrgb ( HIGHLIGHTED_TEXT );
				SetFieldText ( TESTSTATUS_INDEX, "FAILURE" );
				setrgb ( NORMAL_TEXT );
			}
			else
			{
				SetFieldText ( TESTSTATUS_INDEX, "Normal" );
			}

			fLastTestStatus = fSubTestStatus;
		}


		//-------------------------------------------
		// Determine phase of dynamic test
		// update screen
		//-------------------------------------------
		bLoop = TRUE;
		while ( bLoop )
		{
			bLoop = FALSE;
			switch ( TestState )
			{
				case 0xFF:   // First Time ONLY - covers the case were the engine has already been idling
					tTempTime = RunTime;
					tFRTempTime = RunTime;

					if ( (bRunTimeSupport == TRUE) ? (RunTime > 0) : (RPM > 450) )
					{
						if ( SpeedMPH <= 1 )
						{
							TestState = 1;
							tTempTime = RunTime;
							tIdleTime = RunTime;
							bLoop = TRUE;
							break;
						}
					}

					//
					// engine is not running or SpeedMPH >1... don't know if there was an idle
					// set-up to intentionally fall thru to case 0
					//
					TestState = 0;

				case 0:     // wait until idle  (RunTime > 0 or RPM > 450)
					if ( (bRunTimeSupport == TRUE) ? (RunTime > 0) : (RPM > 450) )
					{
						// check 600 seconds cumulative time
						if ( (bDone & CUMULATIVE_TIME) != CUMULATIVE_TIME && RunTime >= 600 )
						{
							tTestCompleteTime = RunTime;
							bDone |= CUMULATIVE_TIME;
							Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
							      "Idle Time = %d;  Speed Time = %d;  Run Time = %d  *RUN TIME DONE*\n",
							      tIdleTime, tAtSpeedTime, RunTime );
						}

						// Check for idle speed
						if ( ((useRPM == FALSE) ? (SpeedMPH <= 1 ) : (RPM > 450)) &&
						     tIdleTime < 30 )
						{
							TestState = 1;
							tIdleTime = 0;
							bLoop = TRUE;
						}
						// Check for drive speed
						else if ( ((useRPM == FALSE) ? (SpeedMPH >= 25 ) : (RPM > 1150)) &&
						          tAtSpeedTime < 300 )
						{
							TestState = 2;
							bLoop = TRUE;
						}
						// Check for test complete
						else if ( bDone == (IDLE_TIME | ATSPEED_TIME | CUMULATIVE_TIME | FEO_TIME) )
						{
							TestState = 3;
							bLoop = TRUE;
						}
						else
						{
							tTempTime = RunTime;
						}
					}
					break;

				case 1:     // 30 seconds continuous time at idle
					// (   (VSS <= 1mph (if supported) or 450 < RPM < 1150 (if VSS not supported)) AND
					//     (RunTime > 0 (if supported) or RPM > 450 (if RUNTIME not supported)) )
					if ( ((useRPM == FALSE) ? (SpeedMPH <= 1 ) : (RPM > 450 && RPM < 1150 )) &&
					     ((bRunTimeSupport == FALSE) ? (RPM > 450) : 1) )
					{
						// check 600 seconds cumulative time
						if ( (bDone & CUMULATIVE_TIME) != CUMULATIVE_TIME && RunTime >= 600 )
						{
							tTestCompleteTime = RunTime;
							bDone |= CUMULATIVE_TIME;
							Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
							      "Idle Time = %d;  Speed Time = %d;  Run Time = %d  *RUN TIME DONE*\n",
							      tIdleTime, tAtSpeedTime, RunTime );
						}

						// check idle time
						tIdleTime = min ( tIdleTime + RunTime - tTempTime, 30 );
						tTempTime = RunTime;

						SetFieldDec ( IDLE_TIMER, tIdleTime );

						if ( tIdleTime >= 30 )
						{
							tTestCompleteTime = RunTime;
							bDone |= IDLE_TIME;
							TestState = 0;
							bLoop = TRUE;
							Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
							      "Idle Time = %d;  Speed Time = %d;  Run Time = %d  *IDLE DONE*\n",
							      tIdleTime, tAtSpeedTime, RunTime );
						}

					}
					else
					{
						TestState = 0;
						bLoop = TRUE;
					}
					break;

				case 2:     // 300 seconds cumulative time at SpeedMPH >= 25 KPH
					// (   (VSS >= 25mph (if supported) or RPM >= 1150 (if VSS not supported)) AND
					//     (RunTime > 0 (if supported) or RPM >= 450 (if RUNTIME not supported)) )
					if ( ((useRPM == FALSE) ? (SpeedMPH >= 25) : (RPM >= 1150 )) &&
					     ((bRunTimeSupport == FALSE) ? (RPM > 450 ) : 1) )
					{
						// check 600 seconds cumulative time
						if ( (bDone & CUMULATIVE_TIME) != CUMULATIVE_TIME && RunTime >= 600 )
						{
							tTestCompleteTime = RunTime;
							bDone |= CUMULATIVE_TIME;
							Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
							      "Idle Time = %d;  Speed Time = %d;  Run Time = %d  *RUN TIME DONE*\n",
							      tIdleTime, tAtSpeedTime, RunTime );
						}

						// check at speed time
						tAtSpeedTime = min ( tAtSpeedTime + RunTime - tTempTime, 300 );
						tTempTime = RunTime;

						SetFieldDec (SPEED_25_MPH_TIMER, tAtSpeedTime );

						if ( tAtSpeedTime >= 300 )
						{
							tTestCompleteTime = RunTime;
							bDone |= ATSPEED_TIME;
							TestState = 0;
							bLoop = TRUE;
							Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
							      "Idle Time = %d;  Speed Time = %d;  Run Time = %d  *AT SPEED DONE*\n",
							      tIdleTime, tAtSpeedTime, RunTime );
						}
					}
					else
					{
						TestState = 0;
						bLoop = TRUE;
					}
					break;

				case 3:     // check for test pass/fail
					if ( bIPT != 0 )
					{
						// check if any OBDCOND counters increment too soon or too late
						bFail = FALSE;
						for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
						{
							if ( (Test10_10_IPT[EcuIndex].Flags&VALIDDATA) != 0 && tObdCondTimestamp[EcuIndex] != 0 )
							{
								if ( tObdCondTimestamp[EcuIndex] < (tTestCompleteTime - 20) )
								{
									Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
									      "ECU %X  OBDCOND incremented too soon (RUNTM = %u)\n",
									      GetEcuId ( EcuIndex ),
									      tObdCondTimestamp[EcuIndex] );
									bFail = TRUE;
								}

								if ( tObdCondTimestamp[EcuIndex] > (tTestCompleteTime + 20) )
								{
									Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
									      "ECU %X  OBDCOND incremented too late (RUNTM = %u)\n",
									      GetEcuId ( EcuIndex ),
									      tObdCondTimestamp[EcuIndex] );
									bFail = TRUE;
								}
							}
						}

						if ( bFail == TRUE )
						{
							Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
							      "Idle Time = %d;  Speed Time = %d;  Run Time = %d\n",
							      tIdleTime, tAtSpeedTime, RunTime );

							free ( pIpt );
							free ( pFEOCNTR );
							gotoxy ( 0, BOTTOM_ROW+2 );
							return FAIL;
						}

						// check for OBDCOND for increment
						for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
						{
							if ( (Test10_10_IPT[EcuIndex].Flags&VALIDDATA) != 0 && tObdCondTimestamp[EcuIndex] == 0 )
							{
								break;      // OBDCOND counter not yet incremented - keep running test
							}
						}

						if ( EcuIndex >= gUserNumEcus )
						{
							Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
							      "Idle Time = %d;  Speed Time = %d;  Run Time = %d  *OBDCOND TEST DONE*\n",
							      tIdleTime, tAtSpeedTime, RunTime );

							free ( pIpt );
							free ( pFEOCNTR );
							gotoxy ( 0, BOTTOM_ROW+2 );
							return PASS;  // Test complete
						}

						// check for timeout
						if ( RunTime >= (tTestCompleteTime + 20) )
						{
							free ( pIpt );
							free ( pFEOCNTR );
							gotoxy ( 0, BOTTOM_ROW+2 );

							if ( gTestPhase == eTestInUseCounters )
							{
								Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
								      "More than 20 seconds has elapsed since the test completed "
								      "and not all the ECUs have incremented OBDCOND\n" );
								Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
								      "Idle Time = %d;  Speed Time = %d;  Run Time = %d\n",
								      tIdleTime, tAtSpeedTime, RunTime );
								return FAIL;
							}
							else
							{
								Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
								      "Idle Time = %d;  Speed Time = %d;  Run Time = %d  *PDTC TEST DONE*\n",
								      tIdleTime, tAtSpeedTime, RunTime );
								return PASS;
							}
						}
					}
					else
					{
						// check for timeout
						if ( RunTime >= (tTestCompleteTime + 20) )
						{
							Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
							      "Idle Time = %d;  Speed Time = %d;  Run Time = %d  *TEST DONE*\n",
							      tIdleTime, tAtSpeedTime, RunTime );

							free ( pIpt );
							free ( pFEOCNTR );
							gotoxy ( 0, BOTTOM_ROW+2 );
							return PASS;
						}
					}

					break;

			} // end switch ( TestState )

			// Check for 10s of fueled operation for PHEV
			if ( gOBDPlugInFlag == TRUE &&
			     (bDone & FEO_TIME) != FEO_TIME &&
			     tFueledRunTime < 10 )
			{
				if ( RPM > 450 )
				{
					tFueledRunTime = min(tFueledRunTime + (RunTime - tFRTempTime), 10);
					tFRTempTime = RunTime;

					SetFieldDec ( PHEV_TIMER, tFueledRunTime );

					if ( tFueledRunTime >= 10 )
					{
						bDone |= FEO_TIME;
						Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
						      "Idle Time = %d;  Speed Time = %d;  Run Time = %d  *FUELED ENGINE TIME DONE*\n",
						      tIdleTime, tAtSpeedTime, RunTime );
					}
				}
				else // RPM < 450
				{
					tFRTempTime = RunTime;
				}
			}
			else if ( gOBDPlugInFlag == FALSE )
			{
				bDone |= FEO_TIME;
			}

		} // end while (bLoop)

		//-------------------------------------------
		// Check for ESC key and sleep
		//-------------------------------------------
		do
		{
			if ( _kbhit () != 0 )
			{
				key = _getch ();
				if ( key == 27 )    // ESC key
				{
					Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "Drive Cycle Test aborted by user\n\n" );
					Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "Idle Time = %d;  Speed Time = %d;  Run Time = %d\n",
					      tIdleTime, tAtSpeedTime, RunTime );

					free ( pIpt );
					free ( pFEOCNTR );
					gotoxy ( 0, BOTTOM_ROW+2 );
					return FAIL;
				}
				else if ( key == 0 || key == 224 )
				{
					key = _getch ();
					if ( key == 73 )  //PGUP
					{
						if ( EcuPage < MAX_ECU_PAGE &&
						     EcuPageOffset+MAX_ECUS_PER_PAGE < gUserNumEcus )
						{
							EcuPageOffset += MAX_ECUS_PER_PAGE;
							EcuPage++;
							ECUPageUpdate = TRUE;
						}
					}
					else if ( key == 81 )  //PGDN
					{
						if ( EcuPage > 1 )
						{
							EcuPageOffset -= MAX_ECUS_PER_PAGE;
							EcuPage--;
							ECUPageUpdate = TRUE;
						}
					}
				}
			}


			tDelayTimeStamp = GetTickCount ();

			Sleep ( min ( 1000 - (tDelayTimeStamp - t1SecTimer), 50 ) );

		} while ( tDelayTimeStamp - t1SecTimer < 1000 );

		if ( ECUPageUpdate == TRUE )
		{
			// Update ECU page number
			SetFieldDec ( ECUPAGE_INDEX, EcuPage );
			ECUPageUpdate = FALSE;

			for ( EcuIndex = 0; EcuIndex < gUserNumEcus && EcuIndex < MAX_ECUS_PER_PAGE; EcuIndex++ )
			{
				// initialize ECU IDs
				SetFieldHex ( ECU_ID+EcuIndex, GetEcuId ( EcuPageOffset+EcuIndex ) );

				// initialize initial OBDCONDs
				SetFieldDec ( INITIAL_OBDCOND+EcuIndex, Test10_10_IPT[EcuPageOffset+EcuIndex].Data[0] );

				// initialize initial IGNCTRs
				SetFieldDec ( INITIAL_IGNCTR+EcuIndex, Test10_10_IPT[EcuPageOffset+EcuIndex].Data[1] );

				if ( gOBDPlugInFlag == TRUE )
				{
					// initialize initial FEOCNTRs
					SetFieldDec ( INITIAL_FEOCNTR+EcuIndex, pTest10_10_FEOCNTR[EcuPageOffset+EcuIndex].Data[0] );
				}

				// if INF 8/B not supported, print 0 (INF8/B will never update)
				if ( bIPT == 0 )
				{
					SetFieldDec ( CURRENT_OBDCOND+EcuIndex, 0 );
					SetFieldDec ( CURRENT_IGNCTR+EcuIndex,  0 );
				}
				else
				{
					SetFieldDec ( CURRENT_OBDCOND+EcuIndex, pIpt[EcuPageOffset+EcuIndex].Data[0] );
					SetFieldDec ( CURRENT_IGNCTR+EcuIndex,  pIpt[EcuPageOffset+EcuIndex].Data[1] );
				}
			}
		}

		t1SecTimer = tDelayTimeStamp;
	}

	Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "Error in Drive Cycle Test\n\n" );

	free ( pIpt );
	free ( pFEOCNTR );
	gotoxy ( 0, BOTTOM_ROW+2 );
	return FAIL;
}


/*
*******************************************************************************
**  DisplayDynamicTest10Results
*******************************************************************************
*/
void DisplayDynamicTest10Results ( void )
{
	unsigned int   EcuIndex;
	unsigned int   bIPT = 0;  // support INF for IPT data, default 0 = no support

	IPT        IPT;

	// Initialize Array
	memset ( &IPT, 0x00, sizeof ( IPT ) );


	for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
	{
		if ( IsDIDSupported ( EcuIndex, INF_TYPE_IPT_GAS ) == TRUE )
		{
			bIPT = INF_TYPE_IPT_GAS;
		}
		else if ( IsDIDSupported ( EcuIndex, INF_TYPE_IPT_DIESEL ) == TRUE )
		{
			bIPT = INF_TYPE_IPT_DIESEL;
		}

		// if IPT data is supported
		if ( bIPT != 0 )
		{
			// ECU IDs
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  supports INF $%X IPT data\n",
			      GetEcuId ( EcuIndex ),
			      bIPT );

			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "         Initial    Current\n" );
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "OBDCOND  %-8d    %-8d\n",
			      Test10_10_IPT[EcuIndex].Data[0],
			      pLast_Ipt[EcuIndex].Data[0] );
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "IGNCTR   %-8d    %-8d\n",
			      Test10_10_IPT[EcuIndex].Data[1],
			      pLast_Ipt[EcuIndex].Data[1] );
		}

		// IPT data is not supported
		else
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  does not support IPT data\n",
			      GetEcuId ( EcuIndex ) );
		}

		bIPT = 0;
	}
}


/*
********************************************************************************
**  AllocateMemoryForTestToVerifyInUseCounters
********************************************************************************
*/
void AllocateMemoryForTestToVerifyInUseCounters ( void )
{
	// Initialize ECU Arrays
	Test10_10_IPT = (IPT *) malloc ( sizeof ( IPT ) * gNumEcusForAlloc );
	if ( Test10_10_IPT == NULL )
	{
		Log ( ERROR_FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "Unable to allocate memory (%d bytes) for Test10_10_IPT.",
		      sizeof ( IPT ) * gNumEcusForAlloc );
		exit ( FAIL );
	}
	else
	{
		memset ( Test10_10_IPT, 0x00, sizeof ( IPT ) * gNumEcusForAlloc );
	}

	pTest10_10_FEOCNTR = (IPT *) malloc ( sizeof ( IPT ) * gNumEcusForAlloc );
	if ( pTest10_10_FEOCNTR == NULL )
	{
		Log ( ERROR_FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "Unable to allocate memory (%d bytes) for pTest10_10_FEOCNTR.",
		      sizeof ( IPT ) * gNumEcusForAlloc );
		exit ( FAIL );
	}
	else
	{
		memset ( pTest10_10_FEOCNTR, 0x00, sizeof ( IPT ) * gNumEcusForAlloc );
	}

	pLast_Ipt = (IPT *) malloc ( sizeof ( IPT ) * gNumEcusForAlloc );
	if ( pLast_Ipt == NULL )
	{
		Log ( ERROR_FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "Unable to allocate memory (%d bytes) for pLast_Ipt.",
		      sizeof ( IPT ) * gNumEcusForAlloc );
		exit ( FAIL );
	}
	else
	{
		memset ( pLast_Ipt, 0x00, sizeof ( IPT ) * gNumEcusForAlloc );
	}
}


/*
*******************************************************************************
**  Free the memory alloc'd by  AllocateMemoryForTestToVerifyInUseCounters
*******************************************************************************
*/
void FreeMemoryForTestToVerifyInUseCounters ( void )
{
	// Test10_10_IPT is used in TestToVerifyPerformanceCounters

	free ( pTest10_10_FEOCNTR );

	// pLast_Ipt is used in TestToVerifyPermanentCodes
}
