/*
********************************************************************************
**  SAE J1699-5 Test Source Code
**
**  Copyright (C) 2002 Drew Technologies. http://j1699-3.sourceforge.net/
**
** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  This source code, when compiled and used with an SAE J2534-compatible pass
**  thru device, is intended to run the tests described in the SAE J1699-5
**  document in an automated manner.
**
**  This computer program is based upon SAE Technical Report J1699,
**  which is provided "AS IS"
**
**  See j1699.c for details of how to build and run this test.
**
********************************************************************************
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <windows.h>
#include "j2534.h"
#include "j1699.h"


/*
*******************************************************************************
**  TestWithNoFaultsAfter3DriveCycles - Tests 9.1 - 9.7 and 9.22 - 9.23
**  Function to run test with no faults after 3 drive cycles
*******************************************************************************
*/
STATUS TestWithNoFaultsAfter3DriveCycles ( void )
{
	BOOL    bSubTestFailed = FALSE;
//	STATUS  RetCode;
	unsigned char i;


//*******************************************************************************
	//Test 9.1
	gTestSubsection = 1;
	gOBDgTestSubsectionFailed = FALSE;

	// Prompt user to complete the second and perform the third drive cycle
	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Complete Two or More Additional Drive Cycles)" );

	printf ( "\n\n\n" );
	Log ( PROMPT, SCREENOUTPUTON, LOGOUTPUTON, ENTER_PROMPT,
	      "Turn key OFF for 30 seconds or longer, as appropriate for the ECU\n"
	      "(This completes two driving cycles)." );

	printf ( "\n\n\n" );
	Log ( PROMPT, SCREENOUTPUTON, LOGOUTPUTON, ENTER_PROMPT,
	      "Start engine or activate propulsion system for Hybrid/PHEV.\n\n"

	      "Let idle for 1 minute or whatever time it takes\n"
	      "to run the monitor and detect that there is no malfunction.\n"
	      "(This starts third driving cycle; however, third driving cycle will not be\n"
	      "completed until key is turned off).\n\n"

	      "Note: Some powertrain control systems have engine controls that can start and\n"
	      "stop the engine without regard to ignition position.\n" );

	do
	{
		printf ( "\n\n\n" );
		Log ( PROMPT, SCREENOUTPUTON, LOGOUTPUTON, ENTER_PROMPT,
		      "Turn key OFF for 30 seconds or longer, as appropriate for the ECU\n"
		      "(This completes the current driving cycles with no fault)." );

		printf ( "\n\n\n" );
		Log ( PROMPT, SCREENOUTPUTON, LOGOUTPUTON, ENTER_PROMPT,
		      "Start engine or activate propulsion system for Hybrid/PHEV.\n\n"

		      "Let idle for 1 minute or whatever time it takes\n"
		      "to run the monitor and detect that there is no malfunction.\n"
		      "(This starts another driving cycle; however, the driving cycle will not be\n"
		      "completed until key is turned off).\n\n"

		      "Note: Some powertrain control systems have engine controls that can start and\n"
		      "stop the engine without regard to ignition position.\n" );
		printf ( "\n\n\n" );
	}
	while ( Log ( PROMPT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_PROMPT,
	              "The MIL should now be OFF.  If it is, the test will continue to\n"
	              "Test 9.2.\n"
	              "If the MIL is still ON, additional driving cycles may be performed\n"
	              "until MIL is off.\n\n"

	              "Is MIL OFF?" ) != 'Y' );

	gOBDEngineRunning = TRUE;

	Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );


//*******************************************************************************
	// Test 9.2, Determine the OBD protocol to use
	gTestSubsection = 2;
	gOBDgTestSubsectionFailed = FALSE;

	//Set flag to indicate DTC should be historical
	gOBDDTCHistorical = TRUE;

	// Set flag to indicate no permanent DTC
	gOBDDTCPermanent = FALSE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Determine Protocol, Engine Running)" );

	if ( DetermineProtocol () != PASS )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "Protocol determination unsuccessful.\n" );
		Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
		return FAIL;
	}

	if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
	{
		if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
		{
			return FAIL;
		}
	}
	else
	{
		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}


//*******************************************************************************
	// Test9.3, Verify pending DTC data
	gTestSubsection = 3;
	gOBDgTestSubsectionFailed = FALSE;
	bSubTestFailed = FALSE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Verify Pending DTCs (Service $19 Subfunction $42 StatusMask $04), Engine Running)" );

	if ( VerifyDTCPendingData () != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
	{
		if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
		{
			return FAIL;
		}
	}
	else
	{
		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}


//*******************************************************************************
	// Test 9.4, Verify stored DTC data
	gTestSubsection = 4;
	gOBDgTestSubsectionFailed = FALSE;
	bSubTestFailed = FALSE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Verify Stored DTCs (Service $19 Subfunction $42 StatusMask $08), Engine Running)" );

	if ( VerifyDTCStoredData () != PASS )
	{
		bSubTestFailed = TRUE;
	}

	// Verify MIL and DTC status is cleared
	if ( VerifyMILData () != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
	{
		if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
		{
			return FAIL;
		}
	}
	else
	{
		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}


//*******************************************************************************
	// Test 9.5, Verify freeze frame support and data
	gTestSubsection = 5;
	gOBDgTestSubsectionFailed = FALSE;
	bSubTestFailed = FALSE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Verify Freeze Frame Data (Service $19 Subfunction $04), Engine Running)" );

	Log ( BLANK, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "\n\n" );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "Request Emission-Related Powertrain Freeze Frame NOT CURRENTLY IMPLEMENTED" );
	Log ( BLANK, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "\n\n" );

//	if ( VerifyFreezeFrameSupportAndData () != PASS )
//	{
//		bSubTestFailed = TRUE;
//	}
//
//	// Link active test to verify communication remained active for ALL protocols
//	if ( VerifyLinkActive () != PASS )
//	{
//		bSubTestFailed = TRUE;
//	}
//
//	if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS )
//	{
//		bSubTestFailed = TRUE;
//	}
//
//	if ( gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
//	{
//		if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
//		{
//			return FAIL;
//		}
//	}
//	else
//	{
//		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
//	}


//*******************************************************************************
	// Test 9.6, Verify permanent codes
	gTestSubsection = 6;
	gOBDgTestSubsectionFailed = FALSE;
	bSubTestFailed = FALSE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Verify Permanent DTCs (Service $19 Subfunction $55), Engine Running)" );

	if ( VerifyPermanentCodeSupport () != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
	{
		if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
		{
			return FAIL;
		}
	}
	else
	{
		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}


//*******************************************************************************
	// Test 9.7, Verify Permanent Codes Drive Cycle
	gTestSubsection = 7;
	gOBDgTestSubsectionFailed = FALSE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Complete Static Test or Continue to Permanent Code Drive Cycle)" );

	if ( gPermDTCSupported != 0 )
	{
		for ( i = 0; i < gPermDTCSupported; i++ )
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "Service $19 Subfunction $55 is supported by:  ECU %X\n",
			      gPermDTCSupECU[i] );
		}

		if ( Log ( PROMPT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_PROMPT,
		           "Do you wish to run the CARB drive cycle portion for Permanent DTCs (Tests 9.8-9.22)?" ) == 'Y' )
		{
			Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );

			// Test 9.8-21, Verify permanent codes
			if ( TestToVerifyPermanentCodes () != PASS )
			{
				bSubTestFailed = TRUE;
			}

			// Test 9.22, Verify permanent code support
			gTestSubsection = 22;
			gOBDgTestSubsectionFailed = FALSE;
			bSubTestFailed = FALSE;

			Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "(Verify Permanent DTCs (Service $19 Subfunction $55), Engine Running)" );

			if ( VerifyPermanentCodeSupport () != PASS )
			{
				bSubTestFailed = TRUE;
			}

			if ( VerifyVehicleInformationChange () != PASS )
			{
				bSubTestFailed = TRUE;
			}

			if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS )
			{
				bSubTestFailed = TRUE;
			}

			if ( gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
			{
				if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
				{
					return FAIL;
				}
			}
			else
			{
				Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
			}
		}

		else
		{
			Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "Service $19 Subfunction $55 was supported but the operator chose not to run\n"
			      "the Permanent Code Drive Cycle test.\n\n" );

			Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
		}
	}
//	// Service $19 Subfunction $55 not supported
//	// IF US, warn if SID $19 $55 not supported
//	else if ( gUserInput.eComplianceType == US_OBDII || gUserInput.eComplianceType == HD_OBD )
//	{
//		Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//		      "Service $19 Subfunction $55 was not supported by any ECU, therefore\n"
//		      "the Permanent Code Drive Cycle was not tested.\n\n" );
//
//		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
//	}
//	// IF not US, just note that SID $19 $55 not supported and Permanent Code Drive Cycle not tested
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "Service $19 Subfunction $55 was not supported by any ECU, therefore\n"
		      "the Permanent Code Drive Cycle was not tested.\n\n" );

		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}


//*******************************************************************************
	// Test 9.23, Clear codes
	gTestSubsection = 23;
	gOBDgTestSubsectionFailed = FALSE;
	bSubTestFailed = FALSE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Clear DTCs (Service $14) with Engine Off)" );

	// Clear Codes Engine OFF
	Log ( PROMPT, SCREENOUTPUTON, LOGOUTPUTON, ENTER_PROMPT,
	      "Turn key OFF for 30 seconds or longer, as appropriate for the ECU.\n" );

	Log ( PROMPT, SCREENOUTPUTON, LOGOUTPUTON, ENTER_PROMPT,
	      "Turn key ON with engine OFF. Do not crank engine.\n" );

	gOBDEngineRunning = FALSE;

	if ( DetermineProtocol () != PASS )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "Protocol determination unsuccessful.\n" );
		Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
		Log ( PROMPT, SCREENOUTPUTON, LOGOUTPUTON, ENTER_PROMPT,
		     "Turn key OFF" );
		return FAIL;
	}

	if ( ClearCodes () != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( VerifyVehicleInformationChange () != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
	{
		if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
		{
			Log ( PROMPT, SCREENOUTPUTON, LOGOUTPUTON, ENTER_PROMPT,
			      "Turn key OFF" );
			return FAIL;
		}
	}
	else
	{
		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}

	return PASS;
}
