/*
********************************************************************************
**  SAE J1699-5 Test Source Code
**
**  Copyright (C) 2002 Drew Technologies. http://j1699-3.sourceforge.net/
**
** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  This source code, when compiled and used with an SAE J2534-compatible pass
**  thru device, is intended to run the tests described in the SAE J1699-5
**  document in an automated manner.
**
**  This computer program is based upon SAE Technical Report J1699,
**  which is provided "AS IS"
**
**  See j1699.c for details of how to build and run this test.
**
********************************************************************************
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <windows.h>
#include <conio.h>
#include "j2534.h"
#include "j1699.h"


/*
*******************************************************************************
**  TestWithPendingDtc - Tests 6.x
**  Function to run test with a pending code induced by a fault
*******************************************************************************
*/
STATUS TestWithPendingDtc ( void )
{
	BOOL          bSubTestFailed = FALSE;
	BOOL          bLoopDone = FALSE;

//*******************************************************************************
	// Test 6.1
	gTestSubsection = 1;
	gOBDgTestSubsectionFailed = FALSE;

	// Prompt user to induce fault and start engine
	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Induce Fault)" );

	Log ( PROMPT, SCREENOUTPUTON, LOGOUTPUTON, ENTER_PROMPT,
	      "Turn key off and disconnect a sensor (e.g. ECT, TP, IAT, MAF, etc.)\n"
	      "that will generate a MIL light with the engine idling for one ECU.\n"
	      "The selected fault must illuminate the MIL using %s driving cycles\n"
	      "(not one like GM Type A) to allow for proper testing of Mode 7 and\n"
	      "freeze frame.\n"
	      "NOTE: You may first start the engine and then disconnect a sensor, in order to avoid starting issues.\n",
	      gUserInput.szNumDCToSetMIL );

	Log ( PROMPT, SCREENOUTPUTON, LOGOUTPUTON, ENTER_PROMPT,
	      "Start engine or activate propulsion system for Hybrid/PHEV.\n\n"
	      "Let idle.\n\n"

	      "NOTE: Some powertrain control systems have engine controls that can start and\n"
	      "stop the engine without regard to ignition position.\n" );

	gOBDEngineRunning = TRUE;

	Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );


//*******************************************************************************
	// Test 6.2, Determine the OBD protocol to use
	gTestSubsection = 2;
	gOBDgTestSubsectionFailed = FALSE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Determine Protocol, Engine Running)" );

	if ( DetermineProtocol () != PASS )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "Protocol determination unsuccessful.\n" );
		Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
		return FAIL;
	}

	if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS ||
	     gOBDgTestSubsectionFailed == TRUE )
	{
		if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
		{
			return FAIL;
		}
	}
	else
	{
		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}


//*******************************************************************************
	// Test 6.3
	gTestSubsection = 3;
	gOBDgTestSubsectionFailed = FALSE;
	bSubTestFailed = FALSE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Verify Pending DTCs (Service $19 Subfunction $42 StatusMask $04), Engine Running)" );

	// flush the STDIN stream of any user input before loop
	clear_keyboard_buffer ();

	// Wait for pending DTC
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTOFF, NO_PROMPT,
	      "Waiting for pending DTC...(press any key to stop test)\n" );

	while ( !_kbhit () )
	{
		if ( IsDTCPending ( (SID_REQ_NORMAL|SID_REQ_ALLOW_NO_RESPONSE) ) == PASS )
		{
			bLoopDone = TRUE;
			break;
		}
		Sleep ( 500 );
	}

	if ( bLoopDone == FALSE )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "User abort\n" );
	}

	// Beep
	printf ( "\007\n" );

	// Flush the STDIN stream of any user input above
	clear_keyboard_buffer ();

	// Set flag to indicate a pending DTC should be present
	gOBDDTCPending = TRUE;

	// Verify there is a pending DTC
	if ( IsDTCPending ( (SID_REQ_NORMAL|SID_REQ_ALLOW_NO_RESPONSE) ) != FAIL )
	{
		// Get DTC list from SID $19 $42 ($04)
		SaveDTCList ( 0x04 );

		// Check if a DTC is stored as well (i.e. GM Type A)
		if ( IsDTCStored ( (SID_REQ_NORMAL|SID_REQ_ALLOW_NO_RESPONSE) ) == PASS )
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "Type A DTC detected, pending without a stored code functionality not tested\n" );
			gOBDDTCStored = TRUE;
		}
	}
	else
	{
		bSubTestFailed = TRUE;
	}

	// Verify pending DTC data
	if ( VerifyDTCPendingData () != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
	{
		if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
		{
			return FAIL;
		}
	}
	else
	{
		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}


//*******************************************************************************
	// Test 6.4, Verify stored DTC data
	gTestSubsection = 4;
	gOBDgTestSubsectionFailed = FALSE;
	bSubTestFailed = FALSE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Verify Stored DTCs (Service $19 Subfunction $42 StatusMask $08), Engine Running)" );

	if ( VerifyDTCStoredData () != PASS )
	{
		bSubTestFailed = TRUE;
	}

	// Verify MIL and DTC status is cleared
	if ( VerifyMILData () != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS )
	{
		bSubTestFailed = TRUE;
	}

	if ( gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
	{
		if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
		{
			return FAIL;
		}
	}
	else
	{
		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
	}


//*******************************************************************************
	// Test 6.5, Verify freeze frame support and data
	gTestSubsection = 5;
	gOBDgTestSubsectionFailed = FALSE;
	bSubTestFailed = FALSE;

	Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "(Verify Freeze Frame Data (Service $19 Subfunction $04), Engine Running)" );

	Log ( BLANK, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "\n\n" );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "Request Emission-Related Powertrain Freeze Frame NOT CURRENTLY IMPLEMENTED" );
	Log ( BLANK, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "\n\n" );

//	if ( VerifyFreezeFrameSupportAndData () != PASS )
//	{
//		bSubTestFailed = TRUE;
//	}
//
//	// Link active test to verify communication remained active for ALL protocols
//	if ( VerifyLinkActive () != PASS )
//	{
//		bSubTestFailed = TRUE;
//	}
//
//	if ( VerifyVehicleState ( gOBDEngineRunning, gOBDHybridFlag ) != PASS )
//	{
//		bSubTestFailed = TRUE;
//	}
//
//	if ( gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
//	{
//		if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
//		{
//			return FAIL;
//		}
//	}
//	else
//	{
//		Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
//	}


//*******************************************************************************
//	if ( gUserInput.eComplianceType == EOBD_WITH_IUMPR ||
//	     gUserInput.eComplianceType == EOBD_NO_IUMPR ||
//	     gUserInput.eComplianceType == HD_EOBD ||
//	     gUserInput.eComplianceType == IOBD_NO_IUMPR ||
//	     gUserInput.eComplianceType == HD_IOBD_NO_IUMPR ||
//	     gUserInput.eComplianceType == OBDBr_NO_IUMPR )
//	{
//		// Test 6.6, Continue to Induce Circuit Fault (EOBD/IOBD/OBDBr only)
//		gTestSubsection = 6;
//		gOBDgTestSubsectionFailed = FALSE;
//		bSubTestFailed = FALSE;
//
//		Log ( SUBSECTION_BEGIN, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//		      "(Continue to induce fault - EOBD/IOBD/OBDBr only)" );
//
//		Log ( PROMPT, SCREENOUTPUTON, LOGOUTPUTON, ENTER_PROMPT,
//		      "Turn key OFF for 30 seconds or longer, as appropriate for the ECU,\n"
//		      "and keep sensor disconnected.\n" );
//
//		Log ( PROMPT, SCREENOUTPUTON, LOGOUTPUTON, ENTER_PROMPT,
//		      "Start engine or activate propulsion system for Hybrid/PHEV.\n\n"
//		      "Let idle for 1 minute or whatever time it takes to set a pending DTC.\n\n"
//		      "NOTE: Some powertrain control systems have engine controls that can start and\n"
//		      "stop the engine without regard to ignition position.\n" );
//
//		if ( gOBDgTestSubsectionFailed == TRUE || bSubTestFailed == TRUE )
//		{
//			if ( Log ( SUBSECTION_FAILED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, YES_NO_ALL_PROMPT, "" ) == 'N' )
//			{
//				return FAIL;
//			}
//		}
//		else
//		{
//			Log ( SUBSECTION_PASSED_RESULT, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT, "" );
//		}
//	}

	return PASS;
}
