/*
********************************************************************************
**  SAE J1699-5 Test Source Code
**
**  Copyright (C) 2002 Drew Technologies. http://j1699-3.sourceforge.net/
**
** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  This source code, when compiled and used with an SAE J2534-compatible pass
**  thru device, is intended to run the tests described in the SAE J1699-5
**  document in an automated manner.
**
**  This computer program is based upon SAE Technical Report J1699,
**  which is provided "AS IS"
**
**  See j1699.c for details of how to build and run this test.
**
********************************************************************************
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <windows.h>
#include "j2534.h"
#include "j1699.h"


STATUS IsDTCIgnored ( unsigned long EcuIndex, unsigned short Size, unsigned char * DTC_Array, BOOL Permanent );

/*
*******************************************************************************
**  VerifyDTCPendingData - Function to verify SID7 DTC pending data
*******************************************************************************
*/
STATUS VerifyDTCPendingData ( void )
{
	SID_REQ       SidReq;
	unsigned long EcuIndex;
	unsigned long NumDTCs;
	unsigned long DataOffset;
	unsigned long ulInit_FailureCount = 0;
	BOOL          DTCFail = FALSE;
	STATUS        RetCode = PASS;
	DTC          *pDTC;

	ulInit_FailureCount = GetFailureCount ();

	// Request SID $19 $42 $33 $04 data
	SidReq.SID    = 0x19;
	SidReq.NumIds = 4;
	SidReq.DID[0] = 0x42;
	SidReq.DID[1] = 0x33;
	SidReq.DID[2] = 0x04;
	SidReq.DID[3] = 0xFF;
	if ( SidRequest ( &SidReq, SID_REQ_NORMAL ) == FAIL )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "SID $19 $42 $33 $04 request failed\n" );
		return FAIL;
	}

	// if a pending DTC is not supposed to be present (Test 5.15, 8.3, 9.3, 10.8, 11.10)
	if ( gOBDDTCPending == FALSE )
	{
		// Verify that SID $19 $42 $33 $04 reports no DTCs pending
		for ( EcuIndex = 0; EcuIndex < gOBDNumEcus; EcuIndex++ )
		{
			// warn for ECUs which don't respond
			if ( gOBDResponse[EcuIndex].PendDTCHeader.DTCSize == 0 )
			{
				Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  No response to SID $19 $42 $33 $04 request!\n",
				      GetEcuId ( EcuIndex ) );
				continue;
			}

			// If ISO15765 protocol, check that DTCs are $00
			if ( gOBDResponse[EcuIndex].PendDTC[1] != 0x00 || gOBDResponse[EcuIndex].PendDTC[2] != 0x00 || gOBDResponse[EcuIndex].PendDTC[3] != 0x00 )
			{
				// if the DTCs  cannot be ignored, fail
				if ( IsDTCIgnored ( EcuIndex, gOBDResponse[EcuIndex].PendDTCHeader.DTCSize, &gOBDResponse[EcuIndex].PendDTC[0], FALSE ) != PASS )
				{
					DTCFail = TRUE;
				}
			}

			if ( DTCFail == TRUE )
			{
				DTCFail = FALSE;

				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  SID $19 $42 $33 $04 indicates DTC pending\n",
				      GetEcuId ( EcuIndex ) );
				RetCode = FAIL;
			}

		}
	}

	// if a DTC is supposed to be present (Test 6.3, 7.3)
	else
	{
		// Verify that SID $19 $42 $33 $04 reports DTCs pending
		NumDTCs = 0;
		for ( EcuIndex = 0; EcuIndex < gOBDNumEcus; EcuIndex++ )
		{
			// warn for ECUs which don't respond
			if ( gOBDResponse[EcuIndex].PendDTCHeader.DTCSize == 0 )
			{
				Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  No response to SID $19 $42 $33 $04 request\n",
				      GetEcuId ( EcuIndex ) );
				continue;
			}

			// Print out all the DTCs
			for ( DataOffset = 0; DataOffset < gOBDResponse[EcuIndex].PendDTCHeader.DTCSize; DataOffset += sizeof ( DTC ) )
			{
				pDTC = (DTC*)&gOBDResponse[EcuIndex].PendDTC[DataOffset];
				if ( pDTC->DTC[0] != 0 || pDTC->DTC[1] != 0 || pDTC->DTC[2])
				{
					Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "ECU %X  Pending DTC $%02X%02X%02X (Severity = $%02X  Status = $%02X) detected\n",
					       GetEcuId ( EcuIndex ),
					       pDTC->DTC[0],    // High Byte of DTC
					       pDTC->DTC[1],    // Middle Byte of DTC
					       pDTC->DTC[2],    // Low Byte of DTC
					       pDTC->Severity,  // Severity of DTC
					       pDTC->Status );  // Status of DTC
//					// Process based on the type of DTC
//					switch ( gOBDResponse[EcuIndex].PendDTC[DataOffset] & 0xC0 )
//					{
//						case 0x00:
//						{
//							Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//							      "ECU %X  Pending DTC P%02X%02X detected\n",
//							      GetEcuId ( EcuIndex ),
//							      gOBDResponse[EcuIndex].PendDTC[DataOffset] & 0x3F,
//							      gOBDResponse[EcuIndex].PendDTC[DataOffset + 1] );
//						}
//						break;
//						case 0x40:
//						{
//							Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//							      "ECU %X  Pending DTC C%02X%02X detected\n",
//							      GetEcuId ( EcuIndex ),
//							      gOBDResponse[EcuIndex].PendDTC[DataOffset] & 0x3F,
//							      gOBDResponse[EcuIndex].PendDTC[DataOffset + 1] );
//						}
//						break;
//						case 0x80:
//						{
//							Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//							      "ECU %X  Pending DTC B%02X%02X detected\n",
//							      GetEcuId ( EcuIndex ),
//							      gOBDResponse[EcuIndex].PendDTC[DataOffset] & 0x3F,
//							      gOBDResponse[EcuIndex].PendDTC[DataOffset + 1] );
//						}
//						break;
//						case 0xC0:
//						{
//							Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//							      "ECU %X  Pending DTC U%02X%02X detected\n",
//							      GetEcuId ( EcuIndex ),
//							      gOBDResponse[EcuIndex].PendDTC[DataOffset] & 0x3F,
//							      gOBDResponse[EcuIndex].PendDTC[DataOffset + 1] );
//						}
//						break;
//					}
					NumDTCs++;
				}
			}
		}

		// If no pending DTCs found, then fail
		if ( NumDTCs == 0 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "No ECU indicates DTC pending\n" );
			return FAIL;
		}
	}

	if ( RetCode != PASS )
	{
		return FAIL;
	}

	// Link active test to verify communication remained active for ALL protocols
	if ( VerifyLinkActive () != PASS )
	{
		return FAIL;
	}

	if ( ulInit_FailureCount != GetFailureCount () )
	{
		// There could have been early/late responses that weren't treated as FAIL
		return FAIL;
	}
	else
	{
		return PASS;
	}
}


/*
*****************************************************************************
**  IsDTCIgnored - Function to check if the DTC(s) are in the ignore list
**  FAIL if any DTC is not on the ignore list
*****************************************************************************
*/
STATUS IsDTCIgnored ( unsigned long  EcuIndex, unsigned short Size, unsigned char * DTC_Array, BOOL Permanent )
{
	STATUS         RetCode = PASS;
	unsigned short ArrayIndex;
	unsigned short ListIndex;
	unsigned long  DTC;
	char           DTC_Type = '#';
	unsigned short ArrayInc = 5;   // initalize Array Increment for Confirmed or Pending DTCs
	unsigned short DTCOffset = 1;  // initalize DTC offset for Confirmed or Pending DTCs (DTCs are preceded by Severity Byte)

	if ( Permanent )
	{
		ArrayInc = 4;
		DTCOffset = 0;
	}

	// check if it's in the ignore list
	for ( ArrayIndex = 0; ArrayIndex < Size; ArrayIndex += ArrayInc )
	{
		DTC = (unsigned short)( ((DTC_Array[ArrayIndex+DTCOffset])<<16) + ((DTC_Array[ArrayIndex+DTCOffset+1])<<8) + DTC_Array[ArrayIndex+DTCOffset+2] );
		// if the DTC is not in the ignore list, fail
		for ( ListIndex = 0; ListIndex < gDTC_Ignore_List_Count; ListIndex++ )
		{
			if ( DTC == gDTC_Ignore_List[ListIndex] )
			{
				// this DTC was found, don't search list anymore
				break;
			}
		}

		// Process based on the type of DTC
		switch ( DTC_Array[ArrayIndex] & 0xC0 )
		{
			case 0x00:
			{
				DTC_Type = 'P';
			}
			break;
			case 0x40:
			{
				DTC_Type = 'C';
			}
			break;
			case 0x80:
			{
				DTC_Type = 'B';
			}
			break;
			case 0xC0:
			{
				DTC_Type = 'U';
			}
			break;
		}
		// If reached the end of the ignore list, the DTC isn't ignored
		if ( ListIndex == gDTC_Ignore_List_Count ||
		     ( gTestPhase != eTestFaultRepaired &&
		       gTestPhase != eTestNoFault3DriveCycle ) )
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  DTC %20X%02X%02X detected\n",
			      GetEcuId ( EcuIndex ),
//			      DTC_Type,
			      DTC_Array[ArrayIndex] & 0x3F,
			      DTC_Array[ArrayIndex + 1] );
			RetCode = FAIL;
		}
		else
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  DTC %20%02X%02X detected but ignored\n",
			      GetEcuId ( EcuIndex ),
			      DTC_Type,
			      DTC_Array[ArrayIndex] & 0x3F,
			      DTC_Array[ArrayIndex + 1] );
		}
	}

	return RetCode;
}
