/*
********************************************************************************
**  SAE J1699-5 Test Source Code
**
**  Copyright (C) 2002 Drew Technologies. http://j1699-3.sourceforge.net/
**
** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  This source code, when compiled and used with an SAE J2534-compatible pass
**  thru device, is intended to run the tests described in the SAE J1699-5
**  document in an automated manner.
**
**  This computer program is based upon SAE Technical Report J1699,
**  which is provided "AS IS"
**
**  See j1699.c for details of how to build and run this test.
**
********************************************************************************
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <windows.h>
#include "j2534.h"
#include "j1699.h"


int          VerifyMidSupportData ( void );


/*
*******************************************************************************
**  VerifyMonitorTestSupportAndResults -
**  Function to verify Mid monitor test support and results
*******************************************************************************
*/
STATUS VerifyMonitorTestSupportAndResults ( void )
{
	unsigned long  EcuIndex;
	unsigned int   IdIndex;
	unsigned long  SidIndex;
	unsigned long  ulInit_FailureCount = 0;
	unsigned short u_tmp;
	signed short   s_tmp;
	long           TestValue;
	long           TestLimitMax;
	long           TestLimitMin;

	unsigned char  fError = FALSE;              // set if an error occured during test
	unsigned char  fSparkIgnMonSup = FALSE;     // set if Spark Ignition Monitor is supported (otherwise Compression)

//	unsigned char  fDataBBit0Supported = FALSE; // set if PID $01 Data B bit 0 is supported by the vehicle
//	unsigned char  fDataCBit0Supported = FALSE; // set if PID $01 Data C bit 0 is supported by the vehicle
	unsigned char  fCATSupported       = FALSE; // set if PID $01 Data C bit 0 is supported by the vehicle
	unsigned char  fHCCATSupported     = FALSE; // set if PID $01 Data C bit 0 is supported by the vehicle
//	unsigned char  fDataCBit1Supported = FALSE; // set if PID $01 Data C bit 1 is supported by the vehicle
	unsigned char  fHCATSupported      = FALSE; // set if PID $01 Data C bit 1 is supported by the vehicle
	unsigned char  fSCRSupported       = FALSE; // set if PID $01 Data C bit 1 is supported by the vehicle
//	unsigned char  fDataCBit2Supported = FALSE; // set if PID $01 Data C bit 2 is supported by the vehicle
	unsigned char  fEVAPSupported      = FALSE; // set if PID $01 Data C bit 2 is supported by the vehicle
//	unsigned char  fDataCBit3Supported = FALSE; // set if PID $01 Data C bit 3 is supported by the vehicle
	unsigned char  fAIRSupported       = FALSE; // set if PID $01 Data C bit 3 is supported by the vehicle
	unsigned char  fBPSupported        = FALSE; // set if PID $01 Data C bit 3 is supported by the vehicle
//	unsigned char  fDataCBit5Supported = FALSE; // set if PID $01 Data C bit 5 is supported by the vehicle
	unsigned char  fO2SSupported       = FALSE; // set if PID $01 Data C bit 5 is supported by the vehicle
	unsigned char  fEGSSupported       = FALSE; // set if PID $01 Data C bit 5 is supported by the vehicle
//	unsigned char  fDataCBit6Supported = FALSE; // set if PID $01 Data C bit 6 is supported by the vehicle
	unsigned char  fHTRSupported       = FALSE; // set if PID $01 Data C bit 6 is supported by the vehicle
	unsigned char  fPMSupported        = FALSE; // set if PID $01 Data C bit 6 is supported by the vehicle
//	unsigned char  fDataCBit7Supported = FALSE; // set if PID $01 Data C bit 7 is supported by the vehicle
	unsigned char  fEGRSupported       = FALSE; // set if PID $01 Data C bit 7 is supported by the vehicle

	unsigned char  f01Supported = FALSE;        // set if MID $01 is supported by the vehicle
	unsigned char  f21Supported = FALSE;        // set if MID $21 is supported by the vehicle
	unsigned char  fA20BSupported = FALSE;      // set if MID $A2 SDTID $0B is supported by the vehicle
	unsigned char  fA20CSupported = FALSE;      // set if MID $A2 SDTID $0C is supported by the vehicle
	unsigned char  fB2Supported = FALSE;        // set if MID $B2 is supported by the vehicle

	unsigned char  f01_10Supported = FALSE;     // set if a MID between $01 and $10 is supported by the ECU
	unsigned char  f21_24Supported = FALSE;     // set if a MID between $21 and $24 is supported by the ECU
	unsigned char  f31_38Supported = FALSE;     // set if a MID between $31 and $38 is supported by the ECU
	unsigned char  f39_3DSupported = FALSE;     // set if a MID between $39 and $3D is supported by the ECU
	unsigned char  f61_64Supported = FALSE;     // set if a MID between $61 and $64 is supported by the ECU
	unsigned char  f71_74Supported = FALSE;     // set if a MID between $71 and $74 is supported by the ECU
	unsigned char  f81_84Supported = FALSE;     // set if a MID between $81 and $84 is supported by the ECU
	unsigned char  fA1_B1Supported = FALSE;     // set if a MID between $A1 and $B1 is supported by the ECU
	unsigned char  fB2_B3Supported = FALSE;     // set if a MID between $B2 and $B3 is supported by the ECU
	unsigned char  f85_86Supported = FALSE;     // set if a MID between $85 and $86 is supported by the ECU
	unsigned char  f90919899Supported = FALSE;  // set if MID $90, $91, $98 or $99 is supported by the ECU

	SID_REQ        SidReq;
	MID           *pMid;

	ulInit_FailureCount = GetFailureCount ();

	// Request MID support data
	if ( RequestDIDSupportData ( MIDREQUEST, TRUE ) != PASS )
	{
		return FAIL;
	}

	// Verify that all MID test data is reset or within limits
	for ( EcuIndex = 0; EcuIndex < gOBDNumEcus; EcuIndex++ )
	{
		// Test 5.14.3
		if ( gTestPhase == eTestNoDTC && gTestSubsection == 14 )
		{
//			// if Spark Ignition Module (PID $F401 Data B bit 3 == 0)
//			if ( (gEcuData[EcuIndex].PID01.Data[1] & 0x08) == 0x00 )
//			{
//				fSparkIgnMonSup = TRUE;
//				gOBDDieselFlag = FALSE;
//			}
//			else
//			{
//				fSparkIgnMonSup = FALSE;
//				gOBDDieselFlag = TRUE;
//			}
			if ( gOBDDieselFlag == FALSE )
			{
				fSparkIgnMonSup = TRUE;
			}
			else
			{
				fSparkIgnMonSup = FALSE;
			}



			// if ECU does not only support CCM requirements (PID $F401 Data B bit 2 == 1)
//			if ( (gEcuData[EcuIndex].PID01.Data[1] & 0x03) != 0x00 ||
//			     (gEcuData[EcuIndex].PID01.Data[2] & 0xFF) != 0x00 )
			if ( (gEcuData[EcuIndex].PIDF501.Data[1] & 0x03) != 0x00 ||
			     (gEcuData[EcuIndex].PIDF501.Data[2] & 0x0F) != 0x00 ||
			     (gEcuData[EcuIndex].PIDF501.Data[3] & 0x0F) != 0x00 ||
			     (gEcuData[EcuIndex].PIDF501.Data[4] & 0x0F) != 0x00 ||
			     (gEcuData[EcuIndex].PIDF501.Data[5] & 0x0F) != 0x00 )
			{
//				// Test PID $F401 Data B bit 3 for each ECU
//				if ( gModelYear >= 2010 )
//				{
//					if ( gOBDDieselFlag == TRUE && fSparkIgnMonSup == TRUE )
//					{
//						Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//						      "ECU %X  PID $F401  Data B bit 3 must = 1 for Compression Ignition Vehicles\n",
//						      GetEcuId ( EcuIndex ) );
//						fError = TRUE;
//					}
//					else if ( gOBDDieselFlag == FALSE && fSparkIgnMonSup == FALSE )
//					{
//						Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//						      "ECU %X  PID $F401  Data B bit 3 must = 0 for Spark Ignition Vehicles\n",
//						      GetEcuId ( EcuIndex ) );
//						fError = TRUE;
//					}
//				}
//				// prior to MY 2010, warn for compression only
//				else
//				{
//					if ( gOBDDieselFlag == TRUE && fSparkIgnMonSup == TRUE )
//					{
//						Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//						      "ECU %X  PID $F401  Data B bit 3 should = 1 for Compression Ignition Vehicles\n",
//						      GetEcuId ( EcuIndex ) );
//					}
//				}
			}


//			if ( (gEcuData[EcuIndex].PID01.Data[2] & 0x10) != 0 )
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "ECU %X  PID $F401  Data C bit 4 is set (Must be 0 for All Vehicles)\n",
//				      GetEcuId ( EcuIndex ) );
//				fError = TRUE;
//			}

//			if ( fSparkIgnMonSup == FALSE && (gEcuData[EcuIndex].PID01.Data[2] & 0x04) != 0 )
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "ECU %X  PID $F401  Data C bit 2 is set (Must be 0 for Compression Ignition Vehicles)\n",
//				      GetEcuId ( EcuIndex ) );
//				fError = TRUE;
//			}

//			// Test Sid1 Data B bit 1 for the vehicle
//			if ( gEcuData[EcuIndex].PID01.Data[1] & 0x01 )
//			{
//				fDataBBit0Supported = TRUE;
//			}

//			// Test Sid1 Data C bit 0 for the vehicle
//			if ( gEcuData[EcuIndex].PID01.Data[2] & 0x01 )
			// Test $F501 CAT/HCCAT for this ECU
			if ( gEcuData[EcuIndex].PIDF501.Data[2] & 0x01 )
			{
				fCATSupported = TRUE;
			}
			if ( gEcuData[EcuIndex].PIDF501.Data[2] & 0x02 )
			{
				fHCCATSupported = TRUE;
			}

//			// Test Sid1 Data C bit 1 for the vehicle
//			if ( gEcuData[EcuIndex].PID01.Data[2] & 0x02 )
			// Test $F501 HCAT/SCR for this ECU
			if ( gEcuData[EcuIndex].PIDF501.Data[2] & 0x04 )
			{
				fHCATSupported = TRUE;
			}
			if ( gEcuData[EcuIndex].PIDF501.Data[2] & 0x08 )
			{
				fSCRSupported = TRUE;
			}

//			// Test Sid1 Data C bit 2 for the vehicle
//			if ( gEcuData[EcuIndex].PID01.Data[2] & 0x04 )
			// Test $F501 EVAP for this ECU
			if ( gEcuData[EcuIndex].PIDF501.Data[3] & 0x01 )
			{
				fEVAPSupported = TRUE;
			}

//			// Test Sid1 Data C bit 3 for the vehicle
//			if ( gEcuData[EcuIndex].PID01.Data[2] & 0x08 )
			// Test $F501 AIR/BP for this ECU
			if ( gEcuData[EcuIndex].PIDF501.Data[3] & 0x02 )
			{
				fAIRSupported = TRUE;
			}
			if ( gEcuData[EcuIndex].PIDF501.Data[3] & 0x04 )
			{
				fBPSupported = TRUE;
			}

//			// Test Sid1 Data C bit 5 for the vehicle
//			if ( gEcuData[EcuIndex].PID01.Data[2] & 0x20 )
			// Test $F501 EGS for this ECU
			if ( gEcuData[EcuIndex].PIDF501.Data[4] & 0x01 )
			{
				fEGSSupported = TRUE;
			}

//			// Test Sid1 Data C bit 6 for the vehicle
//			if ( gEcuData[EcuIndex].PID01.Data[2] & 0x40 )
			// Test $F501 PM for this ECU
			if ( gEcuData[EcuIndex].PIDF501.Data[2] & 0x40 )
			{
				fPMSupported = TRUE;
			}

//			// Test Sid1 Data C bit 7 for the vehicle
//			if ( gEcuData[EcuIndex].PID01.Data[2] & 0x80 )
			// Test $F501 EGR for this ECU
			if ( gEcuData[EcuIndex].PIDF501.Data[4] & 0x04 )
			{
				fEGRSupported = TRUE;
			}

		} // end Test5.14.3

		// For each MID group
		for ( IdIndex = 0xF601; IdIndex < 0xF700; IdIndex++ )
		{
			// skip PID supported PIDs
			if ( IdIndex == 0xF620 || IdIndex == 0xF640 || IdIndex == 0xF660 || IdIndex == 0xF680 ||
			     IdIndex == 0xF6A0 || IdIndex == 0xF6C0 || IdIndex == 0xF6E0 )
			{
				continue;
			}

			// If MID is supported, request it
			if ( IsDIDSupported ( EcuIndex, IdIndex ) == TRUE )
			{
				SidReq.SID = 0x22;
				SidReq.NumIds = 1;
				SidReq.DID[0] = IdIndex;
				if ( SidRequest ( &SidReq, SID_REQ_NORMAL ) == FAIL )
				{
					Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "ECU %X  MID $%04X request\n",
					      GetEcuId ( EcuIndex ),
					      IdIndex);
					fError = TRUE;
					continue;
				}

				if ( gOBDResponse[EcuIndex].MidSize == 0 )
				{
					Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "ECU %X  No MID $%04X data\n",
					      GetEcuId ( EcuIndex ),
					      IdIndex);
					fError = TRUE;
					continue;
				}

				// Check the data that should be reset and / or within limits
				pMid = (MID *)&gOBDResponse[EcuIndex].Mid[0];
				for ( SidIndex = 0; SidIndex < (gOBDResponse[EcuIndex].MidSize / sizeof ( MID )); SidIndex++ )
				{
					// IF not test 11.4 (tests 5.5, 5.14 and 10.6
					if ( gTestPhase != eTestPerformanceCounters )
					{
						// If MID 0x01 - 0x10 TID 1 - 4 OR
						// Test 5.14.3 AND MID $A1 - $B1, don't check for reset
						// Otherwise, values should be zero after a code clear
						if ( !(pMid[SidIndex].MIDLSB <= 0x10 && pMid[SidIndex].SDTID <= 4) &&
						     !( gTestPhase == eTestNoDTC && gTestSubsection == 14 &&
						        ( (pMid[SidIndex].MIDLSB >= 0xA1 && pMid[SidIndex].MIDLSB <= 0xB1) ||
						          (pMid[SidIndex].MIDLSB >= 0x81 && pMid[SidIndex].MIDLSB <= 0x84) ) ) )
						{
							if ( pMid[SidIndex].TVHI    != 0 || pMid[SidIndex].TVLO    != 0 ||
							     pMid[SidIndex].MINTLHI != 0 || pMid[SidIndex].MINTLLO != 0 ||
							     pMid[SidIndex].MAXTLHI != 0 || pMid[SidIndex].MAXTLLO != 0 )
							{
								Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
								      "ECU %X  MID $%02X%02X test value/limits not reset\n",
								      GetEcuId ( EcuIndex ),
								      pMid[SidIndex].MIDMSB,
								      pMid[SidIndex].MIDLSB );
							}
						}


						// Test 5.14.3
						if ( gTestPhase == eTestNoDTC && gTestSubsection == 14 )
						{
							// Check for prohibited MIDs
							if ( ( pMid[SidIndex].MIDLSB >= 0x15 && pMid[SidIndex].MIDLSB <= 0x1F ) ||
							     ( pMid[SidIndex].MIDLSB >= 0x25 && pMid[SidIndex].MIDLSB <= 0x30 ) ||
							       pMid[SidIndex].MIDLSB == 0x3F ||
							     ( pMid[SidIndex].MIDLSB >= 0x55 && pMid[SidIndex].MIDLSB <= 0x5F ) ||
							     ( pMid[SidIndex].MIDLSB >= 0x65 && pMid[SidIndex].MIDLSB <= 0x70 ) ||
							     ( pMid[SidIndex].MIDLSB >= 0x75 && pMid[SidIndex].MIDLSB <= 0x7F ) ||
							     ( pMid[SidIndex].MIDLSB >= 0x87 && pMid[SidIndex].MIDLSB <= 0x8F ) ||
							     ( pMid[SidIndex].MIDLSB >= 0x92 && pMid[SidIndex].MIDLSB <= 0x97 ) ||
							     ( pMid[SidIndex].MIDLSB >= 0x9A && pMid[SidIndex].MIDLSB <= 0x9F ) ||
							     ( pMid[SidIndex].MIDLSB >= 0xB4 && pMid[SidIndex].MIDLSB <= 0xBF ) ||
							     ( pMid[SidIndex].MIDLSB >= 0xC1 && pMid[SidIndex].MIDLSB <= 0xDF ) /*||
							     (gUserInput.eComplianceType == US_OBDII && pMid[SidIndex].MIDLSB >= 0xE1 && pMid[SidIndex].MIDLSB <= 0xFF)*/ )
							{
								Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
								      "ECU %X  MID $%02X%02X support prohibited\n",
								      GetEcuId ( EcuIndex ),
								      pMid[SidIndex].MIDMSB,
								      pMid[SidIndex].MIDLSB );
								fError = TRUE;
							}

							// Check MID ranges common to spark and Compression Ignition vehicles
							else if ( pMid[SidIndex].MIDLSB <= 0x10 )
							{
								f01_10Supported = TRUE;

								// Test 5.14.3 - MID $01 must be supported by all vehicles
								if ( pMid[SidIndex].MIDLSB == 0x01 )
								{
									f01Supported = TRUE;
								}
							}

							else if ( pMid[SidIndex].MIDLSB >= 0x31 && pMid[SidIndex].MIDLSB <= 0x38 )
							{
								f31_38Supported = TRUE;
							}

							else if ( pMid[SidIndex].MIDLSB >= 0x81 && pMid[SidIndex].MIDLSB <= 0x84 )
							{
								f81_84Supported = TRUE;
							}

							// Check MID ranges for Spark Ignition Vehicles
							else if ( fSparkIgnMonSup == TRUE )
							{
								if ( pMid[SidIndex].MIDLSB >= 0x21 && pMid[SidIndex].MIDLSB <= 0x24 )
								{
									f21_24Supported = TRUE;

									if ( pMid[SidIndex].MIDLSB == 0x21 )
									{
										f21Supported = TRUE;
									}
								}

								else if ( pMid[SidIndex].MIDLSB >= 0x39 && pMid[SidIndex].MIDLSB <= 0x3D )
								{
									f39_3DSupported = TRUE;
								}

								else if ( pMid[SidIndex].MIDLSB >= 0x61 && pMid[SidIndex].MIDLSB <= 0x64 )
								{
									f61_64Supported = TRUE;
								}

								else if ( pMid[SidIndex].MIDLSB >= 0x71 && pMid[SidIndex].MIDLSB <= 0x74 )
								{
									f71_74Supported = TRUE;
								}
							}

							// Check MID ranges for Compression Ignition Vehicles
							else
							{
								if ( pMid[SidIndex].MIDLSB >= 0x21 && pMid[SidIndex].MIDLSB <= 0x24 )
								{
									f21_24Supported = TRUE;
								}

								else if ( pMid[SidIndex].MIDLSB >= 0x85 && pMid[SidIndex].MIDLSB <= 0x86 )
								{
									f85_86Supported = TRUE;
								}

								else if ( pMid[SidIndex].MIDLSB == 0x90 ||
								          pMid[SidIndex].MIDLSB == 0x91 ||
								          pMid[SidIndex].MIDLSB == 0x98 ||
								          pMid[SidIndex].MIDLSB == 0x99 )
								{
									f90919899Supported = TRUE;
								}

								if ( pMid[SidIndex].MIDLSB >= 0xB2 && pMid[SidIndex].MIDLSB <= 0xB3 )
								{
									fB2_B3Supported = TRUE;

									if ( pMid[SidIndex].MIDLSB == 0xB2 )
									{
										fB2Supported = TRUE;
									}
								}
							}

						}  // end Test 5.14.3

					}  // end if ( gTestPhase != eTestPerformanceCounters )

					// Tests 5.5, 5.14, 10.6 and 11.4
					// If MID $A1 thru $B1, MID $A2 SDTID $0B or $0C must be supported
					if ( pMid[SidIndex].MIDLSB >= 0xA1 && pMid[SidIndex].MIDLSB <= 0xB1 )
					{
						fA1_B1Supported = TRUE;

						if ( pMid[SidIndex].MIDLSB == 0xA2 )
						{
							if ( pMid[SidIndex].SDTID == 0x0B )
							{
								fA20BSupported = TRUE;
							}

							if ( pMid[SidIndex].SDTID == 0x0C )
							{
								fA20CSupported = TRUE;
							}
						}
					}

					// IF in drive cycle (Test 11.4)
					if ( gTestPhase == eTestPerformanceCounters )
					{
						// Check the value against the limits
						if ( pMid[SidIndex].UASID & 0x80 )
						{
							// Signed values
							s_tmp = (pMid[SidIndex].TVHI << 8) + pMid[SidIndex].TVLO;
							TestValue = s_tmp;

							s_tmp = (pMid[SidIndex].MINTLHI << 8) + pMid[SidIndex].MINTLLO;
							TestLimitMin = s_tmp;

							s_tmp = (pMid[SidIndex].MAXTLHI << 8 ) + pMid[SidIndex].MAXTLLO;
							TestLimitMax = s_tmp;
						}
						else
						{
							// Unsigned values
							u_tmp = (pMid[SidIndex].TVHI << 8) + pMid[SidIndex].TVLO;
							TestValue = u_tmp;

							u_tmp = (pMid[SidIndex].MINTLHI << 8) + pMid[SidIndex].MINTLLO;
							TestLimitMin = u_tmp;

							u_tmp = (pMid[SidIndex].MAXTLHI << 8 ) + pMid[SidIndex].MAXTLLO;
							TestLimitMax = u_tmp;
						}

						if ( TestValue < TestLimitMin )
						{
							Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
							      "ECU %X  MID $%02X%02X test value exceeded min\n",
							      GetEcuId ( EcuIndex ),
							      pMid[SidIndex].MIDMSB,
							      pMid[SidIndex].MIDLSB );
							fError = TRUE;
						}

						if ( TestValue > TestLimitMax )
						{
							Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
							      "ECU %X  MID $%02X%02X test value exceeded max\n",
							      GetEcuId ( EcuIndex ),
							      pMid[SidIndex].MIDMSB,
							      pMid[SidIndex].MIDLSB );
							fError = TRUE;
						}
					} // end if ( gTestPhase == eTestPerformanceCounters )

				} // end for (SidIndex . . .

			} // end if (IsDIDSupported (EcuIndex, IdIndex) == TRUE)

		} // end for (IdIndex . . .

	} // end for (EcuIndex . . .


	// Check for vehicle required MIDs
	// Test 5.14.3
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 14 )
	{
//		if ( fDataBBit0Supported == TRUE &&
//		     (gUserInput.eComplianceType == US_OBDII || gUserInput.eComplianceType == HD_OBD) &&
//		     (fA20BSupported == FALSE || fA20CSupported == FALSE ) )
//		{
//			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//			      "SID $22 MID $F6A2 SDTID $0B or $0C not supported (Required for All Vehicles with PID $F401 Data B bit 0 set)\n" );
//			fError = TRUE;
//		}

		if ( fEGSSupported == TRUE && f01Supported == FALSE )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "SID $22 MID $F601 not supported (Required for All Vehicles with PID $F501 EGS support set)\n" );
			fError = TRUE;
		}

		if ( fEGRSupported == TRUE && f31_38Supported == FALSE )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "SID $22 MID $F631-$F638 not supported (At least one required for All Vehicles with PID $F501 EGR support set)\n" );
			fError = TRUE;
		}

		// IF a spark ignition (gasoline) vehicle
		if ( gOBDDieselFlag == FALSE )
		{
			if ( fCATSupported == TRUE && f21Supported == FALSE )
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "SID $22 MID $F621 not supported (Required for Spark Ignition Vehicles with PID $F501 CAT support set)\n" );
				fError = TRUE;
			}

			if ( fHCATSupported == TRUE && f61_64Supported == FALSE )
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "SID $22 MID $F661-$F664 not supported (At least one required for Spark Ignition Vehicles with PID $F501 HCAT support set)\n" );
				fError = TRUE;
			}

			if ( fEVAPSupported == TRUE && f39_3DSupported == FALSE )
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "SID $22 MID $F639-$F63D not supported (At least one required for Spark Ignition Vehicles with PID $F501 EVAP support set)\n" );
				fError = TRUE;
			}

			if ( fAIRSupported == TRUE && f71_74Supported == FALSE )
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "SID $22 MID $F671-$F674 not supported (At least one required for Spark Ignition Vehicles with PID $F501 AIR support set)\n" );
				fError = TRUE;
			}
		}
		// ELSE a compression ignition (diesel) vehicle
		else
		{
			if ( fPMSupported == TRUE && fB2Supported == FALSE )
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "SID $22 MID $F6B2 not supported (Required for Compression Ignition Vehicles with PID $F501 PM support set)\n" );
				fError = TRUE;
			}

			if ( fSCRSupported == TRUE && f90919899Supported == FALSE )
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "SID $22 MID $F690, $F691, $F698, $F699 not supported (At least one required for Compression Ignition Vehicles with PID $F501 SCR support set)\n" );
				fError = TRUE;
			}

			if ( fBPSupported == TRUE && f85_86Supported == FALSE )
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "SID $22 MID $F685-$F686 not supported (At least one required for Compression Ignition Vehicles with PID $F501 BP support set)\n" );
				fError = TRUE;
			}
		}
	}  // end Test 5.14

//	else if ( (gUserInput.eComplianceType == US_OBDII || gUserInput.eComplianceType == HD_OBD) &&
//	          (fA20BSupported == FALSE || fA20CSupported == FALSE) )
//	{
//		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//		      "SID $22 MID $F6A2 SDTID $0B or $0C not supported by the vehicle\n" );
//		fError = TRUE;
//	}


	// Make sure the required MIDLSB/SDTID values are supported and try group support
	if ( gTestPhase != eTestPerformanceCounters )
	{
		// Link Active test after completion of group request
//		if ( VerifyGroupMonitorTestSupport () == FAIL )
		if ( RequestGroupDIDSupportData ( MIDREQUEST ) == FAIL )
		{
			return FAIL;
		}
	}


	if ( ulInit_FailureCount != GetFailureCount () || fError == TRUE )
	{
		// There could have been early/late responses that weren't treated as FAIL or other failure
		return FAIL;
	}
	else
	{
		return PASS;
	}

}


/*
*******************************************************************************
**   Function:   VerifyMidSupportData
**
**   Purpose:    Verify each controller supports at a minimum one PID.
**               Any ECU that responds that does not support at least
**               one PID is flagged as an error.
**
*******************************************************************************
*/
int VerifyMidSupportData ( void )
{
	int           bReturn = PASS;
	int           bEcuResult;
	unsigned int  EcuIndex;
	unsigned int  Index;

	// For each ECU
	for ( EcuIndex = 0; EcuIndex < gOBDNumEcus; EcuIndex++ )
	{
		bEcuResult = FAIL;
		for ( Index = 0; Index < gOBDResponse[EcuIndex].MidSupportSize; Index++ )
		{
			// If MID is supported, keep looking
			if ( gOBDResponse[EcuIndex].MidSupport[Index].IDBits[0] ||
			     gOBDResponse[EcuIndex].MidSupport[Index].IDBits[1] ||
			     gOBDResponse[EcuIndex].MidSupport[Index].IDBits[2] ||
			     ((gOBDResponse[EcuIndex].MidSupport[Index].IDBits[3]) & 0xFE) != 0x00 )
			{
				bEcuResult = PASS;
				break;
			}
		}

		if ( bEcuResult == FAIL && gOBDResponse[EcuIndex].MidSupportSize > 0 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  MID-Supported MIDs indicate no MIDs supported\n",
			      GetEcuId ( EcuIndex ) );
			bReturn = FAIL;
		}
	}

	return bReturn;
}
