/*
********************************************************************************
**  SAE J1699-5 Test Source Code
**
**   Copyright (C) 2021
**
**  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  This source code, when compiled and used with an SAE J2534-compatible pass
**  thru device, is intended to run the tests described in the SAE J1699-5
**  document in an automated manner.
**
**  This computer program is based upon SAE Technical Report J1699,
**  which is provided "AS IS"
**
**  See j1699.c for details of how to build and run this test.
**
********************************************************************************
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <windows.h>
#include "j2534.h"
#include "j1699.h"


/*
*******************************************************************************
**  VerifyReservedServices -
**  Function to verify that all ECUs respond correctly to reserved/unused services
*******************************************************************************
*/
STATUS VerifyReservedServices ( void )
{
	BOOL bTestFailed = FALSE;
	unsigned char SidIndex;
	SID_REQ SidReq;

	// For each SID
	for ( SidIndex = 0x00; SidIndex <= 0x0F; SidIndex++ )
	{
		switch ( SidIndex )
		{
			case 0x00:
			case 0x0B:
			case 0x0C:
			case 0x0D:
			case 0x0E:
			case 0x0F:
			{
				SidReq.SID = SidIndex;
				SidReq.NumIds = 0;
				if ( SidRequest ( &SidReq, SID_REQ_NORMAL|SID_REQ_IGNORE_NO_RESPONSE ) != FAIL )
				{
					Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "Sid $%02X response\n",
					      SidIndex );
					bTestFailed = TRUE;
					continue;
				}
			}
			break;
		}
	}

	if ( VerifyLinkActive () != PASS )
	{
		bTestFailed = TRUE;
	}

	if ( bTestFailed == TRUE )
	{
		return FAIL;
	}

	return PASS;
}


/*
*******************************************************************************
**  VerifyDeprecatedServices :  Check for a response to a deprecated classic OBD Service request ($1-$A)
*******************************************************************************
*/
STATUS VerifyDeprecatedServices ( unsigned int ID )
{
	SID_REQ     SidReq;
	STATUS      RetCode = PASS;

	// Check if J1979 Classic Services respond
	SidReq.SID = ID;
	if ( ID == 0x0A || ID == 0x07 || ID == 0x03 )
	{
		SidReq.NumIds = 0;
	}
	else
	{
		SidReq.NumIds = 1;
		SidReq.DID[0] = 00;
	}

	if ( SidRequest ( &SidReq, SID_REQ_NORMAL|SID_REQ_IGNORE_NO_RESPONSE ) != FAIL )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "Response to J1979 Classic Service $X.\n",
		      ID );
		RetCode = FAIL;
	}

	return RetCode;
}
