/*
********************************************************************************
**  SAE J1699-5 Test Source Code
**
**  Copyright (C) 2002 Drew Technologies. http://j1699-3.sourceforge.net/
**
** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  This source code, when compiled and used with an SAE J2534-compatible pass
**  thru device, is intended to run the tests described in the SAE J1699-5
**  document in an automated manner.
**
**  This computer program is based upon SAE Technical Report J1699,
**  which is provided "AS IS"
**
**  See j1699.c for details of how to build and run this test.
**
********************************************************************************
*/


/*******************************************************************************
**  Typedefs and function pointers for J2534 API                              **
*******************************************************************************/
typedef long (CALLBACK* PTCONNECT)(unsigned long, unsigned long, unsigned long, unsigned long, unsigned long *);
typedef long (CALLBACK* PTDISCONNECT)(unsigned long);
typedef long (CALLBACK* PTREADMSGS)(unsigned long, void *, unsigned long *, unsigned long);
typedef long (CALLBACK* PTWRITEMSGS)(unsigned long, void *, unsigned long *, unsigned long);
typedef long (CALLBACK* PTSTARTPERIODICMSG)(unsigned long, void *, unsigned long *, unsigned long);
typedef long (CALLBACK* PTSTOPPERIODICMSG)(unsigned long, unsigned long);
typedef long (CALLBACK* PTSTARTMSGFILTER)(unsigned long, unsigned long, void *, void *, void *, unsigned long *);
typedef long (CALLBACK* PTSTOPMSGFILTER)(unsigned long, unsigned long);
typedef long (CALLBACK* PTSETPROGRAMMINGVOLTAGE)(unsigned long, unsigned long, unsigned long);
typedef long (CALLBACK* PTREADVERSION)(unsigned long, char *, char *, char *);
typedef long (CALLBACK* PTGETLASTERROR)(char *);
typedef long (CALLBACK* PTIOCTL)(unsigned long, unsigned long, void *, void *);

/*******************************************************************************
**  J2534 API 4.04                                                            **
*******************************************************************************/
typedef long (CALLBACK* PTOPEN)(void *, unsigned long *);
typedef long (CALLBACK* PTCLOSE)(unsigned long);

extern PTCONNECT PassThruConnect;
extern PTDISCONNECT PassThruDisconnect;
extern PTREADMSGS PassThruReadMsgs;
extern PTWRITEMSGS PassThruWriteMsgs;
extern PTSTARTPERIODICMSG PassThruStartPeriodicMsg;
extern PTSTOPPERIODICMSG PassThruStopPeriodicMsg;
extern PTSTARTMSGFILTER PassThruStartMsgFilter;
extern PTSTOPMSGFILTER PassThruStopMsgFilter;
extern PTSETPROGRAMMINGVOLTAGE PassThruSetProgrammingVoltage;
extern PTREADVERSION PassThruReadVersion;
extern PTGETLASTERROR PassThruGetLastError;
extern PTIOCTL PassThruIoctl;
extern PTOPEN  PassThruOpen;
extern PTCLOSE PassThruClose;


/* maximum number of J2534 devices allowed */
#define MAX_J2534_DEVICES        50

/* Battery voltage limits */
/* Below 11VDC, OBD monitors are not required to run */
#define OBD_BATTERY_MINIMUM      11000

/* Above 18VDC, J1978 scan tools are not required to run */
#define OBD_BATTERY_MAXIMUM      18000

/* Tester present message rate */
/* 4.9 seconds between tester present messages */
#define OBD_TESTER_PRESENT_RATE  2000

/* Time delay to allow for code clearing */
#define CLEAR_CODES_DELAY_MSEC   2000

/* Maximum number of OBD ECUs and protocols */
#define OBD_LEGACY_MAX_ECUS      8
#define OBD_MAX_ECUS             239
#define MAX_ECUS_PER_PAGE        8
#define MAX_ECU_PAGE             30  // OBD_MAX_ECUS / MAX_ECUS_PER_PAGE rounded up
#define OBD_MAX_PROTOCOLS        2

/* OBD response indicator bit */
#define OBD_RESPONSE_BIT         0x40

/* ISO15765 Message Definitions */
#define ISO15765_MAX_BYTES_PER_FRAME 7
#define ISO15765_HEADER_SIZE     4
#define ISO15765_DID_MSB_OFFSET  ISO15765_HEADER_SIZE + 1
#define ISO15765_DID_LSB_OFFSET  ISO15765_HEADER_SIZE + 2
#define ISO15765_DATA_OFFSET     ISO15765_HEADER_SIZE + 3

#define LEGACY_HEADER_SIZE       3


/* earliest allowable model year */
#define MIN_MODEL_YEAR           1981

/* Latest allowable model year */
#define MAX_MODEL_YEAR           2025

/* maximum length for a log filename */
#define MAX_LOGFILENAME          80
#define MAX_USERINPUT            255

/* logging related maximums */
#define MAX_LOG_STRING_SIZE      2048   // max. size of a single log entry
#define MAX_MESSAGE_LOG_SIZE     640    // max. number of message bytes in a single log entry
#define DESIRED_DUMP_SIZE        8      // desired number of transactions to be dumped from ring buffer
#define MAX_TRANSACTION_COUNT    (DESIRED_DUMP_SIZE + 1)  // max. number of transactions in ring buffer
#define MAX_RING_BUFFER_SIZE     16384  // max size of transaction ring buffer


/* Function return value definitions (sometimes treated as bit map, DO NOT CHANGE VALUES!) */
typedef enum { PASS=0x00, FAIL=0x02, ABORT=0x04, ERRORS=0x08, RETRY=0x10, CTRL_C=0x20, EXIT=0x40 } STATUS;

/* NAK Response ID and Codes */
#define NAK                      0x7F
#define NAK_GENERAL_REJECT       0x10
#define NAK_NOT_SUPPORTED        0x11
#define NAK_INVALID_FORMAT       0x12
#define NAK_REPEAT_REQUEST       0x21
#define NAK_SEQUENCE_ERROR       0x22
#define NAK_RESPONSE_PENDING     0x78

/* Tester node address */
#define TESTER_NODE_ADDRESS      0xF1

/* User prompt type definitions */
typedef enum { NO_PROMPT=0, ENTER_PROMPT=1, YES_NO_PROMPT=2, YES_NO_ALL_PROMPT=3, QUIT_CONTINUE_PROMPT=4,
               CUSTOM_PROMPT=5, COMMENT_PROMPT=6 } PROMPTTYPE;

/* User log entry type definitions */
typedef enum { USER_ERROR=1, FAILURE=2, WARNING=3, INFORMATION=4, PROMPT=5,
               SUBSECTION_BEGIN=6, SUBSECTION_PASSED_RESULT=7, SUBSECTION_FAILED_RESULT=8,
               SUBSECTION_INCOMPLETE_RESULT=9, SECTION_PASSED_RESULT=10, SECTION_FAILED_RESULT=11,
               SECTION_INCOMPLETE_RESULT=12, BLANK=13, NETWORK=14, ERROR_FAILURE=15, RESULTS=16,
               COMMENT=17, J2534_FAILURE=20 } LOGTYPE;

/* Screen Output type definitions */
typedef enum { SCREENOUTPUTON=TRUE, SCREENOUTPUTOFF=FALSE } SCREENOUTPUT;

/* Logfile Output type definitions */
typedef enum { LOGOUTPUTON=TRUE, LOGOUTPUTOFF=FALSE } LOGOUTPUT;

/* engine type definitions */
typedef enum { ENGINE_UNKNOWN=0, GASOLINE=1, DIESEL=2 } ENGINETYPE;

/* Powertrain type definitions */
typedef enum { PWRTRN_UNKNOWN=0, CONV=1, SS=2, HEV=3, PHEV=4 } PWRTRNTYPE;

/* vehicle type definitions */
typedef enum { VEHICLE_UNKNOWN=0, LD=1, MD=2, HD=3 } VEHICLETYPE;

/* Compliance test type definitions */
typedef enum { UNKNOWN=0, US_OBDII=1, EOBD_WITH_IUMPR=2, EOBD_NO_IUMPR=3, HD_OBD=4, HD_EOBD=5 , IOBD_NO_IUMPR=6,
               HD_IOBD_NO_IUMPR=7, OBDBr_NO_IUMPR=8, CNOBD_WITH_IUMPR=9} COMPLIANCETYPE;
typedef enum { USOBD=1, EOBD=2, EOBD_250K=3 } SCANTABLETYPE;

typedef enum { eTestNone=0, eTestNoDTC=5, eTestPendingDTC, eTestConfirmedDTC, eTestFaultRepaired,
               eTestNoFault3DriveCycle, eTestInUseCounters, eTestPerformanceCounters } TEST_PHASE;

// DID type enumeration
typedef enum { PIDREQUEST=0, MIDREQUEST=1, TIDREQUEST=2, INFREQUEST=3 } DIDTYPE;


// List of message defines - ORDER MUST MATCH g_rgpcDisplayStrings in j1699.c
#define DSPSTR_PRMPT_MODEL_YEAR         0
#define DSPSTR_PRMPT_ENG_CERT_YEAR      1
#define DSPSTR_PRMPT_OBD_ECU            2
#define DSPSTR_PRMPT_ENG_TYPE           3
#define DSPSTR_PRMPT_PWRTRN_TYPE        4
#define DSPSTR_STMT_COMPLIANCE_TYPE     5
#define DSPSTR_PRMPT_VEH_TYPE           6
#define DSPSTR_PRMPT_ISO15765_TYPE      7
#define DSPSTR_RSLT_TOT_USR_ERROR       8
#define DSPSTR_RSLT_TOT_J2534_FAIL      9
#define DSPSTR_RSLT_TOT_WARN            10
#define DSPSTR_RSLT_TOT_FAIL            11
#define DSPSTR_RSLT_TOT_COMMENTS        12

#define DSPSTR_TOTAL                    13


// Drive Cycle completion flags
#define IDLE_TIME           0x01
#define ATSPEED_TIME        0x02
#define CUMULATIVE_TIME     0x04
#define FEO_TIME            0x08


/* SID 22 InfoTypes */
#define INF_TYPE_VIN_COUNT       0xF801
#define INF_TYPE_VIN             0xF802
#define INF_TYPE_CALID_COUNT     0xF803
#define INF_TYPE_CALID           0xF804
#define INF_TYPE_CVN_COUNT       0xF805
#define INF_TYPE_CVN             0xF806
#define INF_TYPE_IPT_COUNT       0xF807
#define INF_TYPE_IPT_GAS         0xF808
#define INF_TYPE_ECUNAME_COUNT   0xF809
#define INF_TYPE_ECUNAME         0xF80A
#define INF_TYPE_IPT_DIESEL      0xF80B
#define INF_TYPE_ESN_COUNT       0xF80C
#define INF_TYPE_ESN             0xF80D
#define INF_TYPE_EROTAN_COUNT    0xF80E
#define INF_TYPE_EROTAN          0xF80F
#define INF_TYPE_PROTID          0xF810
#define INF_TYPE_GTR             0xF811
#define INF_TYPE_FEOCNTR         0xF812
#define INF_TYPE_TG_EFN          0xF813
#define INF_TYPE_EVAP_DIST       0xF814
#define INF_TYPE_MTRCYCLE        0xF815
#define INF_TYPE_ER_IT           0xF816
#define INF_TYPE_DT_FC           0xF817
#define INF_TYPE_PKE_EOE         0xF818
#define INF_TYPE_PSA             0xF819
#define INF_TYPE_PHEVDD          0xF81A
#define INF_TYPE_PHEVFD          0xF81B
#define INF_TYPE_PHEVGD          0xF81C
#define INF_TYPE_AAF1OCCVD       0xF81D
#define INF_TYPE_AAF2OCCVD       0xF81E
#define INF_TYPE_AAF3OCCVD       0xF81F
#define INF_TYPE_DSOMOCCVD       0xF821
#define INF_TYPE_RTSSCOCCVD      0xF822
#define INF_TYPE_DCTOCCVD        0xF823
#define INF_TYPE_APTWUFOCCVD     0xF824
#define INF_TYPE_OCCT1VD         0xF825
#define INF_TYPE_OCCT2VD         0xF826
#define INF_TYPE_OCCT3VD         0xF827
#define INF_TYPE_OCCT4VD         0xF828
#define INF_TYPE_OCCT5VD         0xF829

#define INF_TYPE_HDGHGA100HD1    0xF841
#define INF_TYPE_HDGHGA100HD2    0xF842
#define INF_TYPE_HDGHGS100HD1    0xF843
#define INF_TYPE_HDGHGS100HD2    0xF844
#define INF_TYPE_HDGHGLD1        0xF845
#define INF_TYPE_HDGHGLD2        0xF846
#define INF_TYPE_HDGHGPSAD       0xF847
#define INF_TYPE_HDGHGPHEVVD     0xF848
#define INF_TYPE_HDGHGPHEVFD     0xF849
#define INF_TYPE_HDGHGPHEVGD     0xF850
#define INF_TYPE_HDGHGATSS       0xF851
#define INF_TYPE_HDGHGATAES      0xF852
#define INF_TYPE_HDGHGATWHR      0xF853
#define INF_TYPE_HDGHGAT1AT2     0xF854
#define INF_TYPE_HDGHGAT3AT4     0xF855
#define INF_TYPE_HDGHGAT5AT6     0xF856
#define INF_TYPE_HDGHGAT7AT8     0xF857
#define INF_TYPE_HDGHGAT9AT10    0xF858
#define INF_TYPE_HDGHGAT11AT12   0xF859
#define INF_TYPE_HDGHGAT13AT14   0xF85A
#define INF_TYPE_HDGHGAT15AT16   0xF85B

#define INF_TYPE_NOx100HMEOA     0xF861
#define INF_TYPE_NOx100HMTA      0xF862
#define INF_TYPE_NOx100HEOEA     0xF863
#define INF_TYPE_NOx100HDTA      0xF864
#define INF_TYPE_NOx100HERTA     0xF865
#define INF_TYPE_NOx100HVFCA     0xF866
#define INF_TYPE_NOx100HMEOS     0xF867
#define INF_TYPE_NOx100HMTS      0xF868
#define INF_TYPE_NOx100HEOES     0xF869
#define INF_TYPE_NOx100HDTS      0xF86A
#define INF_TYPE_NOx100HERTS     0xF86B
#define INF_TYPE_NOx100HVFCS     0xF86C
#define INF_TYPE_NOxVMEOL        0xF86D
#define INF_TYPE_NOxVMTL         0xF86E
#define INF_TYPE_NOxVEOEL        0xF86F
#define INF_TYPE_NOxVDTL         0xF870
#define INF_TYPE_NOxVERTL        0xF871
#define INF_TYPE_NOxVVFCL        0xF872
#define INF_TYPE_NOxEAEOEL       0xF873
#define INF_TYPE_NOxEADTL        0xF874
#define INF_TYPE_NOxEAERTL       0xF875
#define INF_TYPE_NOxEAVFCL       0xF876
#define INF_TYPE_NOxPMRVD        0xF877
#define INF_TYPE_ER              0xF878


#define INF_TYPE_IPT_NODI        28      // number of IPT data items

#define IPT_OBDCOND_INDEX       0   //INF8 & INFB
#define IPT_IGNCNTR_INDEX       1   //INF8 & INFB
#define IPT_CATCOMP1_INDEX      2   //INF8
#define IPT_HCCATCOMP_INDEX     2   //INFB
#define IPT_CATCOND1_INDEX      3   //INF8
#define IPT_HCCATCOND_INDEX     3   //INFB
#define IPT_CATCOMP2_INDEX      4   //INF8
#define IPT_NCATCOMP_INDEX      4   //INFB
#define IPT_CATCOND2_INDEX      5   //INF8
#define IPT_NCATCOND_INDEX      5   //INFB
#define IPT_O2SCOMP1_INDEX      6   //INF8
#define IPT_NADSCOMP_INDEX      6   //INFB
#define IPT_O2SCOND1_INDEX      7   //INF8
#define IPT_NADSCOND_INDEX      7   //INFB
#define IPT_O2SCOMP2_INDEX      8   //INF8
#define IPT_PMCOMP_INDEX        8   //INFB
#define IPT_O2SCOND2_INDEX      9   //INF8
#define IPT_PMCOND_INDEX        9   //INFB
#define IPT_INF8_EGRCOMP_INDEX  10  //INF8
#define IPT_EGSCOMP_INDEX       10  //INFB
#define IPT_INF8_EGRCOND_INDEX  11  //INF8
#define IPT_EGSCOND_INDEX       11  //INFB
#define IPT_AIRCOMP_INDEX       12  //INF8
#define IPT_INFB_EGRCOMP_INDEX  12  //INFB
#define IPT_AIRCOND_INDEX       13  //INF8
#define IPT_INFB_EGRCOND_INDEX  13  //INFB
#define IPT_EVAPCOMP_INDEX      14  //INF8
#define IPT_BPCOMP_INDEX        14  //INFB
#define IPT_EVAPCOND_INDEX      15  //INF8
#define IPT_BPCOND_INDEX        15  //INFB
#define IPT_SO2SCOMP1_INDEX     16  //INF8
#define IPT_FUELCOMP_INDEX      16  //INFB
#define IPT_SO2SCOND1_INDEX     17  //INF8
#define IPT_FUELCOND_INDEX      17  //INFB
#define IPT_SO2SCOMP2_INDEX     18  //INF8 NODI 20 & 28
#define IPT_SO2SCOND2_INDEX     19  //INF8 NODI 20 & 28
#define IPT_AFRICOMP1_INDEX     20  //INF8 NODI 28
#define IPT_AFRICOND1_INDEX     21  //INF8 NODI 28
#define IPT_AFRICOMP2_INDEX     22  //INF8 NODI 28
#define IPT_AFRICOND2_INDEX     23  //INF8 NODI 28
#define IPT_PFCOMP1_INDEX       24  //INF8 NODI 28
#define IPT_PFCOND1_INDEX       25  //INF8 NODI 28
#define IPT_PFCOMP2_INDEX       26  //INF8 NODI 28
#define IPT_PFCOND2_INDEX       27  //INF8 NODI 28


/* SidRequest Flags */
#define SID_REQ_NORMAL                      0x00000000
#define SID_REQ_RETURN_AFTER_ALL_RESPONSES  0x00000001
#define SID_REQ_NO_PERIODIC_DISABLE         0x00000002
#define SID_REQ_ALLOW_NO_RESPONSE           0x00000004
        //   set = no response returns PASS from SidRequest
        // unset = no response returns FAIL from SidRequest
#define SID_REQ_IGNORE_NO_RESPONSE          0x00000008
        //   set = log WARNING on no response in SidRequest
        // unset = don't log WARNING on no response in SidRequest


/* LogMsg Flags */
#define LOG_NORMAL_MSG          0
#define LOG_REQ_MSG             1

/* Macros */
#define ABORT_RETURN   {gOBDTestAborted = TRUE; gOBDTestSectionAborted = TRUE; return ABORT;}


/*
 * ProtocolTag list - the order of this list must match ProtocolInitData[] in InitProtocolList.c
 */
#define ISO15765_I              0   // US OBD-II protocol
#define ISO15765_29_BIT_I       1   // US OBD-II protocol
#define ISO15765_250K_11_BIT_I  2   // EOBD protocol
#define ISO15765_250K_29_BIT_I  3   // EOBD protocol

#define MAX_PROTOCOL_NAME       25
/* Protocol list structure */
typedef struct
{
	unsigned long Protocol;        // generic protocol (for J2534-1)
	unsigned long ChannelID;
	unsigned long InitFlags;       // J2534-1 ConnectFlags
	unsigned long TesterPresentID;
	unsigned long FilterID;
	unsigned long FlowFilterID[OBD_MAX_ECUS];
	unsigned long HeaderSize;      // size of message header
	unsigned long BaudRate;        // link data rate
	unsigned long ProtocolTag;     // tag that uniquely identifies each protocol
	char Name[MAX_PROTOCOL_NAME];  // ASCII string for the protocol name
} PROTOCOL_LIST;


#define MAX_PIDS 0x105

/* OBD message response data structures */
#define MAX_RESP_BUF_SIZE 2048
#define MAX_PID_RESP_SIZE 41
typedef struct
{
	unsigned char PIDMSB;
	unsigned char PIDLSB;
	unsigned char Data[MAX_PID_RESP_SIZE];
} PID;

typedef struct
{
	unsigned char MIDMSB;
	unsigned char MIDLSB;
	unsigned char SDTID;
	unsigned char UASID;
	unsigned char TVHI;
	unsigned char TVLO;
	unsigned char MINTLHI;
	unsigned char MINTLLO;
	unsigned char MAXTLHI;
	unsigned char MAXTLLO;
} MID;

typedef struct
{
	unsigned char INFMSB;
	unsigned char INFLSB;
	unsigned char NumItems;
	unsigned char Data[MAX_RESP_BUF_SIZE];
} INF;

typedef struct
{
	unsigned char  INFMSB;      // INF Type most significant byte
	unsigned char  INFLSB;      // INF Type least significant byte
	unsigned char  NODI;        // number data items
	unsigned short Data[INF_TYPE_IPT_NODI];
	unsigned short Flags;       // application flags. set 1 when valid
} IPT;
#define  VALIDDATA   0x01
#define  IMREADYDONE 0x02
#define  IPTDONE     0x04
#define  ECUDONE     0x80

typedef struct
{
	unsigned char FirstID;
	unsigned char IDBits[4];
} ID_SUPPORT;

typedef struct
{
	unsigned char FirstID;
	unsigned char FrameNumber;
	unsigned char IDBits[4];
} FF_SUPPORT;

typedef struct
{
	unsigned char DTC[3];
	unsigned char RecordNumber;
} FreezeFrame;

typedef struct
{
	unsigned char PIDMSB;
	unsigned char PIDLSB;
	unsigned char FrameNumber;
	unsigned char Data[41];
} FreezeFrameData;

typedef struct
{
	BOOL            DTCSupported;
	unsigned char   DTCStatus;
	unsigned char   DTCSeverity;
	unsigned char   DTCFormat;
	unsigned short  DTCSize;           // number of bytes in array ConfDTC or PendDTC
} DTC_HEADER;

typedef struct
{
	unsigned short Size;
	unsigned char  DTC[MAX_RESP_BUF_SIZE];
} DTC_LIST;

typedef struct
{
	unsigned char Severity;
	unsigned char DTC[3];
	unsigned char Status;
} DTC;

typedef struct
{
	unsigned char DTC[3];
	unsigned char Status;
} DTC_Perm;

typedef struct
{
	unsigned char   Header[4];

	BOOL            bResponseReceived;  // used to check for multiple responses

	BOOL            LinkActive;

	unsigned char   PidSupportSize;
	ID_SUPPORT      PidSupport[16];
	unsigned short  PidSize;
	unsigned char   Pid[246];       // 6 * largest PID (81 - 41 bytes)

	unsigned char   FFSupportSize;
	FF_SUPPORT      FFSupport[8];
	unsigned short  FFSize;
	unsigned char   FF[MAX_RESP_BUF_SIZE];

	DTC_HEADER      ConfDTCHeader;
	unsigned char   ConfDTC[MAX_RESP_BUF_SIZE];

	unsigned short  ClrDTCSize;
	unsigned char   ClrDTCData[8];

	unsigned char   MidSupportSize;
	ID_SUPPORT      MidSupport[8];
	unsigned short  MidSize;
	unsigned char   Mid[MAX_RESP_BUF_SIZE];

	DTC_HEADER      PendDTCHeader;
	unsigned char   PendDTC[MAX_RESP_BUF_SIZE];

	unsigned char   TidSupportSize;
	ID_SUPPORT      TidSupport[8];
	unsigned short  TidSize;
	unsigned char   Tid[MAX_RESP_BUF_SIZE];

	unsigned char   InfSupportSize;
	ID_SUPPORT      InfSupport[8];
	unsigned short  InfSize;
	unsigned char   Inf[MAX_RESP_BUF_SIZE];

	BOOL            PermDTCSupported;
	unsigned char   PermDTCStatus;
	unsigned char   PermDTCFormat;
	unsigned short  PermDTCSize;           // number of bytes in array PermDTC
	unsigned char   PermDTC[MAX_RESP_BUF_SIZE];
} OBD_DATA;


typedef struct
{
	unsigned long   EcuId;                       // ID of the ECU
	BOOL            NAKReceived;
	BOOL            FFReceived;
	unsigned long   ExtendResponseTimeMsecs;
	unsigned long   ResponsePendingDelay;
	unsigned long   AggregateResponseTimeMsecs;  // Total Accumulated Response Time for this ECU
	unsigned long   AggregateResponses;          // Total number of responses from this ECU
	unsigned long   LongestResponsesTime;        // Longest response time from this ECU
	unsigned long   RespTimeOutofRange;          // Count of response times out of range
	unsigned long   RespTimeTooLate;             // Count of response times too long
	unsigned long   RespTimeTooSoon;             // Count of response times too short
} ECU_TIMING_DATA;


#define  VIN_BUFFER_SIZE         18
#define  CVN_BUFFER_SIZE         18
#define  CALID_BUFFER_SIZE       18
#define  ECUNAME_BUFFER_SIZE     18
#define TGEFN_BUFFER_SIZE        13
#define  END_OF_TEST_FORMATTING  27
typedef struct
{
	unsigned long   EcuId;                       // ID of the ECU
	BOOL            VIN_Valid;
	char            VIN_String[VIN_BUFFER_SIZE];
	unsigned int    CVN_Count;
	char           *CVN_String;
	unsigned int    CALID_Count;
	char           *CALID_String;
	char            ECUName_String[ECUNAME_BUFFER_SIZE];
	char            TGEFN_String[TGEFN_BUFFER_SIZE];
	unsigned int    ODB_Compliance_Level;
	unsigned char   ProtocolID;
	BOOL            Diesel;
	PID             PID01;
	PID             PID41;
	unsigned int    PID51;
	PID             PIDF501;
	BOOL            HEV;
	BOOL            PHEV;
} ECU_DATA;


/* Service ID (Mode) request structure */
#define MAX_NUM_IDS 8
typedef struct
{
	unsigned char  SID;
	unsigned char  NumIds;
	unsigned char  Ids[MAX_NUM_IDS];
	unsigned int   DID[MAX_NUM_IDS];
} SID_REQ;


#define MAX_CODE_STR_LEN  10
typedef struct
{
	ENGINETYPE     eEngineType;
	PWRTRNTYPE     ePwrTrnType;
	VEHICLETYPE    eVehicleType;
	COMPLIANCETYPE eComplianceType;     // type of compliance being tested for
	SCANTABLETYPE  eScanTable;          // table of permutations used for OBD protocol scans
	unsigned long  MaxProtocols;        // maximum number of OBD protocols
	char           szNumDCToSetMIL[MAX_CODE_STR_LEN];  // count of drive cycles (either "two" or "three" ) with fault to set MIL
} USER_INPUT;



/*******************************************************************************
**  Local function prototypes                                                 **
*******************************************************************************/
void   clear_keyboard_buffer ( void );

STATUS TestWithNoDtc ( void );
STATUS TestWithPendingDtc ( void );
STATUS TestWithConfirmedDtc ( void );
STATUS TestWithFaultRepaired ( void );
STATUS TestWithNoFaultsAfter3DriveCycles ( void );
STATUS TestToVerifyInUseCounters ( BOOL *pbReEnterTest );
STATUS TestToVerifyPerformanceCounters ( BOOL *pbReEnterTest );

STATUS DetermineProtocol ( void );
unsigned int GetEcuId ( unsigned int EcuIndex );

STATUS SidRequest ( SID_REQ *, unsigned long );

STATUS ConnectProtocol ( void );
STATUS StartPeriodicMsg ( void );
STATUS StopPeriodicMsg ( BOOL bLogError );

STATUS DisconnectProtocol ( void );

STATUS IsDTCPending ( unsigned long Flags );
void   SaveDTCList ( int nSID );

STATUS IsDTCStored ( unsigned long Flags );
void   LogMsg ( PASSTHRU_MSG *, unsigned long );

char   Log ( LOGTYPE LogType, SCREENOUTPUT ScreenOutput, LOGOUTPUT LogOutput, PROMPTTYPE PromptType, const char *LogString, ... );
void   SaveTransactionStart ( void );        // marks the start of a new transaction in the ring buffer
unsigned long GetFailureCount ( void );      // returns global failure count
void   LogStats ( void );
char * substring ( char * str, const char * substr );  // returns loc. of 'substr' in 'str'

STATUS ClearCodes ( void );
STATUS VehicleReport ( void );
STATUS VerifyMILData ( void );
STATUS VerifyMonitorTestSupportAndResults ( void );
STATUS VerifyReversePIDSupport ( void );
STATUS VerifyGroupDiagnosticSupport ( void );
STATUS VerifyGroupMonitorTestSupport ( void );
STATUS VerifyGroupFreezeFrameSupport ( unsigned long *pFreezeFrameDTC );
STATUS VerifyGroupFreezeFrameResponse ( unsigned long *pFreezeFrameDTC );
STATUS VerifyGroupControlSupport ( void );
STATUS VerifyGroupVehicleInformationSupport ( void );
STATUS VerifyDiagnosticSupportAndData ( unsigned char OBDEngineDontCare );
STATUS VerifyDiagnosticBurstSupport ( void );
STATUS VerifyFreezeFrameSupportAndData ( void );
STATUS VerifyDTCStoredData ( void );
STATUS VerifyDTCPendingData ( void );
STATUS VerifyControlSupportAndData ( void );
STATUS VerifyLinkActive ( void );

STATUS VerifyVehicleInformationSupportAndData ( void );
STATUS VerifyIM_Ready ( void );
STATUS VerifyINF8Data ( unsigned long  EcuIndex );
STATUS VerifyINFBData ( unsigned long  EcuIndex );
STATUS VerifyVehicleInformationChange ( void );

STATUS VerifyPermanentCodeSupport ( void );
STATUS TestToVerifyPermanentCodes ( void );

STATUS VerifyReservedServices ( void );
STATUS VerifyDeprecatedServices ( unsigned int ID );

STATUS VerifyVehicleState ( unsigned char ucEngineRunning, unsigned char ucHybrid );

// DID Support Functions
STATUS RequestDIDSupportData ( DIDTYPE DidType, BOOL bVerifyLinkActive );
STATUS RequestGroupDIDSupportData (  DIDTYPE DidType );
STATUS CompareDIDSupport ( unsigned long EcuIndex,        // The index of the current ECU
                           DIDTYPE       DidType,         // Type of DID support array
                           unsigned char ResponseSize,    // Size of group/reverse DID Support response
                           unsigned char CompareSize,     // Size of single/normal DID Support response
                           ID_SUPPORT    ResponseData[],  // Data of group/reverse DID Support response
                           ID_SUPPORT    CompareData[],   // Data of single/normal DID Support response
                           BOOL          bGroup );        // Flag indicating if this is a group DID Support response
BOOL   IsDIDSupported ( unsigned int EcuIndex, unsigned int DidIndex );


/*******************************************************************************
**  Global variables                                                          **
*******************************************************************************/
extern char          gLogFileName[MAX_LOGFILENAME];
extern int           gModelYear;                 // integer version of model year entered by the user
extern int           gVINModelYear;              // integer version of model year entered by the user (used for VIN checks)
extern char          gUserMake[80];              // make of vehicle entered by the user
extern char          gUserModel[80];             // model of vehicle entered by the user
extern unsigned long gUserNumEcus;               // number of ECUs set by the user
extern unsigned long gNumEcusForAlloc;           // number of ECUs used for memory allocation

extern USER_INPUT    gUserInput;                 // structure for user selected compliance test information
extern char         *gEngineTypeStrings[3];      // strings that represent the user selected engine type
extern char         *gPwrTrnTypeStrings[5];      // strings that represent the user selected powertrain type
extern char         *gVehicleTypeStrings[4];     // strings that represent the user selected vehicle type
extern char         *gComplianceTestTypeStrings[10];  // strings that represent the user selected compliance test
extern char         *g_rgpcDisplayStrings[DSPSTR_TOTAL];  // strings used for display/logging

extern BOOL          gDetermineProtocol;

extern BOOL          gOBDTestAborted;            // set if any test was aborted,
                                                 // clear only before running Static and Dynamic tests

extern BOOL          gOBDTestSectionAborted;     // set if a test section (Static/Dynamic) was aborted,
                                                 // clear only before the Static test and before Dynamic tests

extern BOOL          gOBDTestFailed;             // set if any test failed,
                                                 // clear only before running Static and Dynamic tests

extern BOOL          gOBDTestSectionFailed;      // set if a test (static/dynamic) failed,
                                                 // clear only at the beginning of the Static or Dynamic test

extern BOOL          gOBDgTestSubsectionFailed;  // set if a test subsection failed,
                                                 // clear before each subsection (ie test 5.10, test 7.4, etc)

extern BOOL          gOBDDetermined;             // set if a OBD protocol found
extern unsigned int  gOBDRequestDelay;
extern unsigned int  gOBDMaxResponseTimeMsecs;
extern unsigned int  gOBDMinResponseTimeMsecs;   // min response time
extern unsigned int  gOBDListIndex;              // index of the OBD protocol being used
extern unsigned int  gOBDFoundIndex;             // index of OBD protocol found
extern PROTOCOL_LIST gOBDList[OBD_MAX_PROTOCOLS];
extern unsigned int  gOBDNumEcus;                // number of responding ECUs
extern unsigned int  gOBDNumEcusResp;            // number of responding ECUs
extern BOOL          gOBDEngineRunning;          // set if engine should be running
extern BOOL          gOBDEngineWarm;             // set if engine should be warm
extern BOOL          gOBDDTCPending;             // set if a DTC should be pending
extern BOOL          gOBDDTCStored;              // set if a DTC should be stored
extern BOOL          gOBDDTCHistorical;
extern BOOL          gOBDDTCPermanent;           // set if a DTC should be permanent
extern BOOL          gOBDDieselFlag;             // set by the user if this is a compression ignition (diesel) vehicle
extern BOOL          gOBDHybridFlag;             // set by the user if this is a hybrid vehicle
extern BOOL          gOBDPlugInFlag;
extern BOOL          gOBDDynoCertFlag;
extern unsigned long gOBDProtocolOrder;
extern unsigned long gOBDAggregateResponseTimeMsecs;
extern unsigned long gOBDAggregateResponses;
extern unsigned long gLastLogTime;
extern BOOL          gIgnoreUnsupported;
extern BOOL          gSuspendScreenOutput;
extern BOOL          gSuspendLogOutput;
extern OBD_DATA     *gOBDResponse;
extern OBD_DATA     *gOBDCompareResponse;
extern char          gVIN[VIN_BUFFER_SIZE];
extern FILE         *ghLogFile;
extern PASSTHRU_MSG  gTesterPresentMsg;
extern unsigned char gPermDTCSupported;          // count of ECUs supporting SID $19 $55
extern unsigned int  gPermDTCSupECU[OBD_MAX_ECUS];
extern BOOL          gVerifyLink;                // set if in VerifyLinkActive function

extern unsigned char gReverseOrderState[OBD_MAX_ECUS];   // state of reverse order request
#define NOT_REVERSE_ORDER       0  // not reverse order
#define REVERSE_ORDER_REQUESTED 1  // reverse order requested, no response yet
#define REVERSE_ORDER_RESPONSE  2  // first reverse order response received, SupportSize cleared

extern HINSTANCE hDLL;

extern unsigned long    gSid1VariablePidSize;

extern char            *gBanner;
extern const char       gszAPP_REVISION[];

extern unsigned long    gOBDNumEcusCan;
extern unsigned char    gOBDResponseTA[OBD_MAX_ECUS];

extern FILE            *ghTempLogFile;
extern char             gszTempLogFilename[MAX_PATH];

extern DTC_LIST        *gDTCList;

extern unsigned short   gDTC_Ignore_List_Count;
extern unsigned short  *gDTC_Ignore_List;

extern unsigned long    gECU_Ignore_List_Count;
extern unsigned long   *gECU_Ignore_List;

extern unsigned long    gulDeviceID;     // global storage for J2534-1 Device ID

extern TEST_PHASE       gTestPhase;
extern unsigned char    gTestSubsection;  // test subsection to be used in conjunction with gTestPhase

extern ECU_TIMING_DATA *gEcuTimingData;
extern ECU_DATA        *gEcuData;

extern BOOL             gJ2534_2_Mode;

// Option to turn off Tester Present Message
extern BOOL             gPeriodicMsgEnabled;
