/*
********************************************************************************
**  SAE J1699-5 Test Source Code
**
**   Copyright (C) 2021
**
**  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
**  This source code, when compiled and used with an SAE J2534-compatible pass
**  thru device, is intended to run the tests described in the SAE J1699-5
**  document in an automated manner.
**
**  This computer program is based upon SAE Technical Report J1699,
**  which is provided "AS IS"
**
**  See j1699.c for details of how to build and run this test.
**
********************************************************************************
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <windows.h>
#include "j2534.h"
#include "j1699.h"


#define  MAX_NAME_LENGTH  10


/* Function Prototypes */
STATUS VerifyVINData   ( unsigned long EcuIndex );
STATUS VerifyECUNameData ( unsigned long EcuIndex );
STATUS VerifyCALIDData ( unsigned long EcuIndex );
STATUS VerifyCVNData   ( unsigned long EcuIndex );
STATUS VerifyInfD_FData ( unsigned long  EcuIndex );
STATUS VerifyInf10Data ( unsigned long EcuIndex );
STATUS VerifyInf12Data ( unsigned long EcuIndex );
STATUS VerifyInf13Data ( unsigned long EcuIndex );
STATUS VerifyInf14Data ( unsigned long EcuIndex );
STATUS VerifyInf16Data ( unsigned long EcuIndex );
STATUS VerifyInf17Data ( unsigned long EcuIndex );
STATUS VerifyInf18Data ( unsigned long EcuIndex );
STATUS VerifyInf19Data ( unsigned long EcuIndex );
STATUS VerifyInf1AData ( unsigned long EcuIndex );
STATUS VerifyInf1BData ( unsigned long EcuIndex );
STATUS VerifyInf1CData ( unsigned long EcuIndex );
STATUS VerifyInf1DData ( unsigned long EcuIndex );
STATUS VerifyInf1EData ( unsigned long EcuIndex );
STATUS VerifyInf1FData ( unsigned long EcuIndex );
STATUS VerifyInf21Data ( unsigned long EcuIndex );
STATUS VerifyInf22Data ( unsigned long EcuIndex );
STATUS VerifyInf23Data ( unsigned long EcuIndex );
STATUS VerifyInf24Data ( unsigned long EcuIndex );
STATUS VerifyInf25_29Data ( unsigned long EcuIndex );
STATUS VerifyInf41_43_45Data ( unsigned long EcuIndex );
STATUS VerifyInf42_44_46Data ( unsigned long EcuIndex );
STATUS VerifyInf47Data ( unsigned long EcuIndex );
STATUS VerifyInf48Data ( unsigned long EcuIndex );
STATUS VerifyInf49Data ( unsigned long EcuIndex );
STATUS VerifyInf50Data ( unsigned long EcuIndex );
STATUS VerifyInf51Data ( unsigned long EcuIndex );
STATUS VerifyInf52Data ( unsigned long EcuIndex );
STATUS VerifyInf53Data ( unsigned long EcuIndex );
STATUS VerifyInf54_5BData ( unsigned long EcuIndex );
STATUS VerifyInf61_62_67_68_6D_6EData ( unsigned long EcuIndex );
STATUS VerifyInf63_69_6F_73Data ( unsigned long EcuIndex );
STATUS VerifyInf64_6A_70_74Data ( unsigned long EcuIndex );
STATUS VerifyInf65_6B_71_75Data ( unsigned long EcuIndex );
STATUS VerifyInf66_6C_72_76Data ( unsigned long EcuIndex );
STATUS VerifyInf77Data ( unsigned long EcuIndex );
STATUS VerifyInf78Data ( unsigned long EcuIndex );


typedef struct
{
	unsigned long INF16_IGNCNTR_R;
	unsigned long INF16_IGNCNTR_L;

	unsigned long INF16_FEOCNTR_R;
	unsigned long INF16_FEOCNTR_L;

	unsigned long INF16_ERT_R;
	unsigned long INF16_ERT_L;

	unsigned long INF16_IERT_L;

	float INF17_DT_R;
	float INF17_DT_L;

	float INF17_FC_R;
	float INF17_FC_L;

	float INF18_EOE_R;
	float INF18_EOE_L;

	unsigned long INF19_PSA_R;
	unsigned long INF19_PSA_L;

	unsigned long INF19_IPSA_L;

	unsigned long INF19_CPSA_R;
	unsigned long INF19_CPSA_L;

	unsigned long INF41_ERT_A;
	float         INF41_DT_A;
	float         INF41_EOE_A;
	float         INF41_VFC_A;

	float         INF42_EFC_A;
	unsigned long INF42_IERT_A;
	float         INF42_IEFC_A;
	unsigned long INF42_UERT_A;
	unsigned long INF42_PKE_A;
	unsigned long INF42_PTOERT_A;
	float         INF42_PTOFC_A;

	float         INF43_DT_S;
	float         INF43_EOE_S;
	float         INF43_VFC_S;

	float         INF44_EFC_S;

	unsigned long INF47_PSA_A;
	unsigned long INF47_PSA_S;
	unsigned long INF47_PSA_L;
	unsigned long INF47_IPSA_A;
	unsigned long INF47_IPSA_S;
	unsigned long INF47_IPSA_L;
	unsigned long INF47_UPSA_A;
	unsigned long INF47_UPSA_S;
	unsigned long INF47_UPSA_L;

	float         INF61_NOxEOB_A[17];

	float         INF62_NOxTPB_A[17];

	float         INF63_EOEB_A[17];

	float         INF64_DTB_A[17];
} Inf;
Inf  INF_Inf_5_17[OBD_MAX_ECUS];


/*
*******************************************************************************
**  VerifyVehicleInformationSupportAndData -
**  Function to verify INF vehicle info support and data
*******************************************************************************
*/
STATUS VerifyVehicleInformationSupportAndData ( void )
{
	unsigned long  EcuIndex;     // ECU Index
	unsigned long  EcuLoopIndex; // nested ECU Index
	unsigned long  IdIndex;      // INF ID index
	SID_REQ        SidReq;
	INF           *pINF;

	unsigned long  Inf2NumResponses = 0;         // number of INF $02 responses
	unsigned char  fInf4Responded[OBD_MAX_ECUS]; // per ECU INF $4 response indicator
	unsigned char  Inf6NumResponses = 0;         // number of INF $06 responses
	unsigned char  Inf8NumResponses = 0;         // number of INF $08 responses
	unsigned char  fInfAResponded[OBD_MAX_ECUS]; // per ECU INF $A response indicator
	unsigned char  InfBNumResponses = 0;         // number of INF $0B responses
	unsigned char  InfDNumResponses = 0;         // number of INF $0D responses
	unsigned char  Inf13NumResponses = 0;        // number of INF $13 responses

	#define  MAX_INF_ID 0x78
	unsigned int  InfSupportCountList[] =
	{ 0xF802, 0xF804, 0xF806, 0xF808, 0xF80A, 0xF80B, 0xF80D, 0xF812, 0xF813, 0xF814,
	  0xF816, 0xF817, 0xF818, 0xF819, 0xF81A, 0xF81B, 0xF81C, 0xF81D, 0xF81E, 0xF81F,
	  0xF821, 0xF822, 0xF823, 0xF824, 0xF825, 0xF826, 0xF827, 0xF828, 0xF829, 0xF841,
	  0xF842, 0xF843, 0xF844, 0xF845, 0xF846, 0xF847, 0xF848, 0xF849, 0xF84A, 0xF84B,
	  0xF84C, 0xF84D, 0xF84E, 0xF84F, 0xF850, 0xF851, 0xF852, 0xF853, 0xF854, 0xF855,
	  0xF856, 0xF857, 0xF858, 0xF859, 0xF85A, 0xF85B, 0xF861, 0xF862, 0xF863, 0xF864,
	  0xF865, 0xF866, 0xF867, 0xF868, 0xF869, 0xF86A, 0xF86B, 0xF86C, 0xF86D, 0xF86E,
	  0xF86F, 0xF870, 0xF871, 0xF872, 0xF873, 0xF874, 0xF875, 0xF876, 0xF877, 0xF878 };
	unsigned char  InfSupportCount[MAX_INF_ID+1];

	unsigned char  INF2Supported[OBD_MAX_ECUS];  // per ECU support INF $2 indicator
	unsigned char  INF13Supported[OBD_MAX_ECUS]; // per ECU support INF $13 indicator

	BOOL bTestFailed = FALSE;

	unsigned long ulInit_FailureCount = 0;

	unsigned long IGNCNTR;


	ulInit_FailureCount = GetFailureCount ();

	memset ( &InfSupportCount[0], 0, sizeof ( InfSupportCount ) );

	// Request INF support data
	if ( RequestDIDSupportData ( INFREQUEST, TRUE ) != PASS )
	{
		return FAIL;
	}


	// Determine number of ECUs supporting required INFs
	for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
	{
		// initialize the array values for this EcuIndex before use
		fInf4Responded[EcuIndex] = FALSE;
		fInfAResponded[EcuIndex] = FALSE;
		INF2Supported[EcuIndex] = FALSE;
		INF13Supported[EcuIndex] = FALSE;


		for ( IdIndex = 0; IdIndex < 80; IdIndex++ )
		{
			if ( IsDIDSupported ( EcuIndex, InfSupportCountList[IdIndex] ) == TRUE )
			{
				InfSupportCount[InfSupportCountList[IdIndex]-0xF800]++;

				if ( InfSupportCountList[IdIndex] == 0xF802 )
				{
					INF2Supported[EcuIndex] = TRUE;
				}

				// Spark Ignition IPT support
				else if ( InfSupportCountList[IdIndex] == 0xF808 )
				{
//					// if ECU does not only supports CCM requirements (PID $F401 Data B bit 2==1)
//					if ( (gEcuData[EcuIndex].PID01.Data[1] & 0x03) != 0x00 ||
//					     (gEcuData[EcuIndex].PID01.Data[2] & 0xFF) != 0x00 )
//					{
//						// Spark Ignition operator selection
//						if ( gOBDDieselFlag == FALSE )
//						{
//							// Compression Ignition PID $1 DATA_B Bit_3
//							if ( (gEcuData[EcuIndex].PIDF501.Data[1] & 0x08) != 0 /*&&
//							     ( gUserInput.eComplianceType == US_OBDII || gUserInput.eComplianceType == HD_OBD ||
//							       gUserInput.eComplianceType == EOBD_WITH_IUMPR || gUserInput.eComplianceType == HD_EOBD ||
//							       gUserInput.eComplianceType == CNOBD_WITH_IUMPR )*/ )
//							{
//								if ( gModelYear >= 2010 )
//								{
//									Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//									      "ECU %X  PID $F501  DATA_B Bit_3 does not match Operator Selected Ignition Type (Required for MY 2010 and later vehicles)\n",
//									      GetEcuId ( EcuIndex ) );
//									bTestFailed = TRUE;
//								}
//								else
//								{
//									Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//									      "ECU %X  PID $F501  DATA_B Bit_3 does not match Operator Selected Ignition Type (Required for MY 2010 and later vehicles)\n",
//									      GetEcuId ( EcuIndex ) );
//								}
//							}
//						}
//						// Compression Ignition operator selection
//						else
//						{
//							if ( gModelYear >= 2010 )
//							{
//								Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//								      "ECU %X  INF $F808 (IPT)  support does not match Operator Selected Ignition Type (Required for MY 2010 and later vehicles)\n",
//								      GetEcuId ( EcuIndex ) );
//								bTestFailed = TRUE;
//							}
//							else
//							{
//								Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//								      "ECU %X  INF $F808 (IPT)  support does not match Operator Selected Ignition Type (Required for MY 2010 and later vehicles)\n",
//								      GetEcuId ( EcuIndex ) );
//							}
//
//							// Spark Ignition PID $01 DATA_B Bit_3
//							if ( (gEcuData[EcuIndex].PIDF501.Data[1] & 0x08) == 0 )
//							{
//								if ( gModelYear >= 2010 )
//								{
//									Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//									      "ECU %X  PID $F501  DATA_B Bit_3 does not match Operator Selected Ignition Type (Required for MY 2010 and later vehicles)\n",
//									       GetEcuId ( EcuIndex ) );
//									bTestFailed = TRUE;
//								}
//								else
//								{
//									Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//									      "ECU %X  PID $F501  DATA_B Bit_3 does not match Operator Selected Ignition Type (Required for MY 2010 and later vehicles)\n",
//									      GetEcuId ( EcuIndex ) );
//								}
//							}
//						}
//					} // end if ECU does not only supports CCM requirements (PID $F401 Data B bit 2==1)
				} // end if ( IdIndex == 0x08 )

				// Compression Ignition IPT support
				else if ( InfSupportCountList[IdIndex] == 0xF80B )
				{
//					// if ECU does not only supports CCM requirements (PID $F401 Data B bit 2==1)
//					if ( (gEcuData[EcuIndex].PID01.Data[1] & 0x03) != 0x00 ||
//					     (gEcuData[EcuIndex].PID01.Data[2] & 0xFF) != 0x00 )
//					{
//						// Compression Ignition operator selection
//						if ( gOBDDieselFlag == TRUE )
//						{
//							// Compression Ignition PID $01 DATA_B Bit_3
//							if ( (gEcuData[EcuIndex].PIDF501.Data[1] & 0x08) == 0 )
//							{
//								if ( gModelYear >= 2010 )
//								{
//									Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//									       "ECU %X  PID $F501  DATA_B Bit_3 does not match Operator Selected Ignition Type (Required for MY 2010 and later vehicles)\n",
//									       GetEcuId ( EcuIndex ) );
//									bTestFailed = TRUE;
//								}
//								else
//								{
//									Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//									      "ECU %X  PID $F501  DATA_B Bit_3 does not match Operator Selected Ignition Type (Required for MY 2010 and later vehicles)\n",
//									      GetEcuId ( EcuIndex ) );
//								}
//							}
//						}
//						// Spark Ignition operator selection
//						else
//						{
//							if ( gModelYear >= 2010 )
//							{
//								Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//								      "ECU %X  INF $F80B (IPT)  support does not match Operator Selected Ignition Type (Required for MY 2010 and later vehicles)\n",
//								      GetEcuId ( EcuIndex ) );
//								bTestFailed = TRUE;
//							}
//							else
//							{
//								Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//								      "ECU %X  INF $F80B (IPT)  support does not match Operator Selected Ignition Type (Required for MY 2010 and later vehicles)\n",
//								      GetEcuId ( EcuIndex ) );
//							}
//
////							// Compression Ignition PID $01 DATA_B Bit_3
//							if ( (gEcuData[EcuIndex].PIDF501.Data[1] & 0x08) != 0 )
//							{
//								if ( gModelYear >= 2010 )Both INF $F808 and INF $F80B are supported
//								{
//									Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//									      "ECU %X  PID $F501  DATA_B Bit_3 does not match Operator Selected Ignition Type (Required for MY 2010 and later vehicles)\n",
//									      GetEcuId ( EcuIndex ) );
//									bTestFailed = TRUE;
//								}
//								else
//								{
//									Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//									      "ECU %X  PID $F501  DATA_B Bit_3 does not match Operator Selected Ignition Type (Required for MY 2010 and later vehicles)\n",
//									      GetEcuId ( EcuIndex ) );
//								}
//							}
//						} // end gOBDDieselFlag == FALSE
//					} // end if ECU does not only supports CCM requirements (PID $F401 Data B bit 2==1)
				} // end if ( IdIndex == 0x0B )

				else if ( InfSupportCountList[IdIndex] == 0xF813 )
				{
					INF13Supported[EcuIndex] = TRUE;
				}
			} // end if INF supported

			// if INF not supported
			else
			{
				if ( InfSupportCountList[IdIndex] == 0xF80A )
				{
					// Verify that INF $A is supported for MY 2010 and later
					if ( gModelYear >= 2010 /*&&
					     (gUserInput.eComplianceType == US_OBDII || gUserInput.eComplianceType == HD_OBD ||
					      gUserInput.eComplianceType == CNOBD_WITH_IUMPR )*/ )
					{
						Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
						      "ECU %X  INF $F80A (ECUNAME)  not supported! (Required for MY 2010 and later vehicles)\n",
						      GetEcuId ( EcuIndex ) );
						bTestFailed = TRUE;
					}
				}
			} // end if INF notsupported
		} // end for ( IdIndex

		// Check if INF $15 is supported, this is a failure
		if ( IsDIDSupported ( EcuIndex, 0xF815 ) == TRUE )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  Supports INF $F815 (Must not support INF $15)\n",
			      GetEcuId ( EcuIndex ) );
			bTestFailed = TRUE;
		}
	} // end for ( EcuIndex


	//Check support of required INFs

	// Verify that one and only one ECU supports INF $2 (VIN) for US OBD II and HD EOBD
	if ( InfSupportCount[0x02] == 0 )
	{
//		if ( gUserInput.eComplianceType == US_OBDII ||
//		     gUserInput.eComplianceType == HD_OBD   ||
//		     gUserInput.eComplianceType == HD_EOBD )
//		{
//			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//			      "INF $F802 (VIN) not supported by any ECUs\n" );
//			bTestFailed = TRUE;
//		}
//		else
		{
			Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "INF $F802 (VIN) not supported by any ECUs\n" );
		}
	}
	else if ( InfSupportCount[0x02] > 1 )
	{
		for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
		{
			if ( INF2Supported[EcuIndex] == TRUE )
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  INF $F802 (VIN) supported by multiple ECUs\n",
				      GetEcuId ( EcuIndex ) );
			}
		}
		bTestFailed = TRUE;
	}

	// Verify that INF $4 (CALID) is supported by the expected number of ECUs (from user prompt)
	if ( InfSupportCount[0x04] != gUserNumEcus )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "INF $F804 (CALID) not supported by expected number of ECUs ( %d support, %d expected)!\n",
		      InfSupportCount[0x04],
		      gUserNumEcus );
		bTestFailed = TRUE;
	}

	// Verify that INF $6 (CVN) is supported by at least the number of reprogrammable ECUs (from user prompt)
//	if ( InfSupportCount[0x06] < gUserNumEcusReprgm &&
//	     ( gUserInput.eComplianceType == US_OBDII ||
//	       gUserInput.eComplianceType == HD_OBD   ||
//	       gUserInput.eComplianceType == CNOBD_WITH_IUMPR ) )
//	{
//		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//		      "INF $F806 (CVN) supported by less than the expected number of ECUs ( %d support, %d expected)!\n",
//		      InfSupportCount[0x06],
//		      gUserNumEcus );
//		bTestFailed = TRUE;
//	}

	// Verify that INF $8 and INF $B are not both supported
	if ( InfSupportCount[0x08] != 0 && InfSupportCount[0x0B] != 0 /*&&
	     ( gUserInput.eComplianceType == US_OBDII || gUserInput.eComplianceType == HD_OBD ||
	       gUserInput.eComplianceType == EOBD_WITH_IUMPR || gUserInput.eComplianceType == HD_EOBD ||
	       gUserInput.eComplianceType == CNOBD_WITH_IUMPR )*/ )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "Both INF $F808 and INF $F80B are supported!\n" );
		bTestFailed = TRUE;
	}

	if ( InfSupportCount[0x08] == 0 && InfSupportCount[0x0B] == 0 )
	{
		// Verify that INF $8 or INF $B (IPT) is supported by at least one ECU for MY 2007 and later - OBD II
		if ( gModelYear >= 2007 /*&&
		     (gUserInput.eComplianceType == US_OBDII || gUserInput.eComplianceType == HD_OBD) )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "Neither INF $F808 nor INF $F80B is supported! (Support for one required for MY 2007 and later vehicles)\n" );
			bTestFailed = TRUE;
		}
		// Verify that INF $8 or INF $B (IPT) is supported by at least one ECU - EOBD with IUMPR or HD_EOB
		else if ( gUserInput.eComplianceType == EOBD_WITH_IUMPR || gUserInput.eComplianceType == HD_EOBD ||
		          gUserInput.eComplianceType == IOBD_NO_IUMPR   || gUserInput.eComplianceType == HD_IOBD_NO_IUMPR ||
		          gUserInput.eComplianceType == CNOBD_WITH_IUMPR*/ )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "Neither INF $F808 nor INF $F80B is supported!\n" );
			bTestFailed = TRUE;
		}
	}

	// Verify that INF $A is supported for MY 2010 and later
//	if ( gModelYear >= 2010 &&
//	     InfSupportCount[0x0A] != gUserNumEcus &&
//	     ( gUserInput.eComplianceType == US_OBDII ||
//	       gUserInput.eComplianceType == HD_OBD   ||
//	       gUserInput.eComplianceType == CNOBD_WITH_IUMPR ) )
//	{
//		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//		      "INF $F80A (ECU_NAME) is not supported by all ECUs! (Required for MY 2010 and later vehicles)\n" );
//		bTestFailed = TRUE;
//	}

	// Verify that INF $D is supported for MY 2013 and later Heavy Duty vehicles
//	if ( (InfSupportCount[0x0D] == 0 || InfSupportCount[0x0D] > 1) &&
//	     gModelYear >= 2013 &&
//	     gUserInput.eComplianceType == HD_OBD )
//	{
//		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//		      "INF $F80D (ESN) is not supported by one and only one ECU! (Required for MY 2013 and later Heavy Duty vehicles)\n" );
//		bTestFailed = TRUE;
//	}

	// Verify that INF $12 is supported for MY 2014 and later PHEV OBDII vehicles
//	if ( InfSupportCount[0x12] == 0 &&
//	     ( gUserInput.eComplianceType == US_OBDII ||
//	       gUserInput.eComplianceType == CNOBD_WITH_IUMPR ) &&
//	     gModelYear >= 2014 &&
//	     gUserInput.ePwrTrnType == PHEV )
//	{
//		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//		      "INF $F812 (FEOCNTR) is not supported by any ECU! (Required for MY 2014 and later PHEV vehicles)\n" );
//		bTestFailed = TRUE;
//	}

	// Verify that INF $14 is supported for MY 2017 and later OBDII vehicles
//	if ( InfSupportCount[0x14] == 0 &&
//	     gUserInput.eComplianceType == US_OBDII &&
//	     gOBDDieselFlag == FALSE &&
//	     gModelYear >= 2017 )
//	{
//		Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//		      "INF $F814 (EVAP_DIST) is not supported by any ECU! (Required for vehicles that meet the LEV III / Tier 3 EVAP requirements)\n" );
//	}


	// Verify that MY 2019 and later Ligth Duty OBDII vehicles support required INFs
//	if ( gUserInput.eVehicleType == LD &&
//	     gUserInput.eComplianceType == US_OBDII &&
//	     gModelYear >= 2019 )
//	{
//		// Verify that INF $13 is supported for MY 2019 and later Light Duty vehicles
//		if ( InfSupportCount[0x13] == 0 )
//		{
//			if ( gModelYear == 2019 || gModelYear == 2020 )
//			{
//				Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F813 (TG/EFN) is not supported by any ECU! (Required for MY 2019 and later Light Duty vehicles)\n" );
//			}
//			else
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F813 (TG/EFN) is not supported by any ECU! (Required for MY 2019 and later Light Duty vehicles)\n" );
//				bTestFailed = TRUE;
//			}
//		}
//		else if ( InfSupportCount[0x13] > 1  )
//		{
//			if ( gModelYear == 2019 || gModelYear == 2020 )
//			{
//				Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F813 (TG/EFN) supported by more than one ECU!\n" );
//			}
//			else
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F813 (TG/EFN) supported by more than one ECU!\n" );
//				bTestFailed = TRUE;
//			}
//		}
//
//		// Verify that INF $16 is supported for MY 2019 and later Light Duty vehicles
//		if ( InfSupportCount[0x16] == 0 )
//		{
//			if ( gModelYear == 2019 || gModelYear == 2020 )
//			{
//				Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F816 (ERT/IERT) is not supported by any ECU! (Required for MY 2019 and later Light Duty vehicles)\n" );
//			}
//			else
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F816 (ERT/IERT) is not supported by any ECU! (Required for MY 2019 and later Light Duty vehicles)\n" );
//				bTestFailed = TRUE;
//			}
//		}
//
//		// Verify that INF $17 is supported for MY 2019 and later Light Duty vehicles
//		if ( InfSupportCount[0x17] == 0 )
//		{
//			if ( gModelYear == 2019 || gModelYear == 2020 )
//			{
//				Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F817 (DT/FC) is not supported by any ECU! (Required for MY 2019 and later Light Duty vehicles)\n" );
//			}
//			else
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F817 (DT/FC) is not supported by any ECU! (Required for MY 2019 and later Light Duty vehicles)\n" );
//				bTestFailed = TRUE;
//			}
//		}
//
//		// Verify that INF $18 is supported for MY 2019 and later Light Duty vehicles
//		if ( InfSupportCount[0x18] == 0 )
//		{
//			if ( gModelYear == 2019 || gModelYear == 2020 )
//			{
//				Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F818 (PKE/EOE) is not supported by any ECU! (Required for MY 2019 and later Light Duty vehicles)\n" );
//			}
//			else
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F818 (PKE/EOE) is not supported by any ECU! (Required for MY 2019 and later Light Duty vehicles)\n" );
//				bTestFailed = TRUE;
//			}
//		}
//
//		// Verify that INF $19 is supported for MY 2019 and later Light Duty vehicles
//		if ( InfSupportCount[0x19] == 0 )
//		{
//			if ( gModelYear == 2019 || gModelYear == 2020 )
//			{
//				Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F819 (PSA) is not supported by any ECU! (Required for MY 2019 and later Light Duty vehicles)\n" );
//			}
//			else
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F819 (PSA) is not supported by any ECU! (Required for MY 2019 and later Light Duty vehicles)\n" );
//				bTestFailed = TRUE;
//			}
//		}
//
//		// Verify that INF $1A is supported for MY 2019 and later Light Duty PHEV vehicles
//		if ( InfSupportCount[0x1A] == 0 && gUserInput.ePwrTrnType == PHEV )
//		{
//			if ( gModelYear == 2019 || gModelYear == 2020 )
//			{
//				Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F81A (PHEVDD) is not supported by any ECU! (Required for MY 2019 and later Light Duty PHEV vehicles)\n" );
//			}
//			else
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F81A (PHEVDD) is not supported by any ECU! (Required for MY 2019 and later Light Duty PHEV vehicles)\n" );
//				bTestFailed = TRUE;
//			}
//		}
//
//		// Verify that INF $1B is supported for MY 2019 and later Light Duty PHEV vehicles
//		if ( InfSupportCount[0x1B] == 0 && gUserInput.ePwrTrnType == PHEV )
//		{
//			if ( gModelYear == 2019 || gModelYear == 2020 )
//			{
//				Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F81B (CDFC/CIFC) is not supported by any ECU! (Required for MY 2019 and later Light Duty PHEV vehicles)\n" );
//			}
//			else
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F81B (CDFC/CIFC) is not supported by any ECU! (Required for MY 2019 and later Light Duty PHEV vehicles)\n" );
//				bTestFailed = TRUE;
//			}
//		}
//
//		// Verify that INF $1C is supported for MY 2019 and later Light Duty PHEV vehicles
//		if ( InfSupportCount[0x1C] == 0 && gUserInput.ePwrTrnType == PHEV )
//		{
//			if ( gModelYear == 2019 || gModelYear == 2020 )
//			{
//				Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F81C (CDEOGE/CDERGE/GE) is not supported by any ECU! (Required for MY 2019 and later Light Duty PHEV vehicles)\n" );
//			}
//			else
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F81C (CDEOGE/CDERGE/GE) is not supported by any ECU! (Required for MY 2019 and later Light Duty PHEV vehicles)\n" );
//				bTestFailed = TRUE;
//			}
//		}
//	}


//	if ( gUserInput.eVehicleType == LD &&
//	     ( gUserInput.eComplianceType == EOBD_WITH_IUMPR ||
//	       gUserInput.eComplianceType == EOBD_NO_IUMPR ) &&
//	     gModelYear >= 2020 )
//	{
//		// Verify that INF $17 is supported for MY 2020 and later Light Duty vehicles
//		if ( InfSupportCount[0x17] == 0 )
//		{
//			if ( gModelYear == 2020 )
//			{
//				Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F817 (DT/FC) is not supported by any ECU! (Required for MY 2020 and later Light Duty vehicles)\n" );
//			}
//			else
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F817 (DT/FC) is not supported by any ECU! (Required for MY 2020 and later Light Duty vehicles)\n" );
//				bTestFailed = TRUE;
//			}
//		}
//
//		// Verify that INF $1A is supported for MY 2019 and later Light Duty PHEV vehicles
//		if ( InfSupportCount[0x1A] == 0 && gUserInput.ePwrTrnType == PHEV )
//		{
//			if ( gModelYear == 2020 )
//			{
//				Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F81A (PHEVDD) is not supported by any ECU! (Required for MY 2020 and later Light Duty PHEV vehicles)\n" );
//			}
//			else
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F81A (PHEVDD) is not supported by any ECU! (Required for MY 2020 and later Light Duty PHEV vehicles)\n" );
//				bTestFailed = TRUE;
//			}
//		}
//
//		// Verify that INF $1B is supported for MY 2019 and later Light Duty PHEV vehicles
//		if ( InfSupportCount[0x1B] == 0 && gUserInput.ePwrTrnType == PHEV )
//		{
//			if ( gModelYear == 2020 )
//			{
//				Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F81B (CDFC/CIFC) is not supported by any ECU! (Required for MY 2020 and later Light Duty PHEV vehicles)\n" );
//			}
//			else
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F81B (CDFC/CIFC) is not supported by any ECU! (Required for MY 2020 and later Light Duty PHEV vehicles)\n" );
//				bTestFailed = TRUE;
//			}
//		}
//
//		// Verify that INF $1C is supported for MY 2019 and later Light Duty PHEV vehicles
//		if ( InfSupportCount[0x1C] == 0 && gUserInput.ePwrTrnType == PHEV )
//		{
//			if ( gModelYear == 2020 )
//			{
//				Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F81C (CDEOGE/CDERGE/GE) is not supported by any ECU! (Required for MY 2020 and later Light Duty PHEV vehicles)\n" );
//			}
//			else
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "INF $F81C (CDEOGE/CDERGE/GE) is not supported by any ECU! (Required for MY 2020 and later Light Duty PHEV vehicles)\n" );
//				bTestFailed = TRUE;
//			}
//		}
//	}


	// Verify that INFs $F841-F85B and $F861-F878 are supported for MY 2024 and later Heavy Duty vehicles
	if ( gUserInput.eVehicleType == HD &&
	     gModelYear >= 2022 )
	{
		for ( IdIndex = 0x41; IdIndex <= 0x78; IdIndex++ )
		{
			if ( !(IdIndex >= 0x4A && IdIndex <= 0x4F) &&
			     !(IdIndex >= 0x5C && IdIndex <= 0x60) &&
			     InfSupportCount[IdIndex] == 0 )
			{
				if ( InfSupportCount[IdIndex] == 0 )
				{
					if ( gModelYear == 2022 || gModelYear == 2023 )
					{
						Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
						      "INF $F8%02X is not supported by any ECU! (Required for MY 2022 and later Heavy Duty vehicles)\n",
						      IdIndex );
					}
					else
					{
						Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
						      "INF $F8%02X is not supported by any ECU! (Required for MY 2022 and later Heavy Duty vehicles)\n",
						      IdIndex );
						bTestFailed = TRUE;
					}
				}
			}
		}
	}


	// For each INF group verify that all INF data is valid
	// Request user supplied expected OBD ECU responses
	for ( IdIndex = 0xF801; IdIndex < 0xF900; IdIndex++ )
	{
		// If INF is supported by any ECU, request it
		if ( IsDIDSupported ( -1, IdIndex ) == TRUE  )
		{
			SidReq.SID    = 0x22;
			SidReq.NumIds = 1;
			SidReq.DID[0] = IdIndex;

			if ( SidRequest ( &SidReq, SID_REQ_NORMAL ) == FAIL )
			{
				bTestFailed = TRUE;
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "INF $%04X request\n",
				      IdIndex );

				if ( IdIndex != INF_TYPE_CALID ) // IF not CALID, Don't check
				{
					continue;
				}
			}

			for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
			{
				// If INF is not supported, skip to next ECU
				if ( IsDIDSupported ( EcuIndex, IdIndex ) == FALSE )
				{
					continue;
				}

				// INF's 1, 3, 5 & 7 should not be supported by any ECU
				if ( IdIndex ==  0xF801 || IdIndex ==  0xF803 || IdIndex ==  0xF805 || IdIndex ==  0xF807 ||
				     IdIndex ==  0xF809 || IdIndex ==  0xF80C || IdIndex ==  0xF80E )
				{
					Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "ECU %X  INF $%04X is supported (Should not be supported by UDS vehicles)\n",
					      GetEcuId ( EcuIndex ),
					      IdIndex );
					continue;
				}


				// Check the data to see if it is valid
				pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

				if ( gOBDResponse[EcuIndex].InfSize == 0 )
				{
					bTestFailed = TRUE;
					Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "ECU %X  No INF $%04X data\n",
					      GetEcuId ( EcuIndex ),
					      IdIndex );
					continue;
				}

				// Check various INF values for validity
				switch ( IdIndex )
				{
					case INF_TYPE_VIN_COUNT:
					{
					}
					break;

					case INF_TYPE_VIN:
					{
						// Isolated VIN verification logic
						if ( VerifyVINData ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}

						// Check for INF $2 (VIN) support from multiple controllers
						if ( ++Inf2NumResponses != 0x01 /*&&
						     (gUserInput.eComplianceType == US_OBDII || gUserInput.eComplianceType == HD_OBD) )
						{
							bTestFailed = TRUE;
							Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
							      "INF $F802 (VIN) response from multiple controllers!\n" );
						}
						else if ( Inf2NumResponses != 0x01 &&
						          ( gUserInput.eComplianceType == EOBD_WITH_IUMPR || gUserInput.eComplianceType == EOBD_NO_IUMPR ||
						            gUserInput.eComplianceType == IOBD_NO_IUMPR   || gUserInput.eComplianceType == OBDBr_NO_IUMPR ||
						            gUserInput.eComplianceType == CNOBD_WITH_IUMPR)*/ )
						{
							Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
							      "INF $F802 (VIN) response from multiple controllers!\n" );
						}

					}
					break;

					case INF_TYPE_CALID_COUNT:
					{
					}
					break;

					case INF_TYPE_CALID:
					{
						fInf4Responded[EcuIndex] = TRUE;

						if ( VerifyCALIDData ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_CVN_COUNT:
					{
					}
					break;

					case INF_TYPE_CVN:
					{
						Inf6NumResponses++;

						if ( VerifyCVNData ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_IPT_COUNT:
					{
					}
					break;

					case INF_TYPE_IPT_GAS:
					{
						Inf8NumResponses++;

						if ( VerifyINF8Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}

						IGNCNTR = (gOBDResponse[EcuIndex].Inf[5] * 256) + gOBDResponse[EcuIndex].Inf[6];
					}
					break;

					case INF_TYPE_ECUNAME_COUNT:
					{
					}
					break;

					case INF_TYPE_ECUNAME:
					{
						fInfAResponded[EcuIndex] = TRUE;

						if ( VerifyECUNameData ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}

						// Check for duplicate ECU Names
						for ( EcuLoopIndex = 0; EcuLoopIndex < EcuIndex; EcuLoopIndex++ )
						{
							if ( gOBDResponse[EcuIndex].InfSize != 0 &&
							     gOBDResponse[EcuLoopIndex].InfSize != 0 &&
							     _memicmp ( &gOBDResponse[EcuIndex].Inf[0],
							                &gOBDResponse[EcuLoopIndex].Inf[0],
							                gOBDResponse[EcuLoopIndex].InfSize ) == 0 )
							{
								bTestFailed = TRUE;
								Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
								      "ECU %X and ECU %X  INF $F80A, Duplicate ECU names\n",
								      GetEcuId ( EcuIndex ),
								      GetEcuId ( EcuLoopIndex ) );
							}
						}
					}
					break;

					case INF_TYPE_IPT_DIESEL:
					{
						InfBNumResponses++;

						if ( VerifyINFBData ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}

						IGNCNTR = (gOBDResponse[EcuIndex].Inf[5] * 256) + gOBDResponse[EcuIndex].Inf[6];
					}
					break;

					case INF_TYPE_ESN_COUNT:  // INFOTYPE $0C Engine Serial Number Message Count
					{
					}
					break;

					case INF_TYPE_ESN:  // INFOTYPE $0D Engine Serial Number
					{
						if ( VerifyInfD_FData ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_EROTAN_COUNT:  // INFOTYPE $0E Exhaust Regulation or Type Approval Number Message Count
					{
					}
					break;

					case INF_TYPE_EROTAN:  // INFOTYPE $0F Exhaust Regulation or Type Approval
					{
						if ( VerifyInfD_FData ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_PROTID:  // INFOTYPE $10 Protocol Identification
					{
						if ( VerifyInf10Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_FEOCNTR:  // INFOTYPE $12 Fueled Engine Operation Ignition Cycle Counter
					{
						if ( VerifyInf12Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_TG_EFN:  // INFOTYPE $13 Test Grooup/Engine Family Number
					{
						if ( VerifyInf13Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_EVAP_DIST:  // INFOTYPE $14 Distance Traveled Since Evap Monitoring Decision
					{
						if ( VerifyInf14Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_ER_IT:   // INFOTYPE $16 Engine Run/Idle Time
					{
						if ( VerifyInf16Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_DT_FC:   // INFOTYPE $17 Distance/Fuel Used
					{
						if ( VerifyInf17Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_PKE_EOE:  // INFOTYPE $18 PKE/EOE
					{
						if ( VerifyInf18Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_PSA:     // INFOTYPE $19 PSA
					{
						if ( VerifyInf19Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_PHEVDD:  // INFOTYPE $1A PHEV Distance Data
					{
						if ( VerifyInf1AData ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_PHEVFD:  // INFOTYPE $1B PHEV Fuel Data
					{
						if ( VerifyInf1BData ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_PHEVGD:  // INFOTYPE $1C PHEV Grid Data
					{
						if ( VerifyInf1CData ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_AAF1OCCVD:  // INFOTYPE $1D Active Aerodynamic Features 1 Off Cycle Credit Vehicle Data
					{
						if ( VerifyInf1DData ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_AAF2OCCVD:  // INFOTYPE $1E Active Aerodynamic Features 2 Off Cycle Credit Vehicle Data
					{
						if ( VerifyInf1EData ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_AAF3OCCVD:  // INFOTYPE $1F Active Aerodynamic Features 3 Off Cycle Credit Vehicle Data
					{
						if ( VerifyInf1FData ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_DSOMOCCVD:  // INFOTYPE $21 Driver Selectable Operating Modes Off Cycle Credit Vehicle Data
					{
						if ( VerifyInf21Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_RTSSCOCCVD:  // INFOTYPE $22 Run Time for Stop-Start and Coasting Off Cycle Credit Vehicle Data
					{
						if ( VerifyInf22Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_DCTOCCVD:   // INFOTYPE $23 Driver Coaching Technology Off Cycle Credit Vehicle Data
					{
						if ( VerifyInf23Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_APTWUFOCCVD:  // INFOTYPE $24 Active Powertrain Warm Up Features Off Cycle Credit Vehicle Data
					{
						if ( VerifyInf24Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_OCCT1VD:  // INFOTYPE $25 Off Cycle Credit Technology 1 Vehicle Data
					case INF_TYPE_OCCT2VD:  // INFOTYPE $26 Off Cycle Credit Technology 2 Vehicle Data
					case INF_TYPE_OCCT3VD:  // INFOTYPE $27 Off Cycle Credit Technology 3 Vehicle Data
					case INF_TYPE_OCCT4VD:  // INFOTYPE $28 Off Cycle Credit Technology 4 Vehicle Data
					case INF_TYPE_OCCT5VD:  // INFOTYPE $29 Off Cycle Credit Technology 5 Vehicle Data
					{
						if ( VerifyInf25_29Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_HDGHGA100HD1:  // INFOTYPE $41 HD GHG Active 100 Hour Data #1
					case INF_TYPE_HDGHGS100HD1:  // INFOTYPE $43 HD GHG Stored 100 Hour Data #1
					case INF_TYPE_HDGHGLD1:      // INFOTYPE $45 HD GHG Lifetime Data #1
					{
						if ( VerifyInf41_43_45Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_HDGHGA100HD2:  // INFOTYPE $42 HD GHG Active 100 Hour Data #2
					case INF_TYPE_HDGHGS100HD2:  // INFOTYPE $44 HD GHG Stored 100 Hour Data #2
					case INF_TYPE_HDGHGLD2:      // INFOTYPE $46 HD GHG Lifetime Data #2
					{
						if ( VerifyInf42_44_46Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_HDGHGPSAD:     // INFOTYPE $47 HD GHG PSA Data
					{
						if ( VerifyInf47Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_HDGHGPHEVVD:   // INFOTYPE $48 HD GHG Plug-in Hybrid Vehicle Distance Data
					{
						if ( VerifyInf48Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_HDGHGPHEVFD:   // INFOTYPE $49 HD GHG Plug-in Hybrid Vehicle Fuel Data
					{
						if ( VerifyInf49Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_HDGHGPHEVGD:   // INFOTYPE $50 HD GHG Plug-in Hybrid Vehicle Grid Data
					{
						if ( VerifyInf50Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_HDGHGATSS:     // INFOTYPE $51 HD GHG Active Technology: Stop-Start
					{
						if ( VerifyInf51Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_HDGHGATAES:    // INFOTYPE $52 HD GHG Active Technology: Automatic Engine Shutdown
					{
						if ( VerifyInf52Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_HDGHGATWHR:    // INFOTYPE $53 HD GHG Active Technology: Waste Heat Recovery
					{
						if ( VerifyInf53Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_HDGHGAT1AT2:   // INFOTYPE $54 HD GHG Active Technology #1 and Active Technology #2
					case INF_TYPE_HDGHGAT3AT4:   // INFOTYPE $55 HD GHG Active Technology #3 and Active Technology #4
					case INF_TYPE_HDGHGAT5AT6:   // INFOTYPE $56 HD GHG Active Technology #5 and Active Technology #6
					case INF_TYPE_HDGHGAT7AT8:   // INFOTYPE $57 HD GHG Active Technology #7 and Active Technology #8
					case INF_TYPE_HDGHGAT9AT10:  // INFOTYPE $58 HD GHG Active Technology #9 and Active Technology #10
					case INF_TYPE_HDGHGAT11AT12: // INFOTYPE $59 HD GHG Active Technology #11 and Active Technology #12
					case INF_TYPE_HDGHGAT13AT14: // INFOTYPE $5A HD GHG Active Technology #13 and Active Technology #14
					case INF_TYPE_HDGHGAT15AT16: // INFOTYPE $5B HD GHG Active Technology #15 and Active Technology #16
					{
						if ( VerifyInf54_5BData ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_NOx100HMEOA:   // INFOTYPE $61 NOx Binning: 100 Hour NOx Mass - Engine Out Bins (Active)
					case INF_TYPE_NOx100HMTA:    // INFOTYPE $62 NOx Binning: 100 Hour NOx Mass - Tailpipe Bins (Active)
					case INF_TYPE_NOx100HMEOS:   // INFOTYPE $67 NOx Binning: 100 Hour NOx Mass - Engine Out Bins (Stored)
					case INF_TYPE_NOx100HMTS:    // INFOTYPE $68 NOx Binning: 100 Hour NOx Mass - Tailpipe Bins (Stored)
					case INF_TYPE_NOxVMEOL:      // INFOTYPE $6D NOx Binning Valid NOx: NOx Mass - Engine Out Bins (Lifetime)
					case INF_TYPE_NOxVMTL:       // INFOTYPE $6E NOx Binning Valid NOx: NOx Mass - Tailpipe Bins (Lifetime)
					{
						if ( VerifyInf61_62_67_68_6D_6EData ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_NOx100HEOEA:   // INFOTYPE $63 NOx Binning: 100 Hour Engine Output Energy Bins (Active)
					case INF_TYPE_NOx100HEOES:   // INFOTYPE $69 NOx Binning: 100 Hour Engine Output Energy Bins (Stored)
					case INF_TYPE_NOxVEOEL:      // INFOTYPE $6F NOx Binning Valid NOx: Engine Output Energy Bins (Lifetime)
					case INF_TYPE_NOxEAEOEL:     // INFOTYPE $73 NOx Binning Engine Activity: Engine Output Energy Bins (Lifetime)
					{
						if ( VerifyInf63_69_6F_73Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_NOx100HDTA:    // INFOTYPE $64 NOx Binning: 100 Hour Distance traveled Bins (Active)
					case INF_TYPE_NOx100HDTS:    // INFOTYPE $6A NOx Binning: 100 Hour Distance traveled Bins (Stored)
					case INF_TYPE_NOxVDTL:       // INFOTYPE $70 NOx Binning Valid NOx: Distance traveled Bins (Lifetime)
					case INF_TYPE_NOxEADTL:      // INFOTYPE $74 NOx Binning Engine Activity: Distance Traveled Bins (Lifetime)
					{
						if ( VerifyInf64_6A_70_74Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_NOx100HERTA:   // INFOTYPE $65 NOx Binning: 100 Hour Engine Run Time Bins (Active)
					case INF_TYPE_NOx100HERTS:   // INFOTYPE $6B NOx Binning: 100 Hour Engine Run Time Bins (Stored)
					case INF_TYPE_NOxVERTL:      // INFOTYPE $71 NOx Binning Valid NOx: Engine Run Time Bins (Lifetime)
					case INF_TYPE_NOxEAERTL:     // INFOTYPE $75 NOx Binning Engine Activity: Engine Run Time Bins (Lifetime)
					{
						if ( VerifyInf65_6B_71_75Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_NOx100HVFCA:   // INFOTYPE $66 NOx Binning: 100 Hour Vehicle Fuel Consumption Bins (Active)
					case INF_TYPE_NOx100HVFCS:   // INFOTYPE $6C NOx Binning: 100 Hour Vehicle Fuel Consumption Bins (Stored)
					case INF_TYPE_NOxVVFCL:      // INFOTYPE $72 NOx Binning Valid NOx: Vehicle Fuel Consumption Bins (Lifetime)
					case INF_TYPE_NOxEAVFCL:     // INFOTYPE $76 NOx Binning Engine Activity: Vehicle Fuel Consumption Bins (Lifetime)
					{
						if ( VerifyInf66_6C_72_76Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_NOxPMRVD:      // INFOTYPE $77 NOX/PM Regeneration Vehicle Data
					{
						if ( VerifyInf77Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_ER:            // INFOTYPE $78 Engine Ratings
					{
						if ( VerifyInf78Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					default:
					{
						// Non-OBD INF type
					}
					break;
				} // end switch ( index )


				// Verify that every ECU responded to INF $04 request
				if ( IdIndex == 0xF804 && fInf4Responded[EcuIndex] == FALSE )
				{
					Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "ECU %X  Did not report INF $F804 (CALID)!\n",
					      GetEcuId ( EcuIndex ) );
					bTestFailed = TRUE;
				}

				// Verify that INF $A is supported for MY 2010 and later
//				if ( IdIndex == 0xF80A &&
//				     gModelYear >= 2010 &&
//				     fInfAResponded[EcuIndex] == FALSE &&
//				     ( gUserInput.eComplianceType == US_OBDII || gUserInput.eComplianceType == HD_OBD ||
//				       gUserInput.eComplianceType == CNOBD_WITH_IUMPR ) )
//				{
//					Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//					      "ECU %X  Did not report INF $F80A (ECUNAME)! (Required for MY 2010 and later vehicles)\n",
//					       GetEcuId ( EcuIndex ) );
//					bTestFailed = TRUE;
//				}

				// If INFs higher than the max are supported, this is a failure
				if ( IdIndex > 0xF878 )
				{
					Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "ECU %X  Supports INF $%04X (Must not support INF higher than $F878)\n",
					      GetEcuId ( EcuIndex ),
					      IdIndex );
					bTestFailed = TRUE;
				}

			} // end for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )

		} // end if ( IsDIDSupported (-1, IdIndex) == TRUE )

	} // end for ( IdIndex = 0xF801; IdIndex < 0xF900; IdIndex++ )


	// Verify that INF $2 is supported by the correct number of ECUs
	if ( Inf2NumResponses == 0 )
	{
//		if ( gUserInput.eComplianceType == US_OBDII || gUserInput.eComplianceType == HD_OBD )
//		{
//			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//			      "INF $F802 (VIN) must be reported by one ECU\n" );
//			bTestFailed = TRUE;
//		}
//		else
		{
			Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "INF $F802 (VIN) not reported by any ECUs\n" );
		}
	}

	// Verify that INF $6 is supported by the correct number of ECUs
	if ( Inf6NumResponses > gUserNumEcus /*&&
//	     (gUserInput.eComplianceType == US_OBDII ||
//	      gUserInput.eComplianceType == HD_OBD ||
//	      gUserInput.eComplianceType == CNOBD_WITH_IUMPR)*/ )
	{
		Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "INF $F806 (CVN) not reported by expected number of ECUs\n" );
	}

	if ( gOBDDieselFlag == FALSE && Inf8NumResponses == 0 )
	{
		if ( gModelYear >= 2005 /*&&
		     (gUserInput.eComplianceType == US_OBDII || gUserInput.eComplianceType == HD_OBD) )
		{
			// If M/Y 2005 or greater and OBD II, INFO type 8 must be supported
			// If not supported then OEM will present Infotype phase in plan
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "No INF $F808 (IPT) response (Required for Spark Ignition vehicles MY 2005 and later)\n" );
			bTestFailed = TRUE;
		}
		else if ( gUserInput.eComplianceType == EOBD_WITH_IUMPR || gUserInput.eComplianceType == HD_EOBD ||
		          gUserInput.eComplianceType == IOBD_NO_IUMPR   || gUserInput.eComplianceType == HD_IOBD_NO_IUMPR ||
		          gUserInput.eComplianceType == OBDBr_NO_IUMPR  || gUserInput.eComplianceType == CNOBD_WITH_IUMPR*/ )
		{
			// If EOBD with IUMPR, INFO type 8 must be supported */
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "No INF $F808 (IPT) response (Required for Spark Ignition vehicles)\n" );
			bTestFailed = TRUE;
		}
	}

	if ( gOBDDieselFlag == TRUE && InfBNumResponses == 0 )
	{
		if ( gModelYear >= 2010 /*&&
		     (gUserInput.eComplianceType == US_OBDII || gUserInput.eComplianceType == HD_OBD) )
		{
			// If M/Y 2010 or greater, INFO type B must be supported/
			// If not supported then OEM will present Infotype phase in plan/
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "No INF $F80B (IPT) response (Required for Compression Ignition vehicles MY 2010 and later)\n" );
			bTestFailed = TRUE;
		}
		else if ( gUserInput.eComplianceType == EOBD_WITH_IUMPR || gUserInput.eComplianceType == HD_EOBD ||
		          gUserInput.eComplianceType == IOBD_NO_IUMPR   || gUserInput.eComplianceType == HD_IOBD_NO_IUMPR ||
		          gUserInput.eComplianceType == OBDBr_NO_IUMPR  || gUserInput.eComplianceType == CNOBD_WITH_IUMPR*/ )
		{
			// If EOBD with IUMPR, INFO type 8 must be supported
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "No INF $F80B (IPT) response (Required for Compression Ignition vehicles)\n" );
			bTestFailed = TRUE;
		}
	}


//	if ( VerifyGroupVehicleInformationSupport () == FAIL )
	if ( RequestGroupDIDSupportData ( INFREQUEST ) == FAIL )
	{
		bTestFailed = TRUE;
	}


	// If there where any errors in the responses, fail
	if ( bTestFailed == TRUE || ulInit_FailureCount != GetFailureCount () )
	{
		// There could have been early/late responses that weren't treated as FAIL
		return FAIL;
	}
	else
	{
		return PASS;
	}

}


/*
*******************************************************************************
**
**  Function:   VerifyVINData
**
**  Purpose:    Purpose of this function is to verify the VIN format
**              for correct format.  In the event the format fails
**              defined criteria, an error is returned.
**
*******************************************************************************
*/
STATUS VerifyVINData ( unsigned long EcuIndex )
{
	unsigned long VinIndex;
	INF          *pINF;
	STATUS        RetCode = PASS;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F802 (VIN) data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];
	// Copy the VIN into the global array
	memcpy ( &gVIN[0], &pINF[0].Data[0], 17 );

	/// Check that there are no pad bytes included in message
	if ( pINF[0].Data[20] != 0x00 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F802 format error! (Must be 17 chars and no pad bytes)\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// INF $2 NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F802 NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}

	// Print VIN string to log file
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F802  VIN = %s\n",
	      GetEcuId ( EcuIndex ),
	      gVIN );

	// Check all VIN characters for validity
	for ( VinIndex = 0; VinIndex < 17; VinIndex++ )
	{
		if ( gVIN[VinIndex] <  '0' || gVIN[VinIndex] >  'Z' ||
		     gVIN[VinIndex] == 'I' || gVIN[VinIndex] == 'O' ||
		     gVIN[VinIndex] == 'Q' || gVIN[VinIndex] == ':' ||
		     gVIN[VinIndex] == ';' || gVIN[VinIndex] == '<' ||
		     gVIN[VinIndex] == '>' || gVIN[VinIndex] == '=' ||
		     gVIN[VinIndex] == '?' || gVIN[VinIndex] == '@' )
		{
			break;
		}
	}

	if ( VinIndex != 17 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  Invalid VIN information\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}


	// Check that VIN model year character matches user input model year
	if ( ( ( gVIN[9] == '1' && gVINModelYear != 2001 ) ||
	       ( gVIN[9] == '2' && gVINModelYear != 2002 ) ||
	       ( gVIN[9] == '3' && gVINModelYear != 2003 ) ||
	       ( gVIN[9] == '4' && gVINModelYear != 2004 ) ||
	       ( gVIN[9] == '5' && gVINModelYear != 2005 ) ||
	       ( gVIN[9] == '6' && gVINModelYear != 2006 ) ||
	       ( gVIN[9] == '7' && gVINModelYear != 2007 ) ||
	       ( gVIN[9] == '8' && gVINModelYear != 2008 ) ||
	       ( gVIN[9] == '9' && gVINModelYear != 2009 ) ||
	       ( gVIN[9] == 'A' && gVINModelYear != 2010 ) ||
	       ( gVIN[9] == 'B' && gVINModelYear != 1981 && gVINModelYear != 2011 ) ||
	       ( gVIN[9] == 'C' && gVINModelYear != 1982 && gVINModelYear != 2012 ) ||
	       ( gVIN[9] == 'D' && gVINModelYear != 1983 && gVINModelYear != 2013 ) ||
	       ( gVIN[9] == 'E' && gVINModelYear != 1984 && gVINModelYear != 2014 ) ||
	       ( gVIN[9] == 'F' && gVINModelYear != 1985 && gVINModelYear != 2015 ) ||
	       ( gVIN[9] == 'G' && gVINModelYear != 1986 && gVINModelYear != 2016 ) ||
	       ( gVIN[9] == 'H' && gVINModelYear != 1987 && gVINModelYear != 2017 ) ||
	       ( gVIN[9] == 'J' && gVINModelYear != 1988 && gVINModelYear != 2018 ) ||
	       ( gVIN[9] == 'K' && gVINModelYear != 1989 && gVINModelYear != 2019 ) ||
	       ( gVIN[9] == 'L' && gVINModelYear != 1990 && gVINModelYear != 2020 ) ||
	       ( gVIN[9] == 'M' && gVINModelYear != 1991 && gVINModelYear != 2021 ) ||
	       ( gVIN[9] == 'N' && gVINModelYear != 1992 && gVINModelYear != 2022 ) ||
	       ( gVIN[9] == 'P' && gVINModelYear != 1993 && gVINModelYear != 2023 ) ||
	       ( gVIN[9] == 'R' && gVINModelYear != 1994 && gVINModelYear != 2024 ) ||
	       ( gVIN[9] == 'S' && gVINModelYear != 1995 && gVINModelYear != 2025 ) ||
	       ( gVIN[9] == 'T' && gVINModelYear != 1996 ) ||
	       ( gVIN[9] == 'V' && gVINModelYear != 1997 ) ||
	       ( gVIN[9] == 'W' && gVINModelYear != 1998 ) ||
	       ( gVIN[9] == 'X' && gVINModelYear != 1999 ) ||
	       ( gVIN[9] == 'Y' && gVINModelYear != 2000 ) ) /*&&
	     (gUserInput.eComplianceType == US_OBDII || gUserInput.eComplianceType == HD_OBD)*/ )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  VIN year character (%c) does not match user entry (%d)\n",
		      GetEcuId ( EcuIndex ),
		      gVIN[9],
		      gVINModelYear );
		RetCode = FAIL;
	}

	else if ( gVIN[9] == '0' || gVIN[9] == 'I' ||
	          gVIN[9] == 'O' || gVIN[9] == 'Q' ||
	          gVIN[9] == 'U' || gVIN[9] == 'Z' )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  Invalid VIN year character (%c) not currently defined)\n",
		      GetEcuId ( EcuIndex ),
		      gVIN[9] );
		RetCode = FAIL;
	}

	else if ( gVINModelYear < 1981 || gVINModelYear > 2025 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  Model Year %d does not currently have a VIN character defined)\n",
		      GetEcuId ( EcuIndex ),
		      gVINModelYear );
		RetCode = FAIL;
	}

	gEcuData[EcuIndex].VIN_Valid = TRUE;
	strncpy_s ( gEcuData[EcuIndex].VIN_String, VIN_BUFFER_SIZE, gVIN, VIN_BUFFER_SIZE );

	return RetCode;
}


/*
*****************************************************************************
**
**  Function:  VerifyCALIDData
**
**  Purpose:   Purpose of this function is to verify the CALID format
**             for correct format.  In the event the format fails
**             defined criteria, an error is returned.
**
*****************************************************************************
*/
STATUS VerifyCALIDData ( unsigned long EcuIndex )
{
	unsigned long  Inf4NumItems;
	unsigned long  ItemIndex;
	unsigned long  ByteIndex;
	char           buffer[CALID_BUFFER_SIZE];
//	unsigned int   CALID_String_Size;
	INF          *pINF;
	STATUS         RetCode = PASS;
	BOOL bCalidEnd = FALSE;


	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F804 (CALID) data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	Inf4NumItems = pINF->NumItems;

	// INF INF4 NODI must equal number of CALIDs
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( (pINF->NumItems) * 16 != (gOBDResponse[EcuIndex].InfSize - 3) )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F804 NODI = %d mis-matched with Data Size\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		return FAIL;
	}

//	CALID_String_Size = Inf4NumItems * (16 + END_OF_TEST_FORMATTING);
//	// BEGIN TEST LOGGING
//	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//	      "ECU %X  Inf4NumItems = %d    CALID_String_Size = %d\n",
//	      GetEcuId ( EcuIndex ),
//	      Inf4NumItems,
//	      CALID_String_Size );
//	// END TEST LOGGING
//	// allocate memory for copying end-of-test stats
//	if ( gEcuData[EcuIndex].CALID_String != NULL && gEcuData[EcuIndex].CALID_Count != Inf4NumItems )
//	{
//		gEcuData[EcuIndex].CALID_String = realloc ( gEcuData[EcuIndex].CALID_String, CALID_String_Size );
//	}
//	else
//	{
//		gEcuData[EcuIndex].CALID_String = (char *) malloc ( CALID_String_Size );
//	}
//	if ( gEcuData[EcuIndex].CALID_String == NULL )
//	{
//		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//		      "Unable to allocate memory for CALID string\n" );
//		return FAIL;
//	}
//	memset ( gEcuData[EcuIndex].CALID_String, 0x00, CALID_String_Size );
//
//	gEcuData[EcuIndex].CALID_Count = Inf4NumItems;

	for ( ItemIndex = 0; ItemIndex < Inf4NumItems; ItemIndex++ )
	{
		bCalidEnd = FALSE;
		memcpy ( buffer, &(pINF->Data[ItemIndex * 16]), 16 );


		buffer[16] = 0;
		// if there is only one CALID, don't number it
		if ( Inf4NumItems == 1 )
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F804  CALID   = %s\n",
			      GetEcuId ( EcuIndex ),
			      buffer );
		}
		// else, there are more than one CALID, number them
		else
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F804  CALID%02d = %s\n",
			      GetEcuId ( EcuIndex ),
			      ItemIndex+1,
			      buffer );
		}


		for ( ByteIndex = 0; ByteIndex < 16; ByteIndex++ )
		{
			if ( bCalidEnd == FALSE )
			{
				if ( buffer[ByteIndex] < 0x20 || buffer[ByteIndex] > 0x7E )
				{
					if ( buffer[ByteIndex] != 0x00 )
					{
						Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
						      "ECU %X  CALID contains one or more invalid characters\n",
						      GetEcuId ( EcuIndex ) );
						RetCode = FAIL;
						buffer[ByteIndex] = 0;
						break;
					}
					else if ( ByteIndex == 0 )
					{
						Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
						      "ECU %X  CALID can not start with pad byte\n",
						      GetEcuId ( EcuIndex ) );
						RetCode = FAIL;
						break;
					}
					else
					{
						bCalidEnd = TRUE;
					}
				}
			}
			else
			{
				if ( buffer[ByteIndex] != 0 )
				{
					Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "ECU %X  CALID not zero padded on right\n",
					      GetEcuId ( EcuIndex ) );
					RetCode = FAIL;
					buffer[ByteIndex] = 0;
					break;
				}
			}
		} // end for (ByteIndex...


//		// BEGIN TEST LOGGING
//		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//		      "ECU %X  BEFORE    CALID_String Length = %d    Remaining Buffer = %d\n",
//		      GetEcuId ( EcuIndex ),
//		      strlen ( gEcuData[EcuIndex].CALID_String ),
//		      CALID_String_Size - strlen ( gEcuData[EcuIndex].CALID_String ) );
//		// END TEST LOGGING
//		strcat_s ( gEcuData[EcuIndex].CALID_String, CALID_String_Size - strlen ( gEcuData[EcuIndex].CALID_String ), buffer );
//		// BEGIN TEST LOGGING
//		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//		      "ECU %X  AFTER     CALID_String Length = %d    Remaining Buffer = %d\n",
//		      GetEcuId ( EcuIndex ),
//		      strlen ( gEcuData[EcuIndex].CALID_String ),
//		      CALID_String_Size - strlen ( gEcuData[EcuIndex].CALID_String ) );
//		// END TEST LOGGING
//		if ( ItemIndex != Inf4NumItems-1 )
//		{
//			strcat_s ( gEcuData[EcuIndex].CALID_String, CALID_String_Size - strlen ( gEcuData[EcuIndex].CALID_String ), "\n                          " );
//		}

	}


	// response size should be 1 for MY 2009
//	if ( gModelYear >= 2009 &&
//	     pINF[0].NumItems != 1 )
//	{
//		Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//		      "ECU %X  INF $F804 (CALID) NumItems = %d (should be 1 for MY 2009 and later, unless EO approved)\n",
//		      GetEcuId ( EcuIndex ),
//		      pINF[0].NumItems );
//	}

	return RetCode;
}


/*
*****************************************************************************
**
**  Function:  VerifyCVNData
**
**  Purpose:   Purpose of this function is to verify the CVN format
**             for correct format.  In the event the format fails
**             defined criteria, an error is returned.
**
*****************************************************************************
*/
STATUS VerifyCVNData ( unsigned long EcuIndex )
{
	unsigned long  Inf6NumItems;
	unsigned long  ItemIndex;
	unsigned long  ByteIndex;
	char           buffer[CVN_BUFFER_SIZE];
//	unsigned int   CVN_String_Size;
	INF           *pINF;


	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F806 (CVN) data\n",
		      GetEcuId ( EcuIndex ) );
		return PASS;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	Inf6NumItems = pINF->NumItems;

	// INF INF6 NODI must equal number of CVNs
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( (pINF->NumItems) * 4 != (gOBDResponse[EcuIndex].InfSize - 3) )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F806 NODI = %d mis-matched with Data Size\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		return FAIL;
	}


//	// allocate memory for copying end-of-test stats
//	CVN_String_Size = Inf6NumItems * (12 + END_OF_TEST_FORMATTING);
//	if ( gEcuData[EcuIndex].CVN_String != NULL && gEcuData[EcuIndex].CVN_Count != Inf6NumItems )
//	{
//		gEcuData[EcuIndex].CVN_String = realloc ( gEcuData[EcuIndex].CVN_String, CVN_String_Size );
//	}
//	else
//	{
//		gEcuData[EcuIndex].CVN_String = (char *)malloc ( CVN_String_Size );
//	}
//	memset ( gEcuData[EcuIndex].CVN_String, 0x00, CVN_String_Size );
//
//	gEcuData[EcuIndex].CVN_Count = Inf6NumItems;

	for ( ItemIndex = 0, ByteIndex = 0; ItemIndex < Inf6NumItems; ItemIndex++, ByteIndex += 4 )
	{
		sprintf_s ( buffer,
		            CVN_BUFFER_SIZE,
		            "$%02X $%02X $%02X $%02X",
		            pINF->Data[ByteIndex],
		            pINF->Data[ByteIndex+1],
		            pINF->Data[ByteIndex+2],
		            pINF->Data[ByteIndex+3] );

		// if there is only one CVN, don't number it
		if ( Inf6NumItems == 1 )
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F806  CVN   = %s\n",
			      GetEcuId ( EcuIndex ),
			      buffer );
		}
		// else, there are more than one CVN, number them
		else
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F806  CVN%02d = %s\n",
			      GetEcuId ( EcuIndex ),
			      ItemIndex+1,
			      buffer );
		}


//		strcat_s ( gEcuData[EcuIndex].CVN_String, CVN_String_Size - strlen (gEcuData[EcuIndex].CVN_String ), buffer );
//		if ( ItemIndex != Inf6NumItems-1 )
//		{
//			strcat_s ( gEcuData[EcuIndex].CVN_String, CVN_String_Size - strlen ( gEcuData[EcuIndex].CVN_String ), "\n                          " );
//		}

	}

	// response size should be 1 for MY 2009
//	if ( gModelYear >= 2009 &&
//	     pINF[0].NumItems != 1)
//	{
//		Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//		      "ECU %X  INF $F806 (CVN) NumItems = %d (should be 1 for MY 2009 and later, unless EO approved)\n",
//		      GetEcuId ( EcuIndex ),
//		      pINF[0].NumItems );
//	}

	return PASS;
}


/*
*******************************************************************************
**
**  Function:   VerifyECUNameData
**
**  Purpose:    Purpose of this function is to verify the ECU Name format
**              for correct format.  In the event the format fails
**              defined criteria, an error is returned.
**
******************************************************************************
**/
#define TABLE_SIZE  41
#define ACRONYM_STRING_SIZE 45
#define TEXTNAME_STRING_SIZE 16
const char szECUAcronym[TABLE_SIZE][ACRONYM_STRING_SIZE] =
	{ "ABS ABS1 ABS2 ABS3 ABS4 ABS5 ABS6 ABS7 ABS8",
	  "AFCM AFC1 AFC2 AFC3 AFC4 AFC5 AFC6 AFC7 AFC8",
	  "AHCM AHC1 AHC2 AHC3 AHC4 AHC5 AHC6 AHC7 AHC8",
	  "APCM APC1 APC2 APC3 APC4 APC5 APC6 APC7 APC8",
	  "AWDC AWD1 AWD2 AWD3 AWD4 AWD5 AWD6 AWD7 AWD8",
	  "BCCM BCC1 BCC2 BCC3 BCC4 BCC5 BCC6 BCC7 BCC8",
	  "BECM BEC1 BEC2 BEC3 BEC4 BEC5 BEC6 BEC7 BEC8",
	  "BMCM BMC1 BMC2 BMC3 BMC4 BMC5 BMC6 BMC7 BMC8",
	  "BCM BCM1 BCM2 BCM3 BCM4 BCM5 BCM6 BCM7 BCM8",
	  "BSCM BSC1 BSC2 BSC3 BSC4 BSC5 BSC6 BSC7 BSC8",
	  "CATM CAT1 CAT2 CAT3 CAT4 CAT5 CAT6 CAT7 CAT8",
	  "CMPM CMP1 CMP2 CMP3 CMP4 CMP5 CMP6 CMP7 CMP8",
	  "CHCM CHC1 CHC2 CHC3 CHC4 CHC5 CHC6 CHC7 CHC8",
	  "CRCM CRC1 CRC2 CRC3 CRC4 CRC5 CRC6 CRC7 CRC8",
	  "CTCM CTC1 CTC2 CTC3 CTC4 CTC5 CTC6 CTC7 CTC8",
	  "DCDC DCD1 DCD2 DCD3 DCD4 DCD5 DCD6 DCD7 DCD8",
	  "DMCM DMC1 DMC2 DMC3 DMC4 DMC5 DMC6 DMC7 DMC8",
	  "EACC EAC1 EAC2 EAC3 EAC4 EAC5 EAC6 EAC7 EAC8",
	  "EACM EAM1 EAM2 EAM3 EAM4 EAM5 EAM6 EAM7 EAM8",
	  "EBBC EBB1 EBB2 EBB3 EBB4 EBB5 EBB6 EBB7 EBB8",
	  "ECCI ECC1 ECC2 ECC3 ECC4 ECC5 ECC6 ECC7 ECC8",
	  "ECM ECM1 ECM2 ECM3 ECM4 ECM5 ECM6 ECM7 ECM8",
	  "FACM FAC1 FAC2 FAC3 FAC4 FAC5 FAC6 FAC7 FAC8",
	  "FCCM FCC1 FCC2 FCC3 FCC4 FCC5 FCC6 FCC7 FCC8",
	  "FICM FIC1 FIC2 FIC3 FIC4 FIC5 FIC6 FIC7 FIC8",
	  "FPCM FPC1 FPC2 FPC3 FPC4 FPC5 FPC6 FPC7 FPC8",
	  "4WDC 4WD1 4WD2 4WD3 4WD4 4WD5 4WD6 4WD7 4WD8",
	  "GWM GWM1 GWM2 GWM3 GWM4 GWM5 GWM6 GWM7 GWM8",
	  "GPCM GPC1 GPC2 GPC3 GPC4 GPC5 GPC6 GPC7 GPC8",
	  "GSM GSM1 GSM2 GSM3 GSM4 GSM5 GSM6 GSM7 GSM8",
	  "HVAC HVA1 HVA2 HVA3 HVA4 HVA5 HVA6 HVA7 HVA8",
	  "HPCM HPC1 HPC2 HPC3 HPC4 HPC5 HPC6 HPC7 HPC8",
	  "IPC IPC1 IPC2 IPC3 IPC4 IPC5 IPC6 IPC7 IPC8",
	  "OBCC OBC1 OBC2 OBC3 OBC4 OBC5 OBC6 OBC7 OBC8",
	  "PCM PCM1 PCM2 PCM3 PCM4 PCM5 PCM6 PCM7 PCM8",
	  "RDCM RDC1 RDC2 RDC3 RDC4 RDC5 RDC6 RDC7 RDC8",
	  "SGCM SGC1 SGC2 SGC3 SGC4 SGC5 SGC6 SGC7 SGC8",
	  "TACM TAC1 TAC2 TAC3 TAC4 TAC5 TAC6 TAC7 TAC8",
	  "TCCM TCC1 TCC2 TCC3 TCC4 TCC5 TCC6 TCC7 TCC8",
	  "TCM TCM1 TCM2 TCM3 TCM4 TCM5 TCM6 TCM7 TCM8",
	  "VTMC VTM1 VTM2 VTM3 VTM4 VTM5 VTM6 VTM7 VTM8"};

const char szECUTextName[TABLE_SIZE][TEXTNAME_STRING_SIZE] =
	{ "AntiLockBrake",
	  "AltFuelCtrl",
	  "AuxHeatCtrl",
	  "AirPumpCtrl",
	  "AllWhlDrvCtrl",
	  "B+ChargerCtrl",
	  "B+EnergyCtrl",
	  "B+ManageCtrl",
	  "BodyControl",
	  "BrakeSystem",
	  "CatHeaterCtrl",
	  "CamPosCtrl",
	  "ChassisCtrl",
	  "CruiseControl",
	  "CoolTempCtrl",
	  "DCDCConvCtrl",
	  "DriveMotorCtrl",
	  "ElecACCompCtrl",
	  "ExhaustAftCtrl",
	  "ElecBrkBstCtrl",
	  "EmisCritInfo",
	  "EngineControl",
	  "FuelAddCtrl",
	  "FuelCellCtrl",
	  "FuelInjCtrl",
	  "FuelPumpCtrl",
	  "4WhlDrvClCtrl",
	  "Gateway",
	  "GlowPlugCtrl",
	  "GearShiftCtrl",
	  "HVACCtrl",
	  "HybridPtCtrl",
	  "InstPanelClust",
	  "OBChargerCtrl",
	  "PowertrainCtrl",
	  "ReductantCtrl",
	  "Start/GenCtrl",
	  "ThrotActCtrl",
	  "TransfCaseCtrl",
	  "TransmisCtrl",
	  "ThermalMgmt" };

STATUS VerifyECUNameData ( unsigned long  EcuIndex )
{
	INF            *pINF;
	unsigned int    ByteIndex;
	char            buffer[21];
	char           *pString;
	char           *ptoken = NULL;
	char            AcronymBuffer[ACRONYM_STRING_SIZE];
	char            AcronymNumber = 0;
	char            TextNameNumber = 0;
	unsigned int    CompSize;
	unsigned int    TableIndex;
	unsigned int    AcronymTableIndex;
	unsigned char   bTestFailed = FALSE;
	unsigned char   bMatchFound = FALSE;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F80A data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// copy to a protocol independent buffer
	// ECU Name Data must contain 20 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 20 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F80A (ECU NAME) Data Size Error = %d (Must be 20 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// INF INFA NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		bTestFailed = TRUE;
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F80A  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
	}

	memcpy ( buffer, &(pINF->Data[0]), 20 );

	// insert NULL terminator at end of string
	buffer[20] = 0;
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F80A  ECU acronym and name = %s\n",
	      GetEcuId ( EcuIndex ),
	      buffer );
	
	// Check all ECU Acronym characters for validity
	for ( ByteIndex = 0; ByteIndex < 4; ByteIndex++ )
	{
		if ( buffer[ByteIndex] == 0x00 )
		{
			break;
		}
		else if ( !( buffer[ByteIndex] >= '1' && buffer[ByteIndex] <= '9' ) &&
		          !( buffer[ByteIndex] >= 'A' && buffer[ByteIndex] <= 'Z' ) )
		{
			if ( gModelYear >= 2010 )
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  Invalid ECU acronym/number character (byte %d)\n",
				      GetEcuId ( EcuIndex ),
				      ByteIndex );
				bTestFailed = TRUE;
			}
			else
			{
				Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  Invalid ECU acronym/number character (byte %d)\n",
				      GetEcuId ( EcuIndex ),
				      ByteIndex );
			}
		}
	}

	// Check that ECU Acronym is approved
	for ( TableIndex = 0; TableIndex < TABLE_SIZE; TableIndex++ )
	{
		memcpy ( AcronymBuffer, &(szECUAcronym[TableIndex]), ACRONYM_STRING_SIZE );

		// get pointer to first string
		pString = strtok_s ( (char*)&(AcronymBuffer), " ", &ptoken);
		do
		{
			// if found match
			if ( strncmp ( pString, &buffer[0], ByteIndex ) == 0 )
			{
				bMatchFound = TRUE;
				break;
			}

			// get pointer to next string
			pString = strtok_s ( '\0', " ", &ptoken );
		} while ( pString && bMatchFound != TRUE );

		if ( bMatchFound == TRUE )
		{
			break;
		}
	}

	if ( TableIndex == TABLE_SIZE )
	{
		if ( gModelYear >= 2010 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  Not an approved ECU acronym/number\n",
			      GetEcuId ( EcuIndex ) );
			bTestFailed = TRUE;
		}
		else
		{
			Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  Not an approved ECU acronym/number\n",
			      GetEcuId ( EcuIndex ) );
		}
	}

	// if last character was a number
	if ( buffer[ByteIndex-1] >= '1' && buffer[ByteIndex-1] <= '9' )
	{
		// save the number for Acronym/Text Name verification
		AcronymNumber = buffer[ByteIndex-1];
	}

	// save index for Acronym/Text Name verification
	AcronymTableIndex = TableIndex;

	// Check for zero padding of ECU Acronym
	for ( ; ByteIndex < 4; ByteIndex++ )
	{
		if ( buffer[ByteIndex] != 0x00 )
		{
			if ( gModelYear >= 2010 )
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  ECU acronym/number not zero padded on right\n",
				      GetEcuId ( EcuIndex ) );
				bTestFailed = TRUE;
			}
			else
			{
				Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  ECU acronym/number not zero padded on right\n",
				      GetEcuId ( EcuIndex ) );
			}
		}
		else
		{
			buffer[ByteIndex] = ' ';    // replace zero with space for printing to log file
		}
	}

	// Check for invalid delimiter
	if ( buffer[ByteIndex++] != '-' )
	{
		if ( gModelYear >= 2010 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  Invalid ECU Name delimiter character (byte %d)\n",
			      GetEcuId ( EcuIndex ),
			      ByteIndex );
			bTestFailed = TRUE;
		}
		else
		{
			Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  Invalid ECU Name delimiter character (byte %d)\n",
			      GetEcuId ( EcuIndex ),
			      ByteIndex );
		}
	}

	// Check all ECU Text Name characters for validity
	for ( ; ByteIndex < 20; ByteIndex++ )
	{
		if ( buffer[ByteIndex] == 0x00 )
		{
			break;
		}

		if ( !( buffer[ByteIndex] >= '1' && buffer[ByteIndex] <= '9' ) &&
		     !( buffer[ByteIndex] >= 'A' && buffer[ByteIndex] <= 'Z' ) &&
		     !( buffer[ByteIndex] >= 'a' && buffer[ByteIndex] <= 'z' ) &&
		     buffer[ByteIndex] != '+' && buffer[ByteIndex] != '/' )
		{
			if ( gModelYear >= 2010 )
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  Invalid ECU text name character (byte %d)\n",
				      GetEcuId ( EcuIndex ),
				      ByteIndex );
				bTestFailed = TRUE;
			}
			else
			{
				Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  Invalid ECU text name character (byte %d)\n",
				      GetEcuId ( EcuIndex ),
				      ByteIndex );
			}
		}
	}

	// if last character was a number, don't use it in ECU Text Name search
	if ( buffer[ByteIndex-1] >= '1' && buffer[ByteIndex-1] <= '9' )
	{
		// comparison size is buffer size less the Acronym, the Delimiter and the Text Name number
		CompSize = ByteIndex - 6;
		// save the number for Acronym/Text Name verification
		TextNameNumber = buffer[ByteIndex-1];
	}
	else
	{
		// comparison size is buffer size less the Acronym and the Delimiter
		CompSize = ByteIndex - 5;
	}

	// Check that ECU Text Name is approved
	for ( TableIndex = 0; TableIndex < TABLE_SIZE; TableIndex++ )
	{
		// if found match
		if ( strncmp ( &buffer[5], szECUTextName[TableIndex], CompSize ) == 0 )
		{
			break;
		}
	}

	if ( TableIndex == TABLE_SIZE )
	{
		if ( gModelYear >= 2010 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  Not an approved ECU text name\n",
			      GetEcuId ( EcuIndex ) );
			bTestFailed = TRUE;
		}
		else
		{
			Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  Not an approved ECU text name\n",
			      GetEcuId ( EcuIndex ) );
		}
	}

	// Check for zero padding of ECU Text Name
	for ( ; ByteIndex < 20; ByteIndex++ )
	{
		if ( buffer[ByteIndex] != 0 )
		{
			if ( gModelYear >= 2010 )
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  ECU text name not zero padded on right\n",
				      GetEcuId ( EcuIndex ) );
				bTestFailed = TRUE;
			}
			else
			{
				Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  ECU text name not zero padded on right\n",
				      GetEcuId ( EcuIndex ) );
			}
		}
	}

	// Check for ECU Acronym and Text Name match
	if ( AcronymTableIndex != TableIndex ||
	     AcronymNumber != TextNameNumber )
	{
		if ( gModelYear >= 2010 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  ECU acronym and text name do not match\n",
			      GetEcuId ( EcuIndex ) );
			bTestFailed = TRUE;
		}
		else
		{
			Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  ECU acronym and text name do not match\n",
			      GetEcuId ( EcuIndex ) );
		}
	}
	if ( bTestFailed == TRUE )
	{
		return FAIL;
	}
	else
	{
		return PASS;
	}
}


/*
*******************************************************************************
**
**  Function:   VerifyINF8Data
**
**  Purpose:    Purpose of this function is to verify the INF 8 data
**              for correct format.  In the event the format fails
**              defined criteria, an error is returned.
**
*******************************************************************************
*/
STATUS VerifyINF8Data ( unsigned long  EcuIndex )
{
	STATUS eResult=PASS; // function result

	if ( gOBDDieselFlag == TRUE && gModelYear >= 2010 /*&&
	     (gUserInput.eComplianceType == US_OBDII || gUserInput.eComplianceType == HD_OBD)*/ )
	{
		// Compression Ignition vehicles MY 2010 and later should not support INF8
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F808 response! (Not allowed for MY 2010 and later compression ignition vehicles)\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

//	// Check to see if IUMPR is supported for Compliance Type EOBD_NO_IUMPR, IOBD_NO_IUMPR, HD_IOBD_NO_IUMPR or OBDBr_NO_IUMPR
//	if ( gUserInput.eComplianceType == EOBD_NO_IUMPR    ||
//	     gUserInput.eComplianceType == IOBD_NO_IUMPR    ||
//	     gUserInput.eComplianceType == HD_IOBD_NO_IUMPR ||
//	     gUserInput.eComplianceType == OBDBr_NO_IUMPR )
//	{
//		Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//		      "ECU %X  INF $F808 response! (Conflicts with Compliance Type selected)\n",
//		      GetEcuId ( EcuIndex ) );
//	}

	// In-Use Performance Data must contain 32, 40, or 56 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 32 &&
	     gOBDResponse[EcuIndex].InfSize - 3 != 40 &&
	     gOBDResponse[EcuIndex].InfSize - 3 != 56 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F808 (IPT)  Data Size = %d (Must be 32, 40 or 56 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );

		// INF INF8 NODI must equal $10 (16), $14 (20), or $1C (28)
		if ( gOBDResponse[EcuIndex].Inf[2] != 0x10 &&
		     gOBDResponse[EcuIndex].Inf[2] != 0x14 &&
		     gOBDResponse[EcuIndex].Inf[2] != 0x1C )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F808 (IPT)  NODI = %d (Must be 16 ($1C), 20 ($14), or 28 ($1C))\n",
			      GetEcuId ( EcuIndex ),
			      gOBDResponse[EcuIndex].Inf[2] );
		}
		eResult = ERRORS;

	}
	else
	{
		// IF spark ignition (GAS) vehicle
		if ( gOBDDieselFlag == FALSE )
		{
			// If MY 2020 and later CNOBD spark ignition vehicle
			/*if ( gModelYear >= 2020 && gUserInput.eComplianceType == CNOBD_WITH_IUMPR )
			{
				// In-Use Performance Data must contain 56 bytes of data
				// Don't include SID, INF MSB or INF LSB in INFSIZE
				if ( gOBDResponse[EcuIndex].InfSize - 3 != 56 )
				{
					Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "ECU %X  INF $F808 (IPT)  Data Size = %d (Must be 56 bytes!)\n",
					      GetEcuId ( EcuIndex ),
					      (gOBDResponse[EcuIndex].InfSize - 3) );
				}

				// INF INF8 NODI must equal 28 ($1C)
				if ( gOBDResponse[EcuIndex].Inf[2] != 0x1C )
				{
					Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "ECU %X  INF $F808 (IPT)  NODI = %d (Must be 28 ($1C))\n",
					      GetEcuId ( EcuIndex ),
					      gOBDResponse[EcuIndex].Inf[2] );
				}
			}

			// If MY 2019 and later US_OBDII spark ignition vehicle
			else */if ( gModelYear >= 2019 /*&& gUserInput.eComplianceType == US_OBDII*/ )
			{
				// In-Use Performance Data must contain 56 bytes of data
				// Don't include SID, INF MSB or INF LSB in INFSIZE
				if ( gOBDResponse[EcuIndex].InfSize - 3 != 56 )
				{
					Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "ECU %X  INF $F808 (IPT)  Data Size = %d (Must be 56 bytes!)\n",
					      GetEcuId ( EcuIndex ),
					      (gOBDResponse[EcuIndex].InfSize - 3) );
				}

				// INF8 NODI must equal 28 ($1C)
				if ( gOBDResponse[EcuIndex].Inf[2] != 0x1C )
				{
					Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "ECU %X  INF $F808 (IPT)  NODI = %d (Must be 28 ($1C))\n",
					      GetEcuId ( EcuIndex ),
					      gOBDResponse[EcuIndex].Inf[2] );
				}
			}

			// If MY 2010 and later US_OBDII spark ignition vehicle
			else if ( gModelYear >= 2010 /*&& gUserInput.eComplianceType == US_OBDII*/ )
			{
				// In-Use Performance Data must contain 40 bytes of data
				// Don't include SID, INF MSB or INF LSB in INFSIZE
				if ( gOBDResponse[EcuIndex].InfSize - 3 != 40 &&
				     gOBDResponse[EcuIndex].InfSize - 3 != 56 )
				{
					Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "ECU %X  INF $F808 (IPT)  Data Size = %d (Must be 40 or 56 bytes!)\n",
					      GetEcuId ( EcuIndex ),
					      (gOBDResponse[EcuIndex].InfSize - 3) );
					eResult = ERRORS;
				}

				// INF INF8 NODI must equal $14
				if ( gOBDResponse[EcuIndex].Inf[2] != 0x14 &&
				     gOBDResponse[EcuIndex].Inf[2] != 0x1C )
				{
					Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "ECU %X  INF $F808 (IPT)  NODI = %d (Must be 20 ($14) or 28 ($1C))\n",
					      GetEcuId ( EcuIndex ),
					      gOBDResponse[EcuIndex].Inf[2] );
					eResult = ERRORS;
				}
			}
			else
			{
				// INF8 NODI must equal $10 (16), $14 (20), or $1C (28)
				if ( gOBDResponse[EcuIndex].Inf[2] != 0x10 &&
				     gOBDResponse[EcuIndex].Inf[2] != 0x14 &&
				     gOBDResponse[EcuIndex].Inf[2] != 0x1C )
				{
					Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "ECU %X  INF $F808  NODI = %d (Must be 16 ($1C), 20 ($14), or 28 ($1C))\n",
					      GetEcuId ( EcuIndex ),
					      gOBDResponse[EcuIndex].Inf[2] );
					eResult = ERRORS;
				}
			}
		}
		// IF compression ignition (DIESEL) vehicle
		else
		{
			// In-Use Performance Data must contain 32 or 40 bytes of data
			// Don't include SID, INF MSB or INF LSB in INFSIZE
			if ( gOBDResponse[EcuIndex].InfSize - 3 != 32 &&
			     gOBDResponse[EcuIndex].InfSize - 3 != 40 )
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  INF $F808 (IPT)  Data Size = %d (Must be 32 or 40 bytes!)\n",
				      GetEcuId ( EcuIndex ),
				      (gOBDResponse[EcuIndex].InfSize - 3) );
				eResult = ERRORS;
			}

			// INF INF8 NODI must equal $10 or $14
			if ( gOBDResponse[EcuIndex].Inf[2] != 0x10 &&
			     gOBDResponse[EcuIndex].Inf[2] != 0x14 )
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  INF $F808  NODI = %d (Must be 16 ($10) or 20 ($14))\n",
				      GetEcuId ( EcuIndex ),
				      gOBDResponse[EcuIndex].Inf[2] );
				eResult = ERRORS;
			}
		}
	}

	return eResult;
}


/*
*******************************************************************************
**
**  Function:   VerifyINFBData
**
**  Purpose:    Purpose of this function is to verify the INF B data
**              for correct format.  In the event the format fails
**              defined criteria, an error is returned.
**
*******************************************************************************
*/
STATUS VerifyINFBData ( unsigned long  EcuIndex )
{
	if ( gOBDDieselFlag == FALSE && gModelYear >= 2010 /*&&
	     (gUserInput.eComplianceType == US_OBDII || gUserInput.eComplianceType == HD_OBD)*/ )
	{
		// Spark Ignition vehicles MY 2010 and later should not support INFB
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F80B response! (Not allowed for MY 2010 and later spark ignition vehicles)\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

//	// Check to see if IUMPR is supported for Compliance Type EOBD_NO_IUMPR, IOBD_NO_IUMPR, HD_IOBD_NO_IUMPR or OBDBr_NO_IUMPR
//	if ( gUserInput.eComplianceType == EOBD_NO_IUMPR    ||
//	     gUserInput.eComplianceType == IOBD_NO_IUMPR    ||
//	     gUserInput.eComplianceType == HD_IOBD_NO_IUMPR ||
//	     gUserInput.eComplianceType == OBDBr_NO_IUMPR )
//	{
//		Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//		      "ECU %X  INF $F80B response! (Conflicts with Compliance Type selected)\n",
//		      GetEcuId ( EcuIndex ) );
//	}

	// In-Use Performance Data must contain 32 or 36 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 32 &&
	     gOBDResponse[EcuIndex].InfSize - 3 != 36 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F80B (IPT)  Data Size = %d (Must be 32 or 36 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	if ( gOBDResponse[EcuIndex].InfSize - 3 == 32 &&
	     gOBDResponse[EcuIndex].Inf[2] != 0x10 )
	{
		// INF INFB NODI must equal 16 if the data size is 32
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F80B  NODI = %d (Must be 16)\n",
		      GetEcuId ( EcuIndex ),
		      gOBDResponse[EcuIndex].Inf[2] );
		return FAIL;
	}

	if ( gOBDResponse[EcuIndex].InfSize - 3 == 36 &&
	     gOBDResponse[EcuIndex].Inf[2] != 0x12 )
	{
		// INF INFB NODI must equal 18 if the data size is 36
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F80B  NODI = %d (Must be 18)\n",
		      GetEcuId ( EcuIndex ),
		      gOBDResponse[EcuIndex].Inf[2] );
		return FAIL;
	}

	return PASS;
}


/*
*******************************************************************************
**
**  Function:   VerifyInfD_FData
**
**  Purpose:    Purpose of this function is to verify the ESN/EROTAN format
**              for correct format.  In the event the format fails
**              defined criteria, an error is returned.
**
********************************************************************************
*/
STATUS VerifyInfD_FData ( unsigned long EcuIndex )
{
	unsigned int StartOffset = 0;
	unsigned int MsgIndex = 0; // multi-part message index
	unsigned long ByteIndex = 0;
	char rgcBuffer[21];
	STATUS RetCode = PASS;
	INF *pINF;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F8%02X data\n",
		      GetEcuId ( EcuIndex ),
		      gOBDResponse[EcuIndex].Inf[1] );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// copy to a protocol independent buffer

	// ESN Data must contain 17 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 17 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  Data Size = %d (Must be 17 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// INF INFC NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      pINF->NumItems );
		RetCode = FAIL;
	}

	memcpy ( rgcBuffer, &(pINF->Data[0]), 20 );

	// strip of the pad bytes
	for ( ByteIndex = 0; ByteIndex < 17; ByteIndex++ )
	{
		if ( rgcBuffer[ByteIndex] != 0x00 )
		{
			StartOffset = ByteIndex;
			break;
		}
	}

	if ( ByteIndex == 17 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X contains all pad bytes\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB );
		RetCode = FAIL;
	}
	else
	{
		// Check all ESN characters for validity
		for ( ; ByteIndex < 17; ByteIndex++ )
		{
			if ( rgcBuffer[ByteIndex] <  '0' || rgcBuffer[ByteIndex] >  'Z' ||
			     rgcBuffer[ByteIndex] == 'I' || rgcBuffer[ByteIndex] == 'O' ||
			     rgcBuffer[ByteIndex] == 'Q' || rgcBuffer[ByteIndex] == ':' ||
			     rgcBuffer[ByteIndex] == ';' || rgcBuffer[ByteIndex] == '<' ||
			     rgcBuffer[ByteIndex] == '>' || rgcBuffer[ByteIndex] == '=' ||
			     rgcBuffer[ByteIndex] == '?' || rgcBuffer[ByteIndex] == '@' )
			{
				break;
			}
		}

		// NULL terminate string
		rgcBuffer[ByteIndex] = 0x00;

		if ( ByteIndex != 17 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  Invalid or missing INF $F8%02X data\n",
			      GetEcuId ( EcuIndex ),
			      pINF->INFLSB );
			RetCode = FAIL;
		}
	}

	// Print ESN/EROTAN string to log file
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  %s = %s\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      pINF->INFLSB == 0x0D ? "ESN" : "EROTAN",
	      &rgcBuffer[StartOffset] );

	return RetCode;
}


/*
*******************************************************************************
**
**  Function:   VerifyInf10Data
**
**  Purpose:    Purpose of this function is to verify the Protocol Identification data
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf10Data ( unsigned long EcuIndex )
{
	STATUS        RetCode = PASS;
	INF          *pINF;
	static unsigned char  ProtIDCnt = 0;
	unsigned char PROTID;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F810 data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// copy to a protocol independent buffer

	// Protocol Identification Data must contain 1 byte of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 1 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F810  Data Size = %d (Must be 1 byte!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// INF 10 NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F810  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}

	PROTID = gOBDResponse[EcuIndex].Inf[3];

	// log PROTID
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F810  PROTID = %d\n",
	      GetEcuId ( EcuIndex ),
	      PROTID );

	return RetCode;
}


/*
*******************************************************************************
**
**  Function:   VerifyInf11Data
**
**  Purpose:    Purpose of this function is to verify the WWH-OBD GTR Number data
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf11Data ( unsigned long EcuIndex )
{
	STATUS        RetCode = PASS;
	INF          *pINF;
	static unsigned char  ProtIDCnt = 0;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F811 data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// copy to a protocol independent buffer

	// Protocol Identification Data must contain 1 byte of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 11 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F811  Data Size = %d (Must be 11 byte!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// INF $11 NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F811  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}

	pINF->Data[11] = (unsigned char)NULL;

	// log PROTID
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F811  WWH-OBD GTR Number = %s\n",
	      GetEcuId ( EcuIndex ),
	       pINF->Data[0] );

	return RetCode;
}


/*
*******************************************************************************
**
**  Function:   VerifyInf12Data
**
**  Purpose:    Purpose of this function is to verify the FEOCNTR data
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf12Data ( unsigned long EcuIndex )
{
	STATUS        RetCode = PASS;
	INF          *pINF;
	unsigned long FEOCNTR;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F812 data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// copy to a protocol independent buffer

	// FEOCNTR Data must contain 2 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 2 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F812  Data Size = %d (Must be 2 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// INF INF12 NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F812  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}

	FEOCNTR = (gOBDResponse[EcuIndex].Inf[3] * 256) + gOBDResponse[EcuIndex].Inf[4];

	// log FEOCNTR
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F812  FEOCNTR = %d\n",
	      GetEcuId ( EcuIndex ),
	      FEOCNTR );

	return RetCode;
}


/*
*******************************************************************************
**
**  Function:   VerifyInf13Data
**
**  Purpose:    Purpose of this function is to verify the Test Group/Engine Family Number data
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf13Data ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF *pINF;
	unsigned int TG_EFN_index;
	unsigned int Data_index;
	unsigned char cTG_EFN[13];

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F813 data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// TG/EFN Data must contain 12 bytes or less of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( (gOBDResponse[EcuIndex].InfSize - 3) > 12 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F813  Data Size = %d (Must be 1-12 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// INF INF13 NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F813  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}

	// copy valid data to TG_EFN string
	for ( Data_index = 0, TG_EFN_index = 0; Data_index < 12; Data_index++ )
	{
		if ( pINF->Data[Data_index] != 0x00 && (pINF->Data[Data_index] < 0x20 || pINF->Data[Data_index] > 0x7E) )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F813  byte %d  Invalid Character = $%02X (Must be $20-$7E)\n",
			      GetEcuId ( EcuIndex ),
			      Data_index+1,
			      pINF->Data[Data_index] );
			RetCode = FAIL;
		}
		else if ( pINF->Data[Data_index] >= 0x20 && pINF->Data[Data_index] <= 0x7E )
		{
			cTG_EFN[TG_EFN_index++] = pINF->Data[Data_index];
		}
	}
	cTG_EFN[TG_EFN_index] = (unsigned char)NULL;

	// log TG/EFN
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F813  TG/EFN = %s\n",
	      GetEcuId ( EcuIndex ),
	      cTG_EFN );

	// Check that TG/EFN model year character matches user input model year
	if ( ( cTG_EFN[0] == '1' && gModelYear != 2001 ) ||
	     ( cTG_EFN[0] == '2' && gModelYear != 2002 ) ||
	     ( cTG_EFN[0] == '3' && gModelYear != 2003 ) ||
	     ( cTG_EFN[0] == '4' && gModelYear != 2004 ) ||
	     ( cTG_EFN[0] == '5' && gModelYear != 2005 ) ||
	     ( cTG_EFN[0] == '6' && gModelYear != 2006 ) ||
	     ( cTG_EFN[0] == '7' && gModelYear != 2007 ) ||
	     ( cTG_EFN[0] == '8' && gModelYear != 2008 ) ||
	     ( cTG_EFN[0] == '9' && gModelYear != 2009 ) ||
	     ( cTG_EFN[0] == 'A' && gModelYear != 2010 ) ||
	     ( cTG_EFN[0] == 'B' && gModelYear != 1981 && gModelYear != 2011 ) ||
	     ( cTG_EFN[0] == 'C' && gModelYear != 1982 && gModelYear != 2012 ) ||
	     ( cTG_EFN[0] == 'D' && gModelYear != 1983 && gModelYear != 2013 ) ||
	     ( cTG_EFN[0] == 'E' && gModelYear != 1984 && gModelYear != 2014 ) ||
	     ( cTG_EFN[0] == 'F' && gModelYear != 1985 && gModelYear != 2015 ) ||
	     ( cTG_EFN[0] == 'G' && gModelYear != 1986 && gModelYear != 2016 ) ||
	     ( cTG_EFN[0] == 'H' && gModelYear != 1987 && gModelYear != 2017 ) ||
	     ( cTG_EFN[0] == 'J' && gModelYear != 1988 && gModelYear != 2018 ) ||
	     ( cTG_EFN[0] == 'K' && gModelYear != 1989 && gModelYear != 2019 ) ||
	     ( cTG_EFN[0] == 'L' && gModelYear != 1990 && gModelYear != 2020 ) ||
	     ( cTG_EFN[0] == 'M' && gModelYear != 1991 && gModelYear != 2021 ) ||
	     ( cTG_EFN[0] == 'N' && gModelYear != 1992 && gModelYear != 2022 ) ||
	     ( cTG_EFN[0] == 'P' && gModelYear != 1993 && gModelYear != 2023 ) ||
	     ( cTG_EFN[0] == 'R' && gModelYear != 1994 && gModelYear != 2024 ) ||
	     ( cTG_EFN[0] == 'S' && gModelYear != 1995 && gModelYear != 2025 ) ||
	     ( cTG_EFN[0] == 'T' && gModelYear != 1996 ) ||
	     ( cTG_EFN[0] == 'V' && gModelYear != 1997 ) ||
	     ( cTG_EFN[0] == 'W' && gModelYear != 1998 ) ||
	     ( cTG_EFN[0] == 'X' && gModelYear != 1999 ) ||
	     ( cTG_EFN[0] == 'Y' && gModelYear != 2000 ) )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			     "ECU %X  INF $F813 (TG/EFN)  year character (%c) does not match user entry %d\n",
			     GetEcuId(EcuIndex),
			     cTG_EFN[0],
			     gModelYear );
		RetCode = FAIL;
	}


	// Check that TG/EFN model year character matches VIN model year
	if ( cTG_EFN[0] != gVIN[9] &&
	     gUserInput.eEngineType == LD )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F813 (TG/EFN)  year character (%c) does not match INF $F802 (VIN) year character (%c)\n",
		      GetEcuId ( EcuIndex ),
		      cTG_EFN[0],
		      gVIN[9] );
		RetCode = FAIL;
	}

	return RetCode;
}


/*
*******************************************************************************
**
**  Function:   VerifyInf14Data
**
**  Purpose:    Purpose of this function is to verify the EVAP_DIST data
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf14Data ( unsigned long EcuIndex )
{
	STATUS        RetCode = PASS;
	INF          *pINF;
	unsigned long EVAP_DIST;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F814 data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// EVAP_DIST Data must contain 2 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 2 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F814  Data Size = %d (Must be 2 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// INF INF14 NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F814  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}

	EVAP_DIST = (gOBDResponse[EcuIndex].Inf[3] * 256) + gOBDResponse[EcuIndex].Inf[4];

	// log EVAP_DIST
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F814  EVAP_DIST = %d km\n",
	      GetEcuId ( EcuIndex ),
	      EVAP_DIST );

	// if tests 11.7
	if ( gTestPhase == eTestPerformanceCounters && gTestSubsection == 7 && EVAP_DIST == 0xFFFF )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F814 should not return a value of $FFFF after evap monitor runs\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}
	// if tests 5.17
	else if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 && EVAP_DIST != 0xFFFF  && EVAP_DIST != 0x0000 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F814 should return a value of $FFFF or $0000 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	return RetCode;
}


/*
*******************************************************************************
**
**  Function:   VerifyInf16Data
**
**  Purpose:    Purpose of this function is to verify the Engine Run/Idle Time data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf16Data ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF *pINF;
	unsigned long IGNCNTR_R;
	unsigned long IGNCNTR_L;
	unsigned long FEOCNTR_R;
	unsigned long FEOCNTR_L;
	unsigned long ERT_R;
	unsigned long ERT_L;
	unsigned long IERT_R;
	unsigned long IERT_L;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F816 data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// ERT/IERT Data must contain 32 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 32 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F816  Data Size = %d (Must be 32 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// INF INF16 NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F816  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}

	// log IGNCNTR_R
	IGNCNTR_R = (gOBDResponse[EcuIndex].Inf[3] << 24 ) + (gOBDResponse[EcuIndex].Inf[4] << 16 ) + (gOBDResponse[EcuIndex].Inf[5] << 8 ) + (gOBDResponse[EcuIndex].Inf[6] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F816  IGNCNTR-R = %lu\n",
	      GetEcuId ( EcuIndex ),
	      IGNCNTR_R );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		INF_Inf_5_17[EcuIndex].INF16_IGNCNTR_R = IGNCNTR_R;

		if ( IGNCNTR_R > 1 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F816 IGNCNTR-R must return a value of 0 or 1 after a Service $14 code clear\n",
			      GetEcuId ( EcuIndex ) );
			RetCode = FAIL;
		}
	}
	else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 22 &&  // IF Test 9.22
	          IGNCNTR_R <= INF_Inf_5_17[EcuIndex].INF16_IGNCNTR_R )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F816 IGNCNTR-R must return a value greater than the value returned in Test 5.17 (%lu)\n",
		      GetEcuId ( EcuIndex ),
		      INF_Inf_5_17[EcuIndex].INF16_IGNCNTR_R );
		RetCode = FAIL;
	}
	else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	          IGNCNTR_R != 0 && IGNCNTR_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F816 IGNCNTR-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log IGNCNTR_L
	IGNCNTR_L = (gOBDResponse[EcuIndex].Inf[7] << 24 ) + (gOBDResponse[EcuIndex].Inf[8] << 16 ) + (gOBDResponse[EcuIndex].Inf[9] << 8 ) + (gOBDResponse[EcuIndex].Inf[10] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F816  IGNCNTR-L = %lu\n",
	      GetEcuId ( EcuIndex ),
	      IGNCNTR_L );
//	if ( gUserInput.eComplianceType == US_OBDII )
//	{
//		if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
//		{
//			INF_Inf_5_17[EcuIndex].INF16_IGNCNTR_L = IGNCNTR_L;
//
//			if ( IGNCNTR_L < 1 || IGNCNTR_L <= IGNCNTR_R )
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "ECU %X  INF $F816 IGNCNTR-L must return a value of 1 or greater and greater than IGNCNTR-R after a Service $14 code clear\n",
//				      GetEcuId ( EcuIndex ) );
//				RetCode = FAIL;
//			}
//		}
//		else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 22 &&  // IF Test 9.22
//		          IGNCNTR_L <= INF_Inf_5_17[EcuIndex].INF16_IGNCNTR_L )
//		{
//			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//			      "ECU %X  INF $F816 IGNCNTR-L must return a value greater than the value returned in Test 5.17 (%lu)\n",
//			      GetEcuId ( EcuIndex ),
//			      INF_Inf_5_17[EcuIndex].INF16_IGNCNTR_L );
//			RetCode = FAIL;
//		}
//	}

	// log FEOCNTR_R
	FEOCNTR_R = (gOBDResponse[EcuIndex].Inf[11] << 24 ) + (gOBDResponse[EcuIndex].Inf[12] << 16 ) + (gOBDResponse[EcuIndex].Inf[13] << 8 ) + (gOBDResponse[EcuIndex].Inf[14] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F816  FEOCNTR-R = %lu\n",
	      GetEcuId ( EcuIndex ),
	      FEOCNTR_R );
//	if ( gUserInput.eComplianceType == US_OBDII )
//	{
//		if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
//		{
//			INF_Inf_5_17[EcuIndex].INF16_FEOCNTR_R = FEOCNTR_R;
//
//			if ( FEOCNTR_R > 1 )
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "ECU %X  INF $F816 FEOCNTR-R must return a value of 0 or 1 after a Service $14 code clear\n",
//				      GetEcuId ( EcuIndex ) );
//				RetCode = FAIL;
//			}
//		}
//		else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 22 )  // IF Test 9.22
//		{
//			if ( FEOCNTR_R <= INF_Inf_5_17[EcuIndex].INF16_FEOCNTR_R && gOBDHybridFlag != TRUE && gOBDPlugInFlag != TRUE )  // NOT Hybrid/PHEV
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "ECU %X  INF $F816 FEOCNTR-R must return a value greater than the value returned in Test 5.17 (%lu)\n",
//				      GetEcuId ( EcuIndex ),
//				      INF_Inf_5_17[EcuIndex].INF16_FEOCNTR_R );
//				RetCode = FAIL;
//			}
//			else if ( FEOCNTR_R < INF_Inf_5_17[EcuIndex].INF16_FEOCNTR_R  && (gOBDHybridFlag == TRUE || gOBDPlugInFlag == TRUE) )  // Hybrid/PHEV
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "ECU %X  INF $F816 FEOCNTR-R must return a value greater than or equal to the value returned in Test 5.17 (%lu)\n",
//				      GetEcuId ( EcuIndex ),
//				      INF_Inf_5_17[EcuIndex].INF16_FEOCNTR_R );
//				RetCode = FAIL;
//			}
//		}
//		else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
//		          FEOCNTR_R != 0 &&  FEOCNTR_R != 4294967295 )
//		{
//			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//			      "ECU %X  INF $F816 FEOCNTR-R must return a value of 0 after a Service $14 code clear\n",
//			      GetEcuId ( EcuIndex ) );
//			RetCode = FAIL;
//		}
//	}

	// log FEOCNTR_L
	FEOCNTR_L = (gOBDResponse[EcuIndex].Inf[15] << 24 ) + (gOBDResponse[EcuIndex].Inf[16] << 16 ) + (gOBDResponse[EcuIndex].Inf[17] << 8 ) + (gOBDResponse[EcuIndex].Inf[18] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F816  FEOCNTR-L = %lu\n",
	      GetEcuId ( EcuIndex ),
	      FEOCNTR_L );
//	if ( gUserInput.eComplianceType == US_OBDII )
//	{
//		if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
//		{
//			INF_Inf_5_17[EcuIndex].INF16_FEOCNTR_L = FEOCNTR_L;
//
//			if ( (FEOCNTR_L < 1 || FEOCNTR_L <= FEOCNTR_R) && gOBDHybridFlag != TRUE && gOBDPlugInFlag != TRUE )  // NOT Hybrid/PHEV
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "ECU %X  INF $F816 FEOCNTR-L must return a value of 1 or greater and greater than FEOCNTR-R after a Service $14 code clear\n",
//				      GetEcuId ( EcuIndex ) );
//				RetCode = FAIL;
//			}
//		}
//		else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 22 )  // IF Test 9.22
//		{
//			if ( FEOCNTR_L <= INF_Inf_5_17[EcuIndex].INF16_FEOCNTR_L && gOBDHybridFlag != TRUE && gOBDPlugInFlag != TRUE )  // NOT Hybrid/PHEV
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "ECU %X  INF $F816 FEOCNTR-L must return a value greater than the value returned in Test 5.17 (%lu)\n",
//				      GetEcuId ( EcuIndex ),
//				      INF_Inf_5_17[EcuIndex].INF16_FEOCNTR_L );
//				RetCode = FAIL;
//			}
//			else if ( FEOCNTR_L < INF_Inf_5_17[EcuIndex].INF16_FEOCNTR_L  && (gOBDHybridFlag == TRUE || gOBDPlugInFlag == TRUE) )  // Hybrid/PHEV
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "ECU %X  INF $F816 FEOCNTR-L must return a value greater than or equal to the value returned in Test 5.17 (%lu)\n",
//				      GetEcuId ( EcuIndex ),
//				      INF_Inf_5_17[EcuIndex].INF16_FEOCNTR_L );
//				RetCode = FAIL;
//			}
//		}
//	}

	// log ERT_R
	ERT_R = (gOBDResponse[EcuIndex].Inf[19] << 24 ) + (gOBDResponse[EcuIndex].Inf[20] << 16 ) + (gOBDResponse[EcuIndex].Inf[21] << 8 ) + (gOBDResponse[EcuIndex].Inf[22] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F816  ERT-R     = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      ERT_R );
//	if ( gUserInput.eComplianceType == US_OBDII )
//	{
//		if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
//		{
//			INF_Inf_5_17[EcuIndex].INF16_ERT_R = ERT_R;
//
//			if ( ERT_R <= 1 && gOBDHybridFlag != TRUE && gOBDPlugInFlag != TRUE )  // NOT Hybrid/PHEV
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "ECU %X  INF $F816 ERT-R must return a value of greater than 1 after a Service $14 code clear\n",
//				      GetEcuId ( EcuIndex ) );
//				RetCode = FAIL;
//			}
//		}
//		else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 22 )  // IF Test 9.22
//		{
//			if ( ERT_R <= INF_Inf_5_17[EcuIndex].INF16_ERT_R && gOBDHybridFlag != TRUE && gOBDPlugInFlag != TRUE )  // NOT Hybrid/PHEV
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "ECU %X  INF $F816 ERT-R must return a value greater than the value returned in Test 5.17 (%lu)\n",
//				      GetEcuId ( EcuIndex ),
//				      INF_Inf_5_17[EcuIndex].INF16_ERT_R );
//				RetCode = FAIL;
//			}
//			else if ( ERT_R < INF_Inf_5_17[EcuIndex].INF16_ERT_R  && (gOBDHybridFlag == TRUE || gOBDPlugInFlag == TRUE) )  // Hybrid/PHEV
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "ECU %X  INF $F816 ERT-R must return a value greater than or equal to the value returned in Test 5.17 (%lu)\n",
//				      GetEcuId ( EcuIndex ),
//				      INF_Inf_5_17[EcuIndex].INF16_ERT_R );
//				RetCode = FAIL;
//			}
//		}
//		else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
//		          ERT_R != 0 && ERT_R != 4294967295 )
//		{
//			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//			      "ECU %X  INF $F816 ERT-R must return a value of 0 after a Service $14 code clear\n",
//			      GetEcuId ( EcuIndex ) );
//			RetCode = FAIL;
//		}
//	}

	// log ERT_L
	ERT_L = (gOBDResponse[EcuIndex].Inf[23] << 24 ) + (gOBDResponse[EcuIndex].Inf[24] << 16 ) + (gOBDResponse[EcuIndex].Inf[25] << 8 ) + (gOBDResponse[EcuIndex].Inf[26] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F816  ERT-L     = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      ERT_L );
//	if ( gUserInput.eComplianceType == US_OBDII )
//	{
//		if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
//		{
//			INF_Inf_5_17[EcuIndex].INF16_ERT_L = ERT_L;
//
//			if ( ERT_L <= 1 && gOBDHybridFlag != TRUE && gOBDPlugInFlag != TRUE )  // NOT Hybrid/PHEV
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "ECU %X  INF $F816 ERT-L must return a value greater than 1 after a Service $14 code clear\n",
//				      GetEcuId ( EcuIndex ) );
//				RetCode = FAIL;
//			}
//		}
//		else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 22 )  // IF Test 9.22
//		{
//			if ( ERT_L <= INF_Inf_5_17[EcuIndex].INF16_ERT_L && gOBDHybridFlag != TRUE && gOBDPlugInFlag != TRUE )  // NOT Hybrid/PHEV
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "ECU %X  INF $F816 ERT-L must return a value greater than the value returned in Test 5.17 (%lu)\n",
//				      GetEcuId ( EcuIndex ),
//				      INF_Inf_5_17[EcuIndex].INF16_ERT_L );
//				RetCode = FAIL;
//			}
//			else if ( ERT_L < INF_Inf_5_17[EcuIndex].INF16_ERT_L  && (gOBDHybridFlag == TRUE || gOBDPlugInFlag == TRUE) )  // Hybrid/PHEV
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "ECU %X  INF $F816 ERT-L must return a value greater than or equal to the value returned in Test 5.17 (%lu)\n",
//				      GetEcuId ( EcuIndex ),
//				      INF_Inf_5_17[EcuIndex].INF16_ERT_L );
//				RetCode = FAIL;
//			}
//		}
//	}

	// log IERT_R
	IERT_R = (gOBDResponse[EcuIndex].Inf[27] << 24 ) + (gOBDResponse[EcuIndex].Inf[28] << 16 ) + (gOBDResponse[EcuIndex].Inf[29] << 8 ) + (gOBDResponse[EcuIndex].Inf[30] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F816  IERT-R    = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      IERT_R );
//	if ( gUserInput.eComplianceType == US_OBDII )
//	{
//		if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
//		{
//			if ( IERT_R <= 1 && gOBDHybridFlag != TRUE &&  gOBDPlugInFlag != TRUE )  // NOT Hybrid/PHEV
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "ECU %X  INF $F816 IERT_R must return a value of greater than 1 after a Service $14 code clear\n",
//				      GetEcuId ( EcuIndex ) );
//				RetCode = FAIL;
//			}
//		}
//		else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
//		          IERT_R != 0 && IERT_R != 4294967295 )
//		{
//			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//			      "ECU %X  INF $F816 IERT-R must return a value of 0 after a Service $14 code clear\n",
//			      GetEcuId ( EcuIndex ) );
//			RetCode = FAIL;
//		}
//	}

	// log IERT_L
	IERT_L = (gOBDResponse[EcuIndex].Inf[31] << 24 ) + (gOBDResponse[EcuIndex].Inf[32] << 16 ) + (gOBDResponse[EcuIndex].Inf[33] << 8 ) + (gOBDResponse[EcuIndex].Inf[34] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F816  IERT-L    = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      IERT_L );
//	if ( gUserInput.eComplianceType == US_OBDII )
//	{
//		if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
//		{
//			INF_Inf_5_17[EcuIndex].INF16_IERT_L = IERT_L;
//
//			if ( IERT_L <= 1 && gOBDHybridFlag != TRUE && gOBDPlugInFlag != TRUE )  // NOT Hybrid/PHEV
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "ECU %X  INF $F816 IERT-L must return a value of greater than 1 after a Service $14 code clear\n",
//				      GetEcuId ( EcuIndex ) );
//				RetCode = FAIL;
//			}
//		}
//		else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 22 )  // IF Test 9.22
//		{
//			if ( IERT_L <= INF_Inf_5_17[EcuIndex].INF16_IERT_L && gOBDHybridFlag != TRUE && gOBDPlugInFlag != TRUE )  // NOT Hybrid/PHEV
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "ECU %X  INF $F816 IERT-L must return a value greater than the value returned in Test 5.17 (%lu)\n",
//				      GetEcuId ( EcuIndex ),
//				      INF_Inf_5_17[EcuIndex].INF16_IERT_L );
//				RetCode = FAIL;
//			}
//			else if ( IERT_L < INF_Inf_5_17[EcuIndex].INF16_IERT_L  && (gOBDHybridFlag == TRUE || gOBDPlugInFlag == TRUE) )  // Hybrid/PHEV
//			{
//				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//				      "ECU %X  INF $F816 IERT-L must return a value greater than or equal to the value returned in Test 5.17 (%lu)\n",
//				      GetEcuId ( EcuIndex ),
//				      INF_Inf_5_17[EcuIndex].INF16_IERT_L );
//				RetCode = FAIL;
//			}
//		}
//	}

	return RetCode;
}


/*
*******************************************************************************
**
**  Function:   VerifyInf17Data
**
**  Purpose:    Purpose of this function is to verify the Distance Traveled / Fuel Consumed data
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf17Data ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF *pINF;
	float DT_R;
	float DT_L;
	float FC_R;
	float FC_L;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F817 data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// DT/FC Data must contain 16 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 16 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F817  Data Size = %d (Must be 16 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// INF INF17 NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F817  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}

	// log DT_R
	DT_R = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[3] << 24) + (gOBDResponse[EcuIndex].Inf[3] << 16) + (gOBDResponse[EcuIndex].Inf[5] << 8) + (gOBDResponse[EcuIndex].Inf[6])) * 0.1);

	// if it's the max value, not supported
	if ( DT_R == 429496729.5 )
	{
//		if ( gUserInput.eComplianceType == EOBD_NO_IUMPR ||
//		     gUserInput.eComplianceType == EOBD_WITH_IUMPR ||
//		     ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 ) )
//		{
//			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//			      "ECU %X  INF $F817 DT-R not supported\n",
//			      GetEcuId ( EcuIndex ) );
//		}
//		else
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F817 DT-R not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
	}
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F817  DT-R = %.1f km\n",
		      GetEcuId ( EcuIndex ),
		      DT_R );

		if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
		{
			INF_Inf_5_17[EcuIndex].INF17_DT_R = DT_R;  // Save value for comaparison in Test 9.17

			if ( DT_R != 0 )
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  INF $F817 DT-R must return a value of 0 after a Service $14 code clear\n",
				      GetEcuId ( EcuIndex ) );
				RetCode = FAIL;
			}
		}
		else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 22 &&  // IF Test 9.22
		          DT_R <= INF_Inf_5_17[EcuIndex].INF17_DT_R )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F817 DT-R must return a value greater than the value returned in Test 5.17 (%.1f)\n",
			      GetEcuId ( EcuIndex ),
			      INF_Inf_5_17[EcuIndex].INF17_DT_R );
			RetCode = FAIL;
		}
		else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
		          DT_R != 0 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F817 DT-R must return a value of 0 after a Service $14 code clear\n",
			      GetEcuId ( EcuIndex ) );
			RetCode = FAIL;
		}
	}

	// log DT_L
	DT_L = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[7] << 24) + (gOBDResponse[EcuIndex].Inf[8] << 16) + (gOBDResponse[EcuIndex].Inf[9] << 8) + (gOBDResponse[EcuIndex].Inf[10])) * 0.1);

	// if it's the max value, not supported
	if ( DT_L == 429496729.5 )
	{
		if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 )
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F817 DT-L not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
		else
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F817 DT-L not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
	}
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F817  DT-L = %.1f km\n",
		      GetEcuId ( EcuIndex ),
		      DT_L );

		if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
		{
			INF_Inf_5_17[EcuIndex].INF17_DT_L = DT_L;  // Save value for comaparison in Test 9.22
		}
		else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 22 &&  // IF Test 9.22
		          DT_L <= INF_Inf_5_17[EcuIndex].INF17_DT_L )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F817 DT-L must return a value greater than the value returned in Test 5.17 (%.1f)\n",
			      GetEcuId ( EcuIndex ),
			      INF_Inf_5_17[EcuIndex].INF17_DT_L );
			RetCode = FAIL;
		}
	}

	// log FC_R
	FC_R = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[11] << 24) + (gOBDResponse[EcuIndex].Inf[12] << 16) + (gOBDResponse[EcuIndex].Inf[13] << 8) + (gOBDResponse[EcuIndex].Inf[14])) * 0.01);

	// if it's the max value, not supported
	if ( FC_R == 42949672.95 )
	{
//		if ( gUserInput.eComplianceType == EOBD_NO_IUMPR ||
//		     gUserInput.eComplianceType == EOBD_WITH_IUMPR ||
//		     ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 ) )
//		{
//			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//			      "ECU %X  INF $F817 FC-R not supported\n",
//			      GetEcuId ( EcuIndex ) );
//		}
//		else
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F817 FC-R not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
	}
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F817  FC-R = %.2f l\n",
		      GetEcuId ( EcuIndex ),
		      FC_R );

		if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
		{
			INF_Inf_5_17[EcuIndex].INF17_FC_R = FC_R;  // Save value for comaparison in Test 9.22
		}
		else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 22)  // IF Test 9.22
		{
			if ( FC_R <= INF_Inf_5_17[EcuIndex].INF17_FC_R &&
			     gOBDHybridFlag != TRUE && gOBDPlugInFlag != TRUE )  // NOT Hybrid/PHEV
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  INF $F817 FC-R must return a value greater than the value returned in Test 5.17 (%.2f)\n",
				      GetEcuId ( EcuIndex ),
				      INF_Inf_5_17[EcuIndex].INF17_FC_R );
				RetCode = FAIL;
			}
			else if ( FC_R < INF_Inf_5_17[EcuIndex].INF17_FC_R  &&
			          (gOBDHybridFlag == TRUE || gOBDPlugInFlag == TRUE) )  // Hybrid/PHEV
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  INF $F817 FC-R must return a value greater than or equal to the value returned in Test 5.17 (%.2f)\n",
				      GetEcuId ( EcuIndex ),
				      INF_Inf_5_17[EcuIndex].INF17_FC_R );
				RetCode = FAIL;
			}
		}
		else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
		          FC_R != 0 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F817 FC-R must return a value of 0 after a Service $14 code clear\n",
			      GetEcuId ( EcuIndex ) );
			RetCode = FAIL;
		}
	}

	// log FC_L
	FC_L = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[15] << 24) + (gOBDResponse[EcuIndex].Inf[16] << 16) + (gOBDResponse[EcuIndex].Inf[17] << 8) + (gOBDResponse[EcuIndex].Inf[18])) * 0.01);

	// if it's the max value, not supported
	if ( FC_L == 42949672.95 )
	{
		if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 )
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F817 FC-L not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
		else
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F817 FC-L not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
	}
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F817  FC-L = %.2f l\n",
		      GetEcuId ( EcuIndex ),
		      FC_L );

		if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
		{
			INF_Inf_5_17[EcuIndex].INF17_FC_L = FC_L;  // Save value for comaparison in Test 9.22
		}
		else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 17 )  // IF Test 9.17
		{
			if ( FC_L <= INF_Inf_5_17[EcuIndex].INF17_FC_L &&
			     gOBDHybridFlag != TRUE && gOBDPlugInFlag != TRUE )  // NOT Hybrid/PHEV
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  INF $F817 FC-L must return a value greater than the value returned in Test 5.17 (%.2f)\n",
				      GetEcuId ( EcuIndex ),
				      INF_Inf_5_17[EcuIndex].INF17_FC_L );
				RetCode = FAIL;
			}
			else if ( FC_L < INF_Inf_5_17[EcuIndex].INF17_FC_L &&
			          (gOBDHybridFlag == TRUE || gOBDPlugInFlag == TRUE) )  // Hybrid/PHEV
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  INF $F817 FC-L must return a value greater than or equal to the value returned in Test 5.17 (%.2f)\n",
				      GetEcuId ( EcuIndex ),
				      INF_Inf_5_17[EcuIndex].INF17_FC_L );
				RetCode = FAIL;
			}
		}
	}

	return RetCode;
}


/*
*******************************************************************************
**
**  Function:   VerifyInf18Data
**
**  Purpose:    Purpose of this function is to verify the
**              Positive Kinetic Energy and Engine Output Energy data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf18Data ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF *pINF;
	unsigned long  PKE_R;
	unsigned long  PKE_L;
	float EOE_R;
	float EOE_L;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F818 data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// PKE/EO Data must contain 16 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 16 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F818  Data Size = %d (Must be 16 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// INF INF18 NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F818  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}

	// log PKE_R
	PKE_R = (gOBDResponse[EcuIndex].Inf[3] << 24 ) + (gOBDResponse[EcuIndex].Inf[4] << 16 ) + (gOBDResponse[EcuIndex].Inf[5] << 8 ) + (gOBDResponse[EcuIndex].Inf[6] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F818  PKE-R = %lu km/hr2\n",
	      GetEcuId ( EcuIndex ),
	      PKE_R );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 &&  // IF Test 5.17
	     PKE_R != 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F818 PKE-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}
	else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 22 &&  // IF Test 9.22
	          PKE_R == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F818 PKE-R must return a value greater than 0\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}
	else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	          PKE_R != 0 && PKE_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F818 PKE-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log PKE_L
	PKE_L = (gOBDResponse[EcuIndex].Inf[7] << 24 ) + (gOBDResponse[EcuIndex].Inf[8] << 16 ) + (gOBDResponse[EcuIndex].Inf[9] << 8 ) + (gOBDResponse[EcuIndex].Inf[10] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F818  PKE-L = %lu km/hr2\n",
	      GetEcuId ( EcuIndex ),
	      PKE_L );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 22 &&  // IF Test 9.22
	     PKE_L == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F818 PKE-L must return a value greater than 0\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log EOE_R
	EOE_R = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[11] << 24 ) + (gOBDResponse[EcuIndex].Inf[11] << 16 ) + (gOBDResponse[EcuIndex].Inf[13] << 8 ) + (gOBDResponse[EcuIndex].Inf[14] ))) * (float)(0.1);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F818  EOE-R = %.1f kWh\n",
	      GetEcuId ( EcuIndex ),
	      EOE_R );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		INF_Inf_5_17[EcuIndex].INF18_EOE_R = EOE_R;  // Save value for comaparison in Test 9.22
	}
	else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 22 )  // IF Test 9.22
	{
		if ( EOE_R <= INF_Inf_5_17[EcuIndex].INF18_EOE_R &&
		     gUserInput.ePwrTrnType != PHEV && gOBDHybridFlag != TRUE && gOBDPlugInFlag != TRUE )  // NOT Hybrid/PHEV
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F818 EOE-R must return a value greater than the value returned in Test 5.17 (%.1f)\n",
			      GetEcuId ( EcuIndex ),
			      INF_Inf_5_17[EcuIndex].INF18_EOE_R );
			RetCode = FAIL;
		}
		else if ( EOE_R < INF_Inf_5_17[EcuIndex].INF18_EOE_R  &&
		          (gOBDHybridFlag == TRUE || gOBDPlugInFlag == TRUE) )  // Hybrid/PHEV
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F818 EOE-R must return a value greater than or equal to the value returned in Test 5.17 (%lu)\n",
			      GetEcuId ( EcuIndex ),
			      INF_Inf_5_17[EcuIndex].INF18_EOE_R );
			RetCode = FAIL;
		}
	}
	else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	          EOE_R != 0 && EOE_R != 429496729.5 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F818 EOE-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log EOE_L
	EOE_L = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[15] << 24 ) + (gOBDResponse[EcuIndex].Inf[16] << 16 ) + (gOBDResponse[EcuIndex].Inf[17] << 8 ) + (gOBDResponse[EcuIndex].Inf[18] ))) * (float)(0.1);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F818  EOE-L = %.1f kWh\n",
	      GetEcuId ( EcuIndex ),
	      EOE_L );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		INF_Inf_5_17[EcuIndex].INF18_EOE_L = EOE_L;  // Save value for comaparison in Test 9.22
	}
	else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 22 )  // IF Test 9.22
	{
		if ( EOE_L <= INF_Inf_5_17[EcuIndex].INF18_EOE_L &&
		     gOBDHybridFlag != TRUE && gOBDPlugInFlag != TRUE )  // NOT Hybrid/PHEV
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F818 EOE-L must return a value greater than the value returned in Test 5.17 (%.1f)\n",
			      GetEcuId ( EcuIndex ),
			      INF_Inf_5_17[EcuIndex].INF18_EOE_L );
			RetCode = FAIL;
		}
		else if ( EOE_L < INF_Inf_5_17[EcuIndex].INF18_EOE_L  &&
		          (gOBDHybridFlag == TRUE || gOBDPlugInFlag == TRUE) )  // Hybrid/PHEV
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F818 EOE-L must return a value greater than or equal to the value returned in Test 5.17 (%lu)\n",
			      GetEcuId ( EcuIndex ),
			      INF_Inf_5_17[EcuIndex].INF18_EOE_L );
			RetCode = FAIL;
		}
	}

	return RetCode;
}


/*
*******************************************************************************
**
**  Function:   VerifyInf19Data
**
**  Purpose:    Purpose of this function is to verify the Propulsion System Active data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf19Data ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF *pINF;
	unsigned long PSA_R;
	unsigned long PSA_L;
	unsigned long IPSA_R;
	unsigned long IPSA_L;
	unsigned long CPSA_R;
	unsigned long CPSA_L;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F819 data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// PSA Data must contain 24 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 24 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F819  Data Size = %d (Must be 24 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// INF INF19 NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F819  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}

	// log PSA_R
	PSA_R = (gOBDResponse[EcuIndex].Inf[3] << 24 ) + (gOBDResponse[EcuIndex].Inf[4] << 16 ) + (gOBDResponse[EcuIndex].Inf[5] << 8 ) + (gOBDResponse[EcuIndex].Inf[6] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F819  PSA-R  = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      PSA_R );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		INF_Inf_5_17[EcuIndex].INF19_PSA_R = PSA_R;  // Save value for comaparison in Test 9.22
	}
	else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 22 &&  // IF Test 9.22
	          PSA_R <= INF_Inf_5_17[EcuIndex].INF19_PSA_R )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F819 PSA-R must return a value greater than the value returned in Test 5.17 (%lu)\n",
		      GetEcuId ( EcuIndex ),
		      INF_Inf_5_17[EcuIndex].INF19_PSA_R );
		RetCode = FAIL;
	}
	else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	          PSA_R != 0 && PSA_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F819 PSA-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log PSA_L
	PSA_L = (gOBDResponse[EcuIndex].Inf[7] << 24 ) + (gOBDResponse[EcuIndex].Inf[7] << 16 ) + (gOBDResponse[EcuIndex].Inf[9] << 8 ) + (gOBDResponse[EcuIndex].Inf[10] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F819  PSA-L  = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      PSA_L );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		INF_Inf_5_17[EcuIndex].INF19_PSA_L = PSA_L;  // Save value for comaparison in Test 9.22
	}
	else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 22 &&  // IF Test 9.22
	          PSA_L <= INF_Inf_5_17[EcuIndex].INF19_PSA_L )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F819 PSA-L must return a value greater than the value returned in Test 5.17 (%lu)\n",
		      GetEcuId ( EcuIndex ),
		      INF_Inf_5_17[EcuIndex].INF19_PSA_L );
		RetCode = FAIL;
	}

	// log IPSA_R
	IPSA_R = (gOBDResponse[EcuIndex].Inf[11] << 24 ) + (gOBDResponse[EcuIndex].Inf[12] << 16 ) + (gOBDResponse[EcuIndex].Inf[13] << 8 ) + (gOBDResponse[EcuIndex].Inf[14] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F819  IPSA-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      IPSA_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	          IPSA_R != 0 && IPSA_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F819 IPSA-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log IPSA_L
	IPSA_L = (gOBDResponse[EcuIndex].Inf[15] << 24 ) + (gOBDResponse[EcuIndex].Inf[16] << 16 ) + (gOBDResponse[EcuIndex].Inf[17] << 8 ) + (gOBDResponse[EcuIndex].Inf[18] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F819  IPSA-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      IPSA_L );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		INF_Inf_5_17[EcuIndex].INF19_IPSA_L = IPSA_L;  // Save value for comaparison in Test 9.22
	}
	else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 22 &&  // IF Test 9.22
	          IPSA_L <= INF_Inf_5_17[EcuIndex].INF19_IPSA_L )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F819 IPSA-L must return a value greater than the value returned in Test 5.17 (%lu)\n",
		      GetEcuId ( EcuIndex ),
		      INF_Inf_5_17[EcuIndex].INF19_IPSA_L );
		RetCode = FAIL;
	}

	// log CPSA_R
	CPSA_R = (gOBDResponse[EcuIndex].Inf[19] << 24 ) + (gOBDResponse[EcuIndex].Inf[20] << 16 ) + (gOBDResponse[EcuIndex].Inf[21] << 8 ) + (gOBDResponse[EcuIndex].Inf[22] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F819  CPSA-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      CPSA_R );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		INF_Inf_5_17[EcuIndex].INF19_CPSA_R = CPSA_R;  // Save value for comaparison in Test 9.22

		if ( CPSA_R != 0 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F819 CPSA-R must return a value of 0 after a Service $14 code clear\n",
			      GetEcuId ( EcuIndex ),
			      INF_Inf_5_17[EcuIndex].INF19_CPSA_R );
			RetCode = FAIL;
		}
	}
	else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 22 &&  // IF Test 9.22
	          CPSA_R <= INF_Inf_5_17[EcuIndex].INF19_CPSA_R )
	{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F819 CPSA-R must return a value greater than the value returned in Test 5.17 (%lu)\n",
			      GetEcuId ( EcuIndex ),
			      INF_Inf_5_17[EcuIndex].INF19_CPSA_R );
			RetCode = FAIL;
	}
	else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	          CPSA_R != 0 && CPSA_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F819 CPSA-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log CPSA_L
	CPSA_L = (gOBDResponse[EcuIndex].Inf[23] << 24 ) + (gOBDResponse[EcuIndex].Inf[24] << 16 ) + (gOBDResponse[EcuIndex].Inf[25] << 8 ) + (gOBDResponse[EcuIndex].Inf[26] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F819  CPSA-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      CPSA_L );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		INF_Inf_5_17[EcuIndex].INF19_CPSA_L = CPSA_L;  // Save value for comaparison in Test 9.22
	}
	else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 22 &&  // IF Test 9.22
	          CPSA_L <= INF_Inf_5_17[EcuIndex].INF19_CPSA_L )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F819 CPSA-L must return a value greater than the value returned in Test 5.17 (%lu)\n",
		      GetEcuId ( EcuIndex ),
		      INF_Inf_5_17[EcuIndex].INF19_CPSA_R );
		RetCode = FAIL;
	}

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf1AData
**
**  Purpose:    Purpose of this function is to verify the Plug-In Hybrid Vehicle Distance data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf1AData ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF *pINF;
	float CDEODT_R;
	float CDEODT_L;
	float CDERDT_R;
	float CDERDT_L;
	float CIDT_R;
	float CIDT_L;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F81A data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// PHEVDD Data must contain 24 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 24 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81A  Data Size = %d (Must be 24 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// INF INF1A NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81A  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}

	// log CDEODT_R
	CDEODT_R = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[3] << 24) + (gOBDResponse[EcuIndex].Inf[4] << 16) + (gOBDResponse[EcuIndex].Inf[5] << 8) + (gOBDResponse[EcuIndex].Inf[6])) * 0.1);

	// if it's the max value, not supported
	if ( CDEODT_R == 429496729.5 )
	{
//		if ( gUserInput.eComplianceType == EOBD_NO_IUMPR ||
//		     gUserInput.eComplianceType == EOBD_WITH_IUMPR ||
//		     ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 ) )
//		{
//			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//			      "ECU %X  INF $F81A CDEODT-R not supported\n",
//			      GetEcuId ( EcuIndex ) );
//		}
//		else
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81A CDEODT-R not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
	}
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81A  CDEODT-R = %.1f km\n",
		      GetEcuId ( EcuIndex ),
		      CDEODT_R );

		if ( ( ( gTestPhase == eTestNoDTC && gTestSubsection == 17 ) ||  // IF Test 5.17
		       ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 ) ) &&  // OR Test 9.23
		     CDEODT_R != 0 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81A CDEODT-R must return a value of 0 after a Service $14 code clear\n",
			      GetEcuId ( EcuIndex ) );
			RetCode = FAIL;
		}
	}

	// log CDEODT_L
	CDEODT_L = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[7] << 24) + (gOBDResponse[EcuIndex].Inf[8] << 16) + (gOBDResponse[EcuIndex].Inf[9] << 8) + (gOBDResponse[EcuIndex].Inf[10])) * 0.1);

	// if it's the max value, not supported
	if ( CDEODT_L == 429496729.5 )
	{
		if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 )
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81A CDEODT-L not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
		else
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81A CDEODT-L not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
	}
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81A  CDEODT-L = %.1f km\n",
		      GetEcuId ( EcuIndex ),
		      CDEODT_L );
	}

	// log CDERDT_R
	CDERDT_R = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[11] << 24) + (gOBDResponse[EcuIndex].Inf[12] << 16) + (gOBDResponse[EcuIndex].Inf[13] << 8) + (gOBDResponse[EcuIndex].Inf[14])) * 0.1);

	// if it's the max value, not supported
	if ( CDERDT_R == 429496729.5 )
	{
//		if ( gUserInput.eComplianceType == EOBD_NO_IUMPR ||
//		     gUserInput.eComplianceType == EOBD_WITH_IUMPR ||
//		     ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 ) )
//		{
//			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//			      "ECU %X  INF $F81A CDERDT-R not supported\n",
//			      GetEcuId ( EcuIndex ) );
//		}
//		else
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81A CDERDT-R not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
	}
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81A  CDERDT-R = %.1f km\n",
		      GetEcuId ( EcuIndex ),
		      CDERDT_R );

		if ( ( ( gTestPhase == eTestNoDTC && gTestSubsection == 17 ) ||                 // IF Test 5.17
		       ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 ) ) &&  // OR Test 9.23
		     CDERDT_R != 0 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81A CDERDT-R must return a value of 0 after a Service $14 code clear\n",
			      GetEcuId ( EcuIndex ) );
			RetCode = FAIL;
		}
	}

	// log CDERDT_L
	CDERDT_L = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[15] << 24) + (gOBDResponse[EcuIndex].Inf[16] << 16) + (gOBDResponse[EcuIndex].Inf[17] << 8) + (gOBDResponse[EcuIndex].Inf[18])) * 0.1);

	// if it's the max value, not supported
	if ( CDERDT_L == 429496729.5 )
	{
		if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 )
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81A CDERDT-L not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
		else
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81A CDERDT-L not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
	}
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81A  CDERDT-L = %.1f km\n",
		      GetEcuId ( EcuIndex ),
		      CDERDT_L );
	}

	// log CIDT_R
	CIDT_R = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[19] << 24) + (gOBDResponse[EcuIndex].Inf[20] << 16) + (gOBDResponse[EcuIndex].Inf[21] << 8) + (gOBDResponse[EcuIndex].Inf[22])) * 0.1);

	// if it's the max value, not supported
	if ( CIDT_R == 429496729.5 )
	{
//		if ( gUserInput.eComplianceType == EOBD_NO_IUMPR ||
//		     gUserInput.eComplianceType == EOBD_WITH_IUMPR ||
//		     ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 ) )
//		{
//			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//			      "ECU %X  INF $F81A CIDT-R not supported\n",
//			      GetEcuId ( EcuIndex ) );
//		}
//		else
		{
			Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81A CIDT-R not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
	}
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81A  CIDT-R   = %.1f km\n",
		      GetEcuId ( EcuIndex ),
		      CIDT_R );

		if ( ( ( gTestPhase == eTestNoDTC && gTestSubsection == 17 ) ||                 // IF Test 5.17
		       ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 ) ) &&  // OR Test 9.23
		     CIDT_R != 0 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81A CIDT-R must return a value of 0 after a Service $14 code clear\n",
			      GetEcuId ( EcuIndex ) );
			RetCode = FAIL;
		}
	}

	// log CIDT_L
	CIDT_L = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[23] << 24) + (gOBDResponse[EcuIndex].Inf[24] << 16) + (gOBDResponse[EcuIndex].Inf[25] << 8) + (gOBDResponse[EcuIndex].Inf[26])) * 0.1);

	// if it's the max value, not supported
	if ( CIDT_L == 429496729.5 )
	{
		if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 )
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81A CIDT-L not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
		else
		{
			Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81A CIDT-L not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
	}
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81A  CIDT-L   = %.1f km\n",
		      GetEcuId ( EcuIndex ),
		      CIDT_L );
	}

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf1BData
**
**  Purpose:    Purpose of this function is to verify the Plug-In Hybrid Vehicle Fuel data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf1BData ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF *pINF;
	float CDFC_R;
	float CDFC_L;
	float CIFC_R;
	float CIFC_L;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F81B data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// PHEVFD Data must contain 16 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 16 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81B  Data Size = %d (Must be 16 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// INF INF1B NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81B  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}

	// log CDFC_R
	CDFC_R = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[3] << 24) + (gOBDResponse[EcuIndex].Inf[4] << 16) + (gOBDResponse[EcuIndex].Inf[5] << 8) + (gOBDResponse[EcuIndex].Inf[6])) * 0.01);
	// if it's the max value, not supported
	if ( CDFC_R == 42949672.95 )
	{
//		if ( gUserInput.eComplianceType == EOBD_NO_IUMPR ||
//		     gUserInput.eComplianceType == EOBD_WITH_IUMPR ||
//		     ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 ) )
//		{
//			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//			      "ECU %X  INF $F81B CDFC-R not supported\n",
//			      GetEcuId ( EcuIndex ) );
//		}
//		else
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81B CDFC-R not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
	}
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81B  CDFC-R = %.2f l\n",
		      GetEcuId ( EcuIndex ),
		      CDFC_R );

		if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
		     CDFC_R != 0 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81B CDFC-R must return a value of 0 after a Service $14 code clear\n",
			      GetEcuId ( EcuIndex ) );
			RetCode = FAIL;
		}
	}

	// log CDFC_L
	CDFC_L = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[7] << 24) + (gOBDResponse[EcuIndex].Inf[8] << 16) + (gOBDResponse[EcuIndex].Inf[9] << 8) + (gOBDResponse[EcuIndex].Inf[10])) * 0.01);

	// if it's the max value, not supported
	if ( CDFC_L == 42949672.95 )
	{
		if	( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 )
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81B CDFC-L not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
		else
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81B CDFC-L not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
	}
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81B  CDFC-L = %.2f l\n",
		      GetEcuId ( EcuIndex ),
		      CDFC_L );
	}

	// log CIFC_R
	CIFC_R = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[11] << 24) + (gOBDResponse[EcuIndex].Inf[12] << 16) + (gOBDResponse[EcuIndex].Inf[13] << 8) + (gOBDResponse[EcuIndex].Inf[14])) * 0.01);

	// if it's the max value, not supported
	if ( CIFC_R == 42949672.95 )
	{
//		if ( gUserInput.eComplianceType == EOBD_NO_IUMPR ||
//		     gUserInput.eComplianceType == EOBD_WITH_IUMPR ||
//		     ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 ) )
//		{
//			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//			      "ECU %X  INF $F81B CIFC-R not supported\n",
//			      GetEcuId ( EcuIndex ) );
//		}
//		else
		{
			Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81B CIFC-R not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
	}
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81B  CIFC-R = %.2f l\n",
		      GetEcuId ( EcuIndex ),
		      CIFC_R );

		if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
		     CIFC_R != 0 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81B CIFC-R must return a value of 0 after a Service $14 code clear\n",
			      GetEcuId ( EcuIndex ) );
			RetCode = FAIL;
		}
	}

	// log CIFC_L
	CIFC_L = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[15] << 24) + (gOBDResponse[EcuIndex].Inf[16] << 16) + (gOBDResponse[EcuIndex].Inf[17] << 8) + (gOBDResponse[EcuIndex].Inf[18])) * 0.01);

	// if it's the max value, not supported
	if ( CIFC_L == 42949672.95 )
	{
		if	( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 )
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81B CIFC-L not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
		else
		{
			Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81B CIFC-L not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
	}
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81B  CIFC-L = %.2f l\n",
		      GetEcuId ( EcuIndex ),
		      CIFC_L );
	}

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf1CData
**
**  Purpose:    Purpose of this function is to verify the Plug-In Hybrid Vehicle Grid data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf1CData ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF *pINF;
	float CDEOGE_R;
	float CDEOGE_L;
	float CDERGE_R;
	float CDERGE_L;
	float GE_R;
	float GE_L;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F81C data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// PHEVGD Data must contain 24 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 24 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81C  Data Size = %d (Must be 24 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// INF INF1C NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81C  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}

	// log CDEOGE_R
	CDEOGE_R = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[3] << 24) + (gOBDResponse[EcuIndex].Inf[4] << 16) + (gOBDResponse[EcuIndex].Inf[5] << 8) + (gOBDResponse[EcuIndex].Inf[6])) * 0.1);

	// if it's the max value, not supported
	if ( CDEOGE_R == 429496729.5 )
	{
//		if ( gUserInput.eComplianceType == EOBD_NO_IUMPR ||
//		     gUserInput.eComplianceType == EOBD_WITH_IUMPR ||
//		     ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 ) )
//		{
//			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//			      "ECU %X  INF $F81C CDEOGE-R not supported\n",
//			      GetEcuId ( EcuIndex ) );
//		}
//		else
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81C CDEOGE-R not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
	}
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81C  CDEOGE-R = %.1f kWh\n",
		      GetEcuId ( EcuIndex ),
		      CDEOGE_R );

		if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
		     CDEOGE_R != 0 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81C CDEOGE-R must return a value of 0 after a Service $14 code clear\n",
			      GetEcuId ( EcuIndex ) );
			RetCode = FAIL;
		}
	}

	// log CDEOGE_L
	CDEOGE_L = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[7] << 24) + (gOBDResponse[EcuIndex].Inf[8] << 16) + (gOBDResponse[EcuIndex].Inf[9] << 8) + (gOBDResponse[EcuIndex].Inf[10])) * 0.1);

	// if it's the max value, not supported
	if ( CDEOGE_L == 429496729.5 )
	{
		if	( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 )
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81C CDEOGE-L not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
		else
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81C CDEOGE-L not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
	}
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81C  CDEOGE-L = %.1f kWh\n",
		      GetEcuId ( EcuIndex ),
		      CDEOGE_L );
	}

	// log CDERGE_R
	CDERGE_R = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[11] << 24) + (gOBDResponse[EcuIndex].Inf[12] << 16) + (gOBDResponse[EcuIndex].Inf[13] << 8) + (gOBDResponse[EcuIndex].Inf[14])) * 0.1);

	// if it's the max value, not supported
	if ( CDERGE_R == 429496729.5 )
	{
//		if ( gUserInput.eComplianceType == EOBD_NO_IUMPR ||
//		     gUserInput.eComplianceType == EOBD_WITH_IUMPR ||
//		     ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 ) )
//		{
//			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//			      "ECU %X  INF $F81C CDERGE-R not supported\n",
//			      GetEcuId ( EcuIndex ) );
//		}
//		else
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81C CDERGE-R not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
	}
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81C  CDERGE-R = %.1f kWh\n",
		      GetEcuId ( EcuIndex ),
		      CDERGE_R );

		if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
		     CDERGE_R != 0 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81C CDERGE-R must return a value of 0 after a Service $14 code clear\n",
			      GetEcuId ( EcuIndex ) );
			RetCode = FAIL;
		}
	}

	// log CDERGE_L
	CDERGE_L = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[15] << 24) + (gOBDResponse[EcuIndex].Inf[16] << 16) + (gOBDResponse[EcuIndex].Inf[17] << 8) + (gOBDResponse[EcuIndex].Inf[18])) * 0.1);

	// if it's the max value, not supported
	if ( CDERGE_L == 429496729.5 )
	{
		if	( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 )
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81C CDERGE-L not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
		else
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81C CDERGE-L not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
	}
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81C  CDERGE-L = %.1f kWh\n",
		      GetEcuId ( EcuIndex ),
		      CDERGE_L );
	}

	// log GE_R
	GE_R = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[19] << 24) + (gOBDResponse[EcuIndex].Inf[20] << 16) + (gOBDResponse[EcuIndex].Inf[21] << 8) + (gOBDResponse[EcuIndex].Inf[22])) * 0.1);

	// if it's the max value, not supported
	if ( GE_R == 429496729.5 )
	{
//		if ( gUserInput.eComplianceType == EOBD_NO_IUMPR ||
//		     gUserInput.eComplianceType == EOBD_WITH_IUMPR ||
//		     ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 ) )
//		{
//			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//			      "ECU %X  INF $F81C GE-R not supported\n",
//			      GetEcuId ( EcuIndex ) );
//		}
//		else
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81C GE-R not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
	}
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81C  GE-R     = %.1f kWh\n",
		      GetEcuId ( EcuIndex ),
		      GE_R );

		if ( ( ( gTestPhase == eTestNoDTC && gTestSubsection == 17 ) ||                 // IF Test 5.17
		       ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 ) ) &&  // OR Test 9.23
		     GE_R != 0 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81C GE-R must return a value of 0 after a Service $14 code clear\n",
			      GetEcuId ( EcuIndex ) );
			RetCode = FAIL;
		}
	}

	// log GE_L
	GE_L = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[23] << 24) + (gOBDResponse[EcuIndex].Inf[24] << 16) + (gOBDResponse[EcuIndex].Inf[25] << 8) + (gOBDResponse[EcuIndex].Inf[26])) * 0.1);

	// if it's the max value, not supported
	if ( GE_L == 429496729.5 )
	{
		if	( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 )
		{
			Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81C GE-L not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
		else
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F81C GE-L not supported\n",
			      GetEcuId ( EcuIndex ) );
		}
	}
	else
	{
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81C  GE-L     = %.1f kWh\n",
		      GetEcuId ( EcuIndex ),
		      GE_L );
	}

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf1DData
**
**  Purpose:    Purpose of this function is to verify the
**              Active Aerodynamic Features #1 Off-Cycle Credit Vehicle data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf1DData ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF *pINF;
	unsigned long AGSA_TIME1_R;
	unsigned long AGSA_TIME2_R;
	unsigned long AGSA_TIME1_L;
	unsigned long AGSA_TIME2_L;
	unsigned long AGSB_TIME1_R;
	unsigned long AGSB_TIME2_R;
	unsigned long AGSB_TIME1_L;
	unsigned long AGSB_TIME2_L;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F81D data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// AAF1OCCVD Data must contain 32 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 32 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81D  Data Size = %d (Must be 32 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// INF INF1D NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81D  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}

	// log AGSA_TIME1_R
	AGSA_TIME1_R = (gOBDResponse[EcuIndex].Inf[3] << 24 ) + (gOBDResponse[EcuIndex].Inf[4] << 16 ) + (gOBDResponse[EcuIndex].Inf[5] << 8 ) + (gOBDResponse[EcuIndex].Inf[6] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F81D  AGSA_TIME1-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      AGSA_TIME1_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     AGSA_TIME1_R != 0 && AGSA_TIME1_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81D AGSA_TIME1-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log AGSA_TIME2_R
	AGSA_TIME2_R = (gOBDResponse[EcuIndex].Inf[7] << 24 ) + (gOBDResponse[EcuIndex].Inf[8] << 16 ) + (gOBDResponse[EcuIndex].Inf[9] << 8 ) + (gOBDResponse[EcuIndex].Inf[10] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F81D  AGSA_TIME2-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      AGSA_TIME2_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     AGSA_TIME2_R != 0 && AGSA_TIME2_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81D AGSA_TIME2-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log AGSA_TIME1_L
	AGSA_TIME1_L = (gOBDResponse[EcuIndex].Inf[11] << 24 ) + (gOBDResponse[EcuIndex].Inf[12] << 16 ) + (gOBDResponse[EcuIndex].Inf[13] << 8 ) + (gOBDResponse[EcuIndex].Inf[14] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F81D  AGSA_TIME1-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      AGSA_TIME1_L );

	// log AGSA_TIME2_L
	AGSA_TIME2_L = (gOBDResponse[EcuIndex].Inf[15] << 24 ) + (gOBDResponse[EcuIndex].Inf[16] << 16 ) + (gOBDResponse[EcuIndex].Inf[17] << 8 ) + (gOBDResponse[EcuIndex].Inf[18] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F81D  AGSA_TIME2-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      AGSA_TIME2_L );

	// log AGSB_TIME1_R
	AGSB_TIME1_R = (gOBDResponse[EcuIndex].Inf[19] << 24 ) + (gOBDResponse[EcuIndex].Inf[20] << 16 ) + (gOBDResponse[EcuIndex].Inf[21] << 8 ) + (gOBDResponse[EcuIndex].Inf[22] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F81D  AGSB_TIME1-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      AGSB_TIME1_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     AGSB_TIME1_R != 0 && AGSB_TIME1_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81D AGSB_TIME1-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log AGSB_TIME2_R
	AGSB_TIME2_R = (gOBDResponse[EcuIndex].Inf[23] << 24 ) + (gOBDResponse[EcuIndex].Inf[24] << 16 ) + (gOBDResponse[EcuIndex].Inf[25] << 8 ) + (gOBDResponse[EcuIndex].Inf[26] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F81D  AGSB_TIME2-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      AGSB_TIME2_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     AGSB_TIME2_R != 0 && AGSB_TIME2_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81D AGSB_TIME2-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log AGSB_TIME1_L
	AGSB_TIME1_L = (gOBDResponse[EcuIndex].Inf[27] << 24 ) + (gOBDResponse[EcuIndex].Inf[28] << 16 ) + (gOBDResponse[EcuIndex].Inf[29] << 8 ) + (gOBDResponse[EcuIndex].Inf[30] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F81D  AGSB_TIME1-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      AGSB_TIME1_L );

	// log AGSB_TIME2_L
	AGSB_TIME2_L = (gOBDResponse[EcuIndex].Inf[31] << 24 ) + (gOBDResponse[EcuIndex].Inf[32] << 16 ) + (gOBDResponse[EcuIndex].Inf[33] << 8 ) + (gOBDResponse[EcuIndex].Inf[34] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F81D  AGSB_TIME2-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      AGSB_TIME2_L );

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf1EData
**
**  Purpose:    Purpose of this function is to verify the
**              Active Aerodynamic Features #2 Off-Cycle Credit Vehicle data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf1EData ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF *pINF;
	unsigned long VRHC_TIME1_R;
	unsigned long VRHC_TIME2_R;
	unsigned long VRHC_TIME1_L;
	unsigned long VRHC_TIME2_L;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F81E data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// AAF2OCCVD Data must contain 16 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 16 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81E  Data Size = %d (Must be 16 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// INF INF1E NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81E  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}

	// log VRHC_TIME1_R
	VRHC_TIME1_R = (gOBDResponse[EcuIndex].Inf[3] << 24 ) + (gOBDResponse[EcuIndex].Inf[4] << 16 ) + (gOBDResponse[EcuIndex].Inf[5] << 8 ) + (gOBDResponse[EcuIndex].Inf[6] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F81E  VRHC_TIME1-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      VRHC_TIME1_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     VRHC_TIME1_R != 0 && VRHC_TIME1_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81E VRHC_TIME1-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log VRHC_TIME2_R
	VRHC_TIME2_R = (gOBDResponse[EcuIndex].Inf[7] << 24 ) + (gOBDResponse[EcuIndex].Inf[8] << 16 ) + (gOBDResponse[EcuIndex].Inf[9] << 8 ) + (gOBDResponse[EcuIndex].Inf[10] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F81E  VRHC_TIME2-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      VRHC_TIME2_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     VRHC_TIME2_R != 0 && VRHC_TIME2_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81E VRHC_TIME2-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log VRHC_TIME1_L
	VRHC_TIME1_L = (gOBDResponse[EcuIndex].Inf[11] << 24 ) + (gOBDResponse[EcuIndex].Inf[12] << 16 ) + (gOBDResponse[EcuIndex].Inf[13] << 8 ) + (gOBDResponse[EcuIndex].Inf[14] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F81E  VRHC_TIME1-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      VRHC_TIME1_L );

	// log VRHC_TIME2_L
	VRHC_TIME2_L = (gOBDResponse[EcuIndex].Inf[15] << 24 ) + (gOBDResponse[EcuIndex].Inf[16] << 16 ) + (gOBDResponse[EcuIndex].Inf[17] << 8 ) + (gOBDResponse[EcuIndex].Inf[18] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F81E  VRHC_TIME2-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      VRHC_TIME2_L );

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf1FData
**
**  Purpose:    Purpose of this function is to verify the
**              Active Aerodynamic Features #3 Off-Cycle Credit Vehicle data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf1FData ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF *pINF;
	unsigned long AAF1_TIME1_R;
	unsigned long AAF1_TIME2_R;
	unsigned long AAF1_TIME1_L;
	unsigned long AAF1_TIME2_L;
	unsigned long AAF2_TIME1_R;
	unsigned long AAF2_TIME2_R;
	unsigned long AAF2_TIME1_L;
	unsigned long AAF2_TIME2_L;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F81F data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// AAF3OCCVD Data must contain 32 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 32 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81F  Data Size = %d (Must be 32 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// INF INF1F NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81F  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}

	// log AAF1_TIME1_R
	AAF1_TIME1_R = (gOBDResponse[EcuIndex].Inf[3] << 24 ) + (gOBDResponse[EcuIndex].Inf[4] << 16 ) + (gOBDResponse[EcuIndex].Inf[5] << 8 ) + (gOBDResponse[EcuIndex].Inf[6] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F81F  AAF1_TIME1-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      AAF1_TIME1_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     AAF1_TIME1_R != 0 && AAF1_TIME1_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81F AAF1_TIME1-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log AAF1_TIME2_R
	AAF1_TIME2_R = (gOBDResponse[EcuIndex].Inf[7] << 24 ) + (gOBDResponse[EcuIndex].Inf[8] << 16 ) + (gOBDResponse[EcuIndex].Inf[9] << 8 ) + (gOBDResponse[EcuIndex].Inf[10] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F81F  AAF1_TIME2-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      AAF1_TIME2_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     AAF1_TIME2_R != 0 && AAF1_TIME2_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81F AAF1_TIME2-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log AAF1_TIME1_L
	AAF1_TIME1_L = (gOBDResponse[EcuIndex].Inf[11] << 24 ) + (gOBDResponse[EcuIndex].Inf[12] << 16 ) + (gOBDResponse[EcuIndex].Inf[13] << 8 ) + (gOBDResponse[EcuIndex].Inf[14] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F81F  AAF1_TIME1-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      AAF1_TIME1_L );

	// log AAF1_TIME2_L
	AAF1_TIME2_L = (gOBDResponse[EcuIndex].Inf[15] << 24 ) + (gOBDResponse[EcuIndex].Inf[16] << 16 ) + (gOBDResponse[EcuIndex].Inf[17] << 8 ) + (gOBDResponse[EcuIndex].Inf[18] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F81F  AAF1_TIME2-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      AAF1_TIME2_L );

	// log AAF2_TIME1_R
	AAF2_TIME1_R = (gOBDResponse[EcuIndex].Inf[19] << 24 ) + (gOBDResponse[EcuIndex].Inf[20] << 16 ) + (gOBDResponse[EcuIndex].Inf[21] << 8 ) + (gOBDResponse[EcuIndex].Inf[22] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F81F  AAF2_TIME1-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      AAF2_TIME1_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     AAF2_TIME1_R != 0 && AAF2_TIME1_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81F AAF2_TIME1-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log AAF2_TIME2_R
	AAF2_TIME2_R = (gOBDResponse[EcuIndex].Inf[23] << 24 ) + (gOBDResponse[EcuIndex].Inf[24] << 16 ) + (gOBDResponse[EcuIndex].Inf[25] << 8 ) + (gOBDResponse[EcuIndex].Inf[26] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F81F  AAF2_TIME2-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      AAF2_TIME2_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     AAF2_TIME2_R != 0 && AAF2_TIME2_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F81F AAF2_TIME2-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log AAF2_TIME1_L
	AAF2_TIME1_L = (gOBDResponse[EcuIndex].Inf[27] << 24 ) + (gOBDResponse[EcuIndex].Inf[28] << 16 ) + (gOBDResponse[EcuIndex].Inf[29] << 8 ) + (gOBDResponse[EcuIndex].Inf[30] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F81F  AAF2_TIME1-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      AAF2_TIME1_L );

	// log AAF2_TIME2_L
	AAF2_TIME2_L = (gOBDResponse[EcuIndex].Inf[31] << 24 ) + (gOBDResponse[EcuIndex].Inf[32] << 16 ) + (gOBDResponse[EcuIndex].Inf[33] << 8 ) + (gOBDResponse[EcuIndex].Inf[34] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F81F  AAF2_TIME2-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      AAF2_TIME2_L );

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf21Data
**
**  Purpose:    Purpose of this function is to verify the
**              Driver Selectable Operating Modes Off-Cycle Credit Vehicle data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf21Data ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF *pINF;
	unsigned long EDSM_TIME1_R;
	unsigned long EDSM_TIME1_L;
	unsigned long DSM1_TIME1_R;
	unsigned long DSM1_TIME1_L;
	unsigned long DSM2_TIME1_R;
	unsigned long DSM2_TIME1_L;
	unsigned long DSM3_TIME1_R;
	unsigned long DSM3_TIME1_L;
	unsigned long DSM4_TIME1_R;
	unsigned long DSM4_TIME1_L;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F821 data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// DSOMOCCVD Data must contain 40 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 40 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F821  Data Size = %d (Must be 40 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// INF INF21 NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F821  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}

	// log EDSM_TIME1_R
	EDSM_TIME1_R = (gOBDResponse[EcuIndex].Inf[3] << 24 ) + (gOBDResponse[EcuIndex].Inf[4] << 16 ) + (gOBDResponse[EcuIndex].Inf[5] << 8 ) + (gOBDResponse[EcuIndex].Inf[6] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F821  EDSM_TIME1-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      EDSM_TIME1_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     EDSM_TIME1_R != 0 && EDSM_TIME1_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F821 EDSM_TIME1-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log EDSM_TIME1_L
	EDSM_TIME1_L = (gOBDResponse[EcuIndex].Inf[7] << 24 ) + (gOBDResponse[EcuIndex].Inf[8] << 16 ) + (gOBDResponse[EcuIndex].Inf[9] << 8 ) + (gOBDResponse[EcuIndex].Inf[10] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F821  EDSM_TIME1-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      EDSM_TIME1_L );

	// log DSM1_TIME1_R
	DSM1_TIME1_R = (gOBDResponse[EcuIndex].Inf[11] << 24 ) + (gOBDResponse[EcuIndex].Inf[12] << 16 ) + (gOBDResponse[EcuIndex].Inf[13] << 8 ) + (gOBDResponse[EcuIndex].Inf[14] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F821  DSM1_TIME1-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      DSM1_TIME1_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     DSM1_TIME1_R != 0 && DSM1_TIME1_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F821 DSM1_TIME1-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log DSM1_TIME1_L
	DSM1_TIME1_L = (gOBDResponse[EcuIndex].Inf[15] << 24 ) + (gOBDResponse[EcuIndex].Inf[16] << 16 ) + (gOBDResponse[EcuIndex].Inf[17] << 8 ) + (gOBDResponse[EcuIndex].Inf[18] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F821  DSM1_TIME1-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      DSM1_TIME1_L );

	// log DSM2_TIME1_R
	DSM2_TIME1_R = (gOBDResponse[EcuIndex].Inf[19] << 24 ) + (gOBDResponse[EcuIndex].Inf[20] << 16 ) + (gOBDResponse[EcuIndex].Inf[21] << 8 ) + (gOBDResponse[EcuIndex].Inf[22] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F821  DSM2_TIME1-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      DSM2_TIME1_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     DSM2_TIME1_R != 0 && DSM2_TIME1_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F821 DSM2_TIME1-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log DSM2_TIME1_L
	DSM2_TIME1_L = (gOBDResponse[EcuIndex].Inf[23] << 24 ) + (gOBDResponse[EcuIndex].Inf[24] << 16 ) + (gOBDResponse[EcuIndex].Inf[25] << 8 ) + (gOBDResponse[EcuIndex].Inf[26] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F821  DSM2_TIME1-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      DSM2_TIME1_L );

	// log DSM3_TIME1_R
	DSM3_TIME1_R = (gOBDResponse[EcuIndex].Inf[27] << 24 ) + (gOBDResponse[EcuIndex].Inf[28] << 16 ) + (gOBDResponse[EcuIndex].Inf[29] << 8 ) + (gOBDResponse[EcuIndex].Inf[30] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F821  DSM3_TIME1-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      DSM3_TIME1_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     DSM3_TIME1_R != 0 && DSM3_TIME1_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F821 DSM3_TIME1-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log DSM3_TIME1_L
	DSM3_TIME1_L = (gOBDResponse[EcuIndex].Inf[31] << 24 ) + (gOBDResponse[EcuIndex].Inf[32] << 16 ) + (gOBDResponse[EcuIndex].Inf[33] << 8 ) + (gOBDResponse[EcuIndex].Inf[34] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F821  DSM3_TIME1-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      DSM3_TIME1_L );

	// log DSM4_TIME1_R
	DSM4_TIME1_R = (gOBDResponse[EcuIndex].Inf[35] << 24 ) + (gOBDResponse[EcuIndex].Inf[36] << 16 ) + (gOBDResponse[EcuIndex].Inf[37] << 8 ) + (gOBDResponse[EcuIndex].Inf[38] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F821  DSM4_TIME1-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      DSM4_TIME1_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     DSM4_TIME1_R != 0 && DSM4_TIME1_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F821 DSM4_TIME1-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log DSM4_TIME1_L
	DSM4_TIME1_L = (gOBDResponse[EcuIndex].Inf[39] << 24 ) + (gOBDResponse[EcuIndex].Inf[40] << 16 ) + (gOBDResponse[EcuIndex].Inf[41] << 8 ) + (gOBDResponse[EcuIndex].Inf[42] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F821  DSM4_TIME1-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      DSM4_TIME1_L );

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf22Data
**
**  Purpose:    Purpose of this function is to verify the
**              Run Time For Stop-Start and Coasting Off-Cylce Credit Vehicle data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf22Data ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF *pINF;
	unsigned long ISS_TIME1_R;
	unsigned long ISS_TIME1_L;
	unsigned long ERC_TIME1_R;
	unsigned long ERC_TIME1_L;
	unsigned long EOC_TIME1_R;
	unsigned long EOC_TIME1_L;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F822 data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// RTSSCOCCVD Data must contain 24 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 24 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F822  Data Size = %d (Must be 24 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// INF INF22 NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F822  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}

	// log ISS_TIME1_R
	ISS_TIME1_R = (gOBDResponse[EcuIndex].Inf[3] << 24 ) + (gOBDResponse[EcuIndex].Inf[4] << 16 ) + (gOBDResponse[EcuIndex].Inf[5] << 8 ) + (gOBDResponse[EcuIndex].Inf[6] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F822  ISS_TIME1-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      ISS_TIME1_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     ISS_TIME1_R != 0 && ISS_TIME1_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F822 ISS_TIME1-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log ISS_TIME1_L
	ISS_TIME1_L = (gOBDResponse[EcuIndex].Inf[7] << 24 ) + (gOBDResponse[EcuIndex].Inf[8] << 16 ) + (gOBDResponse[EcuIndex].Inf[9] << 8 ) + (gOBDResponse[EcuIndex].Inf[10] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F822  ISS_TIME1-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      ISS_TIME1_L );

	// log ERC_TIME1_R
	ERC_TIME1_R = (gOBDResponse[EcuIndex].Inf[11] << 24 ) + (gOBDResponse[EcuIndex].Inf[12] << 16 ) + (gOBDResponse[EcuIndex].Inf[13] << 8 ) + (gOBDResponse[EcuIndex].Inf[14] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F822  ERC_TIME1-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      ERC_TIME1_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     ERC_TIME1_R != 0 && ERC_TIME1_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F822 ERC_TIME1-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log ERC_TIME1_L
	ERC_TIME1_L = (gOBDResponse[EcuIndex].Inf[15] << 24 ) + (gOBDResponse[EcuIndex].Inf[16] << 16 ) + (gOBDResponse[EcuIndex].Inf[17] << 8 ) + (gOBDResponse[EcuIndex].Inf[18] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F822  ERC_TIME1-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      ERC_TIME1_L );

	// log EOC_TIME1_R
	EOC_TIME1_R = (gOBDResponse[EcuIndex].Inf[19] << 24 ) + (gOBDResponse[EcuIndex].Inf[20] << 16 ) + (gOBDResponse[EcuIndex].Inf[21] << 8 ) + (gOBDResponse[EcuIndex].Inf[22] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F822  EOC_TIME1-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      EOC_TIME1_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     EOC_TIME1_R != 0 && EOC_TIME1_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F822 EOC_TIME1-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log EOC_TIME1_L
	EOC_TIME1_L = (gOBDResponse[EcuIndex].Inf[23] << 24 ) + (gOBDResponse[EcuIndex].Inf[24] << 16 ) + (gOBDResponse[EcuIndex].Inf[25] << 8 ) + (gOBDResponse[EcuIndex].Inf[26] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F822  EOC_TIME1-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      EOC_TIME1_L );

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf23Data
**
**  Purpose:    Purpose of this function is to verify the
**              Drive Coaching Technology Off-Cycle Credit Vehicle data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf23Data ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF *pINF;
	unsigned long DCTE1_CNTS_R;
	unsigned long DCTU1_CNTS_R;
	unsigned long DCTE1_CNTS_L;
	unsigned long DCTU1_CNTS_L;
	unsigned long DCTE2_CNTS_R;
	unsigned long DCTU2_CNTS_R;
	unsigned long DCTE2_CNTS_L;
	unsigned long DCTU2_CNTS_L;
	unsigned long DCTE3_CNTS_R;
	unsigned long DCTU3_CNTS_R;
	unsigned long DCTE3_CNTS_L;
	unsigned long DCTU3_CNTS_L;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F823 data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// DCTOCCVD Data must contain 48 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 48 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F823  Data Size = %d (Must be 48 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// INF INF23 NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F823  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}

	// log DCTE1_CNTS_R
	DCTE1_CNTS_R = (gOBDResponse[EcuIndex].Inf[3] << 24 ) + (gOBDResponse[EcuIndex].Inf[4] << 16 ) + (gOBDResponse[EcuIndex].Inf[5] << 8 ) + (gOBDResponse[EcuIndex].Inf[6] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F823  DCTE1_CNTS-R = %lu\n",
	      GetEcuId ( EcuIndex ),
	      DCTE1_CNTS_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     DCTE1_CNTS_R != 0 && DCTE1_CNTS_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F823 DCTE1_CNTS-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log DCTU1_CNTS_R
	DCTU1_CNTS_R = (gOBDResponse[EcuIndex].Inf[7] << 24 ) + (gOBDResponse[EcuIndex].Inf[8] << 16 ) + (gOBDResponse[EcuIndex].Inf[9] << 8 ) + (gOBDResponse[EcuIndex].Inf[10] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F823  DCTU1_CNTS-R = %lu\n",
	      GetEcuId ( EcuIndex ),
	      DCTU1_CNTS_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     DCTU1_CNTS_R != 0 && DCTU1_CNTS_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F823 DCTU1_CNTS-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log DCTE1_CNTS_L
	DCTE1_CNTS_L = (gOBDResponse[EcuIndex].Inf[11] << 24 ) + (gOBDResponse[EcuIndex].Inf[12] << 16 ) + (gOBDResponse[EcuIndex].Inf[13] << 8 ) + (gOBDResponse[EcuIndex].Inf[14] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F823  DCTE1_CNTS-L = %lu\n",
	      GetEcuId ( EcuIndex ),
	      DCTE1_CNTS_L );

	// log DCTU1_CNTS_L
	DCTU1_CNTS_L = (gOBDResponse[EcuIndex].Inf[15] << 24 ) + (gOBDResponse[EcuIndex].Inf[16] << 16 ) + (gOBDResponse[EcuIndex].Inf[17] << 8 ) + (gOBDResponse[EcuIndex].Inf[18] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F823  DCTU1_CNTS-L = %lu\n",
	      GetEcuId ( EcuIndex ),
	      DCTU1_CNTS_L );

	// log DCTE1_CNTS_R
	DCTE2_CNTS_R = (gOBDResponse[EcuIndex].Inf[19] << 24 ) + (gOBDResponse[EcuIndex].Inf[20] << 16 ) + (gOBDResponse[EcuIndex].Inf[21] << 8 ) + (gOBDResponse[EcuIndex].Inf[22] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F823  DCTE2_CNTS-R = %lu\n",
	      GetEcuId ( EcuIndex ),
	      DCTE2_CNTS_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     DCTE2_CNTS_R != 0 && DCTE2_CNTS_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F823 DCTE2_CNTS-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log DCTU2_CNTS_R
	DCTU2_CNTS_R = (gOBDResponse[EcuIndex].Inf[23] << 24 ) + (gOBDResponse[EcuIndex].Inf[24] << 16 ) + (gOBDResponse[EcuIndex].Inf[25] << 8 ) + (gOBDResponse[EcuIndex].Inf[26] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F823  DCTU2_CNTS-R = %lu\n",
	      GetEcuId ( EcuIndex ),
	      DCTU2_CNTS_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     DCTU2_CNTS_R != 0 && DCTU2_CNTS_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F823 DCTU2_CNTS-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log DCTE2_CNTS_L
	DCTE2_CNTS_L = (gOBDResponse[EcuIndex].Inf[27] << 24 ) + (gOBDResponse[EcuIndex].Inf[28] << 16 ) + (gOBDResponse[EcuIndex].Inf[29] << 8 ) + (gOBDResponse[EcuIndex].Inf[30] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F823  DCTE2_CNTS-L = %lu\n",
	      GetEcuId ( EcuIndex ),
	      DCTE2_CNTS_L );

	// log DCTU2_CNTS_L
	DCTU2_CNTS_L = (gOBDResponse[EcuIndex].Inf[31] << 24 ) + (gOBDResponse[EcuIndex].Inf[32] << 16 ) + (gOBDResponse[EcuIndex].Inf[33] << 8 ) + (gOBDResponse[EcuIndex].Inf[34] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F823  DCTU2_CNTS-L = %lu\n",
	      GetEcuId ( EcuIndex ),
	      DCTU2_CNTS_L );

	// log DCTE3_CNTS_R
	DCTE3_CNTS_R = (gOBDResponse[EcuIndex].Inf[35] << 24 ) + (gOBDResponse[EcuIndex].Inf[36] << 16 ) + (gOBDResponse[EcuIndex].Inf[37] << 8 ) + (gOBDResponse[EcuIndex].Inf[38] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F823  DCTE3_CNTS-R = %lu\n",
	      GetEcuId ( EcuIndex ),
	      DCTE3_CNTS_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     DCTE3_CNTS_R != 0 && DCTE3_CNTS_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F823 DCTE3_CNTS-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log DCTU3_CNTS_R
	DCTU3_CNTS_R = (gOBDResponse[EcuIndex].Inf[39] << 24 ) + (gOBDResponse[EcuIndex].Inf[40] << 16 ) + (gOBDResponse[EcuIndex].Inf[41] << 8 ) + (gOBDResponse[EcuIndex].Inf[42] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F823  DCTU3_CNTS-R = %lu\n",
	      GetEcuId ( EcuIndex ),
	      DCTU3_CNTS_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     DCTU3_CNTS_R != 0 && DCTU3_CNTS_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F823 DCTU3_CNTS-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log DCTE3_CNTS_L
	DCTE3_CNTS_L = (gOBDResponse[EcuIndex].Inf[43] << 24 ) + (gOBDResponse[EcuIndex].Inf[44] << 16 ) + (gOBDResponse[EcuIndex].Inf[45] << 8 ) + (gOBDResponse[EcuIndex].Inf[46] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F823  DCTE3_CNTS-L = %lu\n",
	      GetEcuId ( EcuIndex ),
	      DCTE3_CNTS_L );

	// log DCTU3_CNTS_L
	DCTU3_CNTS_L = (gOBDResponse[EcuIndex].Inf[47] << 24 ) + (gOBDResponse[EcuIndex].Inf[48] << 16 ) + (gOBDResponse[EcuIndex].Inf[49] << 8 ) + (gOBDResponse[EcuIndex].Inf[50] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F823  DCTU3_CNTS-L = %lu\n",
	      GetEcuId ( EcuIndex ),
	      DCTU3_CNTS_L );

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf24Data
**
**  Purpose:    Purpose of this function is to verify the
**              Active Powertrain Warm-Up Features Off-CycleCredit Vehicle data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf24Data ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF *pINF;
	unsigned long EWU_TIME_R;
	unsigned long EWU_TIME_L;
	unsigned long TWU_TIME_R;
	unsigned long TWU_TIME_L;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F824 data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// APTWUFOCCVD Data must contain 16 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 16 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F824  Data Size = %d (Must be 16 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// INF INF24 NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F824  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}

	// log EWU_TIME_R
	EWU_TIME_R = (gOBDResponse[EcuIndex].Inf[3] << 24 ) + (gOBDResponse[EcuIndex].Inf[4] << 16 ) + (gOBDResponse[EcuIndex].Inf[5] << 8 ) + (gOBDResponse[EcuIndex].Inf[6] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F824  EWU_TIME-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      EWU_TIME_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     EWU_TIME_R != 0 && EWU_TIME_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F824 EWU_TIME-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log EWU_TIME_L
	EWU_TIME_L = (gOBDResponse[EcuIndex].Inf[7] << 24 ) + (gOBDResponse[EcuIndex].Inf[8] << 16 ) + (gOBDResponse[EcuIndex].Inf[9] << 8 ) + (gOBDResponse[EcuIndex].Inf[10] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F824  EWU_TIME-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      EWU_TIME_L );

	// log TWU_TIME_R
	TWU_TIME_R = (gOBDResponse[EcuIndex].Inf[11] << 24 ) + (gOBDResponse[EcuIndex].Inf[12] << 16 ) + (gOBDResponse[EcuIndex].Inf[13] << 8 ) + (gOBDResponse[EcuIndex].Inf[14] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F824  TWU_TIME-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      TWU_TIME_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     TWU_TIME_R != 0 && TWU_TIME_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F824 TWU_TIME-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ) );
		RetCode = FAIL;
	}

	// log TWU_TIME_L
	TWU_TIME_L = (gOBDResponse[EcuIndex].Inf[15] << 24 ) + (gOBDResponse[EcuIndex].Inf[16] << 16 ) + (gOBDResponse[EcuIndex].Inf[17] << 8 ) + (gOBDResponse[EcuIndex].Inf[18] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F824  TWU_TIME-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      TWU_TIME_L );

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf25Data
**
**  Purpose:    Purpose of this function is to verify the
**              Off-Cycle Credit Technology Vehicle data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf25_29Data ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF *pINF;
	unsigned long OCCT_TIME1_R;
	unsigned long OCCT_TIME2_R;
	unsigned long OCCT_TIME1_L;
	unsigned long OCCT_TIME2_L;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $%04X data\n",
		      GetEcuId ( EcuIndex ),
		      gOBDResponse[EcuIndex].Inf[0] );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// Data must contain 16 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 16 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  Data Size = %d (Must be 16 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// INF INF2x NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      pINF->NumItems );
		RetCode = FAIL;
	}

	// log AOCCTxT1_R
	OCCT_TIME1_R = (gOBDResponse[EcuIndex].Inf[3] << 24 ) + (gOBDResponse[EcuIndex].Inf[4] << 16 ) + (gOBDResponse[EcuIndex].Inf[5] << 8 ) + (gOBDResponse[EcuIndex].Inf[6] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  OCCT%dT1-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      pINF->INFLSB - 24,
	      OCCT_TIME1_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     OCCT_TIME1_R != 0 && OCCT_TIME1_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X OCCT_TIME1-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB );
		RetCode = FAIL;
	}

	// log AOCCTxT2_R
	OCCT_TIME2_R = (gOBDResponse[EcuIndex].Inf[7] << 24 ) + (gOBDResponse[EcuIndex].Inf[8] << 16 ) + (gOBDResponse[EcuIndex].Inf[9] << 8 ) + (gOBDResponse[EcuIndex].Inf[10] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  OCCT%dT2-R = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      pINF->INFLSB - 24,
	      OCCT_TIME2_R );
	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 &&  // IF Test 9.23
	     OCCT_TIME2_R != 0 && OCCT_TIME2_R != 4294967295 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X OCCT_TIME2-R must return a value of 0 after a Service $14 code clear\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB );
		RetCode = FAIL;
	}

	// log AOCCTxT1_L
	OCCT_TIME1_L = (gOBDResponse[EcuIndex].Inf[11] << 24 ) + (gOBDResponse[EcuIndex].Inf[12] << 16 ) + (gOBDResponse[EcuIndex].Inf[13] << 8 ) + (gOBDResponse[EcuIndex].Inf[14] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  OCCT%dT1-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      pINF->INFLSB - 24,
	      OCCT_TIME1_L );

	// log AOCCTxT2_L
	OCCT_TIME2_L = (gOBDResponse[EcuIndex].Inf[15] << 24 ) + (gOBDResponse[EcuIndex].Inf[16] << 16 ) + (gOBDResponse[EcuIndex].Inf[17] << 8 ) + (gOBDResponse[EcuIndex].Inf[18] );
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  OCCT%dT2-L = %lu sec\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      pINF->INFLSB - 24,
	      OCCT_TIME2_L );

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf41_43_45Data
**
**  Purpose:    Purpose of this function is to verify the
**              Engine Run Time / Distance Traveled / Engine Output Energy / Vehicle Fuel Consumption data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf41_43_45Data ( unsigned long EcuIndex )
{
	STATUS  RetCode = PASS;
	INF    *pINF;
	unsigned long ERT;
	float         DT;
	float         EOE;
	float         VFC;
	char          Letter;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $%04X data\n",
		      GetEcuId ( EcuIndex ),
		      gOBDResponse[EcuIndex].Inf[0] );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// INF $41 and $43 must contain 8 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( (pINF->INFLSB == 0x41 || pINF->INFLSB == 0x43 ) &&
	     gOBDResponse[EcuIndex].InfSize - 3 != 8 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  Data Size = %d (Must be 8 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}
	// INF $45 must contain 16 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	else if ( pINF->INFLSB == 0x45 && gOBDResponse[EcuIndex].InfSize - 3 != 16 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F845  Data Size = %d (Must be 16 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}


	// NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      pINF->NumItems );
		RetCode = FAIL;
	}


	switch ( pINF->INFLSB )
	{
		case 0x41:
		{
			Letter = 'A';
		}
		break;
		case 0x43:
		{
			Letter = 'S';
		}
		break;
		case 0x45:
		{
			Letter = 'L';
		}
		break;
		default:
		{
			Letter = '-';
		}
	}


	// log ERT
	if ( pINF->INFLSB == 0x45 )
	{
		ERT = ((gOBDResponse[EcuIndex].Inf[3] << 24 ) + (gOBDResponse[EcuIndex].Inf[4] << 16) + (gOBDResponse[EcuIndex].Inf[5] << 8 ) + gOBDResponse[EcuIndex].Inf[6]) * 10;
	}
	else
	{
		ERT = ((gOBDResponse[EcuIndex].Inf[3] << 8 ) + gOBDResponse[EcuIndex].Inf[4]) * 10;
	}
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  ERT-%c =  %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Letter,
	      ERT );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		if ( pINF->INFLSB == 0x41 )
		{
			INF_Inf_5_17[EcuIndex].INF41_ERT_A = ERT;
		}

		if ( pINF->INFLSB == 0x41 &&
		     !PHEV &&
		     ERT <= 1 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F8%02X ERT-%c must return a value of greater than 1 sec\n",
			      GetEcuId ( EcuIndex ),
			      pINF->INFLSB,
			      Letter );
			RetCode = FAIL;
		}
		else if ( pINF->INFLSB == 0x45 &&
		          !PHEV &&
		          ERT < INF_Inf_5_17[EcuIndex].INF41_ERT_A )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F845 ERT-L must return a value of greater than ERT-A (%lu)\n",
			      GetEcuId ( EcuIndex ),
			      INF_Inf_5_17[EcuIndex].INF41_ERT_A );
			RetCode = FAIL;
		}
	}


	// log DT
	if ( pINF->INFLSB == 0x45 )
	{
		DT = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[7] << 24 ) + (gOBDResponse[EcuIndex].Inf[8] << 16 ) + (gOBDResponse[EcuIndex].Inf[9] << 8 ) + gOBDResponse[EcuIndex].Inf[10]) * .25);
	}
	else
	{
		DT = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[5] << 8 ) + gOBDResponse[EcuIndex].Inf[6]) * .25);
	}
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  DT-%c  =  %10.3f km\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Letter,
	      DT );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		if ( pINF->INFLSB == 0x41 )
		{
			INF_Inf_5_17[EcuIndex].INF41_DT_A = DT;
			if ( DT != 0 )
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  INF $F841 DT-A must return a value of 0 after a Service $14 code clear\n",
				      GetEcuId ( EcuIndex ) );
				RetCode = FAIL;
			}
		}
		else if ( pINF->INFLSB == 0x43 )
		{
			INF_Inf_5_17[EcuIndex].INF43_DT_S = DT;
		}
		else if ( pINF->INFLSB == 0x45 &&
		          !PHEV &&
		          DT < INF_Inf_5_17[EcuIndex].INF41_DT_A )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F845 DT-L must return a value of greater than DT-A (%.2f)\n",
			      GetEcuId ( EcuIndex ),
			      INF_Inf_5_17[EcuIndex].INF41_DT_A );
			RetCode = FAIL;
		}
	}
	else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 )  // IF Test 9.23
	{
		if ( ( pINF->INFLSB == 0x41 && DT <= INF_Inf_5_17[EcuIndex].INF41_DT_A ) ||
		     ( pINF->INFLSB == 0x43 && DT <= INF_Inf_5_17[EcuIndex].INF43_DT_S ) )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F804X DT-%c must return a value of greater than in Test 5.17 (%.2f)\n",
			      GetEcuId ( EcuIndex ),
			      pINF->INFLSB,
			      Letter,
			      pINF->INFLSB == 0x41 ? INF_Inf_5_17[EcuIndex].INF41_DT_A : INF_Inf_5_17[EcuIndex].INF43_DT_S );
			RetCode = FAIL;
		}
	}


	// log EOE
	if ( pINF->INFLSB == 0x45 )
	{
		EOE = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[11] << 24 ) + (gOBDResponse[EcuIndex].Inf[12] << 16 ) + (gOBDResponse[EcuIndex].Inf[13] << 8 ) + gOBDResponse[EcuIndex].Inf[14]) * .001);
	}
	else
	{
		EOE = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[7] << 8 ) + gOBDResponse[EcuIndex].Inf[8]) * .001);
	}
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  EOE-%c = %11.3f MWh\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Letter,
	      EOE );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		if ( pINF->INFLSB == 0x41 )
		{
			INF_Inf_5_17[EcuIndex].INF41_EOE_A = EOE;
		}
		else if ( pINF->INFLSB == 0x43 )
		{
			INF_Inf_5_17[EcuIndex].INF43_EOE_S = EOE;
		}
		else if ( pINF->INFLSB == 0x45 &&
		          !PHEV &&
		          EOE < INF_Inf_5_17[EcuIndex].INF41_EOE_A )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F845 EOE-L must return a value of greater than EOE-A\n",
			      GetEcuId ( EcuIndex ) );
			RetCode = FAIL;
		}
	}
	else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 )  // IF Test 9.23
	{
		if ( ( pINF->INFLSB == 0x41 && EOE <= INF_Inf_5_17[EcuIndex].INF41_EOE_A ) ||
		     ( pINF->INFLSB == 0x43 && EOE <= INF_Inf_5_17[EcuIndex].INF43_EOE_S ) )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F8%02X EOE-%c must return a value of greater than in Test 5.17 (%.3f)\n",
			      GetEcuId ( EcuIndex ),
			      pINF->INFLSB,
			      Letter,
			      pINF->INFLSB == 0x41 ? INF_Inf_5_17[EcuIndex].INF41_EOE_A : INF_Inf_5_17[EcuIndex].INF43_EOE_S );
			RetCode = FAIL;
		}
	}


	// log VFC
	if ( pINF->INFLSB == 0x45 )
	{
		VFC = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[15] << 24 ) + (gOBDResponse[EcuIndex].Inf[16] << 16 ) + (gOBDResponse[EcuIndex].Inf[17] << 8 ) + gOBDResponse[EcuIndex].Inf[18]) * .5);
	}
	else
	{
		VFC = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[9] << 8 ) + gOBDResponse[EcuIndex].Inf[10]) * .5);
	}
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  VFC-%c =  %10.1f l\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Letter,
	      VFC );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		if ( pINF->INFLSB == 0x41 )
		{
			INF_Inf_5_17[EcuIndex].INF41_VFC_A = VFC;
		}
		else if ( pINF->INFLSB == 0x43 )
		{
			INF_Inf_5_17[EcuIndex].INF43_VFC_S = VFC;
		}
		else if ( pINF->INFLSB == 0x45 &&
		          !PHEV &&
		          VFC < INF_Inf_5_17[EcuIndex].INF41_VFC_A )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F845 VFC-L must return a value of greater than VFC-A\n",
			      GetEcuId ( EcuIndex ) );
			RetCode = FAIL;
		}
	}
	else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 )  // IF Test 9.23
	{
		if ( ( pINF->INFLSB == 0x41 && VFC <= INF_Inf_5_17[EcuIndex].INF41_VFC_A ) ||
		     ( pINF->INFLSB == 0x43 && VFC <= INF_Inf_5_17[EcuIndex].INF43_VFC_S ) )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F8%02X VFC-%c must return a value of greater than in Test 5.17 (%.1f)\n",
			      GetEcuId ( EcuIndex ),
			      pINF->INFLSB,
			      Letter,
			      pINF->INFLSB == 0x41 ? INF_Inf_5_17[EcuIndex].INF41_VFC_A : INF_Inf_5_17[EcuIndex].INF43_VFC_S );
			RetCode = FAIL;
		}
	}

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf42_44_46Data
**
**  Purpose:    Purpose of this function is to verify the
**              Engine Fuel Consumption / Idle Run Time / Engine Idle Fuel Consumption /
**              Urban Speed Run Time / Positive Kinetic Energy Numerator / PTO Run Time /
**              PTO Fuel Consumption data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf42_44_46Data ( unsigned long EcuIndex )
{
	STATUS  RetCode = PASS;
	INF    *pINF;
	float         EFC;
	unsigned long IERT;
	float         IEFC;
	unsigned long UERT;
	unsigned long PKE;
	unsigned long PTOERT;
	float         PTOFC;
	char          Letter;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $%04X data\n",
		      GetEcuId ( EcuIndex ),
		      gOBDResponse[EcuIndex].Inf[0] );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// INF $42 and $44 must contain 16 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( (pINF->INFLSB == 0x42 || pINF->INFLSB == 0x44 ) &&
	     gOBDResponse[EcuIndex].InfSize - 3 != 16 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  Data Size = %d (Must be 16 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}
	// INF $46 must contain 16 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	else if ( pINF->INFLSB == 0x46 && gOBDResponse[EcuIndex].InfSize - 3 != 28 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F846  Data Size = %d (Must be 28 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}


	// NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      pINF->NumItems );
		RetCode = FAIL;
	}


	switch ( pINF->INFLSB )
	{
		case 0x42:
		{
			Letter = 'A';
		}
		break;
		case 0x44:
		{
			Letter = 'S';
		}
		break;
		case 0x46:
		{
			Letter = 'L';
		}
		break;
		default:
		{
			Letter = '-';
		}
	}


	// log EFC
	if ( pINF->INFLSB == 0x46 )
	{
		EFC = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[3] << 24 ) + (gOBDResponse[EcuIndex].Inf[4] << 16) + (gOBDResponse[EcuIndex].Inf[5] << 8 ) + gOBDResponse[EcuIndex].Inf[6]) * .5);
	}
	else
	{
		EFC = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[3] << 8 ) + gOBDResponse[EcuIndex].Inf[4]) * .5);
	}
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  EFC-%c    = %10.1f l\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Letter,
	      EFC );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		if ( pINF->INFLSB == 0x42 )
		{
			INF_Inf_5_17[EcuIndex].INF42_EFC_A = EFC;
			if ( EFC != 0 )
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  INF $F842 ERT-A must return a value of 0 after a Service $14 code clear\n",
				      GetEcuId ( EcuIndex ) );
				RetCode = FAIL;
			}
		}
		else if ( pINF->INFLSB == 0x44 )
		{
			INF_Inf_5_17[EcuIndex].INF44_EFC_S = EFC;
		}
		else if ( pINF->INFLSB == 0x46 &&
		          !PHEV &&
		          EFC < INF_Inf_5_17[EcuIndex].INF42_EFC_A )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F846 EFC-L must return a value of greater than EFC-A (%.1f)\n",
			      GetEcuId ( EcuIndex ),
			      INF_Inf_5_17[EcuIndex].INF42_EFC_A );
			RetCode = FAIL;
		}
	}
	else if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 23 )  // IF Test 9.23
	{
		if ( !PHEV &&
		     ( ( pINF->INFLSB == 0x42 && EFC <= INF_Inf_5_17[EcuIndex].INF42_EFC_A ) ||
		       ( pINF->INFLSB == 0x44 && EFC <= INF_Inf_5_17[EcuIndex].INF44_EFC_S ) ) )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F8%02X EFC-%c must return a value of greater than in Test 5.17 (%d)\n",
			      GetEcuId ( EcuIndex ),
			      pINF->INFLSB,
			      Letter,
			      pINF->INFLSB == 0x41 ? INF_Inf_5_17[EcuIndex].INF42_EFC_A : INF_Inf_5_17[EcuIndex].INF44_EFC_S );
			RetCode = FAIL;
		}
		else if ( PHEV &&
		        ( ( pINF->INFLSB == 0x42 && EFC < INF_Inf_5_17[EcuIndex].INF42_EFC_A ) ||
		          ( pINF->INFLSB == 0x44 && EFC < INF_Inf_5_17[EcuIndex].INF44_EFC_S ) ) )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F8%02X EFC-%c must return a value of greater than or equal to Test 5.17 (%d)\n",
			      GetEcuId ( EcuIndex ),
			      pINF->INFLSB,
			      Letter,
			      pINF->INFLSB == 0x41 ? INF_Inf_5_17[EcuIndex].INF42_EFC_A : INF_Inf_5_17[EcuIndex].INF44_EFC_S );
			RetCode = FAIL;
		}
	}


	// log IERT
	if ( pINF->INFLSB == 0x46 )
	{
		IERT = (gOBDResponse[EcuIndex].Inf[7] << 24 ) + (gOBDResponse[EcuIndex].Inf[8] << 16 ) + (gOBDResponse[EcuIndex].Inf[9] << 8 ) + gOBDResponse[EcuIndex].Inf[10];
	}
	else
	{
		IERT = ((gOBDResponse[EcuIndex].Inf[5] << 8 ) + gOBDResponse[EcuIndex].Inf[6]) * 10;
	}
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  IERT-%c   = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Letter,
	      IERT );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		if ( pINF->INFLSB == 0x42 )
		{
			INF_Inf_5_17[EcuIndex].INF42_IERT_A = IERT;
		}

		if ( pINF->INFLSB == 0x42 &&
		     !PHEV &&
		     IERT <= 1 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F8%02X IERT-%c must return a value of greater than 1\n",
			      GetEcuId ( EcuIndex ),
			      pINF->INFLSB,
			      Letter );
			RetCode = FAIL;
		}
		else if ( pINF->INFLSB == 0x46 &&
		          !PHEV &&
		          IERT < INF_Inf_5_17[EcuIndex].INF42_IERT_A )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F846 IERT-L must return a value of greater than IERT-A (%lu)\n",
			      GetEcuId ( EcuIndex ),
			      INF_Inf_5_17[EcuIndex].INF42_IERT_A );
			RetCode = FAIL;
		}
	}


	// log IEFC
	if ( pINF->INFLSB == 0x46 )
	{
		IEFC = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[11] << 24 ) + (gOBDResponse[EcuIndex].Inf[12] << 16 ) + (gOBDResponse[EcuIndex].Inf[13] << 8 ) + gOBDResponse[EcuIndex].Inf[14]) * .5);
	}
	else
	{
		IEFC = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[7] << 8 ) + gOBDResponse[EcuIndex].Inf[8]) * .5);
	}
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  IEFC-%c   = %10.1f l\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Letter,
	      IEFC );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		if ( pINF->INFLSB == 0x42 )
		{
			INF_Inf_5_17[EcuIndex].INF42_IEFC_A = IEFC;
		}

		if ( pINF->INFLSB == 0x42 &&
		     !PHEV &&
		     IEFC <= 1 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F8%02X IEFC-%c must return a value of greater than 1\n",
			      GetEcuId ( EcuIndex ),
			      pINF->INFLSB,
			      Letter );
			RetCode = FAIL;
		}
		else if ( pINF->INFLSB == 0x46 &&
		          !PHEV &&
		          IEFC < INF_Inf_5_17[EcuIndex].INF42_IEFC_A )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F846 IEFC-L must return a value of greater than IEFC-A (%.1f)\n",
			      GetEcuId ( EcuIndex ),
			      INF_Inf_5_17[EcuIndex].INF42_IEFC_A );
			RetCode = FAIL;
		}
	}


	// log UERT
	if ( pINF->INFLSB == 0x46 )
	{
		UERT = (gOBDResponse[EcuIndex].Inf[15] << 24 ) + (gOBDResponse[EcuIndex].Inf[16] << 16 ) + (gOBDResponse[EcuIndex].Inf[17] << 8 ) + gOBDResponse[EcuIndex].Inf[18];
	}
	else
	{
		UERT = ((gOBDResponse[EcuIndex].Inf[9] << 8 ) + gOBDResponse[EcuIndex].Inf[10]) * 10;
	}
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  UERT-%c   = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Letter,
	      UERT );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		if ( pINF->INFLSB == 0x42 )
		{
			INF_Inf_5_17[EcuIndex].INF42_UERT_A = UERT;
			if ( UERT != 0 )
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  INF $F842 UERT-A must return a value of 0 after a Service $14 code clear\n",
				      GetEcuId ( EcuIndex ) );
				RetCode = FAIL;
			}
		}
		else if ( pINF->INFLSB == 0x46 &&
		          !PHEV &&
		          UERT < INF_Inf_5_17[EcuIndex].INF42_UERT_A )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F846 UERT-L must return a value of greater than UERT-A (%lu)\n",
			      GetEcuId ( EcuIndex ),
			      INF_Inf_5_17[EcuIndex].INF42_UERT_A );
			RetCode = FAIL;
		}
	}


	// log PKE
	if ( pINF->INFLSB == 0x46 )
	{
		PKE = ((gOBDResponse[EcuIndex].Inf[19] << 24) + (gOBDResponse[EcuIndex].Inf[20] << 16) + (gOBDResponse[EcuIndex].Inf[21] << 8 ) + gOBDResponse[EcuIndex].Inf[22]) * 5;
	}
	else
	{
		PKE = (gOBDResponse[EcuIndex].Inf[11] << 24) + (gOBDResponse[EcuIndex].Inf[12] << 16) + (gOBDResponse[EcuIndex].Inf[13] << 8 ) + gOBDResponse[EcuIndex].Inf[14];
	}
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  PKE-%c    = %10lu km2/h2\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Letter,
	      PKE );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17  )  // IF Test 5.17
	{
		if ( pINF->INFLSB == 0x42 )
		{
			INF_Inf_5_17[EcuIndex].INF42_PKE_A = PKE;
			if ( PKE != 0 )
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  INF $F842 PKE-A must return a value of 0 after a Service $14 code clear\n",
				      GetEcuId ( EcuIndex ) );
				RetCode = FAIL;
			}
		}
		else if ( pINF->INFLSB == 0x46 &&
		          !PHEV &&
		          PKE < INF_Inf_5_17[EcuIndex].INF42_PKE_A )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F846 PKE-L must return a value of greater than PKE-A (%lu)\n",
			      GetEcuId ( EcuIndex ),
			      INF_Inf_5_17[EcuIndex].INF42_PKE_A );
			RetCode = FAIL;
		}
	}


	// log PTOERT
	if ( pINF->INFLSB == 0x46 )
	{
		PTOERT = (gOBDResponse[EcuIndex].Inf[23] << 24 ) + (gOBDResponse[EcuIndex].Inf[24] << 16 ) + (gOBDResponse[EcuIndex].Inf[25] << 8 ) + gOBDResponse[EcuIndex].Inf[26];
	}
	else
	{
		PTOERT = ((gOBDResponse[EcuIndex].Inf[15] << 8 ) + gOBDResponse[EcuIndex].Inf[16]) * 10;
	}
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  PTOERT-%c = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Letter,
	      PTOERT );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17  )  // IF Test 5.17
	{
		if ( pINF->INFLSB == 0x42 )
		{
			INF_Inf_5_17[EcuIndex].INF42_PTOERT_A = PTOERT;
			if ( PTOERT != 0 )
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  INF $F842 PTOERT-A must return a value of 0 after a Service $14 code clear\n",
				      GetEcuId ( EcuIndex ) );
				RetCode = FAIL;
			}
		}
		else if ( pINF->INFLSB == 0x46 &&
		          !PHEV &&
		          PTOERT < INF_Inf_5_17[EcuIndex].INF42_PTOERT_A )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F846 PTOERT-L must return a value of greater than PTOERT-A (%lu)\n",
			      GetEcuId ( EcuIndex ),
			      INF_Inf_5_17[EcuIndex].INF42_PTOERT_A );
			RetCode = FAIL;
		}
	}


	// log PTOFC
	if ( pINF->INFLSB == 0x46 )
	{
		PTOFC = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[27] << 24 ) + (gOBDResponse[EcuIndex].Inf[28] << 16 ) + (gOBDResponse[EcuIndex].Inf[29] << 8 ) + gOBDResponse[EcuIndex].Inf[30]) * .5);
	}
	else
	{
		PTOFC = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[17] << 8 ) + gOBDResponse[EcuIndex].Inf[18]) * .5);
	}
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  PTOFC-%c  = %10.1f l\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Letter,
	      PTOFC );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17  )  // IF Test 5.17
	{
		if ( pINF->INFLSB == 0x42 )
		{
			INF_Inf_5_17[EcuIndex].INF42_PTOFC_A = PTOFC;
			if ( PTOFC != 0 )
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  INF $F842 PTOFC-A must return a value of 0 after a Service $14 code clear\n",
				      GetEcuId ( EcuIndex ) );
				RetCode = FAIL;
			}
		}
		else if ( pINF->INFLSB == 0x46 &&
		          !PHEV &&
		          PTOFC < INF_Inf_5_17[EcuIndex].INF42_PTOFC_A )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F846 PTOFC-L must return a value of greater than PTOFC-A (%.1f)\n",
			      GetEcuId ( EcuIndex ),
			      INF_Inf_5_17[EcuIndex].INF42_PTOFC_A );
			RetCode = FAIL;
		}
	}

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf47Data
**
**  Purpose:    Purpose of this function is to verify the
**              Total Propulsion System Active Time data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf47Data ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF *pINF;
	unsigned long PSA_A;
	unsigned long PSA_S;
	unsigned long PSA_L;

	unsigned long IPSA_A;
	unsigned long IPSA_S;
	unsigned long IPSA_L;

	unsigned long UPSA_A;
	unsigned long UPSA_S;
	unsigned long UPSA_L;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F847 data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// must contain 24 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 24 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F847  Data Size = %d (Must be 24 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}


	// NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F847  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}


	// log PSA_A
	PSA_A = ((gOBDResponse[EcuIndex].Inf[3] << 8 ) + gOBDResponse[EcuIndex].Inf[4]) * 10;
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F847  PSA-A  = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      PSA_A );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		INF_Inf_5_17[EcuIndex].INF47_PSA_A = PSA_A;
	}

	// log PSA_S
	PSA_S = ((gOBDResponse[EcuIndex].Inf[5] << 8 ) + gOBDResponse[EcuIndex].Inf[6]) * 10;
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F847  PSA-S  = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      PSA_S );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		INF_Inf_5_17[EcuIndex].INF47_PSA_S = PSA_S;
	}

	// log PSA_L
	PSA_L = (gOBDResponse[EcuIndex].Inf[7] << 24 ) + (gOBDResponse[EcuIndex].Inf[8] << 16) +  (gOBDResponse[EcuIndex].Inf[9] << 8) + gOBDResponse[EcuIndex].Inf[10];
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F847  PSA-L  = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      PSA_L );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		INF_Inf_5_17[EcuIndex].INF47_PSA_L = PSA_L;

		if ( PSA_L <= PSA_A )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F847 PSA-L must return a value of greater than PSA_A\n",
			      GetEcuId ( EcuIndex ) );
			RetCode = FAIL;
		}
	}


	// log IPSA_A
	IPSA_A = ((gOBDResponse[EcuIndex].Inf[11] << 8 ) + gOBDResponse[EcuIndex].Inf[12]) * 10;
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F847  IPSA-A = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      IPSA_A );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		INF_Inf_5_17[EcuIndex].INF47_IPSA_A = IPSA_A;
	}

	// log IPSA_S
	IPSA_S = ((gOBDResponse[EcuIndex].Inf[13] << 8 ) + gOBDResponse[EcuIndex].Inf[14]) * 10;
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F847  IPSA-S = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      IPSA_S );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		INF_Inf_5_17[EcuIndex].INF47_IPSA_S = IPSA_S;
	}

	// log IPSA_L
	IPSA_L = (gOBDResponse[EcuIndex].Inf[15] << 24 ) + (gOBDResponse[EcuIndex].Inf[16] << 16) +  (gOBDResponse[EcuIndex].Inf[17] << 8) + gOBDResponse[EcuIndex].Inf[18];
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F847  IPSA-L = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      IPSA_L );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		INF_Inf_5_17[EcuIndex].INF47_IPSA_L = IPSA_L;

		if ( IPSA_L <= IPSA_A )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F847 IPSA-L must return a value of greater than IPSA_A\n",
			      GetEcuId ( EcuIndex ) );
			RetCode = FAIL;
		}
	}


	// log UPSA_A
	UPSA_A = ((gOBDResponse[EcuIndex].Inf[19] << 8 ) + gOBDResponse[EcuIndex].Inf[20]) * 10;
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F847  UPSA-A = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      UPSA_A );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		INF_Inf_5_17[EcuIndex].INF47_UPSA_A = UPSA_A;

		if ( UPSA_A != 0 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F847 UPSA-A must return a value of 0 after a Service $14 code clear\n",
			      GetEcuId ( EcuIndex ) );
			RetCode = FAIL;
		}
	}

	// log UPSA_S
	UPSA_S = ((gOBDResponse[EcuIndex].Inf[21] << 8 ) + gOBDResponse[EcuIndex].Inf[22]) * 10;
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F847  UPSA-S = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      UPSA_S );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		INF_Inf_5_17[EcuIndex].INF47_UPSA_S = UPSA_S;
	}

	// log UPSA_L
	UPSA_L = (gOBDResponse[EcuIndex].Inf[23] << 24 ) + (gOBDResponse[EcuIndex].Inf[24] << 16) +  (gOBDResponse[EcuIndex].Inf[25] << 8) + gOBDResponse[EcuIndex].Inf[26];
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F847  UPSA-L = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      UPSA_L );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		INF_Inf_5_17[EcuIndex].INF47_UPSA_L = UPSA_L;

		if ( UPSA_L <= UPSA_A )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F847 UPSA-L must return a value of greater than UPSA_A\n",
			      GetEcuId ( EcuIndex ) );
			RetCode = FAIL;
		}
	}

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf48Data
**
**  Purpose:    Purpose of this function is to verify the
**              Charge Depleting Operation data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf48Data ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF   *pINF;
	float  CDEODT_A;
	float  CDEODT_S;
	float  CDEODT_L;

	float  CDERDT_A;
	float  CDERDT_S;
	float  CDERDT_L;

	float  CIDT_A;
	float  CIDT_S;
	float  CIDT_L;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F848 data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// must contain 24 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 24 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F848  Data Size = %d (Must be 24 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}


	// NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F848  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}


	// log CDEODT_A
	CDEODT_A = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[3] << 8 ) + gOBDResponse[EcuIndex].Inf[4]) * .25);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F848  CDEODT-A = %9.3f km\n",
	      GetEcuId ( EcuIndex ),
	      CDEODT_A );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		if ( CDEODT_A != 0 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F848 CDEODT-A must return a value of 0 after a Service $14 code clear\n",
			      GetEcuId ( EcuIndex ) );
			RetCode = FAIL;
		}
	}

	// log CDEODT_S
	CDEODT_S = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[5] << 8 ) + gOBDResponse[EcuIndex].Inf[6]) * .25);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F848  CDEODT-S = %8.3f km\n",
	      GetEcuId ( EcuIndex ),
	      CDEODT_S );

	// log CDEODT_L
	CDEODT_L = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[7] << 24 ) + (gOBDResponse[EcuIndex].Inf[8] << 16) +  (gOBDResponse[EcuIndex].Inf[9] << 8) + gOBDResponse[EcuIndex].Inf[10]) * .005);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F848  CDEODT-L = %8.3f km\n",
	      GetEcuId ( EcuIndex ),
	      CDEODT_L );


	// log CDERDT_A
	CDERDT_A = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[11] << 8 ) + gOBDResponse[EcuIndex].Inf[12]) * .25);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F848  CDERDT-A = %9.3f km\n",
	      GetEcuId ( EcuIndex ),
	      CDERDT_A );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		if ( CDERDT_A != 0 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F848 CDERDT-A must return a value of 0 after a Service $14 code clear\n",
			      GetEcuId ( EcuIndex ) );
			RetCode = FAIL;
		}
	}

	// log CDERDT_S
	CDERDT_S = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[13] << 8 ) + gOBDResponse[EcuIndex].Inf[14]) * .25);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F848  CDERDT-S = %8.3f km\n",
	      GetEcuId ( EcuIndex ),
	      CDERDT_S );

	// log CDERDT_L
	CDERDT_L = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[15] << 24 ) + (gOBDResponse[EcuIndex].Inf[16] << 16) +  (gOBDResponse[EcuIndex].Inf[17] << 8) + gOBDResponse[EcuIndex].Inf[18]) * .005);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F848  CDERDT-L = %8.3f km\n",
	      GetEcuId ( EcuIndex ),
	      CDERDT_L );


	// log CIDT_A
	CIDT_A = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[19] << 8 ) + gOBDResponse[EcuIndex].Inf[20]) * .25);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F848  CIDT-A   = %9.3f km\n",
	      GetEcuId ( EcuIndex ),
	      CIDT_A );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		if ( CIDT_A != 0 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F848 CIDT-A must return a value of 0 after a Service $14 code clear\n",
			      GetEcuId ( EcuIndex ) );
			RetCode = FAIL;
		}
	}

	// log CIDT_S
	CIDT_S = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[21] << 8 ) + gOBDResponse[EcuIndex].Inf[22]) * .25);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F848  CIDT-S   = %9.3f km\n",
	      GetEcuId ( EcuIndex ),
	      CIDT_S );

	// log CIDT_L
	CIDT_L = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[23] << 24 ) + (gOBDResponse[EcuIndex].Inf[24] << 16) +  (gOBDResponse[EcuIndex].Inf[25] << 8) + gOBDResponse[EcuIndex].Inf[26]) * .005);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F848  CIDT-L   = %9.3f km\n",
	      GetEcuId ( EcuIndex ),
	      CIDT_L );

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf49Data
**
**  Purpose:    Purpose of this function is to verify the
**              Total Fuel Consumed data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf49Data ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF   *pINF;
	float  CDFC_A;
	float  CDFC_S;
	float  CDFC_L;

	float  CIFC_A;
	float  CIFC_S;
	float  CIFC_L;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F849 data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// must contain 16 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 16 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F849  Data Size = %d (Must be 16 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}


	// NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F849  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}


	// log CDFC_A
	CDFC_A = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[3] << 8 ) + gOBDResponse[EcuIndex].Inf[4]) * .5);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F849  CDFC-A = %12.1f l\n",
	      GetEcuId ( EcuIndex ),
	      CDFC_A );

	// log CDFC_S
	CDFC_S = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[5] << 8 ) + gOBDResponse[EcuIndex].Inf[6]) * .5);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F849  CDFC-S = %12.1f l\n",
	      GetEcuId ( EcuIndex ),
	      CDFC_S );

	// log CDFC_L
	CDFC_L = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[7] << 24 ) + (gOBDResponse[EcuIndex].Inf[8] << 16) +  (gOBDResponse[EcuIndex].Inf[9] << 8) + gOBDResponse[EcuIndex].Inf[10]) * .5);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F849  CDFC-L = %10.1f l\n",
	      GetEcuId ( EcuIndex ),
	      CDFC_L );


	// log CIFC_A
	CIFC_A = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[11] << 8 ) + gOBDResponse[EcuIndex].Inf[12]) * .5);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F849  CIFC-A = %12.1f l\n",
	      GetEcuId ( EcuIndex ),
	      CIFC_A );

	// log CIFC_S
	CIFC_S = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[13] << 8 ) + gOBDResponse[EcuIndex].Inf[14]) * .5);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F849  CIFC-S = %12.1f l\n",
	      GetEcuId ( EcuIndex ),
	      CIFC_S );

	// log CIFC_L
	CIFC_L = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[15] << 24 ) + (gOBDResponse[EcuIndex].Inf[16] << 16) +  (gOBDResponse[EcuIndex].Inf[17] << 8) + gOBDResponse[EcuIndex].Inf[18]) * .5);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F849  CIFC-L = %10.1f l\n",
	      GetEcuId ( EcuIndex ),
	      CIFC_L );

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf50Data
**
**  Purpose:    Purpose of this function is to verify the
**              Total Grid Energy Consumed In Charge Depleting Operation data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf50Data ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF   *pINF;
	unsigned long CDEOGE_A;
	unsigned long CDEOGE_S;
	unsigned long CDEOGE_L;

	unsigned long CDERGE_A;
	unsigned long CDERGE_S;
	unsigned long CDERGE_L;

	unsigned long GE_A;
	unsigned long GE_S;
	unsigned long GE_L;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F850 data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// must contain 24 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 24 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F850  Data Size = %d (Must be 24 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}


	// NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F850  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}


	// log CDEOGE_A
	CDEOGE_A = (gOBDResponse[EcuIndex].Inf[3] << 8 ) + gOBDResponse[EcuIndex].Inf[4];
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F850  CDEOGE-A = %10lu kWh\n",
	      GetEcuId ( EcuIndex ),
	      CDEOGE_A );

	// log CDEOGE_S
	CDEOGE_S = (gOBDResponse[EcuIndex].Inf[5] << 8 ) + gOBDResponse[EcuIndex].Inf[6];
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F850  CDEOGE-S = %10lu kWh\n",
	      GetEcuId ( EcuIndex ),
	      CDEOGE_S );

	// log CDEOGE_L
	CDEOGE_L = (gOBDResponse[EcuIndex].Inf[7] << 24 ) + (gOBDResponse[EcuIndex].Inf[8] << 16) +  (gOBDResponse[EcuIndex].Inf[9] << 8) + gOBDResponse[EcuIndex].Inf[10];
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F850  CDEOGE-L = %10lu kWh\n",
	      GetEcuId ( EcuIndex ),
	      CDEOGE_L );


	// log CDERGE_A
	CDERGE_A = (gOBDResponse[EcuIndex].Inf[11] << 8 ) + gOBDResponse[EcuIndex].Inf[12];
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F850  CDERGE-A = %10lu kWh\n",
	      GetEcuId ( EcuIndex ),
	      CDERGE_A );

	// log CDERGE_S
	CDERGE_S = (gOBDResponse[EcuIndex].Inf[13] << 8 ) + gOBDResponse[EcuIndex].Inf[14];
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F850  CDERGE-S = %10lu kWh\n",
	      GetEcuId ( EcuIndex ),
	      CDERGE_S );

	// log CDERGE_L
	CDERGE_L = (gOBDResponse[EcuIndex].Inf[15] << 24 ) + (gOBDResponse[EcuIndex].Inf[16] << 16) +  (gOBDResponse[EcuIndex].Inf[17] << 8) + gOBDResponse[EcuIndex].Inf[18];
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F850  CDERGE-L = %10lu kWh\n",
	      GetEcuId ( EcuIndex ),
	      CDERGE_L );


	// log GE_A
	GE_A = (gOBDResponse[EcuIndex].Inf[19] << 8 ) + gOBDResponse[EcuIndex].Inf[20];
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F850  GE-A     = %10lu kWh\n",
	      GetEcuId ( EcuIndex ),
	      GE_A );
	if ( gTestPhase == eTestNoDTC && gTestSubsection == 17 )  // IF Test 5.17
	{
		if ( GE_A != 0 )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F850 GE-A must return a value of 0 after a Service $14 code clear\n",
			      GetEcuId ( EcuIndex ) );
			RetCode = FAIL;
		}
	}

	// log GE_S
	GE_S = (gOBDResponse[EcuIndex].Inf[21] << 8 ) + gOBDResponse[EcuIndex].Inf[22];
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F850  GE-S     = %10lu kWh\n",
	      GetEcuId ( EcuIndex ),
	      GE_S );

	// log GE_L
	GE_L = (gOBDResponse[EcuIndex].Inf[22] << 24 ) + (gOBDResponse[EcuIndex].Inf[23] << 16) +  (gOBDResponse[EcuIndex].Inf[24] << 8) + gOBDResponse[EcuIndex].Inf[25];
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F850  GE-L     = %10lu kWh\n",
	      GetEcuId ( EcuIndex ),
	      GE_L );

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf51Data
**
**  Purpose:    Purpose of this function is to verify the
**              Stop-Start Propulsion System Active Time data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf51Data ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF   *pINF;
	unsigned long ISS_TIME_A;
	unsigned long ISS_TIME_S;
	unsigned long ISS_TIME_L;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F851 data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// must contain 8 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 8 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F851  Data Size = %d (Must be 8 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}


	// NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F851  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}


	// log ISS_TIME_A
	ISS_TIME_A = ((gOBDResponse[EcuIndex].Inf[3] << 8 ) + gOBDResponse[EcuIndex].Inf[4]) * 10;
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F851  ISS_TIME-A = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      ISS_TIME_A );

	// log ISS_TIME_S
	ISS_TIME_S = ((gOBDResponse[EcuIndex].Inf[5] << 8 ) + gOBDResponse[EcuIndex].Inf[6]) * 10;
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F851  ISS_TIME-S = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      ISS_TIME_S );

	// log ISS_TIME_L
	ISS_TIME_L = (gOBDResponse[EcuIndex].Inf[7] << 24 ) + (gOBDResponse[EcuIndex].Inf[8] << 16) +  (gOBDResponse[EcuIndex].Inf[9] << 8) + gOBDResponse[EcuIndex].Inf[10];
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F851  ISS_TIME-L = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      ISS_TIME_L );

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf52Data
**
**  Purpose:    Purpose of this function is to verify the
**              Automatic Engine Shutdown Technology Activation Count data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf52Data ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF   *pINF;
	unsigned long AES_CNT_A;
	unsigned long AES_CNT_S;
	unsigned long AES_CNT_L;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F852 data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// must contain 8 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 8 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F852  Data Size = %d (Must be 8 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}


	// NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F852  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}


	// log AES_CNT_A
	AES_CNT_A = ((gOBDResponse[EcuIndex].Inf[3] << 8 ) + gOBDResponse[EcuIndex].Inf[4]) * 10;
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F852  AES_CNT-A = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      AES_CNT_A );

	// log AES_CNT_S
	AES_CNT_S = ((gOBDResponse[EcuIndex].Inf[5] << 8 ) + gOBDResponse[EcuIndex].Inf[6]) * 10;
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F852  AES_CNT-S = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      AES_CNT_S );

	// log AES_CNT_L
	AES_CNT_L = (gOBDResponse[EcuIndex].Inf[7] << 24 ) + (gOBDResponse[EcuIndex].Inf[8] << 16) +  (gOBDResponse[EcuIndex].Inf[9] << 8) + gOBDResponse[EcuIndex].Inf[10];
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F852  AES_CNT-L = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      AES_CNT_L );

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf53Data
**
**  Purpose:    Purpose of this function is to verify the
**              WHR Propulsion System Active Time / WHR Distance Traveled data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf53Data ( unsigned long EcuIndex )
{
	STATUS         RetCode = PASS;
	INF           *pINF;
	unsigned long  WHR_TIME_A;
	unsigned long  WHR_TIME_S;
	unsigned long  WHR_TIME_L;

	float          WHR_DT_A;
	float          WHR_DT_S;
	float          WHR_DT_L;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F853 data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// must contain 16 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 16 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F853  Data Size = %d (Must be 16 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}


	// NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F853  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}


	// log WHR_TIME_A
	WHR_TIME_A = ((gOBDResponse[EcuIndex].Inf[3] << 8 ) + gOBDResponse[EcuIndex].Inf[4]) * 10;
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F853  WHR_TIME-A = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      WHR_TIME_A );

	// log WHR_TIME_S
	WHR_TIME_S = ((gOBDResponse[EcuIndex].Inf[5] << 8 ) + gOBDResponse[EcuIndex].Inf[6]) * 10;
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F853  WHR_TIME-S = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      WHR_TIME_S );

	// log WHR_TIME_L
	WHR_TIME_L = (gOBDResponse[EcuIndex].Inf[7] << 24 ) + (gOBDResponse[EcuIndex].Inf[8] << 16) +  (gOBDResponse[EcuIndex].Inf[9] << 8) + gOBDResponse[EcuIndex].Inf[10];
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F853  WHR_TIME-L = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      WHR_TIME_L );


	// log WHR_DT_A
	WHR_DT_A = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[11] << 8 ) + gOBDResponse[EcuIndex].Inf[12]) * .25);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F853  WHR_DT-A   = %10.3f km\n",
	      GetEcuId ( EcuIndex ),
	      WHR_DT_A );

	// log WHR_DT_S
	WHR_DT_S = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[13] << 8 ) + gOBDResponse[EcuIndex].Inf[14]) * .25);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F853  WHR_DT-S   = %10.3f km\n",
	      GetEcuId ( EcuIndex ),
	      WHR_DT_S );

	// log WHR_DT_L
	WHR_DT_L = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[15] << 24 ) + (gOBDResponse[EcuIndex].Inf[16] << 16) +  (gOBDResponse[EcuIndex].Inf[17] << 8) + gOBDResponse[EcuIndex].Inf[18]) * .005);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F853  WHR_DT-L   = %10.3f km\n",
	      GetEcuId ( EcuIndex ),
	      WHR_DT_L );

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf54_5BData
**
**  Purpose:    Purpose of this function is to verify the
**              Active Technology #X Propulsion System Active Time /
**              Active Technology #X Distance Traveled data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf54_5BData ( unsigned long EcuIndex )
{
#define  MAX_NUMBER_LENGTH  3

	STATUS        RetCode = PASS;
	INF          *pINF;

	unsigned long AT1_TIME_A;
	unsigned long AT1_TIME_S;
	unsigned long AT1_TIME_L;

	float         AT1_DT_A;
	float         AT1_DT_S;
	float         AT1_DT_L;

	unsigned long AT2_TIME_A;
	unsigned long AT2_TIME_S;
	unsigned long AT2_TIME_L;

	float         AT2_DT_A;
	float         AT2_DT_S;
	float         AT2_DT_L;

	char          Number1[MAX_NUMBER_LENGTH];
	char          Number2[MAX_NUMBER_LENGTH];

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $%04X data\n",
		      GetEcuId ( EcuIndex ),
		      gOBDResponse[EcuIndex].Inf[0] );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// must contain 32 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 32 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  Data Size = %d (Must be 32 bytes!)\n",
		      pINF->INFLSB,
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}


	// NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      pINF->NumItems );
		RetCode = FAIL;
	}


	sprintf_s ( Number1, MAX_NUMBER_LENGTH, "%d", 1 + ((pINF->INFLSB - 0x54) * 2) );
	sprintf_s ( Number2, MAX_NUMBER_LENGTH, "%d", 2 + ((pINF->INFLSB - 0x54) * 2) );


	// log ATx_TIME_A
	AT1_TIME_A = ((gOBDResponse[EcuIndex].Inf[3] << 8 ) + gOBDResponse[EcuIndex].Inf[4]) * 10;
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  AT%s_TIME-A = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Number1,
	      AT1_TIME_A );

	// log ATx_TIME_S
	AT1_TIME_S = ((gOBDResponse[EcuIndex].Inf[5] << 8 ) + gOBDResponse[EcuIndex].Inf[6]) * 10;
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  AT%s_TIME-S = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Number1,
	      AT1_TIME_S );

	// log ATx_TIME_L
	AT1_TIME_L = (gOBDResponse[EcuIndex].Inf[7] << 24 ) + (gOBDResponse[EcuIndex].Inf[8] << 16) +  (gOBDResponse[EcuIndex].Inf[9] << 8) + gOBDResponse[EcuIndex].Inf[10];
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  AT%s_TIME-L = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Number1,
	      AT1_TIME_L );


	// log ATx_DT_A
	AT1_DT_A = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[11] << 8 ) + gOBDResponse[EcuIndex].Inf[12]) * .25);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  AT%s_DT-A   = %10.3f km\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Number1,
	      AT1_DT_A );

	// log ATx_DT_S
	AT1_DT_S = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[13] << 8 ) + gOBDResponse[EcuIndex].Inf[14]) * .25);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  AT%s_DT-S   = %10.3f km\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Number1,
	      AT1_DT_S );

	// log ATx_DT_L
	AT1_DT_L = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[15] << 24 ) + (gOBDResponse[EcuIndex].Inf[16] << 16) +  (gOBDResponse[EcuIndex].Inf[17] << 8) + gOBDResponse[EcuIndex].Inf[18]) * .005);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  AT%s_DT-L   = %10.3f km\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Number1,
	      AT1_DT_L );


	// log ATy_TIME_A
	AT2_TIME_A = ((gOBDResponse[EcuIndex].Inf[19] << 8 ) + gOBDResponse[EcuIndex].Inf[20]) * 10;
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  AT%s_TIME-A = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Number2,
	      AT2_TIME_A );

	// log ATy_TIME_S
	AT2_TIME_S = ((gOBDResponse[EcuIndex].Inf[21] << 8 ) + gOBDResponse[EcuIndex].Inf[22]) * 10;
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  AT%s_TIME-S = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Number2,
	      AT2_TIME_S );

	// log ATy_TIME_L
	AT2_TIME_L = (gOBDResponse[EcuIndex].Inf[23] << 24 ) + (gOBDResponse[EcuIndex].Inf[24] << 16) +  (gOBDResponse[EcuIndex].Inf[25] << 8) + gOBDResponse[EcuIndex].Inf[26];
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  AT%s_TIME-L = %10lu sec\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Number2,
	      AT2_TIME_L );


	// log ATy_DT_A
	AT2_DT_A = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[27] << 8 ) + gOBDResponse[EcuIndex].Inf[28]) * .25);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  AT%s_DT-A   = %10.3f km\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Number2,
	      AT2_DT_A );

	// log ATy_DT_S
	AT2_DT_S = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[29] << 8 ) + gOBDResponse[EcuIndex].Inf[30]) * .25);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  AT%s_DT-S   = %10.3f km\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Number2,
	      AT2_DT_S );

	// log ATy_DT_L
	AT2_DT_L = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[31] << 24 ) + (gOBDResponse[EcuIndex].Inf[32] << 16) +  (gOBDResponse[EcuIndex].Inf[33] << 8) + gOBDResponse[EcuIndex].Inf[34]) * .005);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F8%02X  AT%s_DT-L   = %10.3f km\n",
	      GetEcuId ( EcuIndex ),
	      pINF->INFLSB,
	      Number2,
	      AT2_DT_L );

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf61_62_67_68_6D_6EData
**
**  Purpose:    Purpose of this function is to verify the
**              NOx Binning data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf61_62_67_68_6D_6EData ( unsigned long EcuIndex )
{
	STATUS         RetCode = PASS;
	INF           *pINF;

	float          Bin[17];
	float          Bin_Sum = 0;

	unsigned short binIndex;

	float         *Archive = NULL;

	char           Name[MAX_NAME_LENGTH];
	char           CompareName[MAX_NAME_LENGTH];
	float          Multiplier = (float)1.0;


	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $%04X data\n",
		      GetEcuId ( EcuIndex ),
		      gOBDResponse[EcuIndex].Inf[0] );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// must contain 68 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 68 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  Data Size = %d (Must be 68 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}


	// NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      pINF->NumItems );
		RetCode = FAIL;
	}


	switch ( pINF->INFLSB )
	{
		case 0x61:
		{
			strcpy_s ( Name, MAX_NAME_LENGTH, "NOxEOB-A" );
			Multiplier = (float).000005;
			Archive = &INF_Inf_5_17[EcuIndex].INF61_NOxEOB_A[0];
		}
		break;
		case 0x67:
		{
			strcpy_s ( Name, MAX_NAME_LENGTH, "NOxEOB-S" );
			strcpy_s ( CompareName, MAX_NAME_LENGTH, "NOxEOB-A" );
			Multiplier = (float).000005;
			Archive = &INF_Inf_5_17[EcuIndex].INF61_NOxEOB_A[0];
		}
		break;
		case 0x6D:
		{
			strcpy_s ( Name, MAX_NAME_LENGTH, "NOxEOB-L" );
			strcpy_s ( CompareName, MAX_NAME_LENGTH, "NOxEOB-A" );
			Multiplier = (float).001;
			Archive = &INF_Inf_5_17[EcuIndex].INF61_NOxEOB_A[0];
		}
		break;

		case 0x62:
		{
			strcpy_s ( Name, MAX_NAME_LENGTH, "NOxTPB-A" );
			Multiplier = (float).000002;
			Archive = &INF_Inf_5_17[EcuIndex].INF62_NOxTPB_A[0];
		}
		break;
		case 0x68:
		{
			strcpy_s ( Name, MAX_NAME_LENGTH, "NOxTPB-S" );
			strcpy_s ( CompareName, MAX_NAME_LENGTH, "NOxTPB-A" );
			Multiplier = (float).000002;
			Archive = &INF_Inf_5_17[EcuIndex].INF62_NOxTPB_A[0];
		}
		break;
		case 0x6E:
		{
			strcpy_s ( Name, MAX_NAME_LENGTH, "NOxTPB-L" );
			strcpy_s ( CompareName, MAX_NAME_LENGTH, "NOxTPB-A" );
			Multiplier = (float).001;
			Archive = &INF_Inf_5_17[EcuIndex].INF62_NOxTPB_A[0];
		}
		break;
	}


	for ( binIndex = 0; binIndex < 17; binIndex++ )
	{
		// log bin data
		Bin[binIndex] = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[3 + (binIndex * 4)] << 24 ) + (gOBDResponse[EcuIndex].Inf[4 + (binIndex * 4)] << 16) +  (gOBDResponse[EcuIndex].Inf[5 + (binIndex * 4)] << 8) + gOBDResponse[EcuIndex].Inf[6 + (binIndex * 4)]) * Multiplier);
		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  %s[bin%d] = %11.7f kg\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      Name,
		      binIndex+1,
		      Bin[binIndex] );

		// Save -A values for later
		if ( pINF->INFLSB == 0x61 || pINF->INFLSB == 0x62 )
		{
			Archive[binIndex] = Bin[binIndex];
		}

		// -S and -L values must be greater than -A value
		else if ( ( pINF->INFLSB == 0x6D || pINF->INFLSB == 0x6E ) &&
		          Bin[binIndex] != 0 &&
		          Bin[binIndex] < Archive[binIndex] )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F8%02X %s[bin%d] must be greater than %s[bin%d] (%.7f)\n",
			      GetEcuId ( EcuIndex ),
			      pINF->INFLSB,
			      Name,
			      binIndex+1,
			      CompareName,
			      binIndex+1,
			      Archive[binIndex] );
			RetCode = FAIL;
		}


		// Add bins 2 through 14
		if ( binIndex >= 1 && binIndex <= 13 )
		{
			Bin_Sum += Bin[binIndex];
		}
	}


	// no nox DTCs from SID $19 Subfunction $42 Status Mask $04 or $08 &&
	// Fail if sum of [bin2] through [bin14] != [bin1]
	if ( Bin_Sum < (Bin[0] - (Bin[0]*.01)) || Bin_Sum  > (Bin[0] + (Bin[0]*.01)) )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X the sum of %s[bin2] through %s[bin14] = %.7f (Must be equal to %s[bin1] (%.7f +-1%%))\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      Name,
		      Name,
		      Bin_Sum,
		      Name,
		      Bin[0] );
		RetCode = FAIL;
	}

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf63_69_6F_73Data
**
**  Purpose:    Purpose of this function is to verify the
**              NOx Binning data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf63_69_6F_73Data ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF   *pINF;

	float Bin[17];
	float Bin_Sum = 0;

	unsigned short binIndex;

	char  Name[MAX_NAME_LENGTH];
	float Multiplier = (float)1.0;


	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $%04X data\n",
		      GetEcuId ( EcuIndex ),
		      gOBDResponse[EcuIndex].Inf[0] );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// INF $63 and $69 must contain 34 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( (pINF->INFLSB == 0x63 || pINF->INFLSB == 0x69) &&
	     gOBDResponse[EcuIndex].InfSize - 3 != 34 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  Data Size = %d (Must be 34 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}
	// INF $63 and $69 must contain 68 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	else if ( (pINF->INFLSB == 0x6F || pINF->INFLSB == 0x73) &&
	          gOBDResponse[EcuIndex].InfSize - 3 != 68 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  Data Size = %d (Must be 68 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}


	// NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      pINF->NumItems );
		RetCode = FAIL;
	}


	switch ( pINF->INFLSB )
	{
		case 0x63:
		{
			strcpy_s ( Name, MAX_NAME_LENGTH, "EOEB-A" );
		}
		break;
		case 0x69:
		{
			strcpy_s ( Name, MAX_NAME_LENGTH, "EOEB-S" );
		}
		break;
		case 0x6F:
		{
			strcpy_s ( Name, MAX_NAME_LENGTH, "EOEB-L" );
		}
		break;
		case 0x73:
		{
			strcpy_s ( Name, MAX_NAME_LENGTH, "EOEB-EAL" );
		}
		break;
	}
	Multiplier = (float).001;


	for ( binIndex = 0; binIndex < 17; binIndex++ )
	{
		// log bin data
		if ( pINF->INFLSB == 0x6F || pINF->INFLSB == 0x73 )
		{
			Bin[binIndex] = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[3 + (binIndex * 4)] << 24 ) + (gOBDResponse[EcuIndex].Inf[4 + (binIndex * 4)] << 16) +  (gOBDResponse[EcuIndex].Inf[5 + (binIndex * 4)] << 8) + gOBDResponse[EcuIndex].Inf[6 + (binIndex * 4)]) * Multiplier);
		}
		else
		{
			Bin[binIndex] = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[3 + (binIndex * 2)] << 8 ) + gOBDResponse[EcuIndex].Inf[4 + (binIndex * 2)]) * Multiplier);
		}

		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  %s[bin%d] = %10.6f MWh\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      Name,
		      binIndex+1,
		      Bin[binIndex] );

		// Save -A values for later
		if ( pINF->INFLSB == 0x63 )
		{
			INF_Inf_5_17[EcuIndex].INF63_EOEB_A[binIndex] = Bin[binIndex];
		}

		// -S and -L values must be greater than -A values
		else if ( pINF->INFLSB == 0x6F &&
		          Bin[binIndex] != 0  &&
		          Bin[binIndex] < INF_Inf_5_17[EcuIndex].INF63_EOEB_A[binIndex] )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F8%02X %s[bin%d] must be greater than EOEB-A[bin%d] (%.6f)\n",
			      GetEcuId ( EcuIndex ),
			      pINF->INFLSB,
			      Name,
			      binIndex+1,
			      binIndex+1,
			      INF_Inf_5_17[EcuIndex].INF63_EOEB_A[binIndex] );
			RetCode = FAIL;
		}


		// if INF 63, 69, or 6f, collect the sum of [bin2] through [bin14]
		if ( pINF->INFLSB == 0x63 ||
		     pINF->INFLSB == 0x69 ||
		     pINF->INFLSB == 0x6F )
		{
			if ( binIndex >= 1 && binIndex <= 13 )
			{
				Bin_Sum += Bin[binIndex];
			}
		}
	}

	// if INF 63, 69, or 6f, compare [bin1] against sum of [bin2] through [bin14]
	if ( pINF->INFLSB == 0x63 ||
	     pINF->INFLSB == 0x69 ||
	     pINF->INFLSB == 0x6F )
	{
		// no nox DTCs from SID $19 Subfunction $42 Status Mask $04 or $08 &&
		// Fail if Sum of [bin2] through [bin14] != [bin1]
		if ( Bin_Sum < (Bin[0] - (Bin[0]*.01)) || Bin_Sum  > (Bin[0] + (Bin[0]*.01)) )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F8%02X the sum of %s[bin2] through %s[bin14] = %.3f (Must be equal to %s[bin1] (%.3f +-1%%))\n",
			      GetEcuId ( EcuIndex ),
			      pINF->INFLSB,
			      Name,
			      Name,
			      Bin_Sum,
			      Name,
			      Bin[0] );
			RetCode = FAIL;
		}
	}

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf64_6A_70_74Data
**
**  Purpose:    Purpose of this function is to verify the
**              NOx Binning data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf64_6A_70_74Data ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF   *pINF;

	float Bin[17];
	float Bin_Sum = 0;

	unsigned short binIndex;

	char  Name[MAX_NAME_LENGTH];
	float Multiplier = (float)1.0;


	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $%04X data\n",
		      GetEcuId ( EcuIndex ),
		      gOBDResponse[EcuIndex].Inf[0] );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// INF $64 and $6A must contain 34 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( (pINF->INFLSB == 0x64 || pINF->INFLSB == 0x6A) &&
	     gOBDResponse[EcuIndex].InfSize - 3 != 34 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  Data Size = %d (Must be 34 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}
	// INF $70 and $74 must contain 68 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	else if ( (pINF->INFLSB == 0x70 || pINF->INFLSB == 0x74) &&
	          gOBDResponse[EcuIndex].InfSize - 3 != 68 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  Data Size = %d (Must be 68 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}


	// NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      pINF->NumItems );
		RetCode = FAIL;
	}


	switch ( pINF->INFLSB )
	{
		case 0x64:
		{
			strcpy_s ( Name, MAX_NAME_LENGTH, "DTB-A" );
			Multiplier = (float).25;
		}
		break;
		case 0x6A:
		{
			strcpy_s ( Name, MAX_NAME_LENGTH, "DTB-S" );
			Multiplier = (float).25;
		}
		break;
		case 0x70:
		{
			strcpy_s ( Name, MAX_NAME_LENGTH, "DTB-L" );
			Multiplier = (float).005;
		}
		break;
		case 0x74:
		{
			strcpy_s ( Name, MAX_NAME_LENGTH, "DTB-EAL" );
			Multiplier = (float).005;
		}
		break;
	}


	for ( binIndex = 0; binIndex < 17; binIndex++ )
	{
		// log bin data
		if ( pINF->INFLSB == 0x70 || pINF->INFLSB == 0x74 )
		{
			Bin[binIndex] = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[3 + (binIndex * 4)] << 24 ) + (gOBDResponse[EcuIndex].Inf[4 + (binIndex * 4)] << 16) +  (gOBDResponse[EcuIndex].Inf[5 + (binIndex * 4)] << 8) + gOBDResponse[EcuIndex].Inf[6 + (binIndex * 4)]) * Multiplier);
		}
		else
		{
			Bin[binIndex] = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[3 + (binIndex * 2)] << 8 ) + gOBDResponse[EcuIndex].Inf[4 + (binIndex * 2)]) * Multiplier);
		}

		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  %s[bin%d] = %11.3f km\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      Name,
		      binIndex+1,
		      Bin[binIndex] );

		// Save -A values for later
		if ( pINF->INFLSB == 0x64 )
		{
			INF_Inf_5_17[EcuIndex].INF64_DTB_A[binIndex] = Bin[binIndex];
		}

		// -S and -L values must be greater than -A values
		else if ( pINF->INFLSB == 0x70 &&
		          Bin[binIndex] != 0 &&
		          Bin[binIndex] < INF_Inf_5_17[EcuIndex].INF64_DTB_A[binIndex] )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F8%02X %s[bin%d] must be greater than DTB-A[bin%d] (%.3f)\n",
			      GetEcuId ( EcuIndex ),
			      pINF->INFLSB,
			      Name,
			      binIndex+1,
			      binIndex+1,
			      INF_Inf_5_17[EcuIndex].INF64_DTB_A[binIndex] );
			RetCode = FAIL;
		}


		// if INF 64, 6A, 70, collect the sum of [bin2] through [bin14]
		if ( pINF->INFLSB == 0x64 ||
		     pINF->INFLSB == 0x6A ||
		     pINF->INFLSB == 0x70 )
		{
			if ( binIndex >= 1 && binIndex <= 13 )
			{
				Bin_Sum += Bin[binIndex];
			}
		}
	}

	// if INF 64, 6A, 70, compare [bin1] against sum of [bin2] through [bin14]
	if ( pINF->INFLSB == 0x64 ||
	     pINF->INFLSB == 0x6A ||
	     pINF->INFLSB == 0x70 )
	{
		// no nox DTCs from SID $19 Subfunction $42 Status Mask $04 or $08 &&
		// Fail if sum of [bin2] through [bin14] != [bin1]
		if ( Bin_Sum < (Bin[0] - (Bin[0]*.01)) || Bin_Sum  > (Bin[0] + (Bin[0]*.01)) )
		{
			Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
			      "ECU %X  INF $F8%02X the sum of %s[bin2] through %s[bin14] = %.3f (Must be equal to %s[bin1] (%.3f +-1%%))\n",
			      GetEcuId ( EcuIndex ),
			      pINF->INFLSB,
			      Name,
			      Name,
			      Bin_Sum,
			      Name,
			      Bin[0] );
			RetCode = FAIL;
		}
	}

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf65_6B_71_75Data
**
**  Purpose:    Purpose of this function is to verify the
**              NOx Binning data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf65_6B_71_75Data ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF   *pINF;

	unsigned long Bin[17];
	unsigned long Bin_Sum = 0;

	unsigned int  binIndex;

	char          Name[MAX_NAME_LENGTH];
	unsigned long Multiplier = 1;


	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $%04X data\n",
		      GetEcuId ( EcuIndex ),
		      gOBDResponse[EcuIndex].Inf[0] );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// INF $65 and $6B must contain 34 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( (pINF->INFLSB == 0x65 || pINF->INFLSB == 0x6B) &&
	     gOBDResponse[EcuIndex].InfSize - 3 != 34 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  Data Size = %d (Must be 34 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}
	// INF $71 and $75 must contain 68 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	else if ( (pINF->INFLSB == 0x71 || pINF->INFLSB == 0x75) &&
	          gOBDResponse[EcuIndex].InfSize - 3 != 68 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  Data Size = %d (Must be 68 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}


	// NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      pINF->NumItems );
		RetCode = FAIL;
	}


	switch ( pINF->INFLSB )
	{
		case 0x65:
		{
			strcpy_s ( Name, MAX_NAME_LENGTH, "ERTB-A" );
			Multiplier = 10;
		}
		break;
		case 0x6B:
		{
			strcpy_s ( Name, MAX_NAME_LENGTH, "ERTB-S" );
			Multiplier = 10;
		}
		break;
		case 0x71:
		{
			strcpy_s ( Name, MAX_NAME_LENGTH, "ERTB-L" );
			Multiplier = 1;
		}
		break;
		case 0x75:
		{
			strcpy_s ( Name, MAX_NAME_LENGTH, "ERTB-EAL" );
			Multiplier = 1;
		}
		break;
	}


	for ( binIndex = 0; binIndex < 17; binIndex++ )
	{
		// log bin data
		if ( pINF->INFLSB == 0x71 || pINF->INFLSB == 0x75 )
		{
			Bin[binIndex] = ((gOBDResponse[EcuIndex].Inf[3 + (binIndex * 4)] << 24 ) + (gOBDResponse[EcuIndex].Inf[4 + (binIndex * 4)] << 16) + (gOBDResponse[EcuIndex].Inf[5 + (binIndex * 4)] << 8) + gOBDResponse[EcuIndex].Inf[6 + (binIndex * 4)]) * Multiplier;
		}
		else
		{
			Bin[binIndex] = ((gOBDResponse[EcuIndex].Inf[3 + (binIndex * 2)] << 8 ) + gOBDResponse[EcuIndex].Inf[4 + (binIndex * 2)]) * Multiplier;
		}

		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  %s[bin%d] = %10lu sec\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      Name,
		      binIndex+1,
		      Bin[binIndex] );
	}

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf66_6C_72_76Data
**
**  Purpose:    Purpose of this function is to verify the
**              NOx Binning data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf66_6C_72_76Data ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF   *pINF;

	float Bin[17];
	float Bin_Sum = 0;

	unsigned short binIndex;

	char  Name[MAX_NAME_LENGTH];
	float Multiplier = (float)1.0;


	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F8%02X data\n",
		      GetEcuId ( EcuIndex ),
		      gOBDResponse[EcuIndex].Inf[0] );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// INF $66 and $6C must contain 34 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( (pINF->INFLSB == 0x66 || pINF->INFLSB == 0x6C) &&
	     gOBDResponse[EcuIndex].InfSize - 3 != 34 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  Data Size = %d (Must be 34 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}
	// INF $72 and $76 must contain 68 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	else if ( (pINF->INFLSB == 0x72 || pINF->INFLSB == 0x76) &&
	          gOBDResponse[EcuIndex].InfSize - 3 != 68 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  Data Size = %d (Must be 68 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}


	// NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      pINF->NumItems );
		RetCode = FAIL;
	}


	switch ( pINF->INFLSB )
	{
		case 0x66:
		{
			strcpy_s ( Name, MAX_NAME_LENGTH, "FCB-A" );
		}
		break;
		case 0x6C:
		{
			strcpy_s ( Name, MAX_NAME_LENGTH, "FCB-S" );
		}
		break;
		case 0x72:
		{
			strcpy_s ( Name, MAX_NAME_LENGTH, "IFCB-L" );
		}
		break;
		case 0x76:
		{
			strcpy_s ( Name, MAX_NAME_LENGTH, "FCB-EAL" );
		}
		break;
	}

	Multiplier = (float).5;


	for ( binIndex = 0; binIndex < 17; binIndex++ )
	{
		// log bin data
		if ( pINF->INFLSB == 0x72 || pINF->INFLSB == 0x76 )
		{
			Bin[binIndex] = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[3 + (binIndex * 4)] << 24 ) + (gOBDResponse[EcuIndex].Inf[4 + (binIndex * 4)] << 16) +  (gOBDResponse[EcuIndex].Inf[5 + (binIndex * 4)] << 8) + gOBDResponse[EcuIndex].Inf[6 + (binIndex * 4)]) * Multiplier);
		}
		else
		{
			Bin[binIndex] = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[3 + (binIndex * 2)] << 8 ) + gOBDResponse[EcuIndex].Inf[4 + (binIndex * 2)]) * Multiplier);
		}

		Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F8%02X  %s[bin%d] = %11.1f l\n",
		      GetEcuId ( EcuIndex ),
		      pINF->INFLSB,
		      Name,
		      binIndex+1,
		      Bin[binIndex] );
	}

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf77Data
**
**  Purpose:    Purpose of this function is to verify the
**              Total Run Time / Odometer Reading data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf77Data ( unsigned long EcuIndex )
{
	STATUS        RetCode = PASS;
	INF          *pINF;

	unsigned long NOREDUCT_TIME_L;

	unsigned long COLDEXH_TIME_L;

	float         REGEN1_SDIST;
	float         REGEN1_FDIST;

	float         REGEN2_SDIST;
	float         REGEN2_FDIST;

	float         REGEN3_SDIST;
	float         REGEN3_FDIST;

	unsigned long REGEN_CNTL_L;


	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F877 data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// must contain 36 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 36 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F877  Data Size = %d (Must be 36 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F877  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}

	// log NOREDUCT_TIME_L
	NOREDUCT_TIME_L = (gOBDResponse[EcuIndex].Inf[3] << 24 ) + (gOBDResponse[EcuIndex].Inf[4] << 16) +  (gOBDResponse[EcuIndex].Inf[5] << 8) + gOBDResponse[EcuIndex].Inf[6];
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F877  NOREDUCT_TIME-L = %11lu sec\n",
	      GetEcuId ( EcuIndex ),
	      NOREDUCT_TIME_L );

	// log COLDEXH_TIME_L
	COLDEXH_TIME_L = (gOBDResponse[EcuIndex].Inf[7] << 24 ) + (gOBDResponse[EcuIndex].Inf[8] << 16) +  (gOBDResponse[EcuIndex].Inf[9] << 8) + gOBDResponse[EcuIndex].Inf[10];
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F877  COLDEXH_TIME-L  = %11lu sec\n",
	      GetEcuId ( EcuIndex ),
	      COLDEXH_TIME_L );


	// log REGEN1_SDIST
	REGEN1_SDIST = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[11] << 24 ) + (gOBDResponse[EcuIndex].Inf[12] << 16) +  (gOBDResponse[EcuIndex].Inf[13] << 8) + gOBDResponse[EcuIndex].Inf[14]) * .005);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F877  REGEN1_SDIST-L = %11.3f km\n",
	      GetEcuId ( EcuIndex ),
	      REGEN1_SDIST );

	// log REGEN1_FDIST
	REGEN1_FDIST = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[15] << 24 ) + (gOBDResponse[EcuIndex].Inf[16] << 16) +  (gOBDResponse[EcuIndex].Inf[17] << 8) + gOBDResponse[EcuIndex].Inf[18]) * .005);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F877  REGEN1_FDIST-L = %11.3f km\n",
	      GetEcuId ( EcuIndex ),
	      REGEN1_FDIST );


	// log REGEN2_SDIST
	REGEN2_SDIST = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[19] << 24 ) + (gOBDResponse[EcuIndex].Inf[20] << 16) +  (gOBDResponse[EcuIndex].Inf[21] << 8) + gOBDResponse[EcuIndex].Inf[22]) * .005);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F877  REGEN2_SDIST-L = %11.3f km\n",
	      GetEcuId ( EcuIndex ),
	      REGEN2_SDIST );

	// log REGEN2_FDIST
	REGEN2_FDIST = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[23] << 24 ) + (gOBDResponse[EcuIndex].Inf[24] << 16) +  (gOBDResponse[EcuIndex].Inf[25] << 8) + gOBDResponse[EcuIndex].Inf[26]) * .005);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F877  REGEN2_FDIST-L = %11.3f km\n",
	      GetEcuId ( EcuIndex ),
	      REGEN2_FDIST );


	// log REGEN3_SDIST
	REGEN3_SDIST = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[27] << 24 ) + (gOBDResponse[EcuIndex].Inf[28] << 16) +  (gOBDResponse[EcuIndex].Inf[29] << 8) + gOBDResponse[EcuIndex].Inf[30]) * .005);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F877  REGEN3_SDIST-L = %11.3f km\n",
	      GetEcuId ( EcuIndex ),
	      REGEN3_SDIST );

	// log REGEN3_FDIST
	REGEN3_FDIST = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[31] << 24 ) + (gOBDResponse[EcuIndex].Inf[32] << 16) +  (gOBDResponse[EcuIndex].Inf[33] << 8) + gOBDResponse[EcuIndex].Inf[34]) * .005);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F877  REGEN3_FDIST-L = %11.3f km\n",
	      GetEcuId ( EcuIndex ),
	      REGEN3_FDIST );


	// log REGEN_CNTL_L
	REGEN_CNTL_L = (gOBDResponse[EcuIndex].Inf[35] << 24 ) + (gOBDResponse[EcuIndex].Inf[36] << 16) +  (gOBDResponse[EcuIndex].Inf[37] << 8) + gOBDResponse[EcuIndex].Inf[38];
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F877  REGEN_CNTL-L   = %11lu\n",
	      GetEcuId ( EcuIndex ),
	      REGEN_CNTL_L );

	return RetCode;
}




/*
*******************************************************************************
**
**  Function:   VerifyInf78Data
**
**  Purpose:    Purpose of this function is to verify the
**              Enigne Rated Power / Engine Rated Speed data.
**              In the event the data fails defined criteria,
**              an error is returned.
**
********************************************************************************
*/
STATUS VerifyInf78Data ( unsigned long EcuIndex )
{
	STATUS RetCode = PASS;
	INF   *pINF;

	float  PWR_RATING;
	float  RPM_RATING;

	if ( gOBDResponse[EcuIndex].InfSize == 0 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  No INF $F878 data\n",
		      GetEcuId ( EcuIndex ) );
		return FAIL;
	}

	pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

	// must contain 4 bytes of data
	// Don't include SID, INF MSB or INF LSB in INFSIZE
	if ( gOBDResponse[EcuIndex].InfSize - 3 != 4 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F878  Data Size = %d (Must be 4 bytes!)\n",
		      GetEcuId ( EcuIndex ),
		      (gOBDResponse[EcuIndex].InfSize - 3) );
		return FAIL;
	}

	// NODI must equal $01
	if ( pINF->NumItems != 0x01 )
	{
		Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
		      "ECU %X  INF $F878  NODI = %d (Must be 1)\n",
		      GetEcuId ( EcuIndex ),
		      pINF->NumItems );
		RetCode = FAIL;
	}


	// log PWR_RATING
	PWR_RATING = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[3] << 8 ) + gOBDResponse[EcuIndex].Inf[4]) * .5);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F878  PWR_RATING = %6.1f kW\n",
	      GetEcuId ( EcuIndex ),
	      PWR_RATING );


	// log RPM_RATING
	RPM_RATING = (float)((unsigned int)((gOBDResponse[EcuIndex].Inf[5] << 8) + gOBDResponse[EcuIndex].Inf[6]) * .25);
	Log ( INFORMATION, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
	      "ECU %X  INF $F878  RPM_RATING = %7.2f RPM\n",
	      GetEcuId ( EcuIndex ),
	      RPM_RATING );


	if ( RPM_RATING < 300.00 || RPM_RATING > 5000.00 )
	{
		if ( !( gOBDDieselFlag &&
		        RPM_RATING >= 1.00 && RPM_RATING <= 299.00 ) )
		{
			if ( PHEV &&
			     RPM_RATING >= 1.00 && RPM_RATING <= 299.00 )
			{
				Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  INF $F878  RPM_RATING Must be greater than 300 and less than 5000\n",
				      GetEcuId ( EcuIndex ) );
			}
			else
			{
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "ECU %X  INF $F878  RPM_RATING Must be greater than 300 and less than 5000\n",
				      GetEcuId ( EcuIndex) );
				RetCode = FAIL;
			}
		}
	}

	return RetCode;
}


/*
****************************************************************************
**
**  Function:  VerifyVehicleInformationChange
**
**  Purpose:
**
****************************************************************************
*/
STATUS VerifyVehicleInformationChange ( void )
{
	unsigned long  EcuIndex;     // ECU Index
	unsigned int   IdIndex;      // INF ID index
	unsigned int   MaxId;        // Maximum INF ID
	SID_REQ        SidReq;
	INF           *pINF;

	BOOL bTestFailed = FALSE;

	unsigned long ulInit_FailureCount = 0;


	ulInit_FailureCount = GetFailureCount ();


	// Request INF support data
	if ( RequestDIDSupportData ( INFREQUEST, TRUE ) != PASS )
	{
		return FAIL;
	}


	if ( gTestPhase == eTestNoFault3DriveCycle && gTestSubsection == 22 )  // IF Test 9.22
	{
		if ( gModelYear >= 2022 && gUserInput.eVehicleType == HD )
		{
			MaxId = 0xF844;
		}
		else
		{
			MaxId = 0xF819;
		}
	}
	else
	{
		MaxId = 0xF829;
	}


	// For each INF group verify that all INF data is valid
	//  Request user supplied expected OBD ECU responses
	for ( IdIndex = 0xF816; IdIndex <= MaxId; IdIndex++ )
	{
		// If INF is supported by any ECU, request it
		if ( IsDIDSupported ( -1, IdIndex ) == TRUE  )
		{
			SidReq.SID    = 0x22;
			SidReq.NumIds = 1;
			SidReq.DID[0] = IdIndex;

			if ( SidRequest ( &SidReq, SID_REQ_NORMAL ) == FAIL )
			{
				bTestFailed = TRUE;
				Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
				      "INF $%04X request\n",
				      IdIndex );

				continue;
			}

			for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
			{
				// If INF is not supported, skip to next ECU
				if ( IsDIDSupported ( EcuIndex, IdIndex ) == FALSE )
				{
					continue;
				}


				// Check the data to see if it is valid
				pINF = (INF *)&gOBDResponse[EcuIndex].Inf[0];

				if ( gOBDResponse[EcuIndex].InfSize == 0 )
				{
					bTestFailed = TRUE;
					Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
					      "ECU %X  No INF $%04X data\n",
					      GetEcuId ( EcuIndex ),
					      IdIndex );
					continue;
				}

				// Check various INF values for validity
				switch ( IdIndex )
				{
					case INF_TYPE_ER_IT:  // INFOTYPE $16 Engine Run/Idle Time
					{
						if ( VerifyInf16Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_DT_FC:  // INFOTYPE $17 Distance/Fuel Used
					{
						if ( VerifyInf17Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_PKE_EOE:  // INFOTYPE $18 PKE/EOE
					{
						if ( VerifyInf18Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_PSA:  // INFOTYPE $19 PSA
					{
						if ( VerifyInf19Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_PHEVDD:  // INFOTYPE 0x1A PHEVDD
					{
						if ( VerifyInf1AData ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_PHEVFD:  // INFOTYPE 0x1B PHEVFD
					{
						if ( VerifyInf1BData ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_PHEVGD:  // INFOTYPE 0x1C PHEVGD
					{
						if ( VerifyInf1CData ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_AAF1OCCVD:  // INFOTYPE 0x1D AAF1OCCVD
					{
						if ( VerifyInf1DData ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_AAF2OCCVD:  // INFOTYPE 0x1E AAF2OCCVD
					{
						if ( VerifyInf1EData ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_AAF3OCCVD:  // INFOTYPE 0x1F AAF3OCCVD
					{
						if ( VerifyInf1FData ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_DSOMOCCVD:  // INFOTYPE 0x21 DSOMOCCVD
					{
						if ( VerifyInf21Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_RTSSCOCCVD:  // INFOTYPE 0x22 RTSSCOCCVD
					{
						if ( VerifyInf22Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_DCTOCCVD:  // INFOTYPE 0x23 DCTOCCVD
					{
						if ( VerifyInf23Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_APTWUFOCCVD:  // INFOTYPE 0x24 APTWUFOCCVD
					{
						if ( VerifyInf24Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_OCCT1VD:  // INFOTYPE 0x25 OCCT1VD
					case INF_TYPE_OCCT2VD:  // INFOTYPE 0x26 OCCT2VD
					case INF_TYPE_OCCT3VD:  // INFOTYPE 0x27 OCCT3VD
					case INF_TYPE_OCCT4VD:  // INFOTYPE 0x28 OCCT4VD
					case INF_TYPE_OCCT5VD:  // INFOTYPE 0x29 OCCT5VD
					{
						if ( VerifyInf25_29Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_HDGHGA100HD1:  // INFOTYPE 0x41
					case INF_TYPE_HDGHGS100HD1:  // INFOTYPE 0x43
					{
						if ( VerifyInf41_43_45Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

					case INF_TYPE_HDGHGA100HD2:  // INFOTYPE 0x42
					case INF_TYPE_HDGHGS100HD2:  // INFOTYPE 0x44
					{
						if ( VerifyInf42_44_46Data ( EcuIndex ) != PASS )
						{
							bTestFailed = TRUE;
						}
					}
					break;

				} // end switch ( pINF->INFLSB )
			} // end for ( EcuIndex = 0; EcuIndex < gUserNumEcus; EcuIndex++ )
		} // end if ( IsDIDSupported ( -1, IdIndex)  == TRUE )

		// Verify that required INFs are supported by the appropriate vehicles
		else
		{
			if ( gModelYear >= 2019 )
			{
//				if ( gUserInput.eComplianceType == US_OBDII &&
//				     gUserInput.eVehicleType == LD )
//				{
//					if ( IdIndex <= 0xF819 )
//					{
//						if ( gModelYear == 2019 || gModelYear == 2020 )
//						{
//							Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//							      "INF $%04X is not supported by any ECU! (Required for MY 2019 and later Light Duty vehicles)\n",
//							      IdIndex );
//						}
//						else
//						{
//							Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//							      "INF $%04X is not supported by any ECU! (Required for MY 2019 and later Light Duty vehicles)\n",
//							      IdIndex );
//							bTestFailed = TRUE;
//						}
//					}
//					else if ( IdIndex >= 0xF81A && IdIndex <= 0xF81C &&
//					          gUserInput.ePwrTrnType == PHEV )
//					{
//						if ( gModelYear == 2019 || gModelYear == 2020 )
//						{
//							Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//							      "INF $%04X is not supported by any ECU! (Required for MY 2019 and later Light Duty PHEV vehicles)\n",
//							      IdIndex );
//						}
//						else
//						{
//							Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//							      "INF $%04X is not supported by any ECU! (Required for MY 2019 and later Light Duty PHEV vehicles)\n",
//							      IdIndex );
//							bTestFailed = TRUE;
//						}
//					}
//				}
//
//				else if ( ( gUserInput.eComplianceType == EOBD_WITH_IUMPR ||
//				            gUserInput.eComplianceType == EOBD_NO_IUMPR ) &&
//				          gUserInput.eVehicleType == LD )
//				{
//					if ( IdIndex == 0xF817 )
//					{
//						if ( gModelYear == 2020 )
//						{
//							Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//							      "INF $%04X is not supported by any ECU! (Required for MY 2020 and later Light Duty vehicles)\n",
//							      IdIndex );
//						}
//						else if ( gModelYear > 2020 )
//						{
//							Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//							      "INF $%04X is not supported by any ECU! (Required for MY 2020 and later Light Duty vehicles)\n",
//							      IdIndex );
//							bTestFailed = TRUE;
//						}
//					}
//
//					else if ( IdIndex >= 0xF81A && IdIndex <= 0xF81C &&
//					          gUserInput.ePwrTrnType == PHEV )
//					{
//						if ( gModelYear == 2020 )
//						{
//							Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//							      "INF $%04X is not supported by any ECU! (Required for MY 2020 and later Light Duty PHEV vehicles)\n",
//							      IdIndex );
//						}
//						else if ( gModelYear > 2020 )
//						{
//							Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
//							      "INF $%04X is not supported by any ECU! (Required for MY 2020 and later Light Duty PHEV vehicles)\n",
//							      IdIndex );
//							bTestFailed = TRUE;
//						}
//					}
//				}
			}


			if ( gModelYear >= 2022 && HD )
			{
				if ( IdIndex >= 0xF841 && IdIndex <= 0xF844 )
				{
					if ( gModelYear == 2022 || gModelYear == 2023 )
					{
						Log ( WARNING, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
						      "INF $%04X is not supported by any ECU! (Required for MY 2022 and later Heavy Duty vehicles)\n",
						      IdIndex );
					}
					else
					{
						Log ( FAILURE, SCREENOUTPUTON, LOGOUTPUTON, NO_PROMPT,
						      "INF $%04X is not supported by any ECU! (Required for MY 2022 and later Heavy Duty vehicles)\n",
						      IdIndex );
						bTestFailed = TRUE;
					}
				}
			}
		}


		if ( IdIndex == 0x19 && MaxId == 0xF844 )
		{
			IdIndex = 0xF840;
		}
	} // end for ( IdIndex = 0x16; IdIndex <= 0x19; IdIndex++ )


	// If there where any errors in the responses, fail
	if ( bTestFailed == TRUE || ulInit_FailureCount != GetFailureCount () )
	{
		// There could have been early/late responses that weren't treated as FAIL
		return FAIL;
	}
	else
	{
		return PASS;
	}
}
